Package: ReliaGrowR
Title: Reliability Growth Analysis and Repairable Systems Modeling
Version: 0.7
Authors@R: person("Paul", "Govan", email = "paul.govan2@gmail.com", 
  role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0002-1821-8492"))
Description: Modeling and plotting functions for Reliability Growth Analysis (RGA) and Non-Homogeneous Poisson Process (NHPP) models for repairable systems. RGA models include the Duane (1962) <doi:10.1109/TA.1964.4319640>, NHPP by Crow (1975) (No. AMSAATR138), Piecewise Weibull NHPP by Guo et al. (2010) <doi:10.1109/RAMS.2010.5448029>, and Piecewise Weibull NHPP with Change Point Detection based on the 'segmented' package by Muggeo (2024) <https://cran.r-project.org/package=segmented>. Repairable systems functions include the Mean Cumulative Function (MCF) using the Nelson-Aalen estimator, parametric Power Law and Log-Linear NHPP models, and forecasting.
Imports: graphics, grDevices, plumber, segmented, stats
License: CC BY 4.0
Encoding: UTF-8
Suggests: ellmer, knitr, mcptools, pkgload, rmarkdown, spelling,
        testthat (>= 3.0.0), vdiffr, WeibullR
Language: en-US
URL: https://paulgovan.github.io/ReliaGrowR/,
        https://github.com/paulgovan/ReliaGrowR
Config/testthat/edition: 3
VignetteBuilder: knitr
BugReports: https://github.com/paulgovan/ReliaGrowR/issues
RoxygenNote: 7.3.3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-22 15:01:28 UTC; paulgovan
Author: Paul Govan [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-1821-8492>)
Maintainer: Paul Govan <paul.govan2@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-22 16:30:02 UTC
Built: R 4.5.2; ; 2026-05-22 16:49:36 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(gof,default)
S3method(gof,rga)
S3method(plot,duane)
S3method(plot,duane_predict)
S3method(plot,exposure)
S3method(plot,mcf)
S3method(plot,nhpp)
S3method(plot,nhpp_predict)
S3method(plot,rga)
S3method(plot,rga_predict)
S3method(print,duane)
S3method(print,duane_predict)
S3method(print,exposure)
S3method(print,gof)
S3method(print,mcf)
S3method(print,nhpp)
S3method(print,nhpp_predict)
S3method(print,rdt)
S3method(print,rga)
S3method(print,rga_predict)
export(duane)
export(exposure)
export(gof)
export(grwr_api)
export(mcf)
export(nhpp)
export(overlay_nhpp)
export(overlay_rga)
export(ppplot.rga)
export(predict_duane)
export(predict_nhpp)
export(predict_rga)
export(qqplot.rga)
export(rdt)
export(rga)
export(rga_mcp_server)
export(sim_failures)
export(weibull_to_rga)
import(plumber)
importFrom(grDevices,palette)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(segmented,intercept)
importFrom(segmented,seg.control)
importFrom(segmented,segmented)
importFrom(segmented,slope)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,aggregate)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,lsfit)
importFrom(stats,optim)
importFrom(stats,predict)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,uniroot)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ReliaGrowR <a href="https://paulgovan.github.io/ReliaGrowR/"><img src="man/figures/logo.png" align="right" height="139" alt="ReliaGrowR website" /></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN
status](https://www.r-pkg.org/badges/version/ReliaGrowR)](https://CRAN.R-project.org/package=ReliaGrowR)
[![R-CMD-check](https://github.com/paulgovan/ReliaGrowR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/paulgovan/ReliaGrowR/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/paulgovan/ReliaGrowR/graph/badge.svg)](https://app.codecov.io/gh/paulgovan/ReliaGrowR)
[![](http://cranlogs.r-pkg.org/badges/grand-total/ReliaGrowR)](https://cran.r-project.org/package=ReliaGrowR)
[![](http://cranlogs.r-pkg.org/badges/last-month/ReliaGrowR)](https://cran.r-project.org/package=ReliaGrowR)
[![](https://img.shields.io/badge/doi-10.32614/CRAN.package.ReliaGrowR-green.svg)](https://doi.org/10.32614/CRAN.package.ReliaGrowR)
<!-- badges: end -->

## Introduction

Welcome to **ReliaGrowR**! This package provides modeling and plotting
functions for Reliability Growth Analysis (RGA) and Repairable Systems
Modeling, including:

**Reliability Growth Analysis**

- Duane Analysis
- Crow-AMSAA
- Piecewise NHPP
- Piecewise NHPP with Change Point Detection

**Repairable Systems Modeling**

- Mean Cumulative Function (MCF)
- Power Law NHPP
- Log-Linear NHPP

RGA is focused on improving reliability during development and testing
of products, while repairable systems modeling tracks recurrence
patterns for fielded systems that can be repaired and returned to
service. By analyzing failure data, both approaches help engineers and
researchers identify trends, estimate reliability parameters, and
support decision-making over time.

To learn more about RGA, please view the [RGA
vignette](https://paulgovan.github.io/ReliaGrowR/articles/RGA.html). For
repairable systems modeling, see the [RSA
vignette](https://paulgovan.github.io/ReliaGrowR/articles/RSA.html).

## Installation

To install the release version of ReliaGrowR, use:

``` r
install_packages('ReliaGrowR')
```

You can install the development version of ReliaGrowR like so:

``` r
devtools::install_github('paulgovan/ReliaGrowR')
```

## Example

Here is a basic example of Reliability Growth Analysis. First, load the
package.

``` r
library(ReliaGrowR)
```

Next, suppose a machine has failed at the following times with the
corresponding number of failures.

``` r
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

To perform RGA, use the `rga` function and plot the results. The plot
displays the cumulative failures over time along with the fitted
reliability growth model.

``` r
result <- rga(times, failures)
plot(result, main = "Reliability Growth Analysis", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

<img src="man/figures/README-unnamed-chunk-4-1.png" width="100%" />

## Code of Conduct

Please note that the ReliaGrowR project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ReliaGrowR <a href="https://paulgovan.github.io/ReliaGrowR/"><img src="man/figures/logo.png" align="right" height="139" alt="ReliaGrowR website" /></a>

<!-- badges: start -->

[![Project Status: Active – The project has reached a stable, usable
state and is being actively
developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN
status](https://www.r-pkg.org/badges/version/ReliaGrowR)](https://CRAN.R-project.org/package=ReliaGrowR)
[![R-CMD-check](https://github.com/paulgovan/ReliaGrowR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/paulgovan/ReliaGrowR/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/paulgovan/ReliaGrowR/graph/badge.svg)](https://app.codecov.io/gh/paulgovan/ReliaGrowR)
[![](http://cranlogs.r-pkg.org/badges/grand-total/ReliaGrowR)](https://cran.r-project.org/package=ReliaGrowR)
[![](http://cranlogs.r-pkg.org/badges/last-month/ReliaGrowR)](https://cran.r-project.org/package=ReliaGrowR)
[![](https://img.shields.io/badge/doi-10.32614/CRAN.package.ReliaGrowR-green.svg)](https://doi.org/10.32614/CRAN.package.ReliaGrowR)
<!-- badges: end -->

## Introduction

Welcome to **ReliaGrowR**! This package provides modeling and plotting
functions for Reliability Growth Analysis (RGA) and Repairable Systems
Modeling, including:

**Reliability Growth Analysis**

- Duane Analysis
- Crow-AMSAA
- Piecewise NHPP
- Piecewise NHPP with Change Point Detection
- Forecasting (Duane and Crow-AMSAA)
- Goodness-of-Fit Testing (Cramér-von Mises, K-S)
- Reliability Demonstration Test (RDT) Planning

**Repairable Systems Modeling**

- Mean Cumulative Function (MCF)
- Power Law NHPP
- Log-Linear NHPP
- Forecasting

**AI Integration**

- MCP tools for AI assistants via `rga_mcp_server()`

RGA is focused on improving reliability during development and testing
of products, while repairable systems modeling tracks recurrence
patterns for fielded systems that can be repaired and returned to
service. By analyzing failure data, both approaches help engineers and
researchers identify trends, estimate reliability parameters, and
support decision-making over time.

To learn more about RGA, please view the [RGA
vignette](https://paulgovan.github.io/ReliaGrowR/articles/RGA.html). For
repairable systems modeling, see the [RSA
vignette](https://paulgovan.github.io/ReliaGrowR/articles/RSA.html). For
test planning, see the [RDT
vignette](https://paulgovan.github.io/ReliaGrowR/articles/RDT.html). For
AI integration, see the [MCP
vignette](https://paulgovan.github.io/ReliaGrowR/articles/MCP.html).

## Installation

To install the release version of ReliaGrowR, use:

``` r
install_packages('ReliaGrowR')
```

You can install the development version of ReliaGrowR like so:

``` r
devtools::install_github('paulgovan/ReliaGrowR')
```

## Example

Here is a basic example of Reliability Growth Analysis. First, load the
package.

``` r
library(ReliaGrowR)
```

Next, suppose a machine has failed at the following times with the
corresponding number of failures.

``` r
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

To perform RGA, use the `rga` function and plot the results. The plot
displays the cumulative failures over time along with the fitted
reliability growth model.

``` r
result <- rga(times, failures)
plot(result, main = "Reliability Growth Analysis", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

<img src="man/figures/README-unnamed-chunk-4-1.png" alt="" width="100%" />

## AI Integration via MCP

ReliaGrowR can expose its analysis functions as [Model Context Protocol
(MCP)](https://posit-dev.github.io/mcptools/) tools so AI assistants
like Claude can call them directly. First install the required packages:

``` r
install.packages(c("mcptools", "ellmer"))
```

Then add the server to Claude Code:

``` bash
claude mcp add -s user reliagrowR -- Rscript -e "ReliaGrowR::rga_mcp_server()"
```

Once configured, you can ask Claude to fit models, run forecasts, plan
demonstration tests, and more — all without writing any R code yourself.
See the [MCP
vignette](https://paulgovan.github.io/ReliaGrowR/articles/MCP.html) for
full setup instructions.

## Code of Conduct

Please note that the ReliaGrowR project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

# plumb-rga.R
# Functions rga(), print.rga(), plot.rga(), predict_rga(), print.rga_predict(),
# and plot.rga_predict() are provided by the ReliaGrowR package loaded via
# library(ReliaGrowR) in plumber.R. No local definitions needed.

# Generate the ReliaGrowR hex sticker
#
# Requires: hexSticker, ggplot2
# Usage: source("inst/hex_sticker.R")
#
# Output is saved to man/figures/logo.png

library(ggplot2)
library(hexSticker)

# --- Subplot: reliability growth curve with confidence band ---
set.seed(42)
t <- seq(0.5, 10, length.out = 80)
# Power law: N(t) = lambda * t^beta (beta < 1 = improving system)
y_fit <- 2.5 * t^0.65
y_lo <- y_fit * 0.75
y_hi <- y_fit * 1.30

# Simulated observed data points
t_obs <- c(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
y_obs <- 2.5 * t_obs^0.65 + rnorm(10, 0, 0.6)

df_ribbon <- data.frame(t = t, y = y_fit, lo = y_lo, hi = y_hi)
df_pts <- data.frame(t = t_obs, y = y_obs)

p <- ggplot() +
  geom_ribbon(data = df_ribbon, aes(x = t, ymin = lo, ymax = hi),
              fill = "#4FC3F7", alpha = 0.25) +
  geom_line(data = df_ribbon, aes(x = t, y = y),
            color = "#4FC3F7", linewidth = 1.0) +
  geom_point(data = df_pts, aes(x = t, y = y),
             color = "#B3E5FC", size = 1.2) +
  theme_void() +
  theme_transparent()

# --- Create hex sticker ---
sticker(p,
        package = "ReliaGrowR",
        p_size = 17,
        p_color = "#E3F2FD",
        p_y = 1.48,
        s_x = 1.0,
        s_y = 0.75,
        s_width = 1.2,
        s_height = 0.85,
        h_fill = "#0D1B3E",
        h_color = "#4FC3F7",
        h_size = 1.6,
        url = "paulgovan.github.io/ReliaGrowR",
        u_size = 3.5,
        u_color = "#64B5F6",
        filename = "man/figures/logo.png")

message("Hex sticker saved to man/figures/logo.png")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, message = FALSE, warning = FALSE----------------------------------
library(ReliaGrowR)
library(WeibullR)

## -----------------------------------------------------------------------------
failures <- c(
  500, 600, 650, 700, 730, 760, 790, 810, 830, 845,
  860, 875, 890, 905, 920, 935, 945, 960, 975, 990
)
suspensions <- rep(1000, 80)

n_test_failures <- length(failures)
n_surviving <- length(suspensions)

## -----------------------------------------------------------------------------
obj <- wblr(failures, suspensions,
  col = "steelblue", label = "Test Data", is.plot.cb = FALSE
)
obj <- wblr.fit(obj)
sim_beta <- obj$fit[[1]]$beta
eta_orig <- obj$fit[[1]]$eta
plot(obj,
  main = "Weibull Fit to Test Data",
  xlab = "Operating Time", ylab = "Unreliability"
)

## -----------------------------------------------------------------------------
rga_data <- weibull_to_rga(failures, suspensions)
rga_input <- data.frame(
  times = rga_data$CumulativeTime,
  failures = rga_data$Failures
)

## -----------------------------------------------------------------------------
test_end_cum_time <- max(rga_input$times)
fit <- rga(rga_input, times_type = "cumulative_failure_times")
growth_beta <- fit$betas
growth_lambda <- fit$lambdas

## ----results = "asis"---------------------------------------------------------
plot(fit,
  main = "Reliability Growth Analysis",
  xlab = "Cumulative Time", ylab = "Cumulative Failures"
)

## -----------------------------------------------------------------------------
n_forecast <- 60
n_target <- n_test_failures + n_forecast

t_forecast <- (n_target / growth_lambda)^(1 / growth_beta)
delta_cum_time <- t_forecast - test_end_cum_time
effective_fleet <- n_surviving - n_forecast / 2
per_unit_time <- delta_cum_time / effective_fleet

## ----warnings = FALSE---------------------------------------------------------
forecast_times <- seq(test_end_cum_time, t_forecast, length.out = 50)
fc <- predict_rga(fit, forecast_times)
plot(fc,
  main = "Reliability Growth Forecast",
  xlab = "Cumulative Time", ylab = "Cumulative Failures"
)

## -----------------------------------------------------------------------------
set.seed(123)
sim_result <- sim_failures(
  n = n_forecast,
  runtimes = rep(1000, n_surviving),
  window = per_unit_time,
  beta = sim_beta
)
sim_eta <- attr(sim_result, "weibull_eta")

## -----------------------------------------------------------------------------
sim_fail_times <- sim_result$runtime[sim_result$type == "Failure"]
sim_susp_times <- sim_result$runtime[sim_result$type == "Suspension"]
combined_failures <- c(failures, sim_fail_times)
combined_suspensions <- sim_susp_times

## -----------------------------------------------------------------------------
obj_growth <- wblr(combined_failures, combined_suspensions,
  col = "tomato", label = "With Growth", is.plot.cb = FALSE
)
obj_growth <- wblr.fit(obj_growth)

obj_nogrowth <- wblr(failures, suspensions,
  col = "grey40", label = "Without Growth", is.plot.cb = FALSE
)
obj_nogrowth <- wblr.fit(obj_nogrowth)

plot.wblr(list(obj_nogrowth, obj_growth),
  main = "Weibull Comparison: With vs. Without Reliability Growth",
  is.plot.legend = TRUE
)

## -----------------------------------------------------------------------------
growth_wb_beta <- obj_growth$fit[[1]]$beta
growth_wb_eta <- obj_growth$fit[[1]]$eta
nogrowth_wb_beta <- obj_nogrowth$fit[[1]]$beta
nogrowth_wb_eta <- obj_nogrowth$fit[[1]]$eta

params <- data.frame(
  Scenario = c("Without Growth", "With Growth"),
  Beta = round(c(nogrowth_wb_beta, growth_wb_beta), 3),
  Eta = round(c(nogrowth_wb_eta, growth_wb_eta), 1)
)

knitr::kable(params,
  caption = "Weibull parameters: with vs. without reliability growth",
  col.names = c("Scenario", "\u03b2", "\u03b7"),
  row.names = FALSE
)

## ----mc-helpers, include = FALSE----------------------------------------------
sim_and_fit <- function(test_failures, n_sim, runtimes, window, beta) {
  sim_i <- sim_failures(
    n = n_sim, runtimes = runtimes, window = window, beta = beta
  )
  sim_f <- sim_i$runtime[sim_i$type == "Failure"]
  sim_s <- sim_i$runtime[sim_i$type == "Suspension"]
  obj_i <- wblr(c(test_failures, sim_f), sim_s, is.plot.cb = FALSE)
  obj_i <- wblr.fit(obj_i)
  data.frame(beta = obj_i$fit[[1]]$beta, eta = obj_i$fit[[1]]$eta)
}

## ----mc-loop, message = FALSE, warning = FALSE, results = "hide"--------------
set.seed(99)
n_mc <- 500
mc_results <- vector("list", n_mc)

for (i in seq_len(n_mc)) {
  tryCatch(
    {
      mc_results[[i]] <- sim_and_fit(
        failures, n_forecast, rep(1000, n_surviving), per_unit_time, sim_beta
      )
    },
    error = function(e) NULL
  )
}

mc_df <- do.call(rbind, Filter(Negate(is.null), mc_results))

## ----mc-histograms, fig.height = 4--------------------------------------------
par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))

hist(mc_df$beta,
  breaks = "Sturges", col = "steelblue", border = "white",
  main = "MC Distribution of \u03b2",
  xlab = expression(hat(beta)), ylab = "Count", freq = TRUE
)
abline(v = median(mc_df$beta), lty = 2, lwd = 2)
abline(v = nogrowth_wb_beta, lty = 3, lwd = 2, col = "grey40")

hist(mc_df$eta,
  breaks = "Sturges", col = "tomato", border = "white",
  main = "MC Distribution of \u03b7",
  xlab = expression(hat(eta)), ylab = "Count", freq = TRUE
)
abline(v = median(mc_df$eta), lty = 2, lwd = 2)
abline(v = nogrowth_wb_eta, lty = 3, lwd = 2, col = "grey40")

par(mfrow = c(1, 1))

## ----mc-table-----------------------------------------------------------------
mc_summary <- data.frame(
  Parameter = c("\u03b2", "\u03b7"),
  NoGrowth = round(c(nogrowth_wb_beta, nogrowth_wb_eta), 3),
  Median = round(c(median(mc_df$beta), median(mc_df$eta)), 3),
  Mean = round(c(mean(mc_df$beta), mean(mc_df$eta)), 3),
  CI_lo = round(c(
    quantile(mc_df$beta, 0.025),
    quantile(mc_df$eta, 0.025)
  ), 3),
  CI_hi = round(c(
    quantile(mc_df$beta, 0.975),
    quantile(mc_df$eta, 0.975)
  ), 3)
)

knitr::kable(mc_summary,
  caption = paste0(
    "Monte Carlo summary of Weibull parameters (",
    nrow(mc_df), " valid iterations)"
  ),
  col.names = c(
    "Parameter", "No Growth", "Median", "Mean", "2.5%", "97.5%"
  ),
  row.names = FALSE
)

## ----sens-fail-setup----------------------------------------------------------
# Build failure scenarios by subsampling or extending the base failures
fail_10 <- failures[seq(1, 20, by = 2)]
fail_20 <- failures
fail_30 <- sort(c(failures, seq(505, 995, length.out = 10)))
fail_40 <- sort(c(failures, seq(510, 998, length.out = 20)))

test_fail_scenarios <- list(
  "10 failures" = list(f = fail_10, s = rep(1000, 90)),
  "20 failures" = list(f = fail_20, s = rep(1000, 80)),
  "30 failures" = list(f = fail_30, s = rep(1000, 70)),
  "40 failures" = list(f = fail_40, s = rep(1000, 60))
)

## ----sens-fail-mc, message = FALSE, warning = FALSE, results = "hide"---------
set.seed(42)
n_mc_sens <- 200

sens_fail_list <- lapply(names(test_fail_scenarios), function(sc_name) {
  sc <- test_fail_scenarios[[sc_name]]
  n_f <- length(sc$f)
  n_s <- length(sc$s)
  n_fc <- round(n_s * 0.75) # forecast 75% of survivors

  tryCatch(
    {
      # Fit Weibull
      obj_sc <- wblr(sc$f, sc$s, is.plot.cb = FALSE)
      obj_sc <- wblr.fit(obj_sc)
      beta_sc <- obj_sc$fit[[1]]$beta

      # Fit RGA
      rga_sc <- weibull_to_rga(sc$f, sc$s)
      rga_in <- data.frame(
        times = rga_sc$CumulativeTime,
        failures = rga_sc$Failures
      )
      fit_sc <- rga(rga_in, times_type = "cumulative_failure_times")

      # Forecast
      t_end_sc <- max(rga_in$times)
      n_tgt <- n_f + n_fc
      t_fc <- (n_tgt / fit_sc$lambdas)^(1 / fit_sc$betas)
      delta_sc <- t_fc - t_end_sc
      effective_sc <- n_s - n_fc / 2
      put_sc <- delta_sc / effective_sc

      if (put_sc <= 0) {
        return(NULL)
      }

      # MC loop
      rows <- lapply(seq_len(n_mc_sens), function(i) {
        tryCatch(
          {
            r <- sim_and_fit(sc$f, n_fc, rep(1000, n_s), put_sc, beta_sc)
            cbind(scenario = sc_name, r)
          },
          error = function(e) NULL
        )
      })
      do.call(rbind, Filter(Negate(is.null), rows))
    },
    error = function(e) NULL
  )
})

sens_fail_df <- do.call(rbind, Filter(Negate(is.null), sens_fail_list))
sens_fail_df$scenario <- factor(
  sens_fail_df$scenario,
  levels = names(test_fail_scenarios)
)

## ----sens-fail-boxplot, fig.height = 4----------------------------------------
par(mfrow = c(1, 2), mar = c(5, 4, 3, 1))

fail_cols <- c("#2E86C1", "#27AE60", "#F39C12", "#C0392B")
boxplot(beta ~ scenario,
  data = sens_fail_df,
  col = fail_cols, outline = FALSE,
  main = "Fitted \u03b2 by Test Failures",
  xlab = "Scenario", ylab = expression(hat(beta)),
  las = 2, cex.axis = 0.8
)

boxplot(eta ~ scenario,
  data = sens_fail_df,
  col = fail_cols, outline = FALSE,
  main = "Fitted \u03b7 by Test Failures",
  xlab = "Scenario", ylab = expression(hat(eta)),
  las = 2, cex.axis = 0.8
)

par(mfrow = c(1, 1))

## ----sens-helpers, include = FALSE--------------------------------------------
summarize_mc <- function(df) {
  do.call(rbind, lapply(levels(df$scenario), function(lbl) {
    sub <- df[df$scenario == lbl, ]
    if (nrow(sub) == 0) return(NULL)
    data.frame(
      Scenario = lbl, Valid = nrow(sub),
      Beta_med = round(median(sub$beta), 3),
      Eta_med = round(median(sub$eta), 1),
      Eta_lo = round(quantile(sub$eta, 0.025), 1),
      Eta_hi = round(quantile(sub$eta, 0.975), 1)
    )
  }))
}

## ----sens-fail-table----------------------------------------------------------
sens_fail_tbl <- summarize_mc(sens_fail_df)

knitr::kable(sens_fail_tbl,
  caption = paste0(
    "Growth-adjusted Weibull by test-failure scenario (",
    n_mc_sens, " iterations each)"
  ),
  col.names = c(
    "Scenario", "Valid", "Med. \u03b2",
    "Med. \u03b7", "2.5% \u03b7", "97.5% \u03b7"
  ),
  row.names = FALSE
)

## ----growth-setup-------------------------------------------------------------
growth_scenarios <- c(0.4, 0.6, 0.8, 1.0)
growth_labels <- c(
  "\u03b2g = 0.4 (strong)",
  "\u03b2g = 0.6 (moderate)",
  "\u03b2g = 0.8 (mild)",
  "\u03b2g = 1.0 (none)"
)

## ----growth-mc, message = FALSE, warning = FALSE, results = "hide"------------
set.seed(77)

sens_growth_list <- lapply(seq_along(growth_scenarios), function(k) {
  gb <- growth_scenarios[k]

  # Re-anchor lambda so model matches observed data, then forecast
  lambda_k <- n_test_failures / test_end_cum_time^gb
  t_fc_k <- (n_target / lambda_k)^(1 / gb)
  delta_k <- t_fc_k - test_end_cum_time
  put_k <- delta_k / effective_fleet

  if (put_k <= 0) {
    return(NULL)
  }

  rows <- lapply(seq_len(n_mc_sens), function(i) {
    tryCatch(
      {
        r <- sim_and_fit(
          failures, n_forecast, rep(1000, n_surviving), put_k, sim_beta
        )
        cbind(scenario = growth_labels[k], r)
      },
      error = function(e) NULL
    )
  })
  do.call(rbind, Filter(Negate(is.null), rows))
})

sens_growth_df <- do.call(rbind, Filter(Negate(is.null), sens_growth_list))
sens_growth_df$scenario <- factor(sens_growth_df$scenario, levels = growth_labels)

## ----growth-boxplot, fig.height = 4-------------------------------------------
par(mfrow = c(1, 2), mar = c(5, 4, 3, 1))

growth_cols <- c("#2E86C1", "#27AE60", "#F39C12", "#C0392B")
boxplot(beta ~ scenario,
  data = sens_growth_df,
  col = growth_cols, outline = FALSE,
  main = "Fitted \u03b2 by Growth Strength",
  xlab = "Growth scenario", ylab = expression(hat(beta)),
  las = 2, cex.axis = 0.75
)

boxplot(eta ~ scenario,
  data = sens_growth_df,
  col = growth_cols, outline = FALSE,
  main = "Fitted \u03b7 by Growth Strength",
  xlab = "Growth scenario", ylab = expression(hat(eta)),
  las = 2, cex.axis = 0.75
)

par(mfrow = c(1, 1))

## ----growth-table-------------------------------------------------------------
sens_growth_tbl <- summarize_mc(sens_growth_df)

knitr::kable(sens_growth_tbl,
  caption = paste0(
    "Growth-adjusted Weibull by growth parameter scenario (",
    n_mc_sens, " iterations each)"
  ),
  col.names = c(
    "Scenario", "Valid", "Med. \u03b2",
    "Med. \u03b7", "2.5% \u03b7", "97.5% \u03b7"
  ),
  row.names = FALSE
)

# plumber.R

#* @apiTitle ReliaGrowR API
#* @apiDescription An API for the ReliaGrowR package
#* @apiContact list(name = "API Issues", url = "https://github.com/paulgovan/ReliaGrowR/issues")
#* @apiLicense list(name = "CC-BY-4.0", url = "https://creativecommons.org/licenses/by/4.0/")
#* @apiVersion 0.2

library(ReliaGrowR)

# Source scripts (comment-only stubs kept for organisational clarity)
source("plumb-duane.R")
source("plumb-rga.R")
source("plumb-gof.R")

# ---------------------------------------------------------------------------
# Serialisation helpers — convert S3 objects to plain lists for JSON output.
# (jsonlite has no asJSON method for custom S3 classes, and rga/rga_predict
#  embed lm objects that contain environments which cannot be serialised.)
# ---------------------------------------------------------------------------

.rga_as_list <- function(fit) {
  betas <- if (is.list(fit$betas)) {
    as.numeric(fit$betas$log_times[, "Est."])
  } else {
    as.numeric(fit$betas)
  }
  betas_se <- as.numeric(fit$betas_se)
  lambdas  <- if (is.matrix(fit$lambdas)) {
    as.numeric(fit$lambdas[, "Est."])
  } else {
    as.numeric(fit$lambdas)
  }
  breakpoints <- if (!is.null(fit$breakpoints)) as.numeric(fit$breakpoints) else NULL

  list(
    times         = as.numeric(fit$times),
    failures      = as.numeric(fit$failures),
    n_obs         = fit$n_obs,
    cum_failures  = as.numeric(fit$cum_failures),
    residuals     = as.numeric(fit$residuals),
    logLik        = fit$logLik,
    AIC           = fit$AIC,
    BIC           = fit$BIC,
    method        = fit$method,
    breakpoints   = breakpoints,
    fitted_values = as.numeric(fit$fitted_values),
    lower_bounds  = as.numeric(fit$lower_bounds),
    upper_bounds  = as.numeric(fit$upper_bounds),
    growth_rate   = as.numeric(fit$growth_rate),
    betas         = betas,
    betas_se      = betas_se,
    lambdas       = lambdas
  )
}

.rga_predict_as_list <- function(fc) {
  list(
    times        = as.numeric(fc$times),
    cum_failures = as.numeric(fc$cum_failures),
    lower_bounds = as.numeric(fc$lower_bounds),
    upper_bounds = as.numeric(fc$upper_bounds),
    conf_level   = fc$conf_level,
    model_type   = fc$model_type
  )
}

.duane_as_list <- function(d) {
  cb <- d$Confidence_Bounds
  list(
    times              = as.numeric(d$times),
    failures           = as.numeric(d$failures),
    n_obs              = d$n_obs,
    MTBF               = as.numeric(d$MTBF),
    logLik             = d$logLik,
    AIC                = d$AIC,
    BIC                = d$BIC,
    conf.level         = d$conf.level,
    Cumulative_Time    = as.numeric(d$Cumulative_Time),
    Cumulative_MTBF    = as.numeric(d$Cumulative_MTBF),
    Fitted_Values      = as.numeric(d$Fitted_Values),
    lower_bounds       = as.numeric(d$lower_bounds),
    upper_bounds       = as.numeric(d$upper_bounds),
    Confidence_Bounds  = if (!is.null(cb)) as.data.frame(cb) else NULL,
    Residuals_Log      = as.numeric(d$Residuals_Log),
    Residuals_MTBF     = as.numeric(d$Residuals_MTBF)
  )
}

# ---------------------------------------------------------------------------
# Duane
# ---------------------------------------------------------------------------

#* Run a Duane Analysis
#* @param times Cumulative failure times (comma separated)
#* @param failures The number of failures at each corresponding time in times (comma separated)
#* @param conf.level:numeric Confidence level for intervals (default: 0.95)
#* @post /duane
function(times, failures, conf.level = 0.95) {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  conf.level <- as.numeric(conf.level)

  result <- duane(times = times, failures = failures,
                  conf.level = conf.level)

  return(.duane_as_list(result))
}

#* Plot Method for Duane Analysis
#* @serializer png
#* @param times Cumulative failure times (comma separated)
#* @param failures The number of failures at each corresponding time in times (comma separated)
#* @param conf.level:numeric Confidence level for intervals (default: 0.95)
#* @param log:boolean Whether to use logarithmic scales (default: TRUE)
#* @param conf.int:boolean Whether to plot confidence bounds (default: TRUE)
#* @post /plot.duane
function(times, failures, conf.level = 0.95, log = TRUE, conf.int = TRUE) {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  conf.level <- as.numeric(conf.level)
  log <- as.logical(log)
  conf.int <- as.logical(conf.int)

  result <- duane(times = times, failures = failures,
                  conf.level = conf.level)
  plot(result, log = log, conf.int = conf.int)
}

# ---------------------------------------------------------------------------
# RGA
# ---------------------------------------------------------------------------

#* Run a Crow-AMSAA or Piecewise NHPP Analysis
#* @param times Cumulative failure times (comma separated)
#* @param failures The number of failures at each corresponding time in times (comma separated)
#* @param model_type Type of model: "Crow-AMSAA" or "Piecewise NHPP" (default: "Crow-AMSAA")
#* @param breaks Cumulative time breakpoints for Piecewise NHPP (comma separated, optional)
#* @param conf_level Confidence level for intervals (default: 0.95)
#* @param method Estimation method: "LS" (default) or "MLE"
#* @post /rga
function(times, failures, model_type = "Crow-AMSAA", breaks = NULL,
         conf_level = 0.95, method = "LS") {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  conf_level <- as.numeric(conf_level)
  method <- match.arg(method, c("LS", "MLE"))
  if (!is.null(breaks))
    breaks <- as.numeric(unlist(strsplit(breaks, ",")))

  result <- rga(times = times, failures = failures,
                model_type = model_type, breaks = breaks,
                conf_level = conf_level, method = method)
  return(.rga_as_list(result))
}

#* Plot Method for RGA Objects
#* @serializer png
#* @param times Cumulative failure times (comma separated)
#* @param failures The number of failures at each corresponding time in times (comma separated)
#* @param model_type Type of model: "Crow-AMSAA" or "Piecewise NHPP" (default: "Crow-AMSAA")
#* @param breaks Cumulative time breakpoints for Piecewise NHPP (comma separated, optional)
#* @param conf_level Confidence level for intervals (default: 0.95)
#* @param method Estimation method: "LS" (default) or "MLE"
#* @param conf_bounds:boolean Whether to include confidence bounds (default: TRUE)
#* @param legend:boolean Whether to show the legend (default: TRUE)
#* @param log:boolean Whether to use a log-log scale (default: FALSE)
#* @param legend_pos Position of the legend (default: "bottomright")
#* @post /plot.rga
function(times, failures, model_type = "Crow-AMSAA", breaks = NULL,
         conf_level = 0.95, method = "LS",
         conf_bounds = TRUE, legend = TRUE, log = FALSE,
         legend_pos = "bottomright") {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  model_type <- as.character(model_type)
  conf_level <- as.numeric(conf_level)
  method <- match.arg(method, c("LS", "MLE"))
  if (!is.null(breaks))
    breaks <- as.numeric(unlist(strsplit(breaks, ",")))
  conf_bounds <- as.logical(conf_bounds)
  legend <- as.logical(legend)
  log <- as.logical(log)
  legend_pos <- as.character(legend_pos)

  result <- rga(times = times, failures = failures,
                model_type = model_type, breaks = breaks,
                conf_level = conf_level, method = method)
  plot(result, conf_bounds = conf_bounds, legend = legend,
       log = log, legend_pos = legend_pos)
}

# ---------------------------------------------------------------------------
# predict_rga
# ---------------------------------------------------------------------------

#* Forecast Cumulative Failures from a Fitted RGA Model
#* @param times Cumulative failure times used to fit the model (comma separated)
#* @param failures Number of failures at each time (comma separated)
#* @param predict_times Cumulative times for forecasting (comma separated)
#* @param model_type Type of model: "Crow-AMSAA" or "Piecewise NHPP" (default: "Crow-AMSAA")
#* @param method Estimation method: "LS" (default) or "MLE"
#* @param conf_level Confidence level (default: 0.95)
#* @post /predict_rga
function(times, failures, predict_times, model_type = "Crow-AMSAA",
         method = "LS", conf_level = 0.95) {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  predict_times <- as.numeric(unlist(strsplit(predict_times, ",")))
  conf_level <- as.numeric(conf_level)
  method <- match.arg(method, c("LS", "MLE"))

  fit <- rga(times = times, failures = failures,
             model_type = model_type, method = method,
             conf_level = conf_level)
  fc <- predict_rga(fit, times = predict_times, conf_level = conf_level)
  .rga_predict_as_list(fc)
}

#* Plot Forecast from a Fitted RGA Model
#* @serializer png
#* @param times Cumulative failure times used to fit the model (comma separated)
#* @param failures Number of failures at each time (comma separated)
#* @param predict_times Cumulative times for forecasting (comma separated)
#* @param model_type Type of model: "Crow-AMSAA" or "Piecewise NHPP" (default: "Crow-AMSAA")
#* @param method Estimation method: "LS" (default) or "MLE"
#* @param conf_level Confidence level (default: 0.95)
#* @param conf_bounds:boolean Whether to include confidence bounds (default: TRUE)
#* @param legend:boolean Whether to show the legend (default: TRUE)
#* @param legend_pos Position of the legend (default: "bottomright")
#* @post /plot.predict_rga
function(times, failures, predict_times, model_type = "Crow-AMSAA",
         method = "LS", conf_level = 0.95,
         conf_bounds = TRUE, legend = TRUE, legend_pos = "bottomright") {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  predict_times <- as.numeric(unlist(strsplit(predict_times, ",")))
  conf_level <- as.numeric(conf_level)
  method <- match.arg(method, c("LS", "MLE"))
  conf_bounds <- as.logical(conf_bounds)
  legend <- as.logical(legend)
  legend_pos <- as.character(legend_pos)

  fit <- rga(times = times, failures = failures,
             model_type = model_type, method = method,
             conf_level = conf_level)
  fc <- predict_rga(fit, times = predict_times, conf_level = conf_level)
  plot(fc, conf_bounds = conf_bounds, legend = legend, legend_pos = legend_pos)
}

# ---------------------------------------------------------------------------
# RDT
# ---------------------------------------------------------------------------

#* Calculate Required Sample Size or Test Time for Reliability Demonstration Test
#* @param target Target reliability to demonstrate (e.g., 0.9)
#* @param mission_time Mission time for the reliability target (e.g., 1000 hours)
#* @param conf_level Confidence level for the demonstration (e.g., 0.95)
#* @param beta Shape parameter of the Weibull distribution (default: 1 for Exponential)
#* @param f Allowable failures during the test (non-negative integer, default: 0)
#* @param n Sample size (number of units to test, optional)
#* @param test_time Test time per unit (optional)
#* @post /rdt
function(target, mission_time, conf_level, beta = 1, f = 0,
         n = NULL, test_time = NULL) {

  target <- as.numeric(target)
  mission_time <- as.numeric(mission_time)
  conf_level <- as.numeric(conf_level)
  beta <- as.numeric(beta)
  f <- as.integer(f)
  if (!is.null(n)) n <- as.integer(n)
  if (!is.null(test_time)) test_time <- as.numeric(test_time)

  result <- rdt(target = target, mission_time = mission_time, conf_level = conf_level,
                beta = beta, f = f, n = n, test_time = test_time)
  unclass(result)
}

# ---------------------------------------------------------------------------
# weibull_to_rga
# ---------------------------------------------------------------------------

#* Convert Weibull Failure/Suspension/Interval Data to RGA Format
#* @param failures Exact failure times (comma separated)
#* @param suspensions Right-censored times (comma separated, optional)
#* @param interval_starts Interval lower bounds (comma separated, optional)
#* @param interval_ends Interval upper bounds (comma separated, optional)
#* @post /weibull_to_rga
function(failures, suspensions = NULL, interval_starts = NULL,
         interval_ends = NULL) {

  failures <- as.numeric(unlist(strsplit(failures, ",")))
  if (!is.null(suspensions))
    suspensions <- as.numeric(unlist(strsplit(suspensions, ",")))
  if (!is.null(interval_starts))
    interval_starts <- as.numeric(unlist(strsplit(interval_starts, ",")))
  if (!is.null(interval_ends))
    interval_ends <- as.numeric(unlist(strsplit(interval_ends, ",")))

  weibull_to_rga(failures, suspensions, interval_starts, interval_ends)
}

# ---------------------------------------------------------------------------
# sim_failures
# ---------------------------------------------------------------------------

#* Simulate Failures from a Conditional Weibull Model
#* @param n Number of failures to simulate (positive integer)
#* @param runtimes Current runtimes for each unit (comma separated)
#* @param replace:boolean Sampling with replacement (default: FALSE)
#* @param window:numeric Observation window width (optional)
#* @post /sim_failures
function(n, runtimes, replace = FALSE, window = NULL) {

  n <- as.integer(n)
  runtimes <- as.numeric(unlist(strsplit(runtimes, ",")))
  replace <- as.logical(replace)
  if (!is.null(window)) window <- as.numeric(window)

  sim_failures(n, runtimes, replace = replace, window = window)
}

# ---------------------------------------------------------------------------
# GoF plots
# ---------------------------------------------------------------------------

#* Q-Q Plot for a Fitted RGA Model
#* @serializer png
#* @param times Cumulative failure times (comma separated)
#* @param failures Number of failures at each time (comma separated)
#* @param model_type Type of model: "Crow-AMSAA" or "Piecewise NHPP" (default: "Crow-AMSAA")
#* @param method Estimation method: "LS" (default) or "MLE"
#* @param conf_level Confidence level (default: 0.95)
#* @param main Plot title (default: "Q-Q Plot")
#* @post /qqplot.rga
function(times, failures, model_type = "Crow-AMSAA",
         method = "LS", conf_level = 0.95, main = "Q-Q Plot") {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  conf_level <- as.numeric(conf_level)
  method <- match.arg(method, c("LS", "MLE"))

  fit <- rga(times = times, failures = failures,
             model_type = model_type, method = method,
             conf_level = conf_level)
  qqplot.rga(fit, main = main)
}

#* P-P Plot for a Fitted RGA Model
#* @serializer png
#* @param times Cumulative failure times (comma separated)
#* @param failures Number of failures at each time (comma separated)
#* @param model_type Type of model: "Crow-AMSAA" or "Piecewise NHPP" (default: "Crow-AMSAA")
#* @param method Estimation method: "LS" (default) or "MLE"
#* @param conf_level Confidence level (default: 0.95)
#* @param main Plot title (default: "P-P Plot")
#* @post /ppplot.rga
function(times, failures, model_type = "Crow-AMSAA",
         method = "LS", conf_level = 0.95, main = "P-P Plot") {

  times <- as.numeric(unlist(strsplit(times, ",")))
  failures <- as.numeric(unlist(strsplit(failures, ",")))
  conf_level <- as.numeric(conf_level)
  method <- match.arg(method, c("LS", "MLE"))

  fit <- rga(times = times, failures = failures,
             model_type = model_type, method = method,
             conf_level = conf_level)
  ppplot.rga(fit, main = main)
}

#' ReliaGrowR API
#'
#' This function provides an interface to the ReliaGrowR API.#' This function provides an interface to the ReliaGrowR API.
#' @import plumber
#' @return Launches the ReliaGrowR API on a local server.
#' @examples
#' \dontrun{
#' grwr_api()
#' }
#' @export
grwr_api <- function() {
  # Run the API
  root <- plumber::pr("inst/plumber/plumber.R")
  root %>% plumber::pr_run()
}

#' Reliability Test Data
#'
#' A dataset containing example reliability test data from the military report
#' "Reliability Growth Prediction" (1986) by The Analytical Sciences Corporation.
#' This dataset includes cumulative ETI, failure counts, cumulative MTBF, report
#' numbers, flags, and causes for two different LRUs (G1 and G2).
#'
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G5.0} The data set is a standard data set from a published paper.
#' @srrstats {G5.1} The data set is created within and used to test the package.
#' The data set is exported so that users can confirm tests and run examples.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @format @format ## `testdata`
#' A data frame with 25 rows and 6 variables:
#' \describe{
#'  \item{LRU}{The Line Replaceable Unit identifier (G1 or G2).}
#'  \item{Cum_ETI}{Cumulative Equivalent Test Hours (ETI).}
#'  \item{Failure_Count}{Cumulative number of failures observed.}
#'  \item{Cum_MTBF}{Cumulative Mean Time Between Failures (MTBF).}
#'  \item{Report_No}{Report number associated with the failure.}
#'  \item{Flag}{A flag indicating special conditions or notes.}
#'  \item{Cause}{Cause of the failure (e.g., D for Design, M for Manufacturing, R for Random, NR for No Report).}
#'  }
#'  @usage data(testdata)
#' @examples
#' data(testdata)
#' head(testdata)
#' summary(testdata)
#' str(testdata)
#'
"testdata"

#' Reliability Demonstration Test (RDT) Plan Calculator
#'
#' This function calculates the required test time or sample size for a Reliability
#' Demonstration Test (RDT) based on specified reliability, mission time, confidence
#' level, and Weibull shape parameter.
#'
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.8} Sub-function `print.rdt` is provided for the `rdt` class.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical
#' algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param target Required reliability at mission time (0 < target < 1).
#' @param mission_time Mission duration (time units). Must be greater than 0.
#' @param conf_level Desired confidence level (e.g., 0.9 for 90% confidence). The
#' confidence level must be between 0 and 1 (exclusive).
#' @param beta Weibull shape parameter (beta=1 corresponds to exponential distribution).
#' Must be greater than 0. Default is 1.
#' @param f Number of allowable failures during the test (non-negative integer). Default is 0
#' (zero-failure test plan). Increasing `f` reduces the required test time or sample size at
#' the cost of accepting more observed failures.
#' @param n Sample size (optional, supply if solving for test_time). Must be a positive integer.
#' @param test_time Test time per unit (optional, supply if solving for n). Must be greater than 0.
#' @return The function returns an object of class `rdt` that contains:
#' \item{Distribution}{Type of distribution used (Exponential or Weibull).}
#' \item{Beta}{Weibull shape parameter.}
#' \item{Allowed_Failures}{Number of allowable failures during the test.}
#' \item{Target_Reliability}{Specified target reliability.}
#' \item{Mission_Time}{Specified mission time.}
#' \item{Required_Test_Time}{Calculated required test time (if n is provided).}
#' \item{Input_Sample_Size}{Provided sample size (if test_time is calculated).}
#' \item{Required_Sample_Size}{Calculated required sample size (if test_time is provided).}
#' \item{Input_Test_Time}{Provided test time (if n is calculated).}
#'
#' @examples
#' #' # Example 1: Calculate required test time
#' plan1 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, n = 10)
#' print(plan1)
#' # Example 2: Calculate required sample size
#' plan2 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, test_time = 2000)
#' print(plan2)
#' @importFrom stats qchisq
#' @export
rdt <- function(target, mission_time, conf_level,
                beta = 1, f = 0, n = NULL, test_time = NULL) {
  # Input validation
  if (!is.numeric(target) || length(target) != 1 || !is.finite(target)) {
    stop("'target' must be a single finite numeric value.")
  }
  if (target <= 0 || target >= 1) {
    stop("'target' must be between 0 and 1 (exclusive).")
  }

  if (!is.numeric(mission_time) || length(mission_time) != 1 || !is.finite(mission_time)) {
    stop("'mission_time' must be a single finite numeric value.")
  }
  if (mission_time <= 0) {
    stop("'mission_time' must be greater than 0.")
  }

  if (!is.numeric(conf_level) || length(conf_level) != 1 || !is.finite(conf_level)) {
    stop("'conf_level' must be a single finite numeric value.")
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1 (exclusive).")
  }

  if (!is.numeric(beta) || length(beta) != 1 || !is.finite(beta)) {
    stop("'beta' must be a single finite numeric value.")
  }
  if (beta <= 0) {
    stop("'beta' must be greater than 0.")
  }

  if (!is.numeric(f) || length(f) != 1 || !is.finite(f)) {
    stop("'f' must be a single finite numeric value.")
  }
  if (f < 0 || f != floor(f)) {
    stop("'f' must be a non-negative integer.")
  }

  if (!is.null(n) && !is.null(test_time)) {
    stop("Provide only one of 'n' (sample size) or 'test_time', not both.")
  }
  if (is.null(n) && is.null(test_time)) {
    stop("Provide exactly one of 'n' (sample size) or 'test_time'.")
  }

  if (!is.null(n)) {
    if (!is.numeric(n) || length(n) != 1 || !is.finite(n)) {
      stop("'n' must be a single finite numeric value.")
    }
    if (n <= 0 || n != floor(n)) {
      stop("'n' must be a positive integer.")
    }
  }

  if (!is.null(test_time)) {
    if (!is.numeric(test_time) || length(test_time) != 1 || !is.finite(test_time)) {
      stop("'test_time' must be a single finite numeric value.")
    }
    if (test_time <= 0) {
      stop("'test_time' must be greater than 0.")
    }
  }

  # Scale parameter under H0 (Weibull with specified beta)
  eta0 <- mission_time / (-log(target))^(1 / beta)

  # Chi-squared critical value for f allowed failures; df=2*(f+1) reduces to
  # -log(1-conf_level) when f=0, preserving backward compatibility.
  chi2_val <- qchisq(conf_level, df = 2 * (f + 1))

  if (!is.null(n) & is.null(test_time)) {
    # Solve for required test time
    T_req <- eta0 * (chi2_val / (2 * n))^(1 / beta)
    result <- list(
      Distribution = ifelse(beta == 1, "Exponential", "Weibull"),
      Beta = beta,
      Allowed_Failures = f,
      Target_Reliability = target,
      Mission_Time = mission_time,
      Required_Test_Time = T_req,
      Input_Sample_Size = n
    )
  } else if (is.null(n) & !is.null(test_time)) {
    # Solve for required sample size
    n_req <- ceiling(chi2_val / (2 * (test_time / eta0)^beta))
    result <- list(
      Distribution = ifelse(beta == 1, "Exponential", "Weibull"),
      Beta = beta,
      Allowed_Failures = f,
      Target_Reliability = target,
      Mission_Time = mission_time,
      Required_Sample_Size = n_req,
      Input_Test_Time = test_time
    )
  } else {
    stop("Please provide exactly one of n (sample size) or test_time (test time).")
  }

  class(result) <- "rdt"
  return(result)
}

#' Print method for rdt objects
#'
#' This function provides a formatted print method for objects of class `rdt`.
#'
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.8} Sub-function `print.rdt` is provided for the `rdt` class.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical
#' algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class `rdt`.
#' @param ... Additional arguments (not used).
#' @examples
#' plan <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, n = 10)
#' print(plan)
#' @return Invisibly returns the input object.
#' @export
print.rdt <- function(x, ...) {
  # Input validation
  if (!inherits(x, "rdt")) {
    stop("'x' must be an object of class 'rdt'.")
  }

  cat("Reliability Demonstration Test (RDT) Plan\n")
  cat("-----------------------------------------\n")
  cat("Distribution: ", x$Distribution, "\n")
  cat("Weibull Shape Parameter (Beta): ", x$Beta, "\n")
  cat("Allowed Failures (f): ", x$Allowed_Failures, "\n")
  cat("Target Reliability: ", x$Target_Reliability, "\n")
  cat("Mission Time: ", x$Mission_Time, "\n")

  if (!is.null(x$Required_Test_Time)) {
    cat("Input Sample Size (n): ", x$Input_Sample_Size, "\n")
    cat("Required Test Time (T): ", round(x$Required_Test_Time, 2), "\n")
  } else if (!is.null(x$Required_Sample_Size)) {
    cat("Input Test Time (T): ", x$Input_Test_Time, "\n")
    cat("Required Sample Size (n): ", x$Required_Sample_Size, "\n")
  }
}

#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical
#' algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.

test_that("rdt() input validation errors", {
  # target
  expect_error(
    rdt("a", 100, 0.9, n = 10),
    "'target' must be a single finite numeric value."
  )
  expect_error(
    rdt(c(0.9, 0.8), 100, 0.9, n = 10),
    "'target' must be a single finite numeric value."
  )
  expect_error(
    rdt(Inf, 100, 0.9, n = 10),
    "'target' must be a single finite numeric value."
  )
  # expect_error(rdt(0, 100, 0.9, n = 10),
  #              "'target' must be between 0 and 1 (exclusive).")
  # expect_error(rdt(1, 100, 0.9, n = 10),
  #              "'target' must be between 0 and 1 (exclusive).")

  # mission_time
  expect_error(
    rdt(0.9, "a", 0.9, n = 10),
    "'mission_time' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, c(100, 200), 0.9, n = 10),
    "'mission_time' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, Inf, 0.9, n = 10),
    "'mission_time' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 0, 0.9, n = 10),
    "'mission_time' must be greater than 0."
  )

  # conf_level
  expect_error(
    rdt(0.9, 100, "a", n = 10),
    "'conf_level' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, c(0.9, 0.8), n = 10),
    "'conf_level' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, Inf, n = 10),
    "'conf_level' must be a single finite numeric value."
  )
  # expect_error(rdt(0.9, 100, 0, n = 10),
  #              "'conf_level' must be between 0 and 1 (exclusive).")
  # expect_error(rdt(0.9, 100, 1, n = 10),
  #              "'conf_level' must be between 0 and 1 (exclusive).")

  # beta
  expect_error(
    rdt(0.9, 100, 0.9, beta = "a", n = 10),
    "'beta' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, beta = c(1, 2), n = 10),
    "'beta' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, beta = Inf, n = 10),
    "'beta' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, beta = 0, n = 10),
    "'beta' must be greater than 0."
  )

  # n vs test_time mutual exclusivity
  expect_error(
    rdt(0.9, 100, 0.9, n = 10, test_time = 200),
    "Provide only one of 'n' \\(sample size\\) or 'test_time', not both."
  )
  expect_error(
    rdt(0.9, 100, 0.9),
    "Provide exactly one of 'n' \\(sample size\\) or 'test_time'."
  )

  # n validation
  expect_error(
    rdt(0.9, 100, 0.9, n = "a"),
    "'n' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, n = c(1, 2)),
    "'n' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, n = Inf),
    "'n' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, n = 0),
    "'n' must be a positive integer."
  )
  expect_error(
    rdt(0.9, 100, 0.9, n = 3.5),
    "'n' must be a positive integer."
  )

  # test_time validation
  expect_error(
    rdt(0.9, 100, 0.9, test_time = "a"),
    "'test_time' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, test_time = c(1, 2)),
    "'test_time' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, test_time = Inf),
    "'test_time' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, test_time = 0),
    "'test_time' must be greater than 0."
  )
})

test_that("print.rdt() input validation errors", {
  expect_error(
    print.rdt(list()),
    "'x' must be an object of class 'rdt'."
  )
})

test_that("RDT with sample size input is stable to trivial noise", {
  set.seed(123)

  # Base parameters
  target <- 0.9
  mission_time <- 1000
  conf_level <- 0.9
  beta <- 1
  n <- 10

  base_result <- rdt(target, mission_time, conf_level, beta, n = n)

  # Apply very small noise to continuous parameters
  noise <- function(x) x + rnorm(1, 0, 1e-8)
  noisy_result <- rdt(noise(target), noise(mission_time), noise(conf_level),
    noise(beta),
    n = n
  )

  # Sample size input stays exact
  expect_equal(base_result$Input_Sample_Size, noisy_result$Input_Sample_Size)

  # Required test time should be nearly identical
  expect_equal(base_result$Required_Test_Time,
    noisy_result$Required_Test_Time,
    tolerance = 1e-6
  )
})

test_that("RDT with test time input is stable to trivial noise", {
  set.seed(456)

  # Base parameters
  target <- 0.9
  mission_time <- 1000
  conf_level <- 0.9
  beta <- 1
  test_time <- 2000

  base_result <- rdt(target, mission_time, conf_level, beta, test_time = test_time)

  # Apply very small noise
  noise <- function(x) x + rnorm(1, 0, 1e-8)
  noisy_result <- rdt(noise(target), noise(mission_time), noise(conf_level),
    noise(beta),
    test_time = noise(test_time)
  )

  # Input test time should remain close
  expect_equal(base_result$Input_Test_Time,
    noisy_result$Input_Test_Time,
    tolerance = 1e-6
  )

  # Required sample size is integer-valued; expect identical result
  expect_equal(base_result$Required_Sample_Size, noisy_result$Required_Sample_Size)
})

test_that("print.rdt works for required test time", {
  plan <- rdt(
    target = 0.9, mission_time = 1000,
    conf_level = 0.9, beta = 1, n = 10
  )

  # Ensure output contains expected lines
  expect_output(print(plan), "Reliability Demonstration Test \\(RDT\\) Plan")
  expect_output(print(plan), "Distribution:")
  expect_output(print(plan), "Weibull Shape Parameter")
  expect_output(print(plan), "Target Reliability")
  expect_output(print(plan), "Mission Time")
  expect_output(print(plan), "Input Sample Size")
  expect_output(print(plan), "Required Test Time")

  # Invisibly returns the input object
  expect_invisible(print(plan))
})

test_that("print.rdt works for required sample size", {
  plan <- rdt(
    target = 0.9, mission_time = 1000,
    conf_level = 0.9, beta = 1, test_time = 2000
  )

  expect_output(print(plan), "Reliability Demonstration Test \\(RDT\\) Plan")
  expect_output(print(plan), "Distribution:")
  expect_output(print(plan), "Weibull Shape Parameter")
  expect_output(print(plan), "Target Reliability")
  expect_output(print(plan), "Mission Time")
  expect_output(print(plan), "Input Test Time")
  expect_output(print(plan), "Required Sample Size")

  expect_invisible(print(plan))
})

test_that("print.rdt errors on wrong input", {
  expect_error(print.rdt(123), "'x' must be an object of class 'rdt'")
  expect_error(print.rdt(list()), "'x' must be an object of class 'rdt'")
})

test_that("rdt() validates f parameter", {
  expect_error(
    rdt(0.9, 100, 0.9, f = "a", n = 10),
    "'f' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, f = c(0, 1), n = 10),
    "'f' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, f = Inf, n = 10),
    "'f' must be a single finite numeric value."
  )
  expect_error(
    rdt(0.9, 100, 0.9, f = -1, n = 10),
    "'f' must be a non-negative integer."
  )
  expect_error(
    rdt(0.9, 100, 0.9, f = 1.5, n = 10),
    "'f' must be a non-negative integer."
  )
})

test_that("rdt() f=0 is backward compatible with default", {
  plan_default <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, n = 10)
  plan_f0 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 0, n = 10)
  expect_equal(plan_default$Required_Test_Time, plan_f0$Required_Test_Time)
  expect_equal(plan_f0$Allowed_Failures, 0)

  plan_default2 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, test_time = 2000)
  plan_f0_2 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 0, test_time = 2000)
  expect_equal(plan_default2$Required_Sample_Size, plan_f0_2$Required_Sample_Size)
})

test_that("rdt() f>0 correctness against chi-squared formula", {
  target <- 0.9
  mt <- 1000
  cl <- 0.9
  f <- 2
  eta0 <- mt / (-log(target))

  # Solve for test time
  expected_T <- eta0 * qchisq(cl, df = 2 * (f + 1)) / (2 * 10)
  plan <- rdt(target = target, mission_time = mt, conf_level = cl, beta = 1, f = f, n = 10)
  expect_equal(plan$Required_Test_Time, expected_T, tolerance = 1e-10)
  expect_equal(plan$Allowed_Failures, f)

  # Solve for sample size
  expected_n <- ceiling(qchisq(cl, df = 2 * (f + 1)) / (2 * (2000 / eta0)))
  plan2 <- rdt(target = target, mission_time = mt, conf_level = cl, beta = 1, f = f, test_time = 2000)
  expect_equal(plan2$Required_Sample_Size, expected_n)
  expect_equal(plan2$Allowed_Failures, f)
})

test_that("rdt() allowing more failures increases required test time and sample size", {
  # With fixed n, higher f requires longer per-unit test time (more total evidence needed)
  plan0 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 0, n = 10)
  plan2 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 2, n = 10)
  expect_gt(plan2$Required_Test_Time, plan0$Required_Test_Time)

  # With fixed test_time, higher f requires more units
  plan0_tt <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 0, test_time = 2000)
  plan2_tt <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 2, test_time = 2000)
  expect_gt(plan2_tt$Required_Sample_Size, plan0_tt$Required_Sample_Size)
})

test_that("RDT with f>0 and sample size input is stable to trivial noise", {
  set.seed(789)
  target <- 0.9
  mission_time <- 1000
  conf_level <- 0.9
  beta <- 1
  f <- 2
  n <- 10

  base_result <- rdt(target, mission_time, conf_level, beta, f = f, n = n)

  noise <- function(x) x + rnorm(1, 0, 1e-8)
  noisy_result <- rdt(noise(target), noise(mission_time), noise(conf_level), noise(beta),
    f = f, n = n
  )

  expect_equal(base_result$Input_Sample_Size, noisy_result$Input_Sample_Size)
  expect_equal(base_result$Required_Test_Time, noisy_result$Required_Test_Time, tolerance = 1e-6)
})

test_that("print.rdt shows Allowed Failures line", {
  plan <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 2, n = 10)
  expect_output(print(plan), "Allowed Failures")

  plan2 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, f = 2, test_time = 2000)
  expect_output(print(plan2), "Allowed Failures")
})

#' Mean Cumulative Function for Repairable Systems.
#'
#' Computes the non-parametric Mean Cumulative Function (MCF) for recurrent
#' event data from one or more repairable systems, using the Nelson-Aalen
#' estimator. The MCF estimates the expected cumulative number of events per
#' system as a function of time, properly accounting for system exposure
#' (observation windows).
#'
#' @srrstats {G1.0} The Nelson-Aalen estimator for recurrent events is
#'   described in Nelson (2003) <doi:10.1198/004017002188618563>.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the
#'   documentation.
#' @srrstats {G1.4} \code{roxygen2} documentation is used to document all
#'   functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.7} Both vectors and data frames are accepted as input.
#' @srrstats {G2.13} The function checks for missing data and errors if any
#'   is found.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any
#'   is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if
#'   any are found.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#'
#' @param id A vector of system/unit identifiers. Each unique value represents
#'   a distinct system. Ignored if \code{data} is provided.
#' @param time A numeric vector of event or censoring times. Must be positive
#'   and finite. Ignored if \code{data} is provided.
#' @param event An optional numeric vector of event indicators: 1 for an event,
#'   0 for censoring (end of observation). If \code{NULL} (default), all
#'   observations are treated as events.
#' @param end_time An optional named numeric vector of end-of-observation times
#'   per system, where names correspond to system identifiers. This defines
#'   the actual exposure window for each system. When provided, a system
#'   remains in the risk set until its \code{end_time}, even if its last event
#'   occurred earlier. If \code{NULL} (default), the end of observation is
#'   inferred as the maximum time recorded for each system (from both events
#'   and censoring records).
#' @param data An optional data frame containing columns named \code{id},
#'   \code{time}, and optionally \code{event} and \code{end_time}.
#' @param conf_level Confidence level for bounds (default 0.95).
#' @family Repairable Systems Analysis
#' @return An object of class \code{mcf} containing:
#' \item{time}{Unique event times.}
#' \item{mcf}{MCF values at each event time.}
#' \item{variance}{Variance estimates at each event time.}
#' \item{lower_bounds}{Lower confidence bounds.}
#' \item{upper_bounds}{Upper confidence bounds.}
#' \item{conf_level}{Confidence level used.}
#' \item{n_systems}{Number of distinct systems.}
#' \item{n_events}{Total number of events.}
#' \item{end_times}{Named vector of end-of-observation times per system.}
#'
#' @details
#' The MCF at time \eqn{t} is estimated as:
#' \deqn{\hat{M}(t) = \sum_{t_j \le t} \frac{d_j}{n_j}}
#' where \eqn{d_j} is the number of events at time \eqn{t_j} and \eqn{n_j} is
#' the number of systems still under observation at \eqn{t_j}. Variance is
#' estimated as \eqn{\hat{V}(t) = \sum_{t_j \le t} d_j / n_j^2}.
#'
#' The risk set \eqn{n_j} is determined by each system's **exposure window**.
#' A system is considered at risk at time \eqn{t_j} if its end-of-observation
#' time (from \code{end_time}, censoring records, or last event) is
#' \eqn{\ge t_j}. Specifying \code{end_time} is important when systems were
#' observed beyond their last event -- without it, the MCF may overestimate
#' the true recurrence rate because systems with no late events are assumed to
#' have left observation at their last event time.
#'
#' @examples
#' # Basic usage (end of observation inferred from last event)
#' id <- c(1, 1, 1, 2, 2, 3, 3, 3, 3)
#' time <- c(100, 300, 500, 150, 400, 50, 200, 350, 600)
#' result <- mcf(id, time)
#' print(result)
#' plot(result, main = "Mean Cumulative Function")
#'
#' # With explicit end-of-observation times (exposure-adjusted)
#' end_time <- c("1" = 800, "2" = 800, "3" = 800)
#' result2 <- mcf(id, time, end_time = end_time)
#' print(result2)
#'
#' df <- data.frame(id = id, time = time)
#' result3 <- mcf(data = df)
#' print(result3)
#' @importFrom stats qnorm
#' @importFrom graphics plot lines legend
#' @export
mcf <- function(id = NULL, time = NULL, event = NULL, end_time = NULL,
                data = NULL, conf_level = 0.95) {

  # Extract from data frame if provided
  if (!is.null(data)) {
    if (!is.data.frame(data)) {
      stop("'data' must be a data frame.")
    }
    if (!"id" %in% names(data)) {
      stop("'data' must contain a column named 'id'.")
    }
    if (!"time" %in% names(data)) {
      stop("'data' must contain a column named 'time'.")
    }
    id <- data$id
    time <- data$time
    if ("event" %in% names(data)) {
      event <- data$event
    }
    if ("end_time" %in% names(data) && is.null(end_time)) {
      # Extract per-system end_time from data frame: take max end_time per id
      et_df <- stats::aggregate(data$end_time, by = list(id = data$id), FUN = max)
      end_time <- stats::setNames(et_df$x, et_df$id)
    }
  }

  # Validate id
  if (is.null(id)) stop("'id' must be provided.")
  if (length(id) == 0) stop("'id' cannot be empty.")

  # Validate time
  if (is.null(time)) stop("'time' must be provided.")
  if (!is.numeric(time) || !is.vector(time)) {
    stop("'time' must be a numeric vector.")
  }
  if (length(time) == 0) stop("'time' cannot be empty.")
  if (any(is.na(time)) || any(is.nan(time))) {
    stop("'time' contains missing (NA) or NaN values.")
  }
  if (any(!is.finite(time)) || any(time <= 0)) {
    stop("All values in 'time' must be finite and > 0.")
  }
  if (length(id) != length(time)) {
    stop("'id' and 'time' must have the same length.")
  }

  # Validate event
  if (is.null(event)) {
    event <- rep(1, length(time))
  }
  if (!is.numeric(event) || !is.vector(event)) {
    stop("'event' must be a numeric vector.")
  }
  if (length(event) != length(time)) {
    stop("'event' and 'time' must have the same length.")
  }
  if (any(is.na(event)) || any(is.nan(event))) {
    stop("'event' contains missing (NA) or NaN values.")
  }
  if (!all(event %in% c(0, 1))) {
    stop("'event' must contain only 0 (censored) or 1 (event).")
  }

  # Validate end_time
  if (!is.null(end_time)) {
    if (!is.numeric(end_time)) {
      stop("'end_time' must be a numeric vector.")
    }
    if (any(is.na(end_time)) || any(is.nan(end_time))) {
      stop("'end_time' contains missing (NA) or NaN values.")
    }
    if (any(!is.finite(end_time)) || any(end_time <= 0)) {
      stop("All values in 'end_time' must be finite and > 0.")
    }
  }

  # Validate conf_level
  if (!is.numeric(conf_level) || length(conf_level) != 1) {
    stop("'conf_level' must be a single numeric value.")
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1 (exclusive).")
  }

  # Compute MCF using Nelson-Aalen estimator
  unique_ids <- unique(id)
  n_systems <- length(unique_ids)
  n_events <- sum(event)

  # Determine the end-of-observation time per system
  # Priority: explicit end_time > inferred from data (max observed time)
  inferred_max <- c(tapply(time, id, max))

  if (!is.null(end_time)) {
    # Match end_time to system IDs
    end_time_names <- names(end_time)
    if (is.null(end_time_names)) {
      # Unnamed: assume same order as unique_ids
      if (length(end_time) != n_systems) {
        stop("Unnamed 'end_time' must have one value per system (length ",
             n_systems, ").")
      }
      obs_end <- stats::setNames(end_time, as.character(unique_ids))
    } else {
      obs_end <- inferred_max
      matched <- intersect(as.character(names(obs_end)), end_time_names)
      obs_end[matched] <- pmax(obs_end[matched], end_time[matched])
      # For systems in end_time not in data, ignore silently
      unmatched_et <- setdiff(end_time_names, as.character(names(obs_end)))
      if (length(unmatched_et) > 0) {
        # Systems in end_time but not in event data: still add as at-risk
        for (nm in unmatched_et) {
          obs_end[nm] <- end_time[nm]
          n_systems <- n_systems + 1
        }
      }
    }
  } else {
    obs_end <- inferred_max
  }

  # Get event times only (exclude censoring)
  event_times <- time[event == 1]
  unique_event_times <- sort(unique(event_times))

  mcf_vals <- numeric(length(unique_event_times))
  var_vals <- numeric(length(unique_event_times))

  for (j in seq_along(unique_event_times)) {
    t_j <- unique_event_times[j]
    # Number of events at t_j
    d_j <- sum(event_times == t_j)
    # Number of systems still under observation at t_j
    n_j <- sum(obs_end >= t_j)
    if (n_j > 0) {
      mcf_vals[j] <- d_j / n_j
      var_vals[j] <- d_j / n_j^2
    }
  }

  # Cumulative sums
  mcf_cum <- cumsum(mcf_vals)
  var_cum <- cumsum(var_vals)

  # Confidence bounds
  z_val <- stats::qnorm(1 - (1 - conf_level) / 2)
  se <- sqrt(var_cum)
  lower <- pmax(mcf_cum - z_val * se, 0)
  upper <- mcf_cum + z_val * se

  result <- list(
    time         = unique_event_times,
    mcf          = mcf_cum,
    variance     = var_cum,
    lower_bounds = lower,
    upper_bounds = upper,
    conf_level   = conf_level,
    n_systems    = n_systems,
    n_events     = n_events,
    end_times    = obs_end
  )
  class(result) <- "mcf"
  result
}

#' Print Method for mcf Objects.
#'
#' Prints a summary of the Mean Cumulative Function results.
#'
#' @param x An object of class \code{mcf}.
#' @param ... Additional arguments (not used).
#' @family Repairable Systems Analysis
#' @return Invisibly returns the input object.
#' @examples
#' id <- c(1, 1, 1, 2, 2, 3, 3, 3, 3)
#' time <- c(100, 300, 500, 150, 400, 50, 200, 350, 600)
#' result <- mcf(id, time)
#' print(result)
#' @export
print.mcf <- function(x, ...) {
  if (!inherits(x, "mcf")) {
    stop("'x' must be an object of class 'mcf'.")
  }

  cat("Mean Cumulative Function (MCF)\n")
  cat("------------------------------\n")
  cat(sprintf("Number of systems: %d\n", x$n_systems))
  cat(sprintf("Number of events:  %d\n", x$n_events))
  if (!is.null(x$end_times)) {
    cat(sprintf("Total exposure:    %.2f\n", sum(x$end_times)))
  }
  cat(sprintf("Confidence level:  %.0f%%\n\n", x$conf_level * 100))

  pct <- round(x$conf_level * 100)
  df <- data.frame(
    Time  = x$time,
    MCF   = round(x$mcf, 4),
    Lower = round(x$lower_bounds, 4),
    Upper = round(x$upper_bounds, 4),
    check.names = FALSE
  )
  names(df)[3] <- sprintf("Lower (%d%%)", pct)
  names(df)[4] <- sprintf("Upper (%d%%)", pct)
  print(df, row.names = FALSE)

  invisible(x)
}

#' Plot Method for mcf Objects.
#'
#' Plots the Mean Cumulative Function with optional confidence bounds.
#'
#' @param x An object of class \code{mcf}.
#' @param conf_bounds Logical; include confidence bounds (default: TRUE).
#' @param legend Logical; show the legend (default: TRUE).
#' @param legend_pos Position of the legend (default: "topleft").
#' @param ... Additional arguments passed to \code{plot()}.
#' @family Repairable Systems Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' id <- c(1, 1, 1, 2, 2, 3, 3, 3, 3)
#' time <- c(100, 300, 500, 150, 400, 50, 200, 350, 600)
#' result <- mcf(id, time)
#' plot(result, main = "Mean Cumulative Function")
#' @importFrom graphics plot lines legend
#' @export
plot.mcf <- function(x,
                     conf_bounds = TRUE,
                     legend = TRUE,
                     legend_pos = "topleft",
                     ...) {
  if (!inherits(x, "mcf")) {
    stop("'x' must be an object of class 'mcf'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  # Step function: duplicate points to create steps
  n <- length(x$time)
  if (n == 0) {
    graphics::plot(0, 0, type = "n", ...)
    return(invisible(NULL))
  }

  # Plot MCF as a step function
  graphics::plot(x$time, x$mcf, type = "s", pch = 16, ...)

  if (conf_bounds) {
    graphics::lines(x$time, x$lower_bounds, type = "s", lty = 2)
    graphics::lines(x$time, x$upper_bounds, type = "s", lty = 2)
  }

  if (legend) {
    pct <- round(x$conf_level * 100)
    legend_labels <- "MCF"
    legend_lty <- 1
    legend_pch <- NA

    if (conf_bounds) {
      legend_labels <- c(legend_labels, sprintf("Conf. Bounds (%d%%)", pct))
      legend_lty <- c(legend_lty, 2)
      legend_pch <- c(legend_pch, NA)
    }

    graphics::legend(
      legend_pos,
      legend = legend_labels,
      lty = legend_lty,
      pch = legend_pch,
      bty = "n"
    )
  }

  invisible(NULL)
}

#' @srrstats {G5.0} Tests use standard recurrent event data.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by stop() is unique.
#' @srrstats {G5.4} Correctness tests with known MCF values.
#' @srrstats {G5.8a} Checks for zero-length data.
#' @srrstats {G5.8b} Checks for unsupported data types.
#' @srrstats {G5.8c} Checks for data with NA fields.

# --- Input validation tests ---

test_that("mcf errors on missing id", {
  expect_error(mcf(time = c(1, 2)), "'id' must be provided.", fixed = TRUE)
})

test_that("mcf errors on missing time", {
  expect_error(mcf(id = c(1, 1)), "'time' must be provided.", fixed = TRUE)
})

test_that("mcf errors on empty id", {
  expect_error(mcf(id = c(), time = c()), "'id' must be provided.", fixed = TRUE)
})

test_that("mcf errors on empty time", {
  expect_error(mcf(id = c(1), time = c()), "'time' must be provided.", fixed = TRUE)
})

test_that("mcf errors on non-numeric time", {
  expect_error(
    mcf(id = c(1, 1), time = c("a", "b")),
    "'time' must be a numeric vector.", fixed = TRUE
  )
})

test_that("mcf errors on NA in time", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, NA)),
    "'time' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("mcf errors on NaN in time", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, NaN)),
    "'time' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("mcf errors on non-positive time", {
  expect_error(
    mcf(id = c(1, 1), time = c(0, 100)),
    "All values in 'time' must be finite and > 0.", fixed = TRUE
  )
})

test_that("mcf errors on infinite time", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, Inf)),
    "All values in 'time' must be finite and > 0.", fixed = TRUE
  )
})

test_that("mcf errors on mismatched id and time lengths", {
  expect_error(
    mcf(id = c(1, 2, 3), time = c(100, 200)),
    "'id' and 'time' must have the same length.", fixed = TRUE
  )
})

test_that("mcf errors on non-numeric event", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, 200), event = c("a", "b")),
    "'event' must be a numeric vector.", fixed = TRUE
  )
})

test_that("mcf errors on mismatched event length", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, 200), event = c(1)),
    "'event' and 'time' must have the same length.", fixed = TRUE
  )
})

test_that("mcf errors on NA in event", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, 200), event = c(1, NA)),
    "'event' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("mcf errors on invalid event values", {
  expect_error(
    mcf(id = c(1, 1), time = c(100, 200), event = c(1, 2)),
    "'event' must contain only 0 (censored) or 1 (event).", fixed = TRUE
  )
})

test_that("mcf errors on invalid conf_level", {
  expect_error(
    mcf(id = c(1), time = c(100), conf_level = "a"),
    "'conf_level' must be a single numeric value.", fixed = TRUE
  )
  expect_error(
    mcf(id = c(1), time = c(100), conf_level = 0),
    "'conf_level' must be between 0 and 1 (exclusive).", fixed = TRUE
  )
  expect_error(
    mcf(id = c(1), time = c(100), conf_level = 1),
    "'conf_level' must be between 0 and 1 (exclusive).", fixed = TRUE
  )
})

# --- Data frame input tests ---

test_that("mcf works with data frame input", {
  df <- data.frame(id = c(1, 1, 2, 2), time = c(100, 200, 150, 300))
  result <- mcf(data = df)
  expect_s3_class(result, "mcf")
  expect_equal(result$n_systems, 2)
  expect_equal(result$n_events, 4)
})

test_that("mcf data frame errors on missing columns", {
  expect_error(
    mcf(data = data.frame(x = 1:3)),
    "'data' must contain a column named 'id'.", fixed = TRUE
  )
  expect_error(
    mcf(data = data.frame(id = 1:3)),
    "'data' must contain a column named 'time'.", fixed = TRUE
  )
})

test_that("mcf errors on non-data-frame data", {
  expect_error(
    mcf(data = list(id = 1, time = 1)),
    "'data' must be a data frame.", fixed = TRUE
  )
})

test_that("mcf data frame with event column works", {
  df <- data.frame(
    id = c(1, 1, 2, 2),
    time = c(100, 200, 150, 300),
    event = c(1, 0, 1, 1)
  )
  result <- mcf(data = df)
  expect_s3_class(result, "mcf")
  expect_equal(result$n_events, 3)
})

# --- Correctness tests ---

test_that("mcf computes correct values for simple single-system case", {
  # Single system with events at 100, 200, 300
  id <- c(1, 1, 1)
  time <- c(100, 200, 300)
  result <- mcf(id, time)

  # For single system, MCF = cumulative count: 1, 2, 3
  expect_equal(result$mcf, c(1, 2, 3))
  expect_equal(result$time, c(100, 200, 300))
  expect_equal(result$n_systems, 1)
  expect_equal(result$n_events, 3)
})

test_that("mcf computes correct values for multi-system case", {
  # System 1: events at 100, 300; max_time = 300

# System 2: events at 100, 200; max_time = 200
  # At t=100: d=2, n=2, increment = 1.0
  # At t=200: d=1, n=2, increment = 0.5
  # At t=300: d=1, n=1, increment = 1.0
  id <- c(1, 1, 2, 2)
  time <- c(100, 300, 100, 200)
  result <- mcf(id, time)

  expect_equal(result$time, c(100, 200, 300))
  expect_equal(result$mcf, c(1.0, 1.5, 2.5))
  expect_equal(result$n_systems, 2)
  expect_equal(result$n_events, 4)
})

test_that("mcf with censoring excludes censored observations from events", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 300, 100, 200)
  event <- c(1, 0, 1, 1)  # System 1's second obs is censoring

  result <- mcf(id, time, event)
  # Events at: t=100 (sys 1 + sys 2), t=200 (sys 2)
  # At t=100: d=2, n=2, increment = 1.0
  # At t=200: d=1, n=2, increment = 0.5
  expect_equal(result$time, c(100, 200))
  expect_equal(result$mcf, c(1.0, 1.5))
  expect_equal(result$n_events, 3)
})

test_that("mcf confidence bounds are wider at higher confidence level", {
  id <- c(1, 1, 1, 2, 2, 3, 3, 3)
  time <- c(100, 200, 300, 100, 250, 50, 150, 400)

  result_90 <- mcf(id, time, conf_level = 0.90)
  result_99 <- mcf(id, time, conf_level = 0.99)

  width_90 <- result_90$upper_bounds - result_90$lower_bounds
  width_99 <- result_99$upper_bounds - result_99$lower_bounds

  expect_true(all(width_99 >= width_90))
})

test_that("mcf lower bounds are non-negative", {
  id <- c(1, 1, 2, 2, 3)
  time <- c(100, 200, 150, 300, 50)
  result <- mcf(id, time)
  expect_true(all(result$lower_bounds >= 0))
})

# --- end_time (exposure) tests ---

test_that("mcf with end_time keeps systems in risk set longer", {
  # System 1: event at 100; last event = 100
  # System 2: event at 200; last event = 200
  # Without end_time: at t=200, n=1 (sys1 dropped out at 100)
  # With end_time = 500 for both: at t=200, n=2
  id <- c(1, 2)
  time <- c(100, 200)

  result_no_et <- mcf(id, time)
  result_et <- mcf(id, time, end_time = c("1" = 500, "2" = 500))

  # Without end_time: at t=200, n_j=1 so increment = 1/1 = 1
  # With end_time: at t=200, n_j=2 so increment = 1/2 = 0.5
  # MCF at t=200 should be lower with end_time
  expect_true(result_et$mcf[length(result_et$mcf)] <
                result_no_et$mcf[length(result_no_et$mcf)])
})

test_that("mcf with end_time produces correct values", {
  # System 1: event at 100, observed until 400
  # System 2: event at 100, event at 300, observed until 300
  # At t=100: d=2, n=2, increment = 1.0

  # At t=300: d=1, n=2 (sys1 still at risk until 400), increment = 0.5
  id <- c(1, 2, 2)
  time <- c(100, 100, 300)
  end_time <- c("1" = 400, "2" = 300)

  result <- mcf(id, time, end_time = end_time)

  expect_equal(result$time, c(100, 300))
  expect_equal(result$mcf, c(1.0, 1.5))
})

test_that("mcf with unnamed end_time works", {
  id <- c(1, 2)
  time <- c(100, 200)
  # Unnamed: assumed to be in order of unique(id) = c(1, 2)
  result <- mcf(id, time, end_time = c(500, 500))
  expect_s3_class(result, "mcf")
})

test_that("mcf errors on unnamed end_time with wrong length", {
  id <- c(1, 2)
  time <- c(100, 200)
  expect_error(
    mcf(id, time, end_time = c(500, 500, 500)),
    "Unnamed 'end_time' must have one value per system"
  )
})

test_that("mcf errors on non-numeric end_time", {
  id <- c(1, 2)
  time <- c(100, 200)
  expect_error(
    mcf(id, time, end_time = c("a", "b")),
    "'end_time' must be a numeric vector.", fixed = TRUE
  )
})

test_that("mcf errors on NA in end_time", {
  id <- c(1, 2)
  time <- c(100, 200)
  expect_error(
    mcf(id, time, end_time = c(500, NA)),
    "'end_time' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("mcf errors on non-positive end_time", {
  id <- c(1, 2)
  time <- c(100, 200)
  expect_error(
    mcf(id, time, end_time = c(500, 0)),
    "All values in 'end_time' must be finite and > 0.", fixed = TRUE
  )
})

test_that("mcf output includes end_times", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 200, 150, 300)
  result <- mcf(id, time)
  expect_true("end_times" %in% names(result))
})

test_that("mcf end_times from data frame works", {
  df <- data.frame(
    id = c(1, 1, 2, 2),
    time = c(100, 200, 150, 300),
    end_time = c(500, 500, 500, 500)
  )
  result <- mcf(data = df)
  expect_s3_class(result, "mcf")
  # end_time should be 500 for both systems
  expect_equal(unname(result$end_times), c(500, 500))
})

test_that("mcf with exposure end_times interoperates", {
  # Compute exposure, then use its end_times in MCF
  id <- c(1, 1, 1, 2, 2, 2)
  time <- c(100, 300, 500, 80, 300, 600)
  event <- c(1, 1, 0, 1, 1, 0)

  exp_result <- exposure(id, time, event)
  mcf_result <- mcf(id, time, event, end_time = exp_result$end_times)

  expect_s3_class(mcf_result, "mcf")
  # With censoring at 500 and 600, both systems should be at risk
  # longer than if we only considered event times
  expect_true(all(mcf_result$end_times >= 0))
})

# --- Print and plot tests ---

test_that("print.mcf produces output", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 200, 150, 300)
  result <- mcf(id, time)
  expect_output(print(result), "Mean Cumulative Function")
})

test_that("print.mcf returns invisible", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 200, 150, 300)
  result <- mcf(id, time)
  expect_invisible(print(result))
})

test_that("print.mcf errors on wrong class", {
  expect_error(
    print.mcf(list(a = 1)),
    "'x' must be an object of class 'mcf'.", fixed = TRUE
  )
})

test_that("plot.mcf produces a plot without error", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 200, 150, 300)
  result <- mcf(id, time)
  expect_silent(plot(result, main = "Test MCF"))
})

test_that("plot.mcf validates inputs", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 200, 150, 300)
  result <- mcf(id, time)

  expect_error(plot.mcf(list(a = 1)), "'x' must be an object of class 'mcf'.")
  expect_error(plot(result, conf_bounds = "yes"), "'conf_bounds' must be a single logical value.")
  expect_error(plot(result, legend = "yes"), "'legend' must be a single logical value.")
  expect_error(plot(result, legend_pos = 123), "'legend_pos' must be a single character string.")
})

test_that("plot.mcf works without confidence bounds and legend", {
  id <- c(1, 1, 2, 2)
  time <- c(100, 200, 150, 300)
  result <- mcf(id, time)
  expect_silent(plot(result, conf_bounds = FALSE, legend = FALSE))
})

#' Weibull to RGA
#'
#' Converts Weibull data (failure, suspension, and interval-censored times)
#' into a format suitable for reliability growth analysis (RGA). The function
#' handles exact failure times, right-censored suspensions, and interval-censored data.
#' It approximates interval-censored failures by placing them at the midpoint of the interval.
#' The output is a data frame with cumulative time and failure counts. This format
#' can be used with RGA models such as Crow-AMSAA.
#'
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical
#' algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param failures A numeric vector of exact failure times. Each failure time
#' indicates when an item failed during the observation period.
#' @param suspensions A numeric vector of suspension (right-censored) times. A suspension
#' indicates that the item was removed from observation at that time without failure.
#' This parameter is optional and can be NULL if there are no suspensions.
#' @param interval_starts A numeric vector of interval start times (lower bound of censoring).
#' This parameter is optional and can be NULL if there are no interval-censored data.
#' If provided, it must be the same length as `interval_ends`.
#' @param interval_ends A numeric vector of interval end times (upper bound of censoring).
#' This parameter is optional and can be NULL if there are no interval-censored data.
#' If provided, it must be the same length as `interval_starts`.
#' @return The data frame contains two columns:
#' \item{CumulativeTime}{Cumulative time at each failure event.}
#' \item{Failures}{Number of failures at each cumulative time point.}
#' The function approximates interval-censored failures by placing them at the midpoint
#' of the interval.
#' @family data preparation
#' @examples
#' failures <- c(100, 200, 200, 400)
#' suspensions <- c(250, 350, 450)
#' interval_starts <- c(150, 300)
#' interval_ends <- c(180, 320)
#' result <- weibull_to_rga(failures, suspensions, interval_starts, interval_ends)
#' print(result)
#' @importFrom stats aggregate
#' @export
weibull_to_rga <- function(failures,
                           suspensions = NULL,
                           interval_starts = NULL,
                           interval_ends = NULL) {
  # Validation
  if (any(is.na(failures)) || any(is.nan(failures)) || any(!is.finite(failures))) {
    stop("`failures` contains missing (NA), NaN, or infinite values.")
  }
  if (!is.numeric(failures) || any(failures <= 0)) {
    stop("`failures` must be a numeric vector with positive values.")
  }
  if (any(is.na(suspensions)) || any(is.nan(suspensions)) || any(!is.finite(suspensions))) {
    stop("`suspensions` contains missing (NA), NaN, or infinite values.")
  }
  if (!is.null(suspensions)) {
    if (!is.numeric(suspensions) || any(suspensions <= 0)) {
      stop("`suspensions` must be a numeric vector with positive values.")
    }
  }
  if (!is.null(interval_starts) || !is.null(interval_ends)) {
    if (any(is.na(interval_starts)) || any(is.nan(interval_starts)) || any(!is.finite(interval_starts))) {
      stop("`interval_starts` contains missing (NA), NaN, or infinite values.")
    }
    if (any(is.na(interval_ends)) || any(is.nan(interval_ends)) || any(!is.finite(interval_ends))) {
      stop("`interval_ends` contains missing (NA), NaN, or infinite values.")
    }
    if (is.null(interval_starts) || is.null(interval_ends)) {
      stop("Both `interval_starts` and `interval_ends` must be provided together.")
    }
    if (!is.numeric(interval_starts) || !is.numeric(interval_ends)) {
      stop("Interval bounds must be numeric vectors.")
    }
    if (length(interval_starts) != length(interval_ends)) {
      stop("`interval_starts` and `interval_ends` must have the same length.")
    }
    if (any(interval_starts <= 0) || any(interval_ends <= 0)) {
      stop("Interval bounds must be positive.")
    }

    if (any(interval_starts >= interval_ends)) {
      stop("Each interval start must be strictly less than its corresponding end.")
    }
  }

  # Data prep
  all_times <- c(failures, suspensions)
  all_types <- c(
    rep("Failure", length(failures)),
    rep("Suspension", length(suspensions))
  )

  # Handle interval-censored data (approximate failures at midpoint)
  if (!is.null(interval_starts)) {
    midpoints <- (interval_starts + interval_ends) / 2
    all_times <- c(all_times, midpoints)
    all_types <- c(all_types, rep("IntervalFailure", length(midpoints)))
  }

  data <- data.frame(Time = all_times, Type = all_types)
  data <- data[order(data$Time), ]

  # Cumulative time is just running sum of times in order
  data$CumulativeTime <- cumsum(data$Time)

  # Failure counts by time (Failures + IntervalFailures)
  failure_counts <- stats::aggregate(Type ~ Time,
    data = data,
    FUN = function(x) sum(x %in% c("Failure", "IntervalFailure"))
  )

  result <- merge(failure_counts, data[, c("Time", "CumulativeTime")], by = "Time")
  result <- result[!duplicated(result$Time), ]
  colnames(result)[colnames(result) == "Type"] <- "Failures"

  # Keep only rows where there were actual failures
  result <- result[result$Failures > 0, ]

  # Final columns
  result <- result[, c("CumulativeTime", "Failures")]

  return(result)
}

#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical
#' algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.

test_that("weibull_to_rga() input validation errors", {
  # failures
  expect_error(
    weibull_to_rga("a"),
    "`failures` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, -2, 3)),
    "`failures` must be a numeric vector with positive values."
  )
  expect_error(
    weibull_to_rga(c(1, NA, 3)),
    "`failures` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, NaN, 3)),
    "`failures` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, Inf, 3)),
    "`failures` contains missing \\(NA\\), NaN, or infinite values."
  )

  # suspensions
  expect_error(
    weibull_to_rga(c(1, 2), suspensions = "a"),
    "`suspensions` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), suspensions = c(1, -2)),
    "`suspensions` must be a numeric vector with positive values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), suspensions = c(1, NA)),
    "`suspensions` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), suspensions = c(1, NaN)),
    "`suspensions` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), suspensions = c(1, Inf)),
    "`suspensions` contains missing \\(NA\\), NaN, or infinite values."
  )

  # interval bounds missing
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, 2)),
    "Both `interval_starts` and `interval_ends` must be provided together."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_ends = c(1, 2)),
    "Both `interval_starts` and `interval_ends` must be provided together."
  )

  # interval bounds type
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = "a", interval_ends = 2),
    "`interval_starts` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = 1, interval_ends = "b"),
    "`interval_ends` contains missing \\(NA\\), NaN, or infinite values."
  )

  # interval bounds length mismatch
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, 2), interval_ends = 3),
    "`interval_starts` and `interval_ends` must have the same length."
  )

  # interval bounds positivity
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, -2), interval_ends = c(2, 3)),
    "Interval bounds must be positive."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, 2), interval_ends = c(-2, 3)),
    "Interval bounds must be positive."
  )

  # interval_starts bad values
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, NA), interval_ends = c(2, 3)),
    "`interval_starts` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, NaN), interval_ends = c(2, 3)),
    "`interval_starts` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, Inf), interval_ends = c(2, 3)),
    "`interval_starts` contains missing \\(NA\\), NaN, or infinite values."
  )

  # interval_ends bad values
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, 2), interval_ends = c(NA, 3)),
    "`interval_ends` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, 2), interval_ends = c(NaN, 3)),
    "`interval_ends` contains missing \\(NA\\), NaN, or infinite values."
  )
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(1, 2), interval_ends = c(Inf, 3)),
    "`interval_ends` contains missing \\(NA\\), NaN, or infinite values."
  )

  # interval_starts >= interval_ends
  expect_error(
    weibull_to_rga(c(1, 2), interval_starts = c(5, 10), interval_ends = c(5, 8)),
    "Each interval start must be strictly less than its corresponding end."
  )
})

test_that("weibull_to_rga() works with failures only", {
  res <- weibull_to_rga(c(10, 20, 30))
  expect_s3_class(res, "data.frame")
  expect_equal(names(res), c("CumulativeTime", "Failures"))
  expect_equal(nrow(res), 3)
  expect_true(all(res$Failures == 1))
})

test_that("weibull_to_rga() works with failures and suspensions", {
  res <- weibull_to_rga(c(10, 20), suspensions = c(15, 25))
  # suspensions should not contribute to Failures
  expect_equal(res$Failures, c(1, 1))
  expect_true(all(res$CumulativeTime > 0))
})

test_that("weibull_to_rga() works with interval-censored data", {
  res <- weibull_to_rga(
    failures = c(50, 100),
    interval_starts = c(30, 80),
    interval_ends = c(40, 120)
  )
  # Two failures + two interval failures
  expect_true(all(res$Failures >= 1))
  expect_true(any(diff(res$CumulativeTime) > 0))
})

test_that("weibull_to_rga() works with failures, suspensions, and interval data", {
  res <- weibull_to_rga(
    failures = c(100, 200, 200, 400),
    suspensions = c(250, 350, 450),
    interval_starts = c(150, 300),
    interval_ends = c(180, 320)
  )
  # There should be aggregated failures (since 200 repeats)
  expect_true(any(res$Failures > 1))
  expect_gt(nrow(res), 2)
  expect_equal(names(res), c("CumulativeTime", "Failures"))
})

test_that("weibull_to_rga() handles duplicate failures correctly", {
  res <- weibull_to_rga(c(10, 10, 20))
  # At time 10 there should be 2 failures aggregated
  expect_true(any(res$Failures == 2))
})

# Internal MLE helper for Crow-AMSAA (NHPP power-law) grouped-data likelihood.
# Arguments:
#   cum_time   : numeric vector of cumulative times (cumsum of interval times)
#   failures   : numeric vector of failure counts per interval
#   conf_level : numeric scalar in (0,1)
# Returns a named list of estimates, standard errors, fitted values, CIs,
# residuals, and information criteria.
.fit_mle_crow <- function(cum_time, failures, conf_level) {
  N <- sum(failures)
  T_max <- max(cum_time)
  n_obs <- length(failures)
  t_prev <- c(0, cum_time[-n_obs])

  neg_loglik <- function(par) {
    beta <- par[1]
    lambda <- par[2]
    if (beta <= 0 || lambda <= 0) {
      return(Inf)
    }
    delta_t <- cum_time^beta - t_prev^beta
    if (any(delta_t <= 0)) {
      return(Inf)
    }
    ll <- N * log(lambda) + sum(failures * log(delta_t)) - lambda * T_max^beta
    -ll
  }

  opt <- stats::optim(
    par = c(1, N / T_max),
    fn = neg_loglik,
    method = "L-BFGS-B",
    lower = c(1e-6, 1e-10),
    hessian = TRUE
  )

  beta_hat <- opt$par[1]
  lambda_hat <- opt$par[2]
  loglik <- -opt$value

  vcov_mat <- tryCatch(solve(opt$hessian), error = function(e) matrix(NA_real_, 2, 2))
  beta_se <- if (!anyNA(vcov_mat)) sqrt(max(vcov_mat[1, 1], 0)) else NA_real_

  fitted_values <- lambda_hat * cum_time^beta_hat

  z_val <- stats::qnorm(1 - (1 - conf_level) / 2)
  log_fit <- log(fitted_values)
  grad_mat <- cbind(log(cum_time), 1 / lambda_hat)
  var_lf <- rowSums((grad_mat %*% vcov_mat) * grad_mat)
  hw <- z_val * sqrt(pmax(var_lf, 0))

  list(
    beta          = beta_hat,
    lambda        = lambda_hat,
    betas_se      = beta_se,
    vcov          = vcov_mat,
    fitted_values = fitted_values,
    lower_bounds  = exp(log_fit - hw),
    upper_bounds  = exp(log_fit + hw),
    loglik        = loglik,
    residuals     = cumsum(failures) - fitted_values,
    aic           = -2 * loglik + 4,
    bic           = -2 * loglik + 2 * log(n_obs)
  )
}

.compute_rga_cum_times <- function(times, times_type) {
  if (identical(times_type, "failure_times")) {
    return(cumsum(times))
  }

  if (any(diff(times) <= 0)) {
    stop(
      "When 'times_type = \"cumulative_failure_times\"', 'times' must be strictly increasing."
    )
  }

  times
}

#' Reliability Growth Analysis.
#'
#' This function performs reliability growth analysis using the Crow-AMSAA model by
#' Crow (1975) (AMSAATR138) or piecewise
#' NHPP model by Guo et al. (2010) <doi:10.1109/RAMS.2010.5448029>. It fits
#' a log-log linear regression of cumulative failures versus cumulative time. The
#' function accepts either two numeric vectors (`times`, `failures`) or a data frame
#' containing both. The `Piecewise NHPP` model can automatically detect change points
#' or use user-specified breakpoints.
#'
#' @srrstats {G1.0} Primary references for Crow-AMSAA and Piecewise NHPP models
#' are provided in the description.
#' @srrstats {G1.1} The `rga` function is the first implementation of the Crow-AMSAA
#'  and Piecewise NHPP models within an R package on CRAN.
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.3} See sub-tags for responses.
#' @srrstats {G2.3a} `match.arg()` is used for string inputs.
#' @srrstats {G2.3b} `tolower()` is used for string inputs.
#' @srrstats {G2.4b} Explicit conversion of log-likelihood to continuous is made via `as.numeric()`.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.7} Both one-dimensional vectors and data frames are accepted as input.
#' @srrstats {G2.8} Sub-functions `print.rga` and `plot.rga` are provided for the `rga` class.
#' @srrstats {G2.10} Data extracted from tabular `data.frame` objects are checked to ensure consistent behavior.
#' @srrstats {G2.11} Unit tests check that `data.frame` inputs  are appropriately processed and do not error without reason.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produces expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as parameters change.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#' @srrstats {RE1.2} Documentation includes expected format for inputting predictor variables (`times`, `failures`).
#' @srrstats {RE1.3} Output retains all relevant aspects of input data.
#' @srrstats {RE1.3a} Output retains all relevant aspects of input data.
#' @srrstats {RE1.4} Documentation includes assumptions for the input data (i.e., positive, finite values).
#' @srrstats {RE2.1} NA, NaN, and Inf values in input data results in an error.
#' @srrstats {RE2.2} Missing values in input data results in an error.
#' @srrstats {RE2.4} Function includes check for perfect collinearity between predictor and response variables.
#' @srrstats {RE2.4a} Perfect collinearity between predictor and response variables results in an error.
#' @srrstats {RE2.4b} Perfect collinearity between predictor and response variables results in an error.
#' @srrstats {RE4.0} Software returns a “model” object, which is a modified `lm` model object.
#' @srrstats {RE4.2} Model coefficients are included in the output object.
#' @srrstats {RE4.3} Standard errors on the coefficients are included in the output object.
#' @srrstats {RE4.4} The model specification is included in the output object.
#' @srrstats {RE4.5} The numbers of observations is included in the output object.
#' @srrstats {RE4.8} The Response variable (cumulative failures) is included in the output object.
#' @srrstats {RE4.9} Modeled values of the response variable are included in the output object.
#' @srrstats {RE4.10} Model Residuals, including documentation is included in the output object.
#' @srrstats {RE4.11} Goodness-of-fit statistics (log-likelihood, AIC, BIC) are included
#' in the output object.
#' @srrstats {RE4.13} All input variables are included in the output object.
#' @srrstats {RE4.17} Model objects are extended by a default `print` method which
#' provides an on-screen summary of model (input) parameters and (output) coefficients.
#' @srrstats {RE5.0} Scaling relationships between sizes of input data and
#' speed of algorithm are documented in the function documentation.
#' @srrstats {RE6.0} Model objects have default plot methods.
#' @srrstats {RE6.2} The default plot method produces a plot of the fitted values
#' of the model, with optional visualisation of confidence intervals.
#' @srrstats {RE7.1} Unit tests check for noiseless, exact relationships between
#' predictor (independent) and response (dependent) data.
#' @srrstats {RE7.1a} Unit tests confirm that model fitting is at least as fast
#' or faster than testing with equivalent noisy data.
#' @srrstats {RE7.2} Unit tests demonstrate that output objects retain aspects
#' of input data such as case names.
#' @srrstats {RE7.3} Unit tests demonstrate expected behavior when `rga` object
#' is submitted to the accessor methods `print` and `plot`.
#'
#' @param times Either a numeric vector of failure-time inputs or a data frame
#' containing both time inputs and failure counts. If `times_type = "failure_times"`
#' (default), `times` is treated exactly as in previous versions of the function
#' and is cumulatively summed inside `rga()`. If
#' `times_type = "cumulative_failure_times"`, `times` is treated as already
#' cumulative and is used directly without applying `cumsum()`. If a data frame
#' is provided, it must contain two columns: `times` and `failures`.
#' @param failures A numeric vector of the number of failures at each corresponding time
#' in times. Must be the same length as `times` if both are vectors. All values must be
#' positive and finite. Ignored if `times` is a data frame.
#' @param times_type Character scalar indicating how to interpret `times`.
#'   `"failure_times"` (default) preserves the current behavior and cumulatively
#'   sums `times` inside `rga()`. `"cumulative_failure_times"` treats `times`
#'   as already cumulative and skips that internal `cumsum()`.
#' @param model_type The model type. Either `Crow-AMSAA` (default) or `Piecewise NHPP` with change point detection.
#' @param breaks An optional vector of breakpoints for the `Piecewise NHPP` model.
#' @param conf_level The desired confidence level, which defaults to 95%. The confidence
#' level is the probability that the confidence interval contains the true mean response.
#' @family Reliability Growth Analysis
#' @return The function returns an object of class `rga` that contains:
#' \item{times}{The input time vector, stored exactly as supplied.}
#' \item{cum_times}{The cumulative time vector used for fitting.}
#' \item{times_type}{How `times` was interpreted: `"failure_times"` or `"cumulative_failure_times"`.}
#' \item{failures}{The input number of failures.}
#' \item{n_obs}{The number of observations (failures).}
#' \item{cum_failures}{Cumulative failures.}
#' \item{model}{The fitted model object (lm (linear model) or segmented).}
#' \item{residuals}{Model residuals on the log-log scale. These represent deviations of the observed
#' log cumulative failures from the fitted values and are useful for diagnostic checking.}
#' \item{logLik}{The log-likelihood of the fitted model. The log-likelihood is a
#' measure of model fit, with higher values indicating a better fit.}
#' \item{AIC}{Akaike Information Criterion (AIC). AIC is a measure used for model selection,
#' with lower values indicating a better fit.}
#' \item{BIC}{Bayesian Information Criterion(BIC). BIC is another criterion for model selection}
#' \item{breakpoints}{Breakpoints (log scale) if applicable.}
#' \item{fitted_values}{Fitted cumulative failures on the original scale.}
#' \item{lower_bounds}{Lower confidence bounds (original scale).}
#' \item{upper_bounds}{Upper confidence bounds (original scale).}
#' \item{betas}{Estimated beta(s). Betas are the slopes of the log-log plot.}
#' \item{betas_se}{Standard error(s) of the estimated beta(s).}
#' \item{growth_rate}{Estimated growth rate(s). Growth rates are calculated as 1 - beta.}
#' \item{lambdas}{Estimated lambda(s). Lambdas are the intercepts of the log-log plot.}
#'
#' @details
#' The scaling relationship between the size of input data (numbers of observations)
#' and speed of algorithm execution is approximately linear (O(n)). The function is
#' efficient and can handle large data sets (e.g., thousands of observations) quickly.
#' The function uses the `segmented` package for piecewise regression, which employs
#' an iterative algorithm to estimate breakpoints. The number of iterations required
#' for convergence may vary depending on the data and initial values.
#' In practice, the function typically converges within a few iterations for most data sets.
#' However, in some cases, especially with complex data or poor initial values,
#' it may take more iterations.
#'
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' result1 <- rga(times, failures)
#' print(result1)
#'
#' df <- data.frame(times = times, failures = failures)
#' result2 <- rga(df)
#' print(result2)
#'
#' cum_times <- cumsum(times)
#' result2b <- rga(cum_times, failures, times_type = "cumulative_failure_times")
#' print(result2b)
#'
#' result3 <- rga(times, failures, model_type = "Piecewise NHPP")
#' print(result3)
#'
#' result4 <- rga(times, failures, model_type = "Piecewise NHPP", breaks = c(450))
#' print(result4)
#' @param method Estimation method: \code{"LS"} (default) for least-squares
#'   log-log regression, or \code{"MLE"} for maximum likelihood estimation of
#'   the Crow-AMSAA model. \code{"MLE"} is not supported for
#'   \code{model_type = "Piecewise NHPP"}.
#' @importFrom stats lm predict AIC BIC logLik cor residuals optim qnorm
#' @importFrom segmented segmented slope intercept seg.control
#' @export
rga <- function(times, failures, times_type = c("failure_times", "cumulative_failure_times"),
                model_type = "Crow-AMSAA", breaks = NULL,
                conf_level = 0.95, method = c("LS", "MLE")) {
  if (is.data.frame(times)) {
    if (!all(c("times", "failures") %in% names(times))) {
      stop("If a data frame is provided, it must contain columns 'times' and 'failures'.")
    }
    failures <- times$failures
    times <- times$times
  }

  # Validation checks
  if (!is.numeric(times) || !is.vector(times)) {
    stop("'times' must be a numeric vector.")
  }
  if (!is.numeric(failures) || !is.vector(failures)) {
    stop("'failures' must be a numeric vector.")
  }
  if (any(is.na(times)) || any(is.nan(times))) {
    stop("'times' contains missing (NA) or NaN values.")
  }
  if (any(is.na(failures)) || any(is.nan(failures))) {
    stop("'failures' contains missing (NA) or NaN values.")
  }
  if (length(times) == 0) stop("'times' cannot be empty.")
  if (length(failures) == 0) stop("'failures' cannot be empty.")
  if (length(times) != length(failures)) {
    stop("The length of 'times' and 'failures' must be equal.")
  }
  if (any(!is.finite(times)) || any(times <= 0)) {
    stop("All values in 'times' must be finite and > 0.")
  }
  if (any(!is.finite(failures)) || any(failures <= 0)) {
    stop("All values in 'failures' must be finite and > 0.")
  }

  times_type <- match.arg(times_type)

  if (!is.character(model_type) || length(model_type) != 1) {
    stop("'model_type' must be a single character string.")
  }
  valid_model <- match.arg(
    tolower(model_type),
    tolower(c("crow-amsaa", "piecewise nhpp"))
  )

  if (!is.null(breaks)) {
    if (!is.numeric(breaks) || length(breaks) == 0) {
      stop("'breaks' must be a non-empty numeric vector if provided.")
    }
    if (any(!is.finite(breaks)) || any(breaks <= 0)) {
      stop("All values in 'breaks' must be finite and > 0.")
    }
    if (valid_model != "piecewise nhpp") {
      stop("'breaks' can only be used with the 'Piecewise NHPP' model.")
    }
  }

  if (!is.numeric(conf_level) || length(conf_level) != 1) {
    stop("'conf_level' must be a single numeric value.")
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1 (exclusive).")
  }

  method <- match.arg(method)
  if (method == "MLE" && valid_model == "piecewise nhpp") {
    stop("'method = \"MLE\"' is not supported for model_type = \"Piecewise NHPP\". Use method = \"LS\".")
  }

  # Data prep
  cum_failures <- cumsum(failures)
  cum_time <- .compute_rga_cum_times(times, times_type)
  log_times <- log(cum_time)
  log_cum_failures <- log(cum_failures)

  # Check for perfect collinearity
  cor_val <- suppressWarnings(stats::cor(log_times, log_cum_failures))
  if (is.na(cor_val) || abs(cor_val - 1) < .Machine$double.eps^0.5 ||
    abs(cor_val + 1) < .Machine$double.eps^0.5) {
    stop("Perfect collinearity detected between predictor ('log_times') and response ('log_cum_failures'). Regression cannot be performed.")
  }

  # MLE early return
  if (method == "MLE") {
    mle <- .fit_mle_crow(cum_time, failures, conf_level)
    result <- list(
      times         = times,
      cum_times     = cum_time,
      times_type    = times_type,
      failures      = failures,
      n_obs         = length(failures),
      cum_failures  = cum_failures,
      model         = NULL,
      residuals     = mle$residuals,
      logLik        = mle$loglik,
      AIC           = mle$aic,
      BIC           = mle$bic,
      breakpoints   = NULL,
      fitted_values = mle$fitted_values,
      lower_bounds  = mle$lower_bounds,
      upper_bounds  = mle$upper_bounds,
      growth_rate   = 1 - mle$beta,
      betas         = mle$beta,
      betas_se      = mle$betas_se,
      lambdas       = mle$lambda,
      method        = "MLE",
      vcov          = mle$vcov
    )
    class(result) <- "rga"
    return(result)
  }

  # Fit initial Crow-AMSAA model
  fit <- stats::lm(log_cum_failures ~ log_times)

  if (valid_model == "piecewise nhpp") {
    if (is.null(breaks)) {
      updated_fit <- segmented::segmented(fit, seg.Z = ~log_times)
      breakpoints <- updated_fit$psi[, "Est."]
    } else {
      breakpoints <- log(breaks)
      updated_fit <- segmented::segmented(fit, seg.Z = ~log_times, psi = breakpoints)
    }
    slopes <- segmented::slope(updated_fit)
    intercepts <- segmented::intercept(updated_fit)
    betas <- slopes
    growth_rates <- 1 - slopes$log_times[, "Est."]
    lambdas <- exp(intercepts$log_times)

    # Standard errors
    beta_se <- slopes$log_times[, "St.Err."]
  } else {
    updated_fit <- fit
    breakpoints <- NULL
    smry <- summary(updated_fit)
    slope <- smry$coefficients[2, ]
    intercept <- smry$coefficients[1, ]
    betas <- slope["Estimate"]
    growth_rates <- 1 - betas
    lambdas <- exp(intercept["Estimate"])

    # Standard Error
    beta_se <- slope["Std. Error"]
    lambdas_se <- exp(intercept["Std. Error"])
  }

  # Fit statistics
  loglik <- as.numeric(stats::logLik(updated_fit))
  aic <- stats::AIC(updated_fit)
  bic <- stats::BIC(updated_fit)

  # Predictions values
  fitted_values <- stats::predict(updated_fit)
  residuals <- stats::residuals(updated_fit)
  conf_intervals <- stats::predict(updated_fit, interval = "confidence", level = conf_level)
  lower_bounds <- exp(conf_intervals[, "lwr"])
  upper_bounds <- exp(conf_intervals[, "upr"])

  # Return object
  result <- list(
    times = times,
    cum_times = cum_time,
    times_type = times_type,
    failures = failures,
    n_obs = length(failures),
    cum_failures = cum_failures,
    model = updated_fit,
    residuals = residuals,
    logLik = loglik,
    AIC = aic,
    BIC = bic,
    breakpoints = breakpoints,
    fitted_values = exp(fitted_values),
    lower_bounds = lower_bounds,
    upper_bounds = upper_bounds,
    growth_rate = growth_rates,
    betas = betas,
    betas_se = beta_se,
    lambdas = lambdas,
    method = "LS",
    vcov = NULL
  )
  class(result) <- "rga"
  return(result)
}

#' Print method for rga objects.
#'
#' This function prints a summary of the results from an object of class \code{rga}.
#'
#' @srrstats {G1.0} Primary references for Crow-AMSAA and Piecewise NHPP models
#' are provided in the description.
#' @srrstats {G1.1} The `rga` function is the first implementation of the Crow-AMSAA
#'  and Piecewise NHPP models within an R package on CRAN.
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.3} See sub-tags for responses.
#' @srrstats {G2.3a} `match.arg()` is used for string inputs.
#' @srrstats {G2.3b} `tolower()` is used for string inputs.
#' @srrstats {G2.4b} Explicit conversion of log-likelihood to continuous is made via `as.numeric()`.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.7} Both one-dimensional vectors and data frames are accepted as input.
#' @srrstats {G2.8} Sub-functions `print.rga` and `plot.rga` are provided for the `rga` class.
#' @srrstats {G2.10} Data extracted from tabular `data.frame` objects are checked to ensure consistent behavior.
#' @srrstats {G2.11} Unit tests check that `data.frame` inputs  are appropriately processed and do not error without reason.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produces expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as parameters change.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class \code{rga}, which contains the results from the RGA model.
#' @param ... Additional arguments (not used).
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' result <- rga(times, failures)
#' print(result)
#' @family Reliability Growth Analysis
#' @return Invisibly returns the input object.
#'
#' @export
print.rga <- function(x, ...) {
  # Input validation
  if (!inherits(x, "rga")) {
    stop("'x' must be an object of class 'rga'.")
  }

  cat("Reliability Growth Analysis (RGA)\n")
  cat("---------------------------------\n")

  model_type <- if (is.null(x$breakpoints)) "Crow-AMSAA" else "Piecewise NHPP"
  cat("Model Type:", model_type, "\n")
  est_method <- if (!is.null(x$method)) x$method else "LS"
  cat("Estimation Method:", est_method, "\n\n")

  if (!is.null(x$breakpoints)) {
    cat("Breakpoints (original scale):\n")
    cat(round(exp(x$breakpoints), 4), "\n\n")
  }

  # Number of observations (failures)
  cat(sprintf("\nNumber of observations (failures): %d\n", x$n_obs))

  cat("Parameters (per segment):\n")
  if (model_type == "Piecewise NHPP") {
    growth_rates <- round(x$growth_rate, 4)
    betas <- round(x$betas$log_times[, "Est."], 4)
    betas_se <- round(x$betas_se, 4)
    cat(sprintf("  Growth Rates: %s\n", paste(growth_rates, collapse = ", ")))
    cat(sprintf("  Betas: %s\n", paste(betas, collapse = ", ")))
    cat(sprintf("  Std. Errors (Betas): %s\n", paste(betas_se, collapse = ", ")))

    lambdas <- round(x$lambdas[, "Est."], 4)
    cat(sprintf("  Lambdas: %s\n", paste(lambdas, collapse = ", ")))
  } else {
    cat(sprintf("  Growth Rate: %.4f\n", x$growth_rate))
    cat(sprintf("  Beta: %.4f (SE = %.4f)\n", x$betas, x$betas_se))
    cat(sprintf("  Lambda: %.4f\n", x$lambdas))
  }

  cat("\nGoodness of Fit:\n")
  cat(sprintf("  Log-likelihood: %.2f\n", x$logLik))
  cat(sprintf("  AIC: %.2f\n", x$AIC))
  cat(sprintf("  BIC: %.2f\n", x$BIC))

  invisible(x)
}

#' Plot Method for RGA Objects
#'
#' This function generates plots for objects of class \code{rga}.
#'
#' @srrstats {G1.0} Primary references for Crow-AMSAA and Piecewise NHPP models
#' are provided in the description.
#' @srrstats {G1.1} The `rga` function is the first implementation of the Crow-AMSAA
#'  and Piecewise NHPP models within an R package on CRAN.
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.3} See sub-tags for responses.
#' @srrstats {G2.3a} `match.arg()` is used for string inputs.
#' @srrstats {G2.3b} `tolower()` is used for string inputs.
#' @srrstats {G2.4b} Explicit conversion of log-likelihood to continuous is made via `as.numeric()`.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.7} Both one-dimensional vectors and data frames are accepted as input.
#' @srrstats {G2.8} Sub-functions `print.rga` and `plot.rga` are provided for the `rga` class.
#' @srrstats {G2.10} Data extracted from tabular `data.frame` objects are checked to ensure consistent behavior.
#' @srrstats {G2.11} Unit tests check that `data.frame` inputs  are appropriately processed and do not error without reason.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produces expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as parameters change.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class \code{rga}, which contains the results from the RGA model.
#' @param conf_bounds Logical; include confidence bounds (default: TRUE).
#' @param legend Logical; show the legend (default: TRUE).
#' @param log Logical; use a log-log scale (default: FALSE).
#' @param legend_pos Position of the legend (default: "bottomright").
#' @param ... Additional arguments passed to \code{plot()}.
#' @family Reliability Growth Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' result <- rga(times, failures)
#' plot(result,
#'   main = "Reliability Growth Analysis",
#'   xlab = "Cumulative Time", ylab = "Cumulative Failures"
#' )
#' @importFrom graphics lines abline legend plot
#' @export
plot.rga <- function(x,
                     conf_bounds = TRUE,
                     legend = TRUE,
                     log = FALSE,
                     legend_pos = "bottomright",
                     ...) {
  # Input validation
  if (!inherits(x, "rga")) {
    stop("'x' must be an object of class 'rga'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.logical(log) || length(log) != 1) {
    stop("'log' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  if (!is.null(x$cum_times) && !is.null(x$cum_failures)) {
    times <- x$cum_times
    cum_failures <- x$cum_failures
  } else if (!is.null(x$method) && x$method == "MLE") {
    times <- cumsum(x$times)
    cum_failures <- cumsum(x$failures)
  } else {
    if (!all(c("log_times", "log_cum_failures") %in% names(x$model$model))) {
      stop("The 'rga' object appears malformed or missing model data.")
    }
    times <- exp(x$model$model$log_times)
    cum_failures <- exp(x$model$model$log_cum_failures)
  }

  # Base plot
  plot_args <- list(
    x = times,
    y = cum_failures,
    pch = 16,
    ...
  )
  if (log) plot_args$log <- "xy"
  do.call(graphics::plot, plot_args)

  # Fitted line
  graphics::lines(times, x$fitted_values, ...)

  # Confidence bounds
  if (conf_bounds) {
    graphics::lines(times, x$lower_bounds, lty = 2, ...)
    graphics::lines(times, x$upper_bounds, lty = 2, ...)
  }

  # Breakpoints
  if (!is.null(x$breakpoints)) {
    graphics::abline(v = exp(x$breakpoints), lty = 3)
  }

  # Legend
  if (legend) {
    legend_items <- list(
      labels = c("Observed", "Fitted Line"),
      cols = c("black", "black"),
      pch = c(16, NA),
      lty = c(NA, 1)
    )

    if (conf_bounds) {
      legend_items$labels <- c(legend_items$labels, "Confidence Bounds")
      legend_items$cols <- c(legend_items$cols, "black")
      legend_items$pch <- c(legend_items$pch, NA)
      legend_items$lty <- c(legend_items$lty, 2)
    }

    if (!is.null(x$breakpoints)) {
      legend_items$labels <- c(legend_items$labels, "Change Points")
      legend_items$cols <- c(legend_items$cols, "black")
      legend_items$pch <- c(legend_items$pch, NA)
      legend_items$lty <- c(legend_items$lty, 3)
    }

    graphics::legend(
      legend_pos,
      legend = legend_items$labels,
      col = legend_items$cols,
      pch = legend_items$pch,
      lty = legend_items$lty,
      bty = "n"
    )
  }

  invisible(NULL)
}

#' Overlay Plot for Multiple RGA Models
#'
#' Plots multiple fitted \code{rga} objects on a single set of axes, using
#' distinct colors per model. Observed data points, fitted lines, and optional
#' confidence bounds are drawn for every model. Models may have been fit to
#' different datasets.
#'
#' @srrstats {G1.4} \code{roxygen2} documentation is used to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} List input is validated to contain only \code{rga} objects.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {RE6.0} Model objects have default plot methods.
#' @srrstats {RE6.2} The overlay plot shows fitted values with optional CIs.
#'
#' @param models A named or unnamed list of objects of class \code{rga}.
#'   At least one model must be provided. If the list is named, those names
#'   are used as legend labels; otherwise labels default to
#'   \code{"Model 1"}, \code{"Model 2"}, etc.
#' @param conf_bounds Logical; draw confidence bounds for each model
#'   (default: \code{TRUE}).
#' @param legend Logical; draw a legend (default: \code{TRUE}).
#' @param legend_pos Legend position keyword (default: \code{"bottomright"}).
#' @param colors Optional character vector of colors, one per model. If
#'   \code{NULL} (default), \code{palette()} colors are cycled.
#' @param log Logical; use log-log axes (default: \code{FALSE}).
#' @param ... Additional arguments passed to the initial \code{plot()} call
#'   (e.g., \code{main}, \code{xlab}, \code{ylab}). Not forwarded to subsequent
#'   \code{lines()} or \code{points()} calls.
#' @family Reliability Growth Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' t1 <- c(100, 200, 300, 400, 500)
#' f1 <- c(1, 2, 1, 3, 2)
#' t2 <- c(150, 300, 450, 600, 750)
#' f2 <- c(2, 1, 3, 2, 4)
#' m1 <- rga(t1, f1)
#' m2 <- rga(t2, f2)
#' overlay_rga(list(System_A = m1, System_B = m2),
#'   main = "RGA Overlay", xlab = "Cumulative Time",
#'   ylab = "Cumulative Failures"
#' )
#' @importFrom graphics plot points lines abline legend
#' @importFrom grDevices palette
#' @export
overlay_rga <- function(models,
                        conf_bounds = TRUE,
                        legend = TRUE,
                        legend_pos = "bottomright",
                        colors = NULL,
                        log = FALSE,
                        ...) {
  # Input validation
  if (!identical(class(models), "list") || length(models) == 0) {
    stop("'models' must be a non-empty list of 'rga' objects.")
  }
  not_rga <- !vapply(models, inherits, logical(1), what = "rga")
  if (any(not_rga)) {
    stop("All elements of 'models' must be objects of class 'rga'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.logical(log) || length(log) != 1) {
    stop("'log' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  n_models <- length(models)

  # Resolve model names
  mod_names <- names(models)
  if (is.null(mod_names) || any(mod_names == "")) {
    mod_names <- paste0("Model ", seq_len(n_models))
  }

  # Resolve colors
  if (is.null(colors)) {
    pal <- grDevices::palette()
    colors <- pal[((seq_len(n_models) - 1L) %% length(pal)) + 1L]
  } else {
    if (!is.character(colors) || length(colors) < n_models) {
      stop("'colors' must be a character vector with at least one color per model.")
    }
    colors <- colors[seq_len(n_models)]
  }

  # Extract observed x/y from each model
  extract_xy <- function(x) {
    if (!is.null(x$cum_times) && !is.null(x$cum_failures)) {
      list(times = x$cum_times, cum_failures = x$cum_failures)
    } else if (!is.null(x$method) && x$method == "MLE") {
      list(times = cumsum(x$times), cum_failures = cumsum(x$failures))
    } else {
      list(
        times        = exp(x$model$model$log_times),
        cum_failures = exp(x$model$model$log_cum_failures)
      )
    }
  }
  xy_list <- lapply(models, extract_xy)

  # Compute global axis limits
  all_x <- unlist(lapply(xy_list, `[[`, "times"))
  all_y <- c(
    unlist(lapply(xy_list, `[[`, "cum_failures")),
    unlist(lapply(models, `[[`, "fitted_values"))
  )
  if (conf_bounds) {
    all_y <- c(
      all_y,
      unlist(lapply(models, `[[`, "lower_bounds")),
      unlist(lapply(models, `[[`, "upper_bounds"))
    )
  }
  xlim <- range(all_x, finite = TRUE)
  ylim <- range(all_y, finite = TRUE)
  if (log) ylim[1] <- max(ylim[1], .Machine$double.eps)

  # Base plot using first model's observed data
  plot_args <- list(
    x    = xy_list[[1]]$times,
    y    = xy_list[[1]]$cum_failures,
    xlim = xlim,
    ylim = ylim,
    col  = colors[[1]],
    pch  = 16,
    ...
  )
  if (log) plot_args$log <- "xy"
  do.call(graphics::plot, plot_args)

  # Add remaining models' observed points
  if (n_models > 1L) {
    for (i in seq(2L, n_models)) {
      graphics::points(xy_list[[i]]$times, xy_list[[i]]$cum_failures,
                       pch = 16, col = colors[[i]])
    }
  }

  # Fitted lines, confidence bounds, and breakpoints for all models
  for (i in seq_len(n_models)) {
    xy  <- xy_list[[i]]
    mdl <- models[[i]]
    col <- colors[[i]]

    graphics::lines(xy$times, mdl$fitted_values, col = col, lty = 1)

    if (conf_bounds) {
      graphics::lines(xy$times, mdl$lower_bounds, col = col, lty = 2)
      graphics::lines(xy$times, mdl$upper_bounds, col = col, lty = 2)
    }

    if (!is.null(mdl$breakpoints)) {
      graphics::abline(v = exp(mdl$breakpoints), col = col, lty = 3)
    }
  }

  # Legend
  if (legend) {
    graphics::legend(
      legend_pos,
      legend = mod_names,
      col    = colors,
      pch    = 16,
      lty    = 1,
      bty    = "n"
    )
  }

  invisible(NULL)
}

#' Forecast Cumulative Failures from a Reliability Growth Model
#'
#' Takes a fitted \code{rga} object and a vector of cumulative times, returning
#' predicted cumulative failures with confidence bounds as an \code{rga_predict}
#' S3 object.
#'
#' @srrstats {G1.4} \code{roxygen2} documentation is used to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.8} Sub-functions \code{print.rga_predict} and
#'   \code{plot.rga_predict} are provided for the \code{rga_predict} class.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results
#'   with expected values.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results
#'   with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical
#'   algorithms produce expected results to fixed test data sets.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of
#'   the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected
#'   stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does
#'   not meaningfully change results.
#'
#' @param object An object of class \code{rga} returned by \code{rga()}.
#' @param times A numeric vector of cumulative times at which to forecast.
#'   All values must be finite and > 0. A warning is issued if any value is
#'   at or below the maximum observed cumulative time (hindcasting).
#' @param conf_level The desired confidence level (default \code{0.95}). Must
#'   be a single finite numeric in (0, 1).
#' @family Reliability Growth Analysis
#' @return An object of class \code{rga_predict} containing:
#' \item{times}{The forecast cumulative times.}
#' \item{cum_failures}{Predicted cumulative failures.}
#' \item{lower_bounds}{Lower confidence bounds.}
#' \item{upper_bounds}{Upper confidence bounds.}
#' \item{conf_level}{The confidence level used.}
#' \item{model_type}{Either \code{"Crow-AMSAA"} or \code{"Piecewise NHPP"}.}
#' \item{rga_object}{The original \code{rga} object (used by the plot method).}
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- rga(times, failures)
#' fc <- predict_rga(fit, times = c(1500, 2000))
#' print(fc)
#' @export
predict_rga <- function(object, times, conf_level = 0.95) {
  if (!inherits(object, "rga")) {
    stop("'object' must be an object of class 'rga'.")
  }
  if (!is.numeric(times) || !is.vector(times)) {
    stop("'times' must be a numeric vector.")
  }
  if (length(times) == 0) {
    stop("'times' cannot be empty.")
  }
  if (any(is.na(times)) || any(is.nan(times))) {
    stop("'times' contains missing (NA) or NaN values.")
  }
  if (any(!is.finite(times)) || any(times <= 0)) {
    stop("All values in 'times' must be finite and > 0.")
  }
  if (!is.numeric(conf_level) || length(conf_level) != 1) {
    stop("'conf_level' must be a single numeric value.")
  }
  if (!is.finite(conf_level) || conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1 (exclusive).")
  }

  if (!is.null(object$cum_times)) {
    max_obs_time <- max(object$cum_times)
  } else if (!is.null(object$model)) {
    max_obs_time <- max(exp(object$model$model$log_times))
  } else {
    max_obs_time <- max(cumsum(object$times))
  }
  if (any(times <= max_obs_time)) {
    warning(
      "Some 'times' values are <= the maximum observed cumulative time. ",
      "Hindcasting is allowed but may not be meaningful."
    )
  }

  model_type <- if (is.null(object$breakpoints)) "Crow-AMSAA" else "Piecewise NHPP"

  if (!is.null(object$method) && object$method == "MLE") {
    cum_failures <- object$lambdas * times^object$betas
    log_fitted <- log(cum_failures)
    z_val <- stats::qnorm(1 - (1 - conf_level) / 2)
    grad_mat <- cbind(log(times), 1 / object$lambdas)
    var_lf <- rowSums((grad_mat %*% object$vcov) * grad_mat)
    hw <- z_val * sqrt(pmax(var_lf, 0))
    lower_bounds <- exp(log_fitted - hw)
    upper_bounds <- exp(log_fitted + hw)
  } else {
    newdata <- data.frame(log_times = log(times))
    pred <- stats::predict(object$model,
      newdata = newdata,
      interval = "confidence", level = conf_level
    )
    cum_failures <- exp(pred[, "fit"])
    lower_bounds <- exp(pred[, "lwr"])
    upper_bounds <- exp(pred[, "upr"])
  }

  result <- list(
    times        = times,
    cum_failures = cum_failures,
    lower_bounds = lower_bounds,
    upper_bounds = upper_bounds,
    conf_level   = conf_level,
    model_type   = model_type,
    rga_object   = object
  )
  class(result) <- "rga_predict"
  result
}

#' Print Method for rga_predict Objects
#'
#' Prints a formatted table of forecast cumulative failures with confidence
#' bounds for an \code{rga_predict} object.
#'
#' @srrstats {G1.4} \code{roxygen2} documentation is used to document all functions.
#' @srrstats {G2.8} This method is provided for the \code{rga_predict} class.
#' @srrstats {G5.2} Unit tests demonstrate output content.
#' @srrstats {G5.2b} Unit tests compare output with expected values.
#'
#' @param x An object of class \code{rga_predict}.
#' @param ... Additional arguments (not used).
#' @family Reliability Growth Analysis
#' @return Invisibly returns the input object.
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- rga(times, failures)
#' fc <- predict_rga(fit, times = c(1500, 2000))
#' print(fc)
#' @export
print.rga_predict <- function(x, ...) {
  if (!inherits(x, "rga_predict")) {
    stop("'x' must be an object of class 'rga_predict'.")
  }

  pct <- round(x$conf_level * 100)
  header <- sprintf("Reliability Growth Forecast (%s)", x$model_type)
  cat(header, "\n")
  cat(paste(rep("-", nchar(header) + 1), collapse = ""), "\n")

  df <- data.frame(
    Time           = x$times,
    Cum.Failures   = round(x$cum_failures, 1),
    Lower          = round(x$lower_bounds, 1),
    Upper          = round(x$upper_bounds, 1),
    check.names    = FALSE
  )
  names(df)[3] <- sprintf("Lower (%d%%)", pct)
  names(df)[4] <- sprintf("Upper (%d%%)", pct)

  print(df, row.names = FALSE)

  invisible(x)
}

#' Plot Method for rga_predict Objects
#'
#' Plots observed data, the fitted reliability growth curve, and the forecast
#' with optional confidence bounds for an \code{rga_predict} object.
#'
#' @srrstats {G1.4} \code{roxygen2} documentation is used to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.8} This method is provided for the \code{rga_predict} class.
#' @srrstats {G5.2} Unit tests include smoke tests for this method.
#'
#' @param x An object of class \code{rga_predict}.
#' @param conf_bounds Logical; include confidence bounds (default: \code{TRUE}).
#' @param legend Logical; show the legend (default: \code{TRUE}).
#' @param legend_pos Position of the legend (default: \code{"bottomright"}).
#' @param ... Additional arguments passed to \code{plot()}.
#' @family Reliability Growth Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- rga(times, failures)
#' fc <- predict_rga(fit, times = c(1500, 2000))
#' plot(fc, main = "RGA Forecast", xlab = "Cumulative Time", ylab = "Cumulative Failures")
#' @export
plot.rga_predict <- function(x,
                             conf_bounds = TRUE,
                             legend = TRUE,
                             legend_pos = "bottomright",
                             ...) {
  if (!inherits(x, "rga_predict")) {
    stop("'x' must be an object of class 'rga_predict'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  rga_obj <- x$rga_object
  if (!is.null(rga_obj$cum_times) && !is.null(rga_obj$cum_failures)) {
    obs_times <- rga_obj$cum_times
    obs_cum_failures <- rga_obj$cum_failures
  } else if (!is.null(rga_obj$method) && rga_obj$method == "MLE") {
    obs_times <- cumsum(rga_obj$times)
    obs_cum_failures <- cumsum(rga_obj$failures)
  } else {
    obs_times <- exp(rga_obj$model$model$log_times)
    obs_cum_failures <- exp(rga_obj$model$model$log_cum_failures)
  }

  all_y <- c(obs_cum_failures, rga_obj$fitted_values, x$cum_failures)
  if (conf_bounds) {
    all_y <- c(
      all_y, rga_obj$lower_bounds, rga_obj$upper_bounds,
      x$lower_bounds, x$upper_bounds
    )
  }
  xlim <- range(c(obs_times, x$times))
  ylim <- range(all_y)

  graphics::plot(obs_times, obs_cum_failures,
    xlim = xlim, ylim = ylim, pch = 16, ...
  )

  graphics::lines(obs_times, rga_obj$fitted_values)

  if (conf_bounds) {
    graphics::lines(obs_times, rga_obj$lower_bounds, lty = 3)
    graphics::lines(obs_times, rga_obj$upper_bounds, lty = 3)
  }

  max_obs_time <- max(obs_times)
  graphics::abline(v = max_obs_time, lty = 2, col = "gray")

  graphics::lines(x$times, x$cum_failures, lty = 2)

  if (conf_bounds) {
    graphics::lines(x$times, x$lower_bounds, lty = 3)
    graphics::lines(x$times, x$upper_bounds, lty = 3)
  }

  if (legend) {
    pct <- round(x$conf_level * 100)
    legend_labels <- c("Observed", "Fitted", "Forecast")
    legend_pch <- c(16, NA, NA)
    legend_lty <- c(NA, 1, 2)
    legend_cols <- c("black", "black", "black")

    if (conf_bounds) {
      legend_labels <- c(legend_labels, sprintf("Conf. Bounds (%d%%)", pct))
      legend_pch <- c(legend_pch, NA)
      legend_lty <- c(legend_lty, 3)
      legend_cols <- c(legend_cols, "black")
    }

    graphics::legend(legend_pos,
      legend = legend_labels,
      pch = legend_pch,
      lty = legend_lty,
      col = legend_cols,
      bty = "n",
      cex = 0.85,
      y.intersp = 1.3,
      x.intersp = 0.8
    )
  }

  invisible(NULL)
}

#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produces expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as parameters change.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#' @srrstats {RE7.1} Unit tests check for noiseless, exact relationships between
#' predictor (independent) and response (dependent) data.
#' @srrstats {RE7.1a} Unit tests confirm that model fitting is at least as fast
#' or faster than testing with equivalent noisy data.
#' @srrstats {RE7.2} Unit tests demonstrate that output objects retain aspects
#' of input data such as case names.
#' @srrstats {RE7.3} Unit tests demonstrate expected behavior when `rga` object
#' is submitted to the accessor methods `print` and `plot`.

test_that("data.frame input works the same as separate vectors", {
  times <- c(100, 200, 300)
  failures <- c(1, 2, 1)
  df <- data.frame(times = times, failures = failures)

  res_df <- rga(df)
  res_vec <- rga(times, failures)

  expect_s3_class(res_df, "rga")
  expect_s3_class(res_vec, "rga")
  expect_equal(res_df$betas, res_vec$betas)
  expect_equal(res_df$lambdas, res_vec$lambdas)
})

test_that("data.frame without required columns errors", {
  df1 <- data.frame(x = 1:3, y = 1:3)
  expect_error(
    rga(df1),
    "must contain columns 'times' and 'failures'"
  )

  df2 <- data.frame(times = 1:3)
  expect_error(
    rga(df2),
    "must contain columns 'times' and 'failures'"
  )
})

test_that("NA or NaN in data.frame input throws error", {
  df_na <- data.frame(times = c(100, 200, NA), failures = c(1, 2, 1))
  df_nan <- data.frame(times = c(100, 200, NaN), failures = c(1, 2, 1))
  df_fail_na <- data.frame(times = c(100, 200, 300), failures = c(1, NA, 1))

  expect_error(rga(df_na), "'times' contains missing")
  expect_error(rga(df_nan), "'times' contains missing")
  expect_error(rga(df_fail_na), "'failures' contains missing")
})

test_that("data.frame with non-positive or infinite values errors", {
  df_zero_time <- data.frame(times = c(0, 100, 200), failures = c(1, 2, 1))
  df_neg_fail <- data.frame(times = c(100, 200, 300), failures = c(1, -1, 1))
  df_inf_time <- data.frame(times = c(100, Inf, 200), failures = c(1, 2, 1))

  expect_error(rga(df_zero_time), "must be finite and > 0")
  expect_error(rga(df_neg_fail), "must be finite and > 0")
  expect_error(rga(df_inf_time), "must be finite and > 0")
})

# Helper: a minimal valid pair of inputs
valid_times <- c(100, 200, 300)
valid_failures <- c(1, 2, 1)

test_that("data frame without required columns errors", {
  df_bad <- data.frame(a = 1:3, b = 1:3)
  expect_error(rga(df_bad),
    "If a data frame is provided, it must contain columns 'times' and 'failures'.",
    fixed = TRUE
  )
})

test_that("NA / NaN checks for times and failures", {
  expect_error(rga(c(1, NA), valid_failures),
    "'times' contains missing (NA) or NaN values.",
    fixed = TRUE
  )

  expect_error(rga(valid_times, c(1, NaN, 1)),
    "'failures' contains missing (NA) or NaN values.",
    fixed = TRUE
  )
})

test_that("type checks for times and failures", {
  expect_error(rga(list(1, 2, 3), valid_failures),
    "'times' must be a numeric vector.",
    fixed = TRUE
  )

  expect_error(rga(valid_times, list(1, 2, 3)),
    "'failures' must be a numeric vector.",
    fixed = TRUE
  )
})

test_that("empty vector checks", {
  expect_error(rga(numeric(0), numeric(0)),
    "'times' cannot be empty.",
    fixed = TRUE
  )

  # ensure failures empty triggers its message
  expect_error(rga(valid_times, numeric(0)),
    "'failures' cannot be empty.",
    fixed = TRUE
  )
})

test_that("length mismatch check", {
  expect_error(rga(c(1, 2), c(1, 2, 3)),
    "The length of 'times' and 'failures' must be equal.",
    fixed = TRUE
  )
})

test_that("finite and >0 checks for times and failures", {
  expect_error(rga(c(1, Inf, 3), c(1, 1, 1)),
    "All values in 'times' must be finite and > 0.",
    fixed = TRUE
  )

  expect_error(rga(c(1, 2, 3), c(1, 0, 2)),
    "All values in 'failures' must be finite and > 0.",
    fixed = TRUE
  )
})

test_that("model_type validation errors", {
  # non-single character
  expect_error(rga(valid_times, valid_failures, model_type = c("a", "b")),
    "'model_type' must be a single character string.",
    fixed = TRUE
  )

  # invalid string - match.arg produces its own message; check for 'one of'
  expect_error(
    rga(valid_times, valid_failures, model_type = "not-a-model"),
    "one of"
  )
})

test_that("breaks argument validation", {
  # breaks must be numeric non-empty vector if provided
  expect_error(rga(valid_times, valid_failures, model_type = "Piecewise NHPP", breaks = character(1)),
    "'breaks' must be a non-empty numeric vector if provided.",
    fixed = TRUE
  )

  expect_error(rga(valid_times, valid_failures, model_type = "Piecewise NHPP", breaks = numeric(0)),
    "'breaks' must be a non-empty numeric vector if provided.",
    fixed = TRUE
  )

  # breaks values must be finite and > 0
  expect_error(rga(valid_times, valid_failures, model_type = "Piecewise NHPP", breaks = c(100, -1)),
    "All values in 'breaks' must be finite and > 0.",
    fixed = TRUE
  )

  # breaks only allowed with piecewise model
  expect_error(rga(valid_times, valid_failures, model_type = "Crow-AMSAA", breaks = c(150)),
    "'breaks' can only be used with the 'Piecewise NHPP' model.",
    fixed = TRUE
  )
})

test_that("conf_level validation", {
  expect_error(rga(valid_times, valid_failures, conf_level = c(0.9, 0.95)),
    "'conf_level' must be a single numeric value.",
    fixed = TRUE
  )

  expect_error(rga(valid_times, valid_failures, conf_level = 1),
    "'conf_level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )

  expect_error(rga(valid_times, valid_failures, conf_level = 0),
    "'conf_level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )
})

test_that("cumulative_failure_times input reproduces the default fit", {
  interval_times <- c(100, 200, 300, 400, 500)
  cumulative_times <- cumsum(interval_times)
  failures <- c(1, 2, 1, 3, 2)

  fit_interval <- rga(interval_times, failures)
  fit_cumulative <- rga(
    cumulative_times, failures,
    times_type = "cumulative_failure_times"
  )

  expect_equal(fit_cumulative$times, cumulative_times)
  expect_equal(fit_cumulative$cum_times, cumulative_times)
  expect_equal(fit_cumulative$times_type, "cumulative_failure_times")
  expect_equal(fit_cumulative$betas, fit_interval$betas)
  expect_equal(fit_cumulative$lambdas, fit_interval$lambdas)
  expect_equal(fit_cumulative$fitted_values, fit_interval$fitted_values)
  expect_equal(fit_cumulative$cum_failures, fit_interval$cum_failures)
})

test_that("cumulative_failure_times input must be strictly increasing", {
  expect_error(
    rga(
      c(100, 200, 200),
      c(1, 2, 1),
      times_type = "cumulative_failure_times"
    ),
    "must be strictly increasing"
  )
})

test_that("MLE supports cumulative_failure_times input", {
  interval_times <- c(100, 200, 300, 400, 500)
  cumulative_times <- cumsum(interval_times)
  failures <- c(1, 2, 1, 3, 2)

  fit_interval <- rga(interval_times, failures, method = "MLE")
  fit_cumulative <- rga(
    cumulative_times, failures,
    times_type = "cumulative_failure_times",
    method = "MLE"
  )

  expect_equal(fit_cumulative$cum_times, cumulative_times)
  expect_equal(fit_cumulative$times_type, "cumulative_failure_times")
  expect_equal(fit_cumulative$betas, fit_interval$betas)
  expect_equal(fit_cumulative$lambdas, fit_interval$lambdas)
})

test_that("print.rga errors when input is not rga", {
  expect_error(print.rga(list()),
    "'x' must be an object of class 'rga'.",
    fixed = TRUE
  )
})

test_that("plot.rga argument type checks and malformed object", {
  # Create a minimal valid-looking rga object so argument checks after inherits run
  x_good <- list(
    model = list(model = data.frame(log_times = 1, log_cum_failures = 1)),
    fitted_values = 1,
    lower_bounds = 1,
    upper_bounds = 1
  )
  class(x_good) <- "rga"

  # inherits check (non-rga)
  expect_error(plot.rga(list()),
    "'x' must be an object of class 'rga'.",
    fixed = TRUE
  )

  # conf_bounds must be single logical
  expect_error(plot.rga(x_good, conf_bounds = "nope"),
    "'conf_bounds' must be a single logical value.",
    fixed = TRUE
  )

  # legend must be single logical
  expect_error(plot.rga(x_good, conf_bounds = TRUE, legend = "nope"),
    "'legend' must be a single logical value.",
    fixed = TRUE
  )

  # log must be single logical
  expect_error(plot.rga(x_good, conf_bounds = TRUE, legend = TRUE, log = "nope"),
    "'log' must be a single logical value.",
    fixed = TRUE
  )

  # legend_pos must be single character string
  expect_error(plot.rga(x_good, conf_bounds = TRUE, legend = TRUE, log = FALSE, legend_pos = c("a", "b")),
    "'legend_pos' must be a single character string.",
    fixed = TRUE
  )

  # malformed rga: missing required model$model columns
  x_bad_model <- list(model = list(model = data.frame(foo = 1, bar = 2)))
  class(x_bad_model) <- "rga"
  expect_error(plot.rga(x_bad_model),
    "The 'rga' object appears malformed or missing model data.",
    fixed = TRUE
  )
})

test_that("Crow-AMSAA parameter recovery works (log-log simulation)", {
  set.seed(123)

  # True parameters (log-log)
  beta_true <- 1.3
  lambda_true <- 0.01

  # Observation times (cumulative)
  n <- 200
  t_obs <- seq(50, 5000, length.out = n)
  log_t_obs <- log(t_obs)

  # Mean cumulative failures in log-log space
  log_mu <- log(lambda_true) + beta_true * log_t_obs
  mu <- exp(log_mu)

  # Poisson increments, strictly positive
  failures <- as.integer(pmax(rpois(n, diff(c(0, mu))), 1))

  # Interarrival times (increments)
  times <- c(t_obs[1], diff(t_obs))

  # Fit Crow-AMSAA model
  fit <- rga(times, failures, model_type = "Crow-AMSAA")

  tol_beta <- 0.1
  tol_lambda <- 0.01

  expect_true(abs(fit$betas - beta_true) < tol_beta,
    info = paste("beta estimate:", fit$betas)
  )
  expect_true(abs(fit$lambdas - lambda_true) < tol_lambda,
    info = paste("lambda estimate:", fit$lambdas)
  )
})

test_that("Piecewise NHPP parameter recovery works", {
  # True params
  beta1 <- 0.7
  beta2 <- 1.4
  lambda1 <- 0.02
  tb <- 200
  lambda2 <- lambda1 * tb^(beta1 - beta2) # continuity at breakpoint

  # Generate observation times and interval inputs for rga()
  time_points <- seq(5, 1000, length.out = 200)
  intervals <- c(time_points[1], diff(time_points))

  # True cumulative counts (piecewise power law, continuous at tb)
  cumN <- ifelse(time_points <= tb,
    lambda1 * (time_points^beta1),
    lambda2 * (time_points^beta2)
  )
  failures <- c(cumN[1], diff(cumN))
  stopifnot(all(failures > 0))

  # Run rga with the known breakpoint (simpler, direct parameter recovery)
  res <- rga(
    times = intervals, failures = failures,
    model_type = "Piecewise NHPP", breaks = tb, conf_level = 0.95
  )

  # Helper: extract numeric betas from rga output (handles usual segmented structure)
  get_betas_num <- function(b) {
    if (is.numeric(b)) {
      return(as.numeric(b))
    }
    if (is.list(b) && "log_times" %in% names(b)) {
      mat <- b$log_times
      if ("Est." %in% colnames(mat)) {
        return(as.numeric(mat[, "Est."]))
      }
      if ("Estimate" %in% colnames(mat)) {
        return(as.numeric(mat[, "Estimate"]))
      }
      return(as.numeric(mat[, 1]))
    }
    stop("Can't extract betas from rga output.")
  }

  est_betas <- get_betas_num(res$betas)
  expect_length(est_betas, 2)
  expect_equal(est_betas[1], beta1, tolerance = 1e-6)
  expect_equal(est_betas[2], beta2, tolerance = 1e-6)

  # Lambdas should be numeric vector
  est_lambdas <- as.numeric(res$lambdas)
  expect_length(est_lambdas, 2)
  expect_equal(est_lambdas[1], lambda1, tolerance = 1e-4)
  expect_equal(est_lambdas[2], lambda2, tolerance = 1e-4)

  # Breakpoint (rga stores on log-scale) -> back-transform and compare
  expect_true(!is.null(res$breakpoints))
  recovered_tb <- exp(res$breakpoints[1])
  expect_equal(recovered_tb, tb, tolerance = 1e-6)
})

test_that("rga() handles increasing dataset sizes efficiently", {
  set.seed(123)
  sizes <- c(100, 500, 1000, 5000)
  beta_true <- 1.2
  lambda_true <- 0.01

  for (n in sizes) {
    t_obs <- seq(50, 5000, length.out = n)
    log_mu <- log(lambda_true) + beta_true * log(t_obs)
    mu <- exp(log_mu)
    failures <- as.integer(pmax(rpois(n, diff(c(0, mu))), 1))
    times <- c(t_obs[1], diff(t_obs))

    start_time <- Sys.time()
    fit <- rga(times, failures, model_type = "Crow-AMSAA")
    end_time <- Sys.time()

    runtime <- as.numeric(difftime(end_time, start_time, units = "secs"))
    message(sprintf("n = %d, runtime = %.3f sec", n, runtime))

    expect_true(abs(fit$betas - beta_true) < 0.1)
  }
})

test_that("Crow-AMSAA is robust to small noise in times", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  set.seed(101)

  # True parameters
  beta_true <- 1.2
  lambda_true <- 0.01
  n <- 200
  t_obs <- seq(50, 5000, length.out = n)
  log_mu <- log(lambda_true) + beta_true * log(t_obs)
  mu <- exp(log_mu)
  failures <- as.integer(pmax(rpois(n, diff(c(0, mu))), 1))
  times <- c(t_obs[1], diff(t_obs))

  # Fit on original data
  fit_orig <- rga(times, failures, model_type = "Crow-AMSAA")

  # Add trivial Gaussian noise to times (~0.5% of original value)
  times_noisy <- times * (1 + rnorm(n, mean = 0, sd = 0.005))

  fit_noisy <- rga(times_noisy, failures, model_type = "Crow-AMSAA")

  # Parameters should not change meaningfully
  expect_true(abs(fit_noisy$betas - fit_orig$betas) < 0.02)
  expect_true(abs(fit_noisy$lambdas - fit_orig$lambdas) < 0.001)
})

test_that("Crow-AMSAA is robust to small noise in failures", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  set.seed(202)

  beta_true <- 1.2
  lambda_true <- 0.01
  n <- 200
  t_obs <- seq(50, 5000, length.out = n)
  log_mu <- log(lambda_true) + beta_true * log(t_obs)
  mu <- exp(log_mu)
  failures <- as.integer(pmax(rpois(n, diff(c(0, mu))), 1))
  times <- c(t_obs[1], diff(t_obs))

  fit_orig <- rga(times, failures, model_type = "Crow-AMSAA")

  # Add tiny Gaussian noise to failures and round to integers
  failures_noisy <- as.integer(pmax(failures + rnorm(n, mean = 0, sd = 0.05), 1))

  fit_noisy <- rga(times, failures_noisy, model_type = "Crow-AMSAA")

  expect_true(abs(fit_noisy$betas - fit_orig$betas) < 0.05)
  expect_true(abs(fit_noisy$lambdas - fit_orig$lambdas) < 0.01)
})

test_that("Piecewise NHPP is robust to small noise in times and failures", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  set.seed(303)

  beta_true <- c(0.8, 1.5)
  lambda_true <- c(0.02, NA)
  break_time <- 2500
  n <- 200
  t_obs <- seq(100, 5000, length.out = n)
  log_lambda2 <- log(lambda_true[1]) + beta_true[1] * log(break_time) - beta_true[2] * log(break_time)
  lambda_true[2] <- exp(log_lambda2)

  log_mu <- numeric(n)
  for (i in seq_along(t_obs)) {
    if (t_obs[i] <= break_time) {
      log_mu[i] <- log(lambda_true[1]) + beta_true[1] * log(t_obs[i])
    } else {
      log_mu[i] <- log_lambda2 + beta_true[2] * log(t_obs[i])
    }
  }
  mu <- exp(log_mu)
  failures <- as.integer(pmax(rpois(n, diff(c(0, mu))), 1))
  times <- c(t_obs[1], diff(t_obs))

  fit_orig <- rga(times, failures, model_type = "Piecewise NHPP", breaks = c(break_time))

  # Add small noise
  times_noisy <- times * (1 + rnorm(n, 0, 0.005))
  failures_noisy <- as.integer(pmax(failures + rnorm(n, 0, 0.05), 1))

  fit_noisy <- rga(times_noisy, failures_noisy, model_type = "Piecewise NHPP", breaks = c(break_time))

  slopes_orig <- fit_orig$betas$log_times[, "Est."]
  slopes_noisy <- fit_noisy$betas$log_times[, "Est."]
  intercepts_orig <- fit_orig$lambdas
  intercepts_noisy <- fit_noisy$lambdas

  expect_true(all(abs(slopes_noisy - slopes_orig) < 0.05))
  expect_true(all(abs(intercepts_noisy - intercepts_orig) < 0.01))
})

test_that("rga works with input from testdata (Crow-AMSAA)", {
  # testdata is lazy-loaded via LazyData: true; no explicit data() call needed
  g1 <- subset(testdata, LRU == "G1")
  times <- g1$Cum_ETI
  failures <- g1$Failure_Count

  fit <- rga(times, failures, model_type = "Crow-AMSAA", conf_level = 0.95)

  expect_true(all(fit$fitted_values > 0))
  expect_true(all(fit$lower_bounds > 0))
  expect_true(all(fit$upper_bounds > 0))
})

# Create a simple dataset for testing
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
rga_obj <- rga(times, failures)

test_that("print.rga works correctly", {
  # Output should include key phrases
  expect_output(print(rga_obj), "Reliability Growth Analysis")
  expect_output(print(rga_obj), "Model Type: Crow-AMSAA")
  expect_output(print(rga_obj), "Log-likelihood:")
  expect_output(print(rga_obj), "AIC:")
  expect_output(print(rga_obj), "BIC:")

  # Should invisibly return the same object
  expect_invisible(print(rga_obj))
})

test_that("plot.rga basic functionality works", {
  # Just checks that no error is thrown
  expect_silent(plot(rga_obj))

  # With options
  expect_silent(plot(rga_obj, conf_bounds = FALSE, legend = FALSE))
  expect_silent(plot(rga_obj, log = TRUE))
  expect_silent(plot(rga_obj, legend_pos = "topright"))
})

test_that("plot.rga argument validation works", {
  expect_error(plot(rga_obj, conf_bounds = "yes"), "'conf_bounds' must be a single logical value")
  expect_error(plot(rga_obj, legend = c(TRUE, FALSE)), "'legend' must be a single logical value")
  expect_error(plot(rga_obj, log = NA), "missing value where TRUE/FALSE needed")
  expect_error(plot(rga_obj, legend_pos = 1), "'legend_pos' must be a single character string")
})

# Optional: visual regression tests (requires vdiffr)
if (requireNamespace("vdiffr", quietly = TRUE)) {
  test_that("plot.rga visual output is stable", {
    vdiffr::expect_doppelganger("Crow-AMSAA plot", function() plot(rga_obj))
  })
}

test_that("rga() errors on perfect (noiseless) collinearity between predictor and response", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  # Perfect power-law relationship between cumulative time and cumulative failures
  n <- 200
  cum_time <- seq(1, n)
  lambda <- 2.0
  beta <- 0.75
  cum_failures <- lambda * cum_time^beta

  # failures per interval (must be positive)
  failures <- diff(c(0, cum_failures))

  # Expect perfect-collinearity error
  expect_error(
    rga(times = rep(1, n), failures = failures),
    regexp = "Perfect collinearity detected"
  )

  # also check with data.frame input
  df <- data.frame(times = rep(1, n), failures = failures)
  expect_error(
    rga(df),
    regexp = "Perfect collinearity detected"
  )
})

test_that("rga() fits near-noiseless data and is at least as fast as noisy data (Crow-AMSAA)", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  set.seed(42)
  n <- 800
  cum_time <- seq(1, n)
  lambda <- 5.0
  beta <- 0.6
  cum_failures <- lambda * cum_time^beta
  failures_per_interval <- diff(c(0, cum_failures))

  # Create a near-noiseless version
  near_noise <- rnorm(n, mean = 0, sd = 1e-2)
  cum_failures_near <- cum_failures * exp(near_noise)
  failures_near <- diff(c(0, cum_failures_near))

  # Create a clearly noisy version
  big_noise <- rnorm(n, mean = 0, sd = 0.1)
  cum_failures_noisy <- cum_failures * exp(big_noise)
  failures_noisy <- diff(c(0, cum_failures_noisy))

  # Ensure positiveness
  failures_near <- pmax(failures_near, .Machine$double.eps)
  failures_noisy <- pmax(failures_noisy, .Machine$double.eps)

  # Timing: Crow-AMSAA model (default)
  t_near <- system.time({
    fit_near <- rga(times = rep(1, n), failures = failures_near, model_type = "Crow-AMSAA")
  })[["elapsed"]]

  t_noisy <- system.time({
    fit_noisy <- rga(times = rep(1, n), failures = failures_noisy, model_type = "Crow-AMSAA")
  })[["elapsed"]]

  # Require near-noiseless to be <= 1.2 * noisy (allow CI variability)
  expect_true(
    t_near <= t_noisy * 1.2 + 0.1,
    info = sprintf("Noiseless fit took %.3fs vs noisy %.3fs", t_near, t_noisy)
  )
})

test_that("rga() fits near-noiseless data with user-supplied breaks (Piecewise NHPP) and is reasonably fast", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  set.seed(101)
  n <- 400
  cum_time <- seq(1, n)
  lambda <- 3.0
  beta <- 0.8
  cum_failures <- lambda * cum_time^beta
  failures <- diff(c(0, cum_failures))

  # near-noiseless
  near_noise <- rnorm(n, mean = 0, sd = 5e-3) # changed from 1e-4 to 5e-3
  cum_failures_near <- cum_failures * exp(near_noise)
  failures_near <- diff(c(0, cum_failures_near))
  failures_near <- pmax(failures_near, .Machine$double.eps)

  big_noise <- rnorm(n, mean = 0, sd = 0.1)
  cum_failures_noisy <- cum_failures * exp(big_noise)
  failures_noisy <- diff(c(0, cum_failures_noisy))
  failures_noisy <- pmax(failures_noisy, .Machine$double.eps)

  # choose a breakpoint roughly in the middle (on original scale)
  break_pt <- floor(n / 2)

  t_near_pw <- system.time({
    fit_near_pw <- rga(
      times = rep(1, n), failures = failures_near,
      model_type = "Piecewise NHPP", breaks = break_pt
    )
  })[["elapsed"]]

  t_noisy_pw <- system.time({
    fit_noisy_pw <- rga(
      times = rep(1, n), failures = failures_noisy,
      model_type = "Piecewise NHPP", breaks = break_pt
    )
  })[["elapsed"]]

  # Require near-noiseless to be <= 1.2 * noisy (allow CI variability)
  expect_true(
    t_near_pw <= t_noisy_pw * 1.2 + 0.1,
    info = sprintf("Noiseless fit took %.3fs vs noisy %.3fs", t_near_pw, t_noisy_pw)
  )
})

# --------------------------------------------------------------------------- #
# predict_rga() tests                                                        #
# --------------------------------------------------------------------------- #

# Shared fixture for forecast tests
fc_times <- c(100, 200, 300, 400, 500)
fc_failures <- c(1, 2, 1, 3, 2)
fc_fit <- rga(fc_times, fc_failures)

test_that("predict_rga: input validation — wrong class", {
  expect_error(predict_rga(list()), "'object' must be an object of class 'rga'.",
    fixed = TRUE
  )
})

test_that("predict_rga: input validation — non-numeric times", {
  expect_error(predict_rga(fc_fit, times = "600"),
    "'times' must be a numeric vector.",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = list(600)),
    "'times' must be a numeric vector.",
    fixed = TRUE
  )
})

test_that("predict_rga: input validation — empty times", {
  expect_error(predict_rga(fc_fit, times = numeric(0)),
    "'times' cannot be empty.",
    fixed = TRUE
  )
})

test_that("predict_rga: input validation — NA / NaN in times", {
  expect_error(predict_rga(fc_fit, times = c(600, NA)),
    "'times' contains missing (NA) or NaN values.",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = c(600, NaN)),
    "'times' contains missing (NA) or NaN values.",
    fixed = TRUE
  )
})

test_that("predict_rga: input validation — non-finite / non-positive times", {
  expect_error(predict_rga(fc_fit, times = c(600, Inf)),
    "All values in 'times' must be finite and > 0.",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = c(600, -1)),
    "All values in 'times' must be finite and > 0.",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = c(600, 0)),
    "All values in 'times' must be finite and > 0.",
    fixed = TRUE
  )
})

test_that("predict_rga: input validation — bad conf_level", {
  expect_error(predict_rga(fc_fit, times = c(600), conf_level = c(0.9, 0.95)),
    "'conf_level' must be a single numeric value.",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = c(600), conf_level = 0),
    "'conf_level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = c(600), conf_level = 1),
    "'conf_level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )
  expect_error(predict_rga(fc_fit, times = c(600), conf_level = Inf),
    "'conf_level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )
})

test_that("predict_rga: hindcast warning when times <= max observed", {
  max_obs <- sum(fc_times) # cumulative time of last observation = 1500
  expect_warning(
    predict_rga(fc_fit, times = c(max_obs - 1, max_obs)),
    "Hindcasting is allowed but may not be meaningful."
  )
})

test_that("predict_rga uses stored cumulative times for cumulative_failure_times input", {
  fit_cumulative <- rga(
    cumsum(fc_times), fc_failures,
    times_type = "cumulative_failure_times"
  )

  expect_warning(
    predict_rga(fit_cumulative, times = c(max(fit_cumulative$cum_times))),
    "Hindcasting is allowed but may not be meaningful."
  )
})

test_that("predict_rga: Crow-AMSAA cum_failures == lambda * T^beta", {
  fc <- predict_rga(fc_fit, times = c(2000, 3000, 4000))
  expected <- fc_fit$lambdas * c(2000, 3000, 4000)^fc_fit$betas
  expect_equal(fc$cum_failures, expected, tolerance = 1e-6, ignore_attr = TRUE)

  # Bounds ordering: lower <= fit <= upper
  expect_true(all(fc$lower_bounds <= fc$cum_failures))
  expect_true(all(fc$cum_failures <= fc$upper_bounds))
})

test_that("predict_rga: Piecewise NHPP forecast is numeric beyond last breakpoint", {
  fit_pw <- suppressWarnings(
    rga(fc_times, fc_failures, model_type = "Piecewise NHPP", breaks = c(450))
  )
  fc_pw <- predict_rga(fit_pw, times = c(2000, 3000))

  expect_s3_class(fc_pw, "rga_predict")
  expect_equal(fc_pw$model_type, "Piecewise NHPP")
  expect_true(is.numeric(fc_pw$cum_failures))
  expect_length(fc_pw$cum_failures, 2)
  expect_true(all(is.finite(fc_pw$cum_failures)))
})

test_that("predict_rga: monotonicity — larger times yield larger cum_failures", {
  fc <- predict_rga(fc_fit, times = c(2000, 3000, 4000, 5000))
  expect_true(all(diff(fc$cum_failures) > 0))
})

test_that("predict_rga: wider conf_level produces wider bounds", {
  fc90 <- predict_rga(fc_fit, times = c(2000, 3000), conf_level = 0.90)
  fc99 <- predict_rga(fc_fit, times = c(2000, 3000), conf_level = 0.99)

  width90 <- fc90$upper_bounds - fc90$lower_bounds
  width99 <- fc99$upper_bounds - fc99$lower_bounds
  expect_true(all(width99 > width90))
})

test_that("print.rga_predict: smoke test and invisibility", {
  fc <- predict_rga(fc_fit, times = c(2000, 3000, 4000))
  expect_output(print(fc), "Reliability Growth Forecast")
  expect_output(print(fc), "Crow-AMSAA")
  expect_output(print(fc), "Cum.Failures")
  expect_invisible(print(fc))
})

test_that("print.rga_predict: errors on wrong class", {
  expect_error(print.rga_predict(list()),
    "'x' must be an object of class 'rga_predict'.",
    fixed = TRUE
  )
})

test_that("plot.rga_predict: smoke test — no error with default args", {
  fc <- predict_rga(fc_fit, times = c(2000, 3000, 4000))
  expect_silent(plot(fc))
  expect_silent(plot(fc, conf_bounds = FALSE))
  expect_silent(plot(fc, legend = FALSE))
  expect_silent(plot(fc, legend_pos = "topright"))
})

test_that("plot.rga_predict: invisibly returns NULL", {
  fc <- predict_rga(fc_fit, times = c(2000, 3000))
  expect_invisible(plot(fc))
})

test_that("plot.rga_predict: argument validation", {
  fc <- predict_rga(fc_fit, times = c(2000, 3000))
  expect_error(plot.rga_predict(list()),
    "'x' must be an object of class 'rga_predict'.",
    fixed = TRUE
  )
  expect_error(plot(fc, conf_bounds = "yes"),
    "'conf_bounds' must be a single logical value.",
    fixed = TRUE
  )
  expect_error(plot(fc, legend = "yes"),
    "'legend' must be a single logical value.",
    fixed = TRUE
  )
  expect_error(plot(fc, legend_pos = 1),
    "'legend_pos' must be a single character string.",
    fixed = TRUE
  )
})

# --------------------------------------------------------------------------- #

test_that("rga output retains row / case names from input", {
  # Case 1: Named numeric vectors
  times <- c(A = 100, B = 200, C = 300, D = 400, E = 500)
  failures <- c(A = 1, B = 2, C = 1, D = 3, E = 2)

  res_named <- rga(times, failures)

  # input names preserved in stored inputs
  expect_equal(names(res_named$times), names(times))
  expect_equal(names(res_named$cum_times), names(times))
  expect_equal(names(res_named$failures), names(failures))

  # Case 2: Piecewise NHPP

  # Use a dataset that can reasonably produce a breakpoint (small example)
  times2 <- c(t1 = 10, t2 = 20, t3 = 30, t4 = 40, t5 = 50)
  failures2 <- c(t1 = 1, t2 = 1, t3 = 2, t4 = 3, t5 = 5)

  res_piecewise <- rga(times2, failures2, model_type = "Piecewise NHPP", breaks = c(30))

  expect_equal(names(res_piecewise$times), names(times2))
  expect_equal(names(res_piecewise$failures), names(failures2))
})

# --------------------------------------------------------------------------- #
# MLE estimation tests                                                         #
# --------------------------------------------------------------------------- #

test_that("MLE + Piecewise NHPP errors with 'is not supported'", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  expect_error(
    rga(times, failures, model_type = "Piecewise NHPP", method = "MLE"),
    "is not supported"
  )
})

test_that("invalid method value errors via match.arg", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  expect_error(
    rga(times, failures, method = "OLS"),
    "should be one of"
  )
})

test_that("MLE returns correct class and fields", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures, method = "MLE")

  expect_s3_class(fit, "rga")
  expect_equal(fit$method, "MLE")
  expect_null(fit$model)
  expect_true(is.matrix(fit$vcov))
  expect_equal(dim(fit$vcov), c(2L, 2L))
  expect_true(all(fit$lower_bounds <= fit$fitted_values + 1e-10))
  expect_true(all(fit$fitted_values <= fit$upper_bounds + 1e-10))
})

test_that("LS still has method='LS' and vcov=NULL", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures)

  expect_equal(fit$method, "LS")
  expect_null(fit$vcov)
})

test_that("MLE logLik/AIC/BIC are finite and AIC == -2*logLik + 4", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures, method = "MLE")

  expect_true(is.finite(fit$logLik))
  expect_true(is.finite(fit$AIC))
  expect_true(is.finite(fit$BIC))
  expect_equal(fit$AIC, -2 * fit$logLik + 4, tolerance = 1e-10)
})

test_that("print.rga shows Estimation Method for MLE and LS", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit_mle <- rga(times, failures, method = "MLE")
  fit_ls <- rga(times, failures, method = "LS")

  expect_output(print(fit_mle), "Estimation Method: MLE")
  expect_output(print(fit_ls), "Estimation Method: LS")
})

test_that("MLE parameter recovery (n=10 wide-interval NHPP simulation)", {
  #' @srrstats {G5.6} Parameter recovery test for MLE
  skip_on_cran()
  set.seed(42)

  # Wide intervals ensure >> 100 expected failures per interval, so:
  # (a) pmax(rpois, 1) floor never triggers (no bias), and
  # (b) each interval is highly informative, giving tight MLE estimates.
  beta_true <- 0.8
  lambda_true <- 5.0
  t_obs <- seq(200, 2000, by = 200) # 10 intervals, each 200 units wide
  mu <- lambda_true * t_obs^beta_true
  failures <- as.integer(pmax(rpois(length(t_obs), diff(c(0, mu))), 1))
  times <- c(t_obs[1], diff(t_obs))

  fit <- rga(times, failures, method = "MLE")

  expect_true(abs(fit$betas - beta_true) < 0.10,
    info = paste("beta estimate:", fit$betas)
  )
  expect_true(abs(fit$lambdas - lambda_true) < 0.50,
    info = paste("lambda estimate:", fit$lambdas)
  )
})

test_that("predict_rga with MLE: finite, positive, monotone", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures, method = "MLE")
  fc <- predict_rga(fit, times = c(2000, 3000, 4000))

  expect_true(all(is.finite(fc$cum_failures)))
  expect_true(all(fc$cum_failures > 0))
  expect_true(all(diff(fc$cum_failures) > 0))
})

test_that("predict_rga MLE formula: lambda * T^beta", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures, method = "MLE")
  fc <- predict_rga(fit, times = c(2000, 3000, 4000))

  expected <- fit$lambdas * c(2000, 3000, 4000)^fit$betas
  expect_equal(fc$cum_failures, expected, tolerance = 1e-6, ignore_attr = TRUE)
})

test_that("predict_rga MLE wider conf_level yields wider bounds", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures, method = "MLE")
  fc90 <- predict_rga(fit, times = c(2000, 3000), conf_level = 0.90)
  fc99 <- predict_rga(fit, times = c(2000, 3000), conf_level = 0.99)

  width90 <- fc90$upper_bounds - fc90$lower_bounds
  width99 <- fc99$upper_bounds - fc99$lower_bounds
  expect_true(all(width99 > width90))
})

test_that("plot.rga and plot.rga_predict with MLE: no error", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)
  fit <- rga(times, failures, method = "MLE")
  fc <- predict_rga(fit, times = c(2000, 3000, 4000))

  expect_silent(plot(fit))
  expect_silent(plot(fc))
})

# ---- overlay_rga tests ----
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by stop() is unique.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {RE6.0} Model objects have overlay plot methods.
#' @srrstats {RE6.2} The overlay plot shows fitted values with optional CIs.

test_that("overlay_rga: non-list input errors", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  expect_error(overlay_rga(m), "'models' must be a non-empty list of 'rga' objects.", fixed = TRUE)
})

test_that("overlay_rga: empty list errors", {
  expect_error(overlay_rga(list()), "'models' must be a non-empty list of 'rga' objects.", fixed = TRUE)
})

test_that("overlay_rga: non-rga element errors", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  expect_error(overlay_rga(list(m, list(a = 1))),
               "All elements of 'models' must be objects of class 'rga'.", fixed = TRUE)
})

test_that("overlay_rga: conf_bounds wrong type errors", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  expect_error(overlay_rga(list(m), conf_bounds = "yes"),
               "'conf_bounds' must be a single logical value.", fixed = TRUE)
})

test_that("overlay_rga: legend wrong type errors", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  expect_error(overlay_rga(list(m), legend = 1L),
               "'legend' must be a single logical value.", fixed = TRUE)
})

test_that("overlay_rga: log wrong type errors", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  expect_error(overlay_rga(list(m), log = "yes"),
               "'log' must be a single logical value.", fixed = TRUE)
})

test_that("overlay_rga: legend_pos wrong length errors", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  expect_error(overlay_rga(list(m), legend_pos = c("a", "b")),
               "'legend_pos' must be a single character string.", fixed = TRUE)
})

test_that("overlay_rga: colors too short errors", {
  m1 <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  expect_error(overlay_rga(list(m1, m2), colors = "black"),
               "'colors' must be a character vector with at least one color per model.", fixed = TRUE)
})

test_that("overlay_rga: single model renders without error", {
  m <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_rga(list(m)))
})

test_that("overlay_rga: two named models render and return NULL", {
  m1 <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  pdf(NULL)
  on.exit(dev.off())
  result <- overlay_rga(list(System_A = m1, System_B = m2))
  expect_null(result)
})

test_that("overlay_rga: two unnamed models render without error", {
  m1 <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_rga(list(m1, m2)))
})

test_that("overlay_rga: conf_bounds = FALSE renders without error", {
  m1 <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_rga(list(m1, m2), conf_bounds = FALSE))
})

test_that("overlay_rga: log = TRUE renders without error", {
  m1 <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_rga(list(m1, m2), log = TRUE))
})

test_that("overlay_rga: custom colors render without error", {
  m1 <- rga(c(100, 200, 300, 400, 500), c(1, 2, 1, 3, 2))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_rga(list(m1, m2), colors = c("steelblue", "tomato")))
})

test_that("overlay_rga: models with different-length datasets render without error", {
  m1 <- rga(c(100, 200, 300), c(1, 2, 1))
  m2 <- rga(c(150, 300, 450, 600, 750), c(2, 1, 3, 2, 4))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_rga(list(m1, m2)))
})

.expected_failures_weibull <- function(eta, runtimes, window, beta) {
  delta_h <- ((runtimes + window) / eta)^beta - (runtimes / eta)^beta
  sum(1 - exp(-delta_h))
}

.solve_weibull_eta <- function(target_failures, runtimes, window, beta) {
  target_failures <- min(target_failures, length(runtimes))

  objective <- function(log_eta) {
    .expected_failures_weibull(exp(log_eta), runtimes, window, beta) - target_failures
  }

  lower <- log(max(min(runtimes) * 1e-6, 1e-8))
  upper <- log(max(runtimes + window) * 1e6)

  for (i in seq_len(25)) {
    if (objective(lower) >= 0) {
      break
    }
    lower <- lower - 2
  }
  for (i in seq_len(25)) {
    if (objective(upper) <= 0) {
      break
    }
    upper <- upper + 2
  }

  exp(stats::uniroot(objective, c(lower, upper))$root)
}

#' Simulate Failures from a Conditional Weibull Model
#'
#' Simulates which units in a non-failed population fail next by using a
#' Weibull life model conditional on each unit's current runtime. When a
#' positive `window` is supplied, the function calibrates a Weibull scale
#' parameter (unless provided directly) so that the expected number of failures
#' within the window matches `n`. Units are then sampled with probability
#' proportional to their conditional Weibull failure probability over the
#' window, and failure times are drawn from the truncated conditional Weibull
#' distribution. The full fleet is returned: selected units are labelled
#' `"Failure"` and the remaining units are labelled `"Suspension"`.
#'
#' When `window = NULL`, the function returns the current fleet state at the
#' supplied runtimes. In this case, failing units are selected using relative
#' Weibull hazard weights implied by `beta`.
#'
#' @srrstats {G1.0} This function implements conditional Weibull simulation,
#' a standard reliability-modeling technique.
#' @srrstats {G1.4} [`roxygen2`](https://roxygen2.r-lib.org/) documentation is
#' used to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.13} The function checks for missing data and errors.
#' @srrstats {G5.2} Unit tests demonstrate error messages.
#' @srrstats {G5.4} Unit tests include correctness tests with fixed data.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks.
#'
#' @param n Positive integer. Number of failures to simulate.
#' @param runtimes Numeric vector of positive values. The current operating
#'   runtime of each unit in the non-failed population.
#' @param replace Logical scalar. If `TRUE`, sampling is done with replacement
#'   (a unit may be selected more than once). Default is `FALSE`.
#' @param window `NULL` or a single positive numeric. The width of the
#'   observation window. When `NULL` (default), event times equal current
#'   runtimes. When provided, failure times are sampled from the conditional
#'   Weibull distribution over `(runtime, runtime + window]`, and suspension
#'   times are `runtime + window`.
#' @param beta Positive numeric scalar. Weibull shape parameter used to model
#'   the age-dependent hazard. Defaults to `1`, corresponding to an
#'   exponential-process assumption.
#' @param eta `NULL` or a single positive numeric. Weibull scale parameter.
#'   When `NULL` and `window` is supplied, the scale is calibrated so that the
#'   expected number of failures across the fleet during the window matches
#'   `n`.
#' @return A data frame with `length(runtimes)` rows sorted ascending by
#'   `runtime`, containing:
#'   \item{index}{Integer index of the unit in `runtimes`.}
#'   \item{runtime}{Simulated event time.}
#'   \item{type}{Character; `"Failure"` for selected units, `"Suspension"` for
#'     the rest.}
#'   The returned object also carries attributes `weibull_beta` and
#'   `weibull_eta` describing the Weibull parameters used for the simulation.
#' @family data preparation
#' @examples
#' set.seed(42)
#' runtimes <- c(100, 500, 200, 800, 300)
#' result <- sim_failures(2, runtimes, beta = 1.5)
#' print(result)
#'
#' # With an observation window
#' set.seed(42)
#' result_w <- sim_failures(2, runtimes, window = 50, beta = 1.5)
#' print(result_w)
#' @importFrom stats runif uniroot
#' @export
sim_failures <- function(n, runtimes, replace = FALSE, window = NULL,
                         beta = 1, eta = NULL) {
  # Validate n
  if (!is.numeric(n) || length(n) != 1) {
    stop("'n' must be a single numeric value.")
  }
  if (is.na(n) || is.nan(n) || !is.finite(n)) {
    stop("'n' must be a finite positive integer.")
  }
  if (n != floor(n) || n < 1) {
    stop("'n' must be a positive integer.")
  }

  # Validate runtimes
  if (!is.numeric(runtimes)) {
    stop("'runtimes' must be a numeric vector.")
  }
  if (length(runtimes) == 0) {
    stop("'runtimes' cannot be empty.")
  }
  if (any(is.na(runtimes)) || any(is.nan(runtimes))) {
    stop("'runtimes' contains missing (NA) or NaN values.")
  }
  if (any(!is.finite(runtimes)) || any(runtimes <= 0)) {
    stop("All values in 'runtimes' must be finite and > 0.")
  }

  # Validate replace
  if (!is.logical(replace) || length(replace) != 1 || is.na(replace)) {
    stop("'replace' must be a single logical value (TRUE or FALSE).")
  }

  # Validate n vs length(runtimes) when sampling without replacement
  if (!replace && n > length(runtimes)) {
    stop("'n' cannot exceed the number of units in 'runtimes' when replace = FALSE.")
  }

  # Validate beta
  if (!is.numeric(beta) || length(beta) != 1) {
    stop("'beta' must be a single numeric value.")
  }
  if (is.na(beta) || is.nan(beta) || !is.finite(beta)) {
    stop("'beta' must be a finite positive numeric value.")
  }
  if (beta <= 0) {
    stop("'beta' must be a positive numeric value.")
  }

  # Validate window
  if (!is.null(window)) {
    if (!is.numeric(window) || length(window) != 1) {
      stop("'window' must be a single positive numeric value.")
    }
    if (is.na(window) || is.nan(window) || !is.finite(window)) {
      stop("'window' must be a finite positive numeric value.")
    }
    if (window <= 0) {
      stop("'window' must be a positive numeric value.")
    }
  }

  # Validate eta
  if (!is.null(eta)) {
    if (!is.numeric(eta) || length(eta) != 1) {
      stop("'eta' must be a single positive numeric value.")
    }
    if (is.na(eta) || is.nan(eta) || !is.finite(eta)) {
      stop("'eta' must be a finite positive numeric value.")
    }
    if (eta <= 0) {
      stop("'eta' must be a positive numeric value.")
    }
  }

  target_failures <- min(n, length(runtimes))

  if (is.null(window)) {
    # Without a forward window, return the current fleet state while weighting
    # selection by the relative Weibull hazard implied by beta.
    prob <- runtimes^(beta - 1)
    if (any(!is.finite(prob)) || sum(prob) <= 0) {
      prob <- rep(1, length(runtimes))
    }
    eta_used <- eta
  } else {
    eta_used <- if (is.null(eta)) {
      .solve_weibull_eta(target_failures, runtimes, window, beta)
    } else {
      eta
    }
    delta_h <- ((runtimes + window) / eta_used)^beta - (runtimes / eta_used)^beta
    prob <- 1 - exp(-delta_h)
    if (any(!is.finite(prob)) || sum(prob) <= 0) {
      stop("Unable to compute valid Weibull failure probabilities from 'runtimes'.")
    }
  }

  idx <- sample(seq_along(runtimes), size = n, replace = replace, prob = prob)

  fail_idx <- unique(idx)
  susp_idx <- setdiff(seq_along(runtimes), fail_idx)

  # Compute event times
  if (is.null(window)) {
    fail_times <- runtimes[fail_idx]
    susp_times <- runtimes[susp_idx]
  } else {
    fail_runtimes <- runtimes[fail_idx]
    h_start <- (fail_runtimes / eta_used)^beta
    delta_fail <- ((fail_runtimes + window) / eta_used)^beta - h_start
    u <- stats::runif(length(fail_idx))
    fail_times <- eta_used * (h_start - log1p(-u * (1 - exp(-delta_fail))))^(1 / beta)
    susp_times <- runtimes[susp_idx] + window
  }

  result <- data.frame(
    index = c(fail_idx, susp_idx),
    runtime = c(fail_times, susp_times),
    type = c(
      rep("Failure", length(fail_idx)),
      rep("Suspension", length(susp_idx))
    )
  )
  result <- result[order(result$runtime), ]
  rownames(result) <- NULL
  attr(result, "weibull_beta") <- beta
  attr(result, "weibull_eta") <- eta_used

  return(result)
}

test_that("sim_failures returns correct output structure and attributes", {
  set.seed(1)
  runtimes <- c(100, 500, 200, 800, 300)
  result <- sim_failures(2, runtimes, window = 50, beta = 1.5)

  expect_s3_class(result, "data.frame")
  expect_named(result, c("index", "runtime", "type"))
  expect_equal(nrow(result), 5)
  expect_type(result$index, "integer")
  expect_type(result$runtime, "double")
  expect_type(result$type, "character")
  expect_true(all(result$type %in% c("Failure", "Suspension")))
  expect_true(all(diff(result$runtime) >= 0))
  expect_equal(attr(result, "weibull_beta"), 1.5)
  expect_true(is.numeric(attr(result, "weibull_eta")))
  expect_gt(attr(result, "weibull_eta"), 0)
})

test_that("sim_failures with beta > 1 favors older units", {
  set.seed(99)
  runtimes <- c(10, 1000)
  n_rep <- 5000
  draws <- replicate(n_rep, {
    res <- sim_failures(1, runtimes, window = 50, beta = 2)
    res$index[res$type == "Failure"]
  })
  prop_unit2 <- mean(draws == 2)
  expect_gt(prop_unit2, 0.85)
})

test_that("sim_failures with beta < 1 favors younger units", {
  set.seed(99)
  runtimes <- c(10, 1000)
  n_rep <- 5000
  draws <- replicate(n_rep, {
    res <- sim_failures(1, runtimes, window = 50, beta = 0.5)
    res$index[res$type == "Failure"]
  })
  prop_unit1 <- mean(draws == 1)
  expect_gt(prop_unit1, 0.70)
})

test_that("sim_failures errors: n is not numeric or not scalar", {
  expect_error(sim_failures("2", c(1, 2, 3)), "'n' must be a single numeric value.")
  expect_error(sim_failures(c(1, 2), c(1, 2, 3)), "'n' must be a single numeric value.")
})

test_that("sim_failures errors: n is NA, NaN, or Inf", {
  expect_error(sim_failures(NA_real_, c(1, 2, 3)), "'n' must be a finite positive integer.")
  expect_error(sim_failures(NaN, c(1, 2, 3)), "'n' must be a finite positive integer.")
  expect_error(sim_failures(Inf, c(1, 2, 3)), "'n' must be a finite positive integer.")
})

test_that("sim_failures errors: n is not a positive whole number", {
  expect_error(sim_failures(0, c(1, 2, 3)), "'n' must be a positive integer.")
  expect_error(sim_failures(-1, c(1, 2, 3)), "'n' must be a positive integer.")
  expect_error(sim_failures(1.5, c(1, 2, 3)), "'n' must be a positive integer.")
})

test_that("sim_failures errors: runtimes is not a numeric vector", {
  expect_error(sim_failures(1, c("a", "b")), "'runtimes' must be a numeric vector.")
  expect_error(sim_failures(1, TRUE), "'runtimes' must be a numeric vector.")
})

test_that("sim_failures errors: runtimes is empty", {
  expect_error(sim_failures(1, numeric(0)), "'runtimes' cannot be empty.")
})

test_that("sim_failures errors: runtimes contains NA or NaN", {
  expect_error(sim_failures(1, c(1, NA, 3)), "'runtimes' contains missing \\(NA\\) or NaN values.")
  expect_error(sim_failures(1, c(1, NaN, 3)), "'runtimes' contains missing \\(NA\\) or NaN values.")
})

test_that("sim_failures errors: runtimes contains non-finite or <= 0 values", {
  expect_error(sim_failures(1, c(1, Inf, 3)), "All values in 'runtimes' must be finite and > 0.")
  expect_error(sim_failures(1, c(0, 1, 2)), "All values in 'runtimes' must be finite and > 0.")
  expect_error(sim_failures(1, c(-1, 1, 2)), "All values in 'runtimes' must be finite and > 0.")
})

test_that("sim_failures errors: replace is not a logical scalar", {
  expect_error(sim_failures(1, c(1, 2, 3), replace = 1), "'replace' must be a single logical value \\(TRUE or FALSE\\).")
  expect_error(sim_failures(1, c(1, 2, 3), replace = NA), "'replace' must be a single logical value \\(TRUE or FALSE\\).")
  expect_error(sim_failures(1, c(1, 2, 3), replace = c(TRUE, FALSE)), "'replace' must be a single logical value \\(TRUE or FALSE\\).")
})

test_that("sim_failures errors: beta validation", {
  expect_error(sim_failures(1, c(1, 2, 3), beta = "2"), "'beta' must be a single numeric value.")
  expect_error(sim_failures(1, c(1, 2, 3), beta = NA_real_), "'beta' must be a finite positive numeric value.")
  expect_error(sim_failures(1, c(1, 2, 3), beta = Inf), "'beta' must be a finite positive numeric value.")
  expect_error(sim_failures(1, c(1, 2, 3), beta = 0), "'beta' must be a positive numeric value.")
})

test_that("sim_failures errors: eta validation", {
  expect_error(sim_failures(1, c(1, 2, 3), eta = "50"), "'eta' must be a single positive numeric value.")
  expect_error(sim_failures(1, c(1, 2, 3), eta = NA_real_), "'eta' must be a finite positive numeric value.")
  expect_error(sim_failures(1, c(1, 2, 3), eta = Inf), "'eta' must be a finite positive numeric value.")
  expect_error(sim_failures(1, c(1, 2, 3), eta = 0), "'eta' must be a positive numeric value.")
})

test_that("sim_failures errors: n > length(runtimes) without replacement", {
  expect_error(
    sim_failures(4, c(100, 200, 300), replace = FALSE),
    "'n' cannot exceed the number of units in 'runtimes' when replace = FALSE."
  )

  set.seed(1)
  result <- sim_failures(4, c(100, 200, 300), replace = TRUE, window = 50, beta = 1.5)
  expect_lte(nrow(result), 3)
})

test_that("sim_failures is reproducible with the same seed", {
  runtimes <- c(100, 500, 200, 800, 300)

  set.seed(42)
  r1 <- sim_failures(3, runtimes, window = 50, beta = 1.5)

  set.seed(42)
  r2 <- sim_failures(3, runtimes, window = 50, beta = 1.5)

  expect_equal(r1, r2)
})

test_that("sim_failures edge case: n = 1 returns full fleet", {
  set.seed(7)
  runtimes <- c(100, 500, 200)
  result <- sim_failures(1, runtimes, window = 50, beta = 1.5)
  expect_equal(nrow(result), 3)
  expect_named(result, c("index", "runtime", "type"))
})

test_that("sim_failures edge case: n = length(runtimes) returns all units without replacement", {
  set.seed(7)
  runtimes <- c(100, 500, 200, 800, 300)
  result <- sim_failures(length(runtimes), runtimes, replace = FALSE, window = 50, beta = 1.5)
  expect_equal(nrow(result), length(runtimes))
  expect_equal(sort(result$index), seq_along(runtimes))
})

test_that("sim_failures type labels: correct counts of Failure and Suspension", {
  set.seed(42)
  runtimes <- c(100, 500, 200, 800, 300)
  n <- 2
  result <- sim_failures(n, runtimes, replace = FALSE, window = 50, beta = 1.5)

  n_failures <- sum(result$type == "Failure")
  n_suspensions <- sum(result$type == "Suspension")

  expect_equal(n_failures, n)
  expect_equal(n_suspensions, length(runtimes) - n)
})

test_that("sim_failures window: failure times are in (runtime, runtime + window]", {
  set.seed(42)
  runtimes <- c(100, 500, 200, 800, 300)
  w <- 50
  result <- sim_failures(2, runtimes, window = w, beta = 1.5)

  failures <- result[result$type == "Failure", ]
  for (i in seq_len(nrow(failures))) {
    orig <- runtimes[failures$index[i]]
    expect_gt(failures$runtime[i], orig)
    expect_lte(failures$runtime[i], orig + w)
  }
})

test_that("sim_failures window: suspension times equal runtime + window exactly", {
  set.seed(42)
  runtimes <- c(100, 500, 200, 800, 300)
  w <- 50
  result <- sim_failures(2, runtimes, window = w, beta = 1.5)

  suspensions <- result[result$type == "Suspension", ]
  for (i in seq_len(nrow(suspensions))) {
    orig <- runtimes[suspensions$index[i]]
    expect_equal(suspensions$runtime[i], orig + w)
  }
})

test_that("sim_failures window = NULL: runtimes unchanged", {
  set.seed(42)
  runtimes <- c(100, 500, 200, 800, 300)
  result <- sim_failures(2, runtimes, beta = 1.5)

  for (i in seq_len(nrow(result))) {
    expect_equal(result$runtime[i], runtimes[result$index[i]])
  }
})

test_that("sim_failures window validation errors", {
  runtimes <- c(100, 500, 200)

  expect_error(
    sim_failures(1, runtimes, window = "50"),
    "'window' must be a single positive numeric value."
  )
  expect_error(
    sim_failures(1, runtimes, window = c(10, 20)),
    "'window' must be a single positive numeric value."
  )
  expect_error(
    sim_failures(1, runtimes, window = NA_real_),
    "'window' must be a finite positive numeric value."
  )
  expect_error(
    sim_failures(1, runtimes, window = Inf),
    "'window' must be a finite positive numeric value."
  )
  expect_error(
    sim_failures(1, runtimes, window = 0),
    "'window' must be a positive numeric value."
  )
  expect_error(
    sim_failures(1, runtimes, window = -5),
    "'window' must be a positive numeric value."
  )
})

test_that("sim_failures auto-calibrates eta to the target failure count", {
  set.seed(42)
  runtimes <- c(100, 500, 200, 800, 300)
  w <- 50
  beta <- 1.5
  result <- sim_failures(2, runtimes, window = w, beta = beta)
  eta <- attr(result, "weibull_eta")

  delta_h <- ((runtimes + w) / eta)^beta - (runtimes / eta)^beta
  expected_failures <- sum(1 - exp(-delta_h))

  expect_equal(expected_failures, 2, tolerance = 1e-5)
})

test_that("sim_failures preserves an explicit eta when provided", {
  set.seed(42)
  runtimes <- c(100, 500, 200, 800, 300)
  result <- sim_failures(2, runtimes, window = 50, beta = 1.5, eta = 900)

  expect_equal(attr(result, "weibull_eta"), 900)
})

# Internal MLE helper for Power Law NHPP.
# Fits N(t) = lambda * t^beta using maximum likelihood.
# Arguments:
#   cum_time   : numeric vector of cumulative event times
#   events     : numeric vector of event counts per time
#   conf_level : numeric scalar in (0,1)
# Returns a named list of estimates, SEs, fitted values, CIs, and GOF stats.
.fit_mle_power <- function(cum_time, events, conf_level) {
  N <- sum(events)
  T_max <- max(cum_time)
  n_obs <- length(events)
  t_prev <- c(0, cum_time[-n_obs])

  neg_loglik <- function(par) {
    beta <- par[1]
    lambda <- par[2]
    if (beta <= 0 || lambda <= 0) return(Inf)
    delta_t <- cum_time^beta - t_prev^beta
    if (any(delta_t <= 0)) return(Inf)
    ll <- N * log(lambda) + sum(events * log(delta_t)) - lambda * T_max^beta
    -ll
  }

  opt <- stats::optim(
    par = c(1, N / T_max),
    fn = neg_loglik,
    method = "L-BFGS-B",
    lower = c(1e-6, 1e-10),
    hessian = TRUE
  )

  beta_hat <- opt$par[1]
  lambda_hat <- opt$par[2]
  loglik <- -opt$value

  vcov_mat <- tryCatch(solve(opt$hessian), error = function(e) matrix(NA_real_, 2, 2))
  beta_se <- if (!anyNA(vcov_mat)) sqrt(max(vcov_mat[1, 1], 0)) else NA_real_

  fitted_values <- lambda_hat * cum_time^beta_hat

  z_val <- stats::qnorm(1 - (1 - conf_level) / 2)
  log_fit <- log(fitted_values)
  grad_mat <- cbind(log(cum_time), 1 / lambda_hat)
  var_lf <- rowSums((grad_mat %*% vcov_mat) * grad_mat)
  hw <- z_val * sqrt(pmax(var_lf, 0))

  list(
    beta          = beta_hat,
    lambda        = lambda_hat,
    beta_se       = beta_se,
    vcov          = vcov_mat,
    fitted_values = fitted_values,
    lower_bounds  = exp(log_fit - hw),
    upper_bounds  = exp(log_fit + hw),
    loglik        = loglik,
    residuals     = cumsum(events) - fitted_values,
    aic           = -2 * loglik + 4,
    bic           = -2 * loglik + 2 * log(n_obs)
  )
}

# Internal MLE helper for Log-Linear NHPP.
# Fits intensity lambda(t) = exp(a + b*t), cumulative Lambda(t) = (exp(a)/b)(exp(bt) - 1).
# For grouped data with event counts at cumulative times.
.fit_mle_loglinear <- function(cum_time, events, conf_level) {
  N <- sum(events)
  T_max <- max(cum_time)
  n_obs <- length(events)
  t_prev <- c(0, cum_time[-n_obs])

  neg_loglik <- function(par) {
    a <- par[1]
    b <- par[2]
    if (abs(b) < 1e-12) {
      # HPP case: intensity = exp(a), cumulative = exp(a)*t
      delta_Lambda <- exp(a) * (cum_time - t_prev)
      if (any(!is.finite(delta_Lambda)) || any(delta_Lambda <= 0)) return(1e30)
      ll <- sum(events * log(delta_Lambda)) - exp(a) * T_max
    } else {
      # Log-linear: Lambda(t) = (exp(a)/b)(exp(bt) - 1)
      bt_curr <- b * cum_time
      bt_prev <- b * t_prev
      # Guard against overflow
      if (any(abs(bt_curr) > 500) || any(abs(bt_prev) > 500)) return(1e30)
      Lambda_curr <- (exp(a) / b) * (exp(bt_curr) - 1)
      Lambda_prev <- (exp(a) / b) * (exp(bt_prev) - 1)
      delta_Lambda <- Lambda_curr - Lambda_prev
      if (any(!is.finite(delta_Lambda)) || any(delta_Lambda <= 0)) return(1e30)
      total_Lambda <- (exp(a) / b) * (exp(b * T_max) - 1)
      if (!is.finite(total_Lambda)) return(1e30)
      ll <- sum(events * log(delta_Lambda)) - total_Lambda
    }
    if (!is.finite(ll)) return(1e30)
    -ll
  }

  # Use Nelder-Mead first to find a robust starting point
  opt_nm <- stats::optim(
    par = c(log(N / T_max), 1e-4),
    fn = neg_loglik,
    method = "Nelder-Mead"
  )

  opt <- stats::optim(
    par = opt_nm$par,
    fn = neg_loglik,
    method = "L-BFGS-B",
    lower = c(-20, -5),
    upper = c(20, 5),
    hessian = TRUE
  )

  a_hat <- opt$par[1]
  b_hat <- opt$par[2]
  loglik <- -opt$value

  vcov_mat <- tryCatch(solve(opt$hessian), error = function(e) matrix(NA_real_, 2, 2))
  a_se <- if (!anyNA(vcov_mat)) sqrt(max(vcov_mat[1, 1], 0)) else NA_real_
  b_se <- if (!anyNA(vcov_mat)) sqrt(max(vcov_mat[2, 2], 0)) else NA_real_

  # Fitted values: Lambda(t) = (exp(a)/b)(exp(bt) - 1)
  if (abs(b_hat) < 1e-12) {
    fitted_values <- exp(a_hat) * cum_time
  } else {
    fitted_values <- (exp(a_hat) / b_hat) * (exp(b_hat * cum_time) - 1)
  }

  # Delta method for confidence intervals on log(Lambda)
  z_val <- stats::qnorm(1 - (1 - conf_level) / 2)
  log_fit <- log(pmax(fitted_values, 1e-300))

  # Gradient of log(Lambda) w.r.t. (a, b)
  if (abs(b_hat) < 1e-12) {
    grad_a <- rep(1, n_obs)
    grad_b <- cum_time / 2
  } else {
    # d/da log(Lambda) = 1
    grad_a <- rep(1, n_obs)
    # d/db log(Lambda) = (t*exp(bt))/(exp(bt)-1) - 1/b
    ebt <- exp(b_hat * cum_time)
    grad_b <- (cum_time * ebt) / (ebt - 1) - 1 / b_hat
  }
  grad_mat <- cbind(grad_a, grad_b)
  var_lf <- rowSums((grad_mat %*% vcov_mat) * grad_mat)
  hw <- z_val * sqrt(pmax(var_lf, 0))

  list(
    a             = a_hat,
    b             = b_hat,
    a_se          = a_se,
    b_se          = b_se,
    vcov          = vcov_mat,
    fitted_values = fitted_values,
    lower_bounds  = exp(log_fit - hw),
    upper_bounds  = exp(log_fit + hw),
    loglik        = loglik,
    residuals     = cumsum(events) - fitted_values,
    aic           = -2 * loglik + 4,
    bic           = -2 * loglik + 2 * log(n_obs)
  )
}


#' Non-Homogeneous Poisson Process Model for Repairable Systems.
#'
#' Fits a parametric NHPP model to recurrent event data from repairable systems.
#' Supported models include the Power Law process and the Log-Linear process.
#' The Power Law model can also be fit as a piecewise (segmented) model with
#' automatic change point detection or user-specified breakpoints.
#'
#' @srrstats {G1.0} The Power Law NHPP is described in Crow (1975), and the
#'   Log-Linear NHPP is described in Cox and Lewis (1966).
#' @srrstats {G1.3} All statistical terminology is explicitly defined.
#' @srrstats {G1.4} \code{roxygen2} documentation is used.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.3a} \code{match.arg()} is used for string inputs.
#' @srrstats {G2.7} Both vectors and data frames are accepted as input.
#' @srrstats {G2.13} The function checks for missing data.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.16} NA and NaN values result in an error.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#' @srrstats {RE2.1} NA, NaN, and Inf values in input result in an error.
#' @srrstats {RE4.0} Software returns a model object.
#' @srrstats {RE4.2} Model coefficients are included in the output.
#' @srrstats {RE4.11} Goodness-of-fit statistics are included.
#'
#' @param time A numeric vector of cumulative event times, or a data frame
#'   containing columns \code{time} and optionally \code{event}. All values
#'   must be positive, finite, and strictly increasing.
#' @param event An optional numeric vector of event counts at each time. If
#'   \code{NULL} (default), each time is treated as a single event.
#' @param data An optional data frame containing columns \code{time} and
#'   optionally \code{event}.
#' @param model_type Model type: \code{"Power Law"} (default) or
#'   \code{"Log-Linear"}.
#' @param breaks Optional vector of breakpoints for piecewise Power Law model.
#' @param method Estimation method: \code{"MLE"} (default) or \code{"LS"}.
#'   \code{"LS"} is not supported for \code{"Log-Linear"} models.
#' @param conf_level Confidence level for bounds (default 0.95).
#' @family Repairable Systems Analysis
#' @return An object of class \code{nhpp} containing:
#' \item{time}{The input cumulative event times.}
#' \item{event}{The event counts.}
#' \item{cum_events}{Cumulative event counts.}
#' \item{n_obs}{Number of observations.}
#' \item{model}{Fitted model object (lm or segmented), or NULL for MLE.}
#' \item{model_type}{\code{"Power Law"} or \code{"Log-Linear"}.}
#' \item{method}{\code{"MLE"} or \code{"LS"}.}
#' \item{params}{Named list of estimated parameters.}
#' \item{params_se}{Named list of standard errors.}
#' \item{vcov}{Variance-covariance matrix (MLE only).}
#' \item{fitted_values}{Fitted cumulative events.}
#' \item{lower_bounds}{Lower confidence bounds.}
#' \item{upper_bounds}{Upper confidence bounds.}
#' \item{residuals}{Model residuals.}
#' \item{logLik}{Log-likelihood.}
#' \item{AIC}{Akaike Information Criterion.}
#' \item{BIC}{Bayesian Information Criterion.}
#' \item{breakpoints}{Breakpoints (log scale) if piecewise model.}
#' \item{conf_level}{Confidence level used.}
#'
#' @details
#' The **Power Law NHPP** models the cumulative number of events as
#' \eqn{N(t) = \lambda t^\beta}{N(t) = lambda * t^beta}. The parameter
#' \eqn{\beta > 1} indicates a deteriorating system (increasing event rate),
#' \eqn{\beta < 1} an improving system, and \eqn{\beta = 1} a constant rate
#' (HPP).
#'
#' The **Log-Linear NHPP** models the intensity as
#' \eqn{\lambda(t) = \exp(a + bt)}{lambda(t) = exp(a + b*t)} with cumulative
#' function \eqn{\Lambda(t) = \frac{e^a}{b}(e^{bt} - 1)}{Lambda(t) = (exp(a)/b)(exp(bt) - 1)}.
#'
#' @examples
#' time <- c(200, 400, 600, 800, 1000)
#' event <- c(3, 5, 4, 7, 6)
#' result <- nhpp(time, event)
#' print(result)
#' plot(result, main = "Power Law NHPP")
#'
#' result_ll <- nhpp(time, event, model_type = "Log-Linear")
#' print(result_ll)
#' @importFrom stats lm predict AIC BIC logLik cor residuals optim qnorm
#' @importFrom segmented segmented slope intercept seg.control
#' @export
nhpp <- function(time, event = NULL, data = NULL,
                 model_type = "Power Law",
                 breaks = NULL,
                 method = c("MLE", "LS"),
                 conf_level = 0.95) {

  # Extract from data frame
  if (!is.null(data)) {
    if (!is.data.frame(data)) {
      stop("'data' must be a data frame.")
    }
    if (!"time" %in% names(data)) {
      stop("'data' must contain a column named 'time'.")
    }
    time <- data$time
    if ("event" %in% names(data)) {
      event <- data$event
    }
  }

  # Validate time
  if (!is.numeric(time) || !is.vector(time)) {
    stop("'time' must be a numeric vector.")
  }
  if (length(time) == 0) stop("'time' cannot be empty.")
  if (any(is.na(time)) || any(is.nan(time))) {
    stop("'time' contains missing (NA) or NaN values.")
  }
  if (any(!is.finite(time)) || any(time <= 0)) {
    stop("All values in 'time' must be finite and > 0.")
  }
  if (is.unsorted(time, strictly = TRUE)) {
    stop("'time' must be strictly increasing.")
  }

  # Validate event
  if (is.null(event)) {
    event <- rep(1, length(time))
  }
  if (!is.numeric(event) || !is.vector(event)) {
    stop("'event' must be a numeric vector.")
  }
  if (length(event) != length(time)) {
    stop("'event' and 'time' must have the same length.")
  }
  if (any(is.na(event)) || any(is.nan(event))) {
    stop("'event' contains missing (NA) or NaN values.")
  }
  if (any(!is.finite(event)) || any(event <= 0)) {
    stop("All values in 'event' must be finite and > 0.")
  }

  # Validate model_type
  if (!is.character(model_type) || length(model_type) != 1) {
    stop("'model_type' must be a single character string.")
  }
  valid_model <- match.arg(
    tolower(model_type),
    tolower(c("power law", "log-linear"))
  )

  method <- match.arg(method)

  # Validate method/model combinations
  if (method == "LS" && valid_model == "log-linear") {
    stop("'method = \"LS\"' is not supported for model_type = \"Log-Linear\". Use method = \"MLE\".")
  }
  if (method == "MLE" && valid_model == "power law" && !is.null(breaks)) {
    stop("'method = \"MLE\"' is not supported for piecewise Power Law models. Use method = \"LS\".")
  }

  # Validate breaks
  if (!is.null(breaks)) {
    if (!is.numeric(breaks) || length(breaks) == 0) {
      stop("'breaks' must be a non-empty numeric vector if provided.")
    }
    if (any(!is.finite(breaks)) || any(breaks <= 0)) {
      stop("All values in 'breaks' must be finite and > 0.")
    }
    if (valid_model != "power law") {
      stop("'breaks' can only be used with the 'Power Law' model.")
    }
  }

  # Validate conf_level
  if (!is.numeric(conf_level) || length(conf_level) != 1) {
    stop("'conf_level' must be a single numeric value.")
  }
  if (conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1 (exclusive).")
  }

  cum_events <- cumsum(event)
  n_obs <- length(event)

  # ---- Log-Linear MLE ----
  if (valid_model == "log-linear") {
    mle <- .fit_mle_loglinear(time, event, conf_level)
    result <- list(
      time          = time,
      event         = event,
      cum_events    = cum_events,
      n_obs         = n_obs,
      model         = NULL,
      model_type    = "Log-Linear",
      method        = "MLE",
      params        = list(a = mle$a, b = mle$b),
      params_se     = list(a_se = mle$a_se, b_se = mle$b_se),
      vcov          = mle$vcov,
      fitted_values = mle$fitted_values,
      lower_bounds  = mle$lower_bounds,
      upper_bounds  = mle$upper_bounds,
      residuals     = mle$residuals,
      logLik        = mle$loglik,
      AIC           = mle$aic,
      BIC           = mle$bic,
      breakpoints   = NULL,
      conf_level    = conf_level
    )
    class(result) <- "nhpp"
    return(result)
  }

  # ---- Power Law ----
  log_times <- log(time)
  log_cum_events <- log(cum_events)

  if (method == "MLE") {
    mle <- .fit_mle_power(time, event, conf_level)
    result <- list(
      time          = time,
      event         = event,
      cum_events    = cum_events,
      n_obs         = n_obs,
      model         = NULL,
      model_type    = "Power Law",
      method        = "MLE",
      params        = list(beta = mle$beta, lambda = mle$lambda),
      params_se     = list(beta_se = mle$beta_se),
      vcov          = mle$vcov,
      fitted_values = mle$fitted_values,
      lower_bounds  = mle$lower_bounds,
      upper_bounds  = mle$upper_bounds,
      residuals     = mle$residuals,
      logLik        = mle$loglik,
      AIC           = mle$aic,
      BIC           = mle$bic,
      breakpoints   = NULL,
      conf_level    = conf_level
    )
    class(result) <- "nhpp"
    return(result)
  }

  # ---- Power Law LS ----
  # Check for perfect collinearity
  cor_val <- suppressWarnings(stats::cor(log_times, log_cum_events))
  if (is.na(cor_val) || abs(cor_val - 1) < .Machine$double.eps^0.5 ||
    abs(cor_val + 1) < .Machine$double.eps^0.5) {
    stop("Perfect collinearity detected between 'log(time)' and 'log(cum_events)'. Regression cannot be performed.")
  }

  fit <- stats::lm(log_cum_events ~ log_times)

  if (!is.null(breaks)) {
    # Piecewise model
    breakpoints_log <- log(breaks)
    updated_fit <- segmented::segmented(fit, seg.Z = ~log_times, psi = breakpoints_log)

    slopes <- segmented::slope(updated_fit)
    intercepts <- segmented::intercept(updated_fit)
    betas <- slopes$log_times[, "Est."]
    beta_se <- slopes$log_times[, "St.Err."]
    lambdas <- exp(intercepts$log_times[, "Est."])

    params <- list(betas = betas, lambdas = lambdas)
    params_se <- list(betas_se = beta_se)
    bp <- updated_fit$psi[, "Est."]
  } else {
    # Simple model
    updated_fit <- fit
    bp <- NULL

    smry <- summary(updated_fit)
    slope_row <- smry$coefficients[2, ]
    intercept_row <- smry$coefficients[1, ]
    betas <- slope_row["Estimate"]
    beta_se <- slope_row["Std. Error"]
    lambdas <- exp(intercept_row["Estimate"])

    params <- list(beta = betas, lambda = lambdas)
    params_se <- list(beta_se = beta_se)
  }

  loglik <- as.numeric(stats::logLik(updated_fit))
  aic <- stats::AIC(updated_fit)
  bic <- stats::BIC(updated_fit)

  fitted_log <- stats::predict(updated_fit)
  resid <- stats::residuals(updated_fit)
  conf_intervals <- stats::predict(updated_fit, interval = "confidence", level = conf_level)

  result <- list(
    time          = time,
    event         = event,
    cum_events    = cum_events,
    n_obs         = n_obs,
    model         = updated_fit,
    model_type    = "Power Law",
    method        = "LS",
    params        = params,
    params_se     = params_se,
    vcov          = NULL,
    fitted_values = exp(fitted_log),
    lower_bounds  = exp(conf_intervals[, "lwr"]),
    upper_bounds  = exp(conf_intervals[, "upr"]),
    residuals     = resid,
    logLik        = loglik,
    AIC           = aic,
    BIC           = bic,
    breakpoints   = bp,
    conf_level    = conf_level
  )
  class(result) <- "nhpp"
  result
}


#' Print Method for nhpp Objects.
#'
#' Prints a summary of the NHPP model results.
#'
#' @param x An object of class \code{nhpp}.
#' @param ... Additional arguments (not used).
#' @family Repairable Systems Analysis
#' @return Invisibly returns the input object.
#' @examples
#' time <- c(200, 400, 600, 800, 1000)
#' event <- c(3, 5, 4, 7, 6)
#' result <- nhpp(time, event)
#' print(result)
#' @export
print.nhpp <- function(x, ...) {
  if (!inherits(x, "nhpp")) {
    stop("'x' must be an object of class 'nhpp'.")
  }

  cat("Non-Homogeneous Poisson Process (NHPP)\n")
  cat("---------------------------------------\n")
  cat("Model Type:", x$model_type, "\n")
  cat("Estimation Method:", x$method, "\n")
  cat(sprintf("Number of observations: %d\n\n", x$n_obs))

  if (!is.null(x$breakpoints)) {
    cat("Breakpoints (original scale):\n")
    cat(round(exp(x$breakpoints), 4), "\n\n")
  }

  cat("Parameters:\n")
  if (x$model_type == "Log-Linear") {
    cat(sprintf("  a: %.4f (SE = %.4f)\n", x$params$a, x$params_se$a_se))
    cat(sprintf("  b: %.4f (SE = %.4f)\n", x$params$b, x$params_se$b_se))
  } else if (!is.null(x$breakpoints)) {
    # Piecewise
    cat(sprintf("  Betas: %s\n", paste(round(x$params$betas, 4), collapse = ", ")))
    cat(sprintf("  Std. Errors (Betas): %s\n", paste(round(x$params_se$betas_se, 4), collapse = ", ")))
    cat(sprintf("  Lambdas: %s\n", paste(round(x$params$lambdas, 4), collapse = ", ")))
  } else {
    cat(sprintf("  Beta: %.4f (SE = %.4f)\n", x$params$beta, x$params_se$beta_se))
    cat(sprintf("  Lambda: %.4f\n", x$params$lambda))
  }

  cat("\nGoodness of Fit:\n")
  cat(sprintf("  Log-likelihood: %.2f\n", x$logLik))
  cat(sprintf("  AIC: %.2f\n", x$AIC))
  cat(sprintf("  BIC: %.2f\n", x$BIC))

  invisible(x)
}


#' Plot Method for nhpp Objects.
#'
#' Plots observed cumulative events with the fitted NHPP model and optional
#' confidence bounds.
#'
#' @param x An object of class \code{nhpp}.
#' @param conf_bounds Logical; include confidence bounds (default: TRUE).
#' @param legend Logical; show the legend (default: TRUE).
#' @param legend_pos Position of the legend (default: "topleft").
#' @param ... Additional arguments passed to \code{plot()}.
#' @family Repairable Systems Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' time <- c(200, 400, 600, 800, 1000)
#' event <- c(3, 5, 4, 7, 6)
#' result <- nhpp(time, event)
#' plot(result, main = "Power Law NHPP", xlab = "Time", ylab = "Cumulative Events")
#' @importFrom graphics plot lines abline legend
#' @export
plot.nhpp <- function(x,
                      conf_bounds = TRUE,
                      legend = TRUE,
                      legend_pos = "topleft",
                      ...) {
  if (!inherits(x, "nhpp")) {
    stop("'x' must be an object of class 'nhpp'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  graphics::plot(x$time, x$cum_events, pch = 16, ...)
  graphics::lines(x$time, x$fitted_values)

  if (conf_bounds) {
    graphics::lines(x$time, x$lower_bounds, lty = 2)
    graphics::lines(x$time, x$upper_bounds, lty = 2)
  }

  if (!is.null(x$breakpoints)) {
    graphics::abline(v = exp(x$breakpoints), lty = 3)
  }

  if (legend) {
    pct <- round(x$conf_level * 100)
    legend_labels <- c("Observed", "Fitted")
    legend_pch <- c(16, NA)
    legend_lty <- c(NA, 1)

    if (conf_bounds) {
      legend_labels <- c(legend_labels, sprintf("Conf. Bounds (%d%%)", pct))
      legend_pch <- c(legend_pch, NA)
      legend_lty <- c(legend_lty, 2)
    }

    if (!is.null(x$breakpoints)) {
      legend_labels <- c(legend_labels, "Change Points")
      legend_pch <- c(legend_pch, NA)
      legend_lty <- c(legend_lty, 3)
    }

    graphics::legend(
      legend_pos,
      legend = legend_labels,
      pch = legend_pch,
      lty = legend_lty,
      bty = "n"
    )
  }

  invisible(NULL)
}


#' Overlay Plot for Multiple NHPP Models
#'
#' Plots multiple fitted \code{nhpp} objects on a single set of axes, using
#' distinct colors per model. Observed data points, fitted lines, and optional
#' confidence bounds are drawn for every model. Models may have been fit to
#' different datasets.
#'
#' @srrstats {G1.4} \code{roxygen2} documentation is used to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} List input is validated to contain only \code{nhpp} objects.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {RE6.0} Model objects have default plot methods.
#' @srrstats {RE6.2} The overlay plot shows fitted values with optional CIs.
#'
#' @param models A named or unnamed list of objects of class \code{nhpp}.
#'   At least one model must be provided. If the list is named, those names
#'   are used as legend labels; otherwise labels default to
#'   \code{"Model 1"}, \code{"Model 2"}, etc.
#' @param conf_bounds Logical; draw confidence bounds for each model
#'   (default: \code{TRUE}).
#' @param legend Logical; draw a legend (default: \code{TRUE}).
#' @param legend_pos Legend position keyword (default: \code{"topleft"}).
#' @param colors Optional character vector of colors, one per model. If
#'   \code{NULL} (default), \code{palette()} colors are cycled.
#' @param ... Additional arguments passed to the initial \code{plot()} call
#'   (e.g., \code{main}, \code{xlab}, \code{ylab}). Not forwarded to subsequent
#'   \code{lines()} or \code{points()} calls.
#' @family Repairable Systems Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' t1 <- c(200, 400, 600, 800, 1000)
#' e1 <- c(3, 5, 4, 7, 6)
#' t2 <- c(300, 600, 900, 1200, 1500)
#' e2 <- c(4, 6, 5, 8, 7)
#' m1 <- nhpp(t1, e1)
#' m2 <- nhpp(t2, e2)
#' overlay_nhpp(list(System_A = m1, System_B = m2),
#'   main = "NHPP Overlay", xlab = "Time",
#'   ylab = "Cumulative Events"
#' )
#' @importFrom graphics plot points lines abline legend
#' @importFrom grDevices palette
#' @export
overlay_nhpp <- function(models,
                         conf_bounds = TRUE,
                         legend = TRUE,
                         legend_pos = "topleft",
                         colors = NULL,
                         ...) {
  # Input validation
  if (!identical(class(models), "list") || length(models) == 0) {
    stop("'models' must be a non-empty list of 'nhpp' objects.")
  }
  not_nhpp <- !vapply(models, inherits, logical(1), what = "nhpp")
  if (any(not_nhpp)) {
    stop("All elements of 'models' must be objects of class 'nhpp'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  n_models <- length(models)

  # Resolve model names
  mod_names <- names(models)
  if (is.null(mod_names) || any(mod_names == "")) {
    mod_names <- paste0("Model ", seq_len(n_models))
  }

  # Resolve colors
  if (is.null(colors)) {
    pal <- grDevices::palette()
    colors <- pal[((seq_len(n_models) - 1L) %% length(pal)) + 1L]
  } else {
    if (!is.character(colors) || length(colors) < n_models) {
      stop("'colors' must be a character vector with at least one color per model.")
    }
    colors <- colors[seq_len(n_models)]
  }

  # Compute global axis limits
  all_x <- unlist(lapply(models, `[[`, "time"))
  all_y <- c(
    unlist(lapply(models, `[[`, "cum_events")),
    unlist(lapply(models, `[[`, "fitted_values"))
  )
  if (conf_bounds) {
    all_y <- c(
      all_y,
      unlist(lapply(models, `[[`, "lower_bounds")),
      unlist(lapply(models, `[[`, "upper_bounds"))
    )
  }
  xlim <- range(all_x, finite = TRUE)
  ylim <- range(all_y, finite = TRUE)

  # Base plot using first model's observed data
  graphics::plot(
    models[[1]]$time, models[[1]]$cum_events,
    xlim = xlim, ylim = ylim,
    col = colors[[1]], pch = 16,
    ...
  )

  # Add remaining models' observed points
  if (n_models > 1L) {
    for (i in seq(2L, n_models)) {
      graphics::points(models[[i]]$time, models[[i]]$cum_events,
                       pch = 16, col = colors[[i]])
    }
  }

  # Fitted lines, confidence bounds, and breakpoints for all models
  for (i in seq_len(n_models)) {
    mdl <- models[[i]]
    col <- colors[[i]]

    graphics::lines(mdl$time, mdl$fitted_values, col = col, lty = 1)

    if (conf_bounds) {
      graphics::lines(mdl$time, mdl$lower_bounds, col = col, lty = 2)
      graphics::lines(mdl$time, mdl$upper_bounds, col = col, lty = 2)
    }

    if (!is.null(mdl$breakpoints)) {
      graphics::abline(v = exp(mdl$breakpoints), col = col, lty = 3)
    }
  }

  # Legend
  if (legend) {
    graphics::legend(
      legend_pos,
      legend = mod_names,
      col    = colors,
      pch    = 16,
      lty    = 1,
      bty    = "n"
    )
  }

  invisible(NULL)
}


#' Forecast Cumulative Events from an NHPP Model.
#'
#' Takes a fitted \code{nhpp} object and a vector of future cumulative times,
#' returning predicted cumulative events with confidence bounds.
#'
#' @srrstats {G1.4} \code{roxygen2} documentation is used.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.13} The function checks for missing data.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#'
#' @param object An object of class \code{nhpp} returned by \code{nhpp()}.
#' @param time A numeric vector of cumulative times at which to forecast.
#'   All values must be finite and > 0.
#' @param conf_level Confidence level (default 0.95).
#' @family Repairable Systems Analysis
#' @return An object of class \code{nhpp_predict} containing:
#' \item{time}{Forecast times.}
#' \item{cum_events}{Predicted cumulative events.}
#' \item{lower_bounds}{Lower confidence bounds.}
#' \item{upper_bounds}{Upper confidence bounds.}
#' \item{conf_level}{Confidence level used.}
#' \item{model_type}{Model type.}
#' \item{nhpp_object}{The original nhpp object.}
#' @examples
#' time <- c(200, 400, 600, 800, 1000)
#' event <- c(3, 5, 4, 7, 6)
#' fit <- nhpp(time, event)
#' fc <- predict_nhpp(fit, time = c(1500, 2000))
#' print(fc)
#' plot(fc, main = "NHPP Forecast", xlab = "Time", ylab = "Cumulative Events")
#' @export
predict_nhpp <- function(object, time, conf_level = 0.95) {
  if (!inherits(object, "nhpp")) {
    stop("'object' must be an object of class 'nhpp'.")
  }
  if (!is.numeric(time) || !is.vector(time)) {
    stop("'time' must be a numeric vector.")
  }
  if (length(time) == 0) {
    stop("'time' cannot be empty.")
  }
  if (any(is.na(time)) || any(is.nan(time))) {
    stop("'time' contains missing (NA) or NaN values.")
  }
  if (any(!is.finite(time)) || any(time <= 0)) {
    stop("All values in forecast 'time' must be finite and > 0.")
  }
  if (!is.numeric(conf_level) || length(conf_level) != 1) {
    stop("'conf_level' must be a single numeric value.")
  }
  if (!is.finite(conf_level) || conf_level <= 0 || conf_level >= 1) {
    stop("'conf_level' must be between 0 and 1 (exclusive).")
  }

  max_obs_time <- max(object$time)
  if (any(time <= max_obs_time)) {
    warning(
      "Some 'time' values are <= the maximum observed time. ",
      "Hindcasting is allowed but may not be meaningful."
    )
  }

  z_val <- stats::qnorm(1 - (1 - conf_level) / 2)

  if (object$model_type == "Log-Linear") {
    a <- object$params$a
    b <- object$params$b
    if (abs(b) < 1e-12) {
      cum_events <- exp(a) * time
    } else {
      cum_events <- (exp(a) / b) * (exp(b * time) - 1)
    }
    log_fit <- log(pmax(cum_events, 1e-300))

    if (abs(b) < 1e-12) {
      grad_a <- rep(1, length(time))
      grad_b <- time / 2
    } else {
      grad_a <- rep(1, length(time))
      ebt <- exp(b * time)
      grad_b <- (time * ebt) / (ebt - 1) - 1 / b
    }
    grad_mat <- cbind(grad_a, grad_b)
    var_lf <- rowSums((grad_mat %*% object$vcov) * grad_mat)
    hw <- z_val * sqrt(pmax(var_lf, 0))
    lower_bounds <- exp(log_fit - hw)
    upper_bounds <- exp(log_fit + hw)

  } else if (object$method == "MLE") {
    beta <- object$params$beta
    lambda <- object$params$lambda
    cum_events <- lambda * time^beta
    log_fit <- log(cum_events)
    grad_mat <- cbind(log(time), 1 / lambda)
    var_lf <- rowSums((grad_mat %*% object$vcov) * grad_mat)
    hw <- z_val * sqrt(pmax(var_lf, 0))
    lower_bounds <- exp(log_fit - hw)
    upper_bounds <- exp(log_fit + hw)

  } else {
    # LS (including segmented)
    newdata <- data.frame(log_times = log(time))
    pred <- stats::predict(object$model, newdata = newdata,
                           interval = "confidence", level = conf_level)
    cum_events <- exp(pred[, "fit"])
    lower_bounds <- exp(pred[, "lwr"])
    upper_bounds <- exp(pred[, "upr"])
  }

  result <- list(
    time         = time,
    cum_events   = cum_events,
    lower_bounds = lower_bounds,
    upper_bounds = upper_bounds,
    conf_level   = conf_level,
    model_type   = object$model_type,
    nhpp_object  = object
  )
  class(result) <- "nhpp_predict"
  result
}


#' Print Method for nhpp_predict Objects.
#'
#' Prints a formatted table of forecast cumulative events with confidence bounds.
#'
#' @param x An object of class \code{nhpp_predict}.
#' @param ... Additional arguments (not used).
#' @family Repairable Systems Analysis
#' @return Invisibly returns the input object.
#' @examples
#' time <- c(200, 400, 600, 800, 1000)
#' event <- c(3, 5, 4, 7, 6)
#' fit <- nhpp(time, event)
#' fc <- predict_nhpp(fit, time = c(1500, 2000))
#' print(fc)
#' @export
print.nhpp_predict <- function(x, ...) {
  if (!inherits(x, "nhpp_predict")) {
    stop("'x' must be an object of class 'nhpp_predict'.")
  }

  pct <- round(x$conf_level * 100)
  header <- sprintf("NHPP Forecast (%s)", x$model_type)
  cat(header, "\n")
  cat(paste(rep("-", nchar(header) + 1), collapse = ""), "\n")

  df <- data.frame(
    Time       = x$time,
    Cum.Events = round(x$cum_events, 1),
    Lower      = round(x$lower_bounds, 1),
    Upper      = round(x$upper_bounds, 1),
    check.names = FALSE
  )
  names(df)[3] <- sprintf("Lower (%d%%)", pct)
  names(df)[4] <- sprintf("Upper (%d%%)", pct)

  print(df, row.names = FALSE)

  invisible(x)
}


#' Plot Method for nhpp_predict Objects.
#'
#' Plots observed data, fitted model, and forecast with optional confidence bounds.
#'
#' @param x An object of class \code{nhpp_predict}.
#' @param conf_bounds Logical; include confidence bounds (default: TRUE).
#' @param legend Logical; show the legend (default: TRUE).
#' @param legend_pos Position of the legend (default: "topleft").
#' @param ... Additional arguments passed to \code{plot()}.
#' @family Repairable Systems Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' time <- c(200, 400, 600, 800, 1000)
#' event <- c(3, 5, 4, 7, 6)
#' fit <- nhpp(time, event)
#' fc <- predict_nhpp(fit, time = c(1500, 2000))
#' plot(fc, main = "NHPP Forecast", xlab = "Time", ylab = "Cumulative Events")
#' @importFrom graphics plot lines abline legend
#' @export
plot.nhpp_predict <- function(x,
                              conf_bounds = TRUE,
                              legend = TRUE,
                              legend_pos = "topleft",
                              ...) {
  if (!inherits(x, "nhpp_predict")) {
    stop("'x' must be an object of class 'nhpp_predict'.")
  }
  if (!is.logical(conf_bounds) || length(conf_bounds) != 1) {
    stop("'conf_bounds' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string.")
  }

  nhpp_obj <- x$nhpp_object
  obs_times <- nhpp_obj$time
  obs_cum_events <- nhpp_obj$cum_events

  all_y <- c(obs_cum_events, nhpp_obj$fitted_values, x$cum_events)
  if (conf_bounds) {
    all_y <- c(all_y, nhpp_obj$lower_bounds, nhpp_obj$upper_bounds,
               x$lower_bounds, x$upper_bounds)
  }
  xlim <- range(c(obs_times, x$time))
  ylim <- range(all_y)

  graphics::plot(obs_times, obs_cum_events,
                 xlim = xlim, ylim = ylim, pch = 16, ...)
  graphics::lines(obs_times, nhpp_obj$fitted_values)

  if (conf_bounds) {
    graphics::lines(obs_times, nhpp_obj$lower_bounds, lty = 3)
    graphics::lines(obs_times, nhpp_obj$upper_bounds, lty = 3)
  }

  max_obs_time <- max(obs_times)
  graphics::abline(v = max_obs_time, lty = 2, col = "gray")

  graphics::lines(x$time, x$cum_events, lty = 2)

  if (conf_bounds) {
    graphics::lines(x$time, x$lower_bounds, lty = 3)
    graphics::lines(x$time, x$upper_bounds, lty = 3)
  }

  if (legend) {
    pct <- round(x$conf_level * 100)
    legend_labels <- c("Observed", "Fitted", "Forecast")
    legend_pch <- c(16, NA, NA)
    legend_lty <- c(NA, 1, 2)

    if (conf_bounds) {
      legend_labels <- c(legend_labels, sprintf("Conf. Bounds (%d%%)", pct))
      legend_pch <- c(legend_pch, NA)
      legend_lty <- c(legend_lty, 3)
    }

    graphics::legend(legend_pos,
                     legend = legend_labels,
                     pch = legend_pch,
                     lty = legend_lty,
                     bty = "n",
                     cex = 0.85,
                     y.intersp = 1.3,
                     x.intersp = 0.8)
  }

  invisible(NULL)
}

#' @srrstats {G5.0} Tests use standard data sets.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by stop() is unique.
#' @srrstats {G5.4} Correctness tests with known parameter values.
#' @srrstats {G5.6} Parameter recovery tests.
#' @srrstats {G5.8a} Checks for zero-length data.
#' @srrstats {G5.8b} Checks for unsupported data types.
#' @srrstats {G5.8c} Checks for data with NA fields.
#' @srrstats {G5.9a} Noise susceptibility tests.

# --- Input validation tests ---

test_that("nhpp errors on non-numeric time", {
  expect_error(
    nhpp(time = c("a", "b"), event = c(1, 1)),
    "'time' must be a numeric vector.", fixed = TRUE
  )
})

test_that("nhpp errors on empty time", {
  expect_error(nhpp(time = c()), "'time' must be a numeric vector.", fixed = TRUE)
})

test_that("nhpp errors on NA in time", {
  expect_error(
    nhpp(time = c(100, NA), event = c(1, 1)),
    "'time' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("nhpp errors on NaN in time", {
  expect_error(
    nhpp(time = c(100, NaN), event = c(1, 1)),
    "'time' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("nhpp errors on non-positive time", {
  expect_error(
    nhpp(time = c(0, 100)),
    "All values in 'time' must be finite and > 0.", fixed = TRUE
  )
})

test_that("nhpp errors on infinite time", {
  expect_error(
    nhpp(time = c(100, Inf)),
    "All values in 'time' must be finite and > 0.", fixed = TRUE
  )
})

test_that("nhpp errors on non-increasing time", {
  expect_error(
    nhpp(time = c(200, 100)),
    "'time' must be strictly increasing.", fixed = TRUE
  )
})

test_that("nhpp errors on non-numeric event", {
  expect_error(
    nhpp(time = c(100, 200), event = c("a", "b")),
    "'event' must be a numeric vector.", fixed = TRUE
  )
})

test_that("nhpp errors on mismatched event length", {
  expect_error(
    nhpp(time = c(100, 200), event = c(1)),
    "'event' and 'time' must have the same length.", fixed = TRUE
  )
})

test_that("nhpp errors on NA in event", {
  expect_error(
    nhpp(time = c(100, 200), event = c(1, NA)),
    "'event' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("nhpp errors on non-positive event", {
  expect_error(
    nhpp(time = c(100, 200), event = c(1, 0)),
    "All values in 'event' must be finite and > 0.", fixed = TRUE
  )
})

test_that("nhpp errors on invalid model_type", {
  expect_error(
    nhpp(time = c(100, 200, 300), event = c(1, 2, 1), model_type = 123),
    "'model_type' must be a single character string.", fixed = TRUE
  )
})

test_that("nhpp errors on LS with Log-Linear", {
  expect_error(
    nhpp(time = c(100, 200, 300), event = c(1, 2, 1),
         model_type = "Log-Linear", method = "LS"),
    "'method = \"LS\"' is not supported for model_type = \"Log-Linear\".", fixed = TRUE
  )
})

test_that("nhpp errors on MLE with piecewise Power Law", {
  expect_error(
    nhpp(time = c(100, 200, 300, 400, 500), event = c(1, 2, 1, 3, 2),
         breaks = c(250), method = "MLE"),
    "'method = \"MLE\"' is not supported for piecewise Power Law models.", fixed = TRUE
  )
})

test_that("nhpp errors on invalid breaks", {
  expect_error(
    nhpp(time = c(100, 200, 300, 400, 500), event = c(1, 2, 1, 3, 2),
         breaks = numeric(0), method = "LS"),
    "'breaks' must be a non-empty numeric vector if provided.", fixed = TRUE
  )
  expect_error(
    nhpp(time = c(100, 200, 300, 400, 500), event = c(1, 2, 1, 3, 2),
         breaks = c(-1), method = "LS"),
    "All values in 'breaks' must be finite and > 0.", fixed = TRUE
  )
})

test_that("nhpp errors on breaks with Log-Linear", {
  expect_error(
    nhpp(time = c(100, 200, 300, 400, 500), event = c(1, 2, 1, 3, 2),
         breaks = c(250), model_type = "Log-Linear"),
    "'breaks' can only be used with the 'Power Law' model.", fixed = TRUE
  )
})

test_that("nhpp errors on invalid conf_level", {
  expect_error(
    nhpp(time = c(100, 200, 300), event = c(1, 2, 1), conf_level = "a"),
    "'conf_level' must be a single numeric value.", fixed = TRUE
  )
  expect_error(
    nhpp(time = c(100, 200, 300), event = c(1, 2, 1), conf_level = 0),
    "'conf_level' must be between 0 and 1 (exclusive).", fixed = TRUE
  )
})

# --- Data frame input tests ---

test_that("nhpp works with data frame input", {
  df <- data.frame(time = c(100, 200, 300, 400, 500), event = c(1, 2, 1, 3, 2))
  result <- nhpp(data = df)
  expect_s3_class(result, "nhpp")
})

test_that("nhpp data frame errors on missing time column", {
  expect_error(
    nhpp(data = data.frame(x = 1:3)),
    "'data' must contain a column named 'time'.", fixed = TRUE
  )
})

test_that("nhpp data frame errors on non-data-frame", {
  expect_error(
    nhpp(data = list(time = 1:3)),
    "'data' must be a data frame.", fixed = TRUE
  )
})

# --- Power Law MLE correctness ---

test_that("Power Law MLE returns valid nhpp object", {
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)
  result <- nhpp(time, event, method = "MLE")

  expect_s3_class(result, "nhpp")
  expect_equal(result$model_type, "Power Law")
  expect_equal(result$method, "MLE")
  expect_true(!is.null(result$params$beta))
  expect_true(!is.null(result$params$lambda))
  expect_true(result$params$beta > 0)
  expect_true(result$params$lambda > 0)
  expect_equal(result$n_obs, 5)
  expect_equal(length(result$fitted_values), 5)
})

test_that("Power Law MLE parameter recovery", {
  # Generate data from known power law: N(t) = 0.02 * t^1.2
  set.seed(42)
  true_beta <- 1.2
  true_lambda <- 0.02
  time <- seq(100, 2000, by = 100)
  expected <- true_lambda * time^true_beta
  event <- pmax(1, round(diff(c(0, expected)) + rnorm(length(time), 0, 0.5)))

  result <- nhpp(time, event, method = "MLE")

  expect_equal(result$params$beta, true_beta, tolerance = 0.3)
  expect_equal(result$params$lambda, true_lambda, tolerance = 0.05)
})

test_that("Power Law MLE has valid GOF statistics", {
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)
  result <- nhpp(time, event, method = "MLE")

  expect_true(is.finite(result$logLik))
  expect_true(is.finite(result$AIC))
  expect_true(is.finite(result$BIC))
})

# --- Power Law LS correctness ---

test_that("Power Law LS returns valid nhpp object", {
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)
  result <- nhpp(time, event, method = "LS")

  expect_s3_class(result, "nhpp")
  expect_equal(result$model_type, "Power Law")
  expect_equal(result$method, "LS")
  expect_true(!is.null(result$params$beta))
  expect_true(!is.null(result$params$lambda))
  expect_true(!is.null(result$model))
})

test_that("Power Law LS and MLE produce similar estimates", {
  time <- c(200, 400, 600, 800, 1000, 1200, 1400, 1600, 1800, 2000)
  event <- c(3, 5, 4, 7, 6, 8, 5, 9, 7, 10)

  result_ls <- nhpp(time, event, method = "LS")
  result_mle <- nhpp(time, event, method = "MLE")

  expect_equal(unname(result_ls$params$beta), result_mle$params$beta, tolerance = 0.5)
})

# --- Power Law LS with default event ---

test_that("nhpp uses default event of all 1s when event is NULL", {
  time <- c(100, 200, 300, 400, 500)
  result <- nhpp(time)
  expect_s3_class(result, "nhpp")
  expect_equal(result$event, rep(1, 5))
  expect_equal(result$cum_events, 1:5)
})

# --- Log-Linear MLE correctness ---

test_that("Log-Linear MLE returns valid nhpp object", {
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)
  result <- nhpp(time, event, model_type = "Log-Linear")

  expect_s3_class(result, "nhpp")
  expect_equal(result$model_type, "Log-Linear")
  expect_equal(result$method, "MLE")
  expect_true(!is.null(result$params$a))
  expect_true(!is.null(result$params$b))
  expect_true(is.finite(result$logLik))
})

test_that("Log-Linear MLE fitted values are positive", {
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)
  result <- nhpp(time, event, model_type = "Log-Linear")
  expect_true(all(result$fitted_values > 0))
})

# --- Segmented Power Law LS ---

test_that("Piecewise Power Law with breaks works", {
  time <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000,
            1100, 1200, 1300, 1400, 1500)
  event <- c(1, 1, 2, 4, 4, 1, 1, 2, 1, 4, 1, 1, 3, 3, 4)
  result <- nhpp(time, event, breaks = c(500), method = "LS")

  expect_s3_class(result, "nhpp")
  expect_true(!is.null(result$breakpoints))
  expect_true(length(result$params$betas) > 1)
})

# --- Confidence bounds ---

test_that("nhpp confidence bounds widen with higher conf_level", {
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)

  result_90 <- nhpp(time, event, conf_level = 0.90)
  result_99 <- nhpp(time, event, conf_level = 0.99)

  width_90 <- result_90$upper_bounds - result_90$lower_bounds
  width_99 <- result_99$upper_bounds - result_99$lower_bounds

  expect_true(all(width_99 >= width_90))
})

# --- Noise susceptibility ---

test_that("Power Law MLE is stable with small noise", {
  skip_on_cran()
  set.seed(123)
  time <- c(200, 400, 600, 800, 1000)
  event <- c(3, 5, 4, 7, 6)

  result_clean <- nhpp(time, event, method = "MLE")

  # Add small noise to event counts (round to keep integers)
  event_noisy <- pmax(1, round(event + rnorm(5, 0, 0.3)))
  result_noisy <- nhpp(time, event_noisy, method = "MLE")

  expect_equal(result_clean$params$beta, result_noisy$params$beta, tolerance = 0.5)
})

# --- predict_nhpp tests ---

test_that("predict_nhpp errors on wrong class", {
  expect_error(
    predict_nhpp(list(a = 1), time = c(1500)),
    "'object' must be an object of class 'nhpp'.", fixed = TRUE
  )
})

test_that("predict_nhpp errors on non-numeric time", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(
    predict_nhpp(fit, time = c("a")),
    "'time' must be a numeric vector.", fixed = TRUE
  )
})

test_that("predict_nhpp errors on empty time", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(
    predict_nhpp(fit, time = c()),
    "'time' must be a numeric vector.", fixed = TRUE
  )
})

test_that("predict_nhpp errors on NA in time", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(
    predict_nhpp(fit, time = c(1500, NA)),
    "'time' contains missing (NA) or NaN values.", fixed = TRUE
  )
})

test_that("predict_nhpp errors on non-positive time", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(
    predict_nhpp(fit, time = c(0, 1500)),
    "All values in forecast 'time' must be finite and > 0.", fixed = TRUE
  )
})

test_that("predict_nhpp errors on invalid conf_level", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(
    predict_nhpp(fit, time = c(1500), conf_level = 1.5),
    "'conf_level' must be between 0 and 1 (exclusive).", fixed = TRUE
  )
})

test_that("predict_nhpp warns on hindcasting", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_warning(
    predict_nhpp(fit, time = c(500)),
    "Some 'time' values are <= the maximum observed time"
  )
})

test_that("predict_nhpp returns valid nhpp_predict for Power Law MLE", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6), method = "MLE")
  fc <- predict_nhpp(fit, time = c(1500, 2000))

  expect_s3_class(fc, "nhpp_predict")
  expect_equal(length(fc$cum_events), 2)
  expect_true(all(fc$cum_events > 0))
  expect_true(all(fc$lower_bounds < fc$cum_events))
  expect_true(all(fc$upper_bounds > fc$cum_events))
})

test_that("predict_nhpp returns valid nhpp_predict for Power Law LS", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6), method = "LS")
  fc <- predict_nhpp(fit, time = c(1500, 2000))

  expect_s3_class(fc, "nhpp_predict")
  expect_equal(length(fc$cum_events), 2)
  expect_true(all(fc$cum_events > 0))
})

test_that("predict_nhpp returns valid nhpp_predict for Log-Linear", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6),
              model_type = "Log-Linear")
  fc <- predict_nhpp(fit, time = c(1500, 2000))

  expect_s3_class(fc, "nhpp_predict")
  expect_equal(length(fc$cum_events), 2)
  expect_true(all(fc$cum_events > 0))
})

test_that("predict_nhpp forecast is monotonically increasing for Power Law", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6), method = "MLE")
  fc <- predict_nhpp(fit, time = c(1200, 1500, 2000, 3000))
  expect_true(all(diff(fc$cum_events) > 0))
})

# --- Print and plot tests ---

test_that("print.nhpp produces output for Power Law MLE", {
  result <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6), method = "MLE")
  expect_output(print(result), "Non-Homogeneous Poisson Process")
  expect_output(print(result), "Power Law")
  expect_output(print(result), "Beta:")
})

test_that("print.nhpp produces output for Log-Linear", {
  result <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6),
                 model_type = "Log-Linear")
  expect_output(print(result), "Log-Linear")
})

test_that("print.nhpp returns invisible", {
  result <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_invisible(print(result))
})

test_that("print.nhpp errors on wrong class", {
  expect_error(
    print.nhpp(list(a = 1)),
    "'x' must be an object of class 'nhpp'.", fixed = TRUE
  )
})

test_that("plot.nhpp produces a plot without error", {
  result <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_silent(plot(result, main = "Test NHPP"))
})

test_that("plot.nhpp validates inputs", {
  result <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(plot.nhpp(list(a = 1)), "'x' must be an object of class 'nhpp'.")
  expect_error(plot(result, conf_bounds = "yes"), "'conf_bounds' must be a single logical value.")
  expect_error(plot(result, legend = "yes"), "'legend' must be a single logical value.")
  expect_error(plot(result, legend_pos = 123), "'legend_pos' must be a single character string.")
})

test_that("plot.nhpp works without conf bounds and legend", {
  result <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_silent(plot(result, conf_bounds = FALSE, legend = FALSE))
})

test_that("print.nhpp_predict produces output", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  fc <- predict_nhpp(fit, time = c(1500, 2000))
  expect_output(print(fc), "NHPP Forecast")
})

test_that("print.nhpp_predict returns invisible", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  fc <- predict_nhpp(fit, time = c(1500, 2000))
  expect_invisible(print(fc))
})

test_that("print.nhpp_predict errors on wrong class", {
  expect_error(
    print.nhpp_predict(list(a = 1)),
    "'x' must be an object of class 'nhpp_predict'.", fixed = TRUE
  )
})

test_that("plot.nhpp_predict produces a plot without error", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  fc <- predict_nhpp(fit, time = c(1500, 2000))
  expect_silent(plot(fc, main = "Test Forecast"))
})

test_that("plot.nhpp_predict validates inputs", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  fc <- predict_nhpp(fit, time = c(1500, 2000))

  expect_error(plot.nhpp_predict(list(a = 1)), "'x' must be an object of class 'nhpp_predict'.")
  expect_error(plot(fc, conf_bounds = "yes"), "'conf_bounds' must be a single logical value.")
  expect_error(plot(fc, legend = "yes"), "'legend' must be a single logical value.")
  expect_error(plot(fc, legend_pos = 123), "'legend_pos' must be a single character string.")
})

test_that("plot.nhpp_predict works without conf bounds and legend", {
  fit <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  fc <- predict_nhpp(fit, time = c(1500, 2000))
  expect_silent(plot(fc, conf_bounds = FALSE, legend = FALSE))
})

# ---- overlay_nhpp tests ----
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by stop() is unique.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {RE6.0} Model objects have overlay plot methods.
#' @srrstats {RE6.2} The overlay plot shows fitted values with optional CIs.

test_that("overlay_nhpp: non-list input errors", {
  m <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(overlay_nhpp(m), "'models' must be a non-empty list of 'nhpp' objects.", fixed = TRUE)
})

test_that("overlay_nhpp: empty list errors", {
  expect_error(overlay_nhpp(list()), "'models' must be a non-empty list of 'nhpp' objects.", fixed = TRUE)
})

test_that("overlay_nhpp: non-nhpp element errors", {
  m <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(overlay_nhpp(list(m, list(a = 1))),
               "All elements of 'models' must be objects of class 'nhpp'.", fixed = TRUE)
})

test_that("overlay_nhpp: conf_bounds wrong type errors", {
  m <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(overlay_nhpp(list(m), conf_bounds = "yes"),
               "'conf_bounds' must be a single logical value.", fixed = TRUE)
})

test_that("overlay_nhpp: legend wrong type errors", {
  m <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(overlay_nhpp(list(m), legend = 1L),
               "'legend' must be a single logical value.", fixed = TRUE)
})

test_that("overlay_nhpp: legend_pos wrong length errors", {
  m <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  expect_error(overlay_nhpp(list(m), legend_pos = c("a", "b")),
               "'legend_pos' must be a single character string.", fixed = TRUE)
})

test_that("overlay_nhpp: colors too short errors", {
  m1 <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  m2 <- nhpp(c(300, 600, 900, 1200, 1500), c(4, 6, 5, 8, 7))
  expect_error(overlay_nhpp(list(m1, m2), colors = "black"),
               "'colors' must be a character vector with at least one color per model.", fixed = TRUE)
})

test_that("overlay_nhpp: single model renders without error", {
  m <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_nhpp(list(m)))
})

test_that("overlay_nhpp: two named models render and return NULL", {
  m1 <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  m2 <- nhpp(c(300, 600, 900, 1200, 1500), c(4, 6, 5, 8, 7))
  pdf(NULL)
  on.exit(dev.off())
  result <- overlay_nhpp(list(System_A = m1, System_B = m2))
  expect_null(result)
})

test_that("overlay_nhpp: two unnamed models render without error", {
  m1 <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  m2 <- nhpp(c(300, 600, 900, 1200, 1500), c(4, 6, 5, 8, 7))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_nhpp(list(m1, m2)))
})

test_that("overlay_nhpp: conf_bounds = FALSE renders without error", {
  m1 <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  m2 <- nhpp(c(300, 600, 900, 1200, 1500), c(4, 6, 5, 8, 7))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_nhpp(list(m1, m2), conf_bounds = FALSE))
})

test_that("overlay_nhpp: custom colors render without error", {
  m1 <- nhpp(c(200, 400, 600, 800, 1000), c(3, 5, 4, 7, 6))
  m2 <- nhpp(c(300, 600, 900, 1200, 1500), c(4, 6, 5, 8, 7))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_nhpp(list(m1, m2), colors = c("steelblue", "tomato")))
})

test_that("overlay_nhpp: models with different-length datasets render without error", {
  m1 <- nhpp(c(200, 400, 600), c(3, 5, 4))
  m2 <- nhpp(c(300, 600, 900, 1200, 1500), c(4, 6, 5, 8, 7))
  pdf(NULL)
  on.exit(dev.off())
  expect_silent(overlay_nhpp(list(m1, m2)))
})

#' Exposure Analysis for Repairable Systems.
#'
#' Computes exposure (total operating time at risk) across one or more
#' repairable systems as a function of time. Exposure is defined as the
#' total accumulated observation time summed across all systems still under
#' observation. The function also computes the number of systems at risk
#' and the event rate (events per unit exposure) at each event time.
#'
#' @srrstats {G1.0} Exposure analysis for repairable systems is described
#'   in Meeker and Escobar (1998) <doi:10.1002/9781118032985>.
#' @srrstats {G1.3} All statistical terminology is explicitly defined.
#' @srrstats {G1.4} \code{roxygen2} documentation is used.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.7} Both vectors and data frames are accepted as input.
#' @srrstats {G2.13} The function checks for missing data.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.16} NA and NaN values result in an error.
#' @srrstats {G5.2a} Every message produced by \code{stop()} is unique.
#'
#' @param id A vector of system/unit identifiers. Each unique value
#'   represents a distinct system.
#' @param time A numeric vector of event or censoring times. Must be
#'   positive and finite.
#' @param event An optional numeric vector of event indicators: 1 for an
#'   event, 0 for censoring (end of observation). If \code{NULL} (default),
#'   all observations are treated as events, and the maximum time per system
#'   is treated as the end of observation.
#' @param data An optional data frame containing columns named \code{id},
#'   \code{time}, and optionally \code{event}.
#' @family Repairable Systems Analysis
#' @return An object of class \code{exposure} containing:
#' \item{time}{Sorted unique event times (excluding censoring-only times).}
#' \item{n_at_risk}{Number of systems under observation at each event time.}
#' \item{cum_exposure}{Cumulative total exposure (system-time) up to each
#'   event time.}
#' \item{cum_events}{Cumulative number of events up to each event time.}
#' \item{event_rate}{Cumulative event rate (cum_events / cum_exposure) at
#'   each event time.}
#' \item{total_exposure}{Total exposure across all systems and the full
#'   observation period.}
#' \item{total_events}{Total number of events.}
#' \item{n_systems}{Number of distinct systems.}
#' \item{end_times}{Named numeric vector of end-of-observation times per
#'   system. Can be passed directly to \code{mcf(end_time = ...)} to ensure
#'   the MCF properly accounts for system exposure.}
#'
#' @details
#' **Exposure** is the total amount of operating time during which events
#' can occur. For a fleet of \eqn{k} systems observed up to times
#' \eqn{T_1, T_2, \ldots, T_k}, the total exposure is
#' \eqn{E = \sum_{i=1}^{k} T_i}{E = T_1 + T_2 + ... + T_k}.
#'
#' The **cumulative exposure** at time \eqn{t} is
#' \eqn{E(t) = \sum_{i=1}^{k} \min(t, T_i)}{E(t) = sum of min(t, T_i)},
#' i.e., each system contributes time up to the lesser of \eqn{t} or its
#' observation end.
#'
#' The **event rate** at time \eqn{t} is the cumulative number of events
#' divided by the cumulative exposure: \eqn{r(t) = N(t) / E(t)}.
#'
#' @examples
#' id   <- c(1, 1, 1, 2, 2, 2, 3, 3, 3, 3)
#' time <- c(100, 350, 500, 80, 300, 600, 150, 250, 400, 700)
#' result <- exposure(id, time)
#' print(result)
#' plot(result)
#'
#' # With censoring
#' id    <- c(1, 1, 1, 2, 2, 2, 3, 3, 3)
#' time  <- c(100, 350, 500, 80, 300, 400, 150, 250, 700)
#' event <- c(  1,   1,   0,  1,   1,   0,   1,   1,   1)
#' result2 <- exposure(id, time, event)
#' print(result2)
#' @importFrom stats aggregate
#' @importFrom graphics plot lines par axis mtext legend
#' @export
exposure <- function(id = NULL, time = NULL, event = NULL, data = NULL) {

  # Extract from data frame if provided
  if (!is.null(data)) {
    if (!is.data.frame(data)) {
      stop("'data' must be a data frame when provided to exposure().")
    }
    if (!"id" %in% names(data)) {
      stop("'data' must contain a column named 'id' for exposure().")
    }
    if (!"time" %in% names(data)) {
      stop("'data' must contain a column named 'time' for exposure().")
    }
    id <- data$id
    time <- data$time
    if ("event" %in% names(data)) {
      event <- data$event
    }
  }

  # Validate id
  if (is.null(id)) stop("'id' must be provided to exposure().")
  if (length(id) == 0) stop("'id' cannot be empty in exposure().")

  # Validate time
  if (is.null(time)) stop("'time' must be provided to exposure().")
  if (!is.numeric(time) || !is.vector(time)) {
    stop("'time' must be a numeric vector in exposure().")
  }
  if (length(time) == 0) stop("'time' cannot be empty in exposure().")
  if (any(is.na(time)) || any(is.nan(time))) {
    stop("'time' contains missing (NA) or NaN values in exposure().")
  }
  if (any(!is.finite(time)) || any(time <= 0)) {
    stop("All values in 'time' must be finite and > 0 in exposure().")
  }
  if (length(id) != length(time)) {
    stop("'id' and 'time' must have the same length in exposure().")
  }

  # Validate event
  if (is.null(event)) {
    event <- rep(1, length(time))
  }
  if (!is.numeric(event) || !is.vector(event)) {
    stop("'event' must be a numeric vector in exposure().")
  }
  if (length(event) != length(time)) {
    stop("'event' and 'time' must have the same length in exposure().")
  }
  if (any(is.na(event)) || any(is.nan(event))) {
    stop("'event' contains missing (NA) or NaN values in exposure().")
  }
  if (!all(event %in% c(0, 1))) {
    stop("'event' must contain only 0 (censored) or 1 (event) in exposure().")
  }

  # Core calculations
  unique_ids <- unique(id)
  n_systems <- length(unique_ids)
  total_events <- sum(event)

  # Max observation time per system (includes censoring times)
  max_times <- tapply(time, id, max)

  # Total exposure = sum of all max observation times
  total_exposure <- sum(max_times)

  # Get event times only
  event_times <- time[event == 1]
  unique_event_times <- sort(unique(event_times))

  n_times <- length(unique_event_times)
  n_at_risk <- numeric(n_times)
  cum_exposure <- numeric(n_times)
  cum_events_vec <- numeric(n_times)

  cum_ev <- 0
  for (j in seq_along(unique_event_times)) {
    t_j <- unique_event_times[j]

    # Number of systems still under observation at t_j
    n_at_risk[j] <- sum(max_times >= t_j)

    # Cumulative exposure up to t_j: each system contributes min(t_j, T_i)
    cum_exposure[j] <- sum(pmin(t_j, max_times))

    # Cumulative events up to t_j
    cum_ev <- cum_ev + sum(event_times == t_j)
    cum_events_vec[j] <- cum_ev
  }

  # Event rate = cum_events / cum_exposure
  event_rate <- ifelse(cum_exposure > 0, cum_events_vec / cum_exposure, 0)

  result <- list(
    time           = unique_event_times,
    n_at_risk      = n_at_risk,
    cum_exposure   = cum_exposure,
    cum_events     = cum_events_vec,
    event_rate     = event_rate,
    total_exposure = total_exposure,
    total_events   = total_events,
    n_systems      = n_systems,
    end_times      = max_times
  )
  class(result) <- "exposure"
  result
}


#' Print Method for exposure Objects.
#'
#' Prints a summary of the exposure analysis results.
#'
#' @param x An object of class \code{exposure}.
#' @param ... Additional arguments (not used).
#' @family Repairable Systems Analysis
#' @return Invisibly returns the input object.
#' @examples
#' id   <- c(1, 1, 2, 2)
#' time <- c(100, 200, 150, 300)
#' result <- exposure(id, time)
#' print(result)
#' @export
print.exposure <- function(x, ...) {
  if (!inherits(x, "exposure")) {
    stop("'x' must be an object of class 'exposure'.")
  }

  cat("Exposure Analysis for Repairable Systems\n")
  cat("-----------------------------------------\n")
  cat(sprintf("Number of systems:  %d\n", x$n_systems))
  cat(sprintf("Total events:       %d\n", x$total_events))
  cat(sprintf("Total exposure:     %.2f\n", x$total_exposure))
  cat(sprintf("Overall event rate: %.6f\n\n", x$total_events / x$total_exposure))

  df <- data.frame(
    Time         = x$time,
    At.Risk      = x$n_at_risk,
    Cum.Exposure = round(x$cum_exposure, 2),
    Cum.Events   = x$cum_events,
    Event.Rate   = round(x$event_rate, 6),
    check.names  = FALSE
  )
  names(df) <- c("Time", "At Risk", "Cum. Exposure", "Cum. Events", "Event Rate")
  print(df, row.names = FALSE)

  invisible(x)
}


#' Plot Method for exposure Objects.
#'
#' Produces a multi-panel plot of exposure analysis results. The default
#' layout shows cumulative exposure and cumulative events versus time
#' (top panel), the number of systems at risk over time (middle panel),
#' and the event rate over time (bottom panel). Alternatively, a single
#' \code{which} panel can be selected.
#'
#' @param x An object of class \code{exposure}.
#' @param which Character string selecting which panel(s) to plot. One of
#'   \code{"all"} (default), \code{"exposure"}, \code{"at_risk"}, or
#'   \code{"event_rate"}.
#' @param legend Logical; show the legend (default: TRUE).
#' @param legend_pos Position of the legend (default: "topleft").
#' @param ... Additional arguments passed to the underlying \code{plot()}.
#' @family Repairable Systems Analysis
#' @return Invisibly returns \code{NULL}.
#' @examples
#' id   <- c(1, 1, 1, 2, 2, 2, 3, 3, 3, 3)
#' time <- c(100, 350, 500, 80, 300, 600, 150, 250, 400, 700)
#' result <- exposure(id, time)
#' plot(result)
#' plot(result, which = "exposure")
#' @importFrom graphics plot lines par axis mtext legend
#' @export
plot.exposure <- function(x,
                          which = c("all", "exposure", "at_risk", "event_rate"),
                          legend = TRUE,
                          legend_pos = "topleft",
                          ...) {
  if (!inherits(x, "exposure")) {
    stop("'x' must be an object of class 'exposure'.")
  }
  which <- match.arg(which)
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value in plot.exposure().")
  }
  if (!is.character(legend_pos) || length(legend_pos) != 1) {
    stop("'legend_pos' must be a single character string in plot.exposure().")
  }

  if (which == "all") {
    old_par <- graphics::par(mfrow = c(3, 1), mar = c(4, 4, 2, 4))
    on.exit(graphics::par(old_par), add = TRUE)

    # Panel 1: Cumulative exposure and cumulative events
    .plot_exposure_panel(x, legend = legend, legend_pos = legend_pos, ...)

    # Panel 2: Number at risk
    .plot_at_risk_panel(x, ...)

    # Panel 3: Event rate
    .plot_event_rate_panel(x, ...)

  } else if (which == "exposure") {
    .plot_exposure_panel(x, legend = legend, legend_pos = legend_pos, ...)
  } else if (which == "at_risk") {
    .plot_at_risk_panel(x, ...)
  } else {
    .plot_event_rate_panel(x, ...)
  }

  invisible(NULL)
}


# Internal: Plot cumulative exposure and cumulative events on dual y-axes.
.plot_exposure_panel <- function(x, legend = TRUE, legend_pos = "topleft", ...) {
  # Cumulative exposure (left axis)
  graphics::plot(x$time, x$cum_exposure,
                 type = "s", lwd = 2,
                 xlab = "Time",
                 ylab = "Cumulative Exposure",
                 main = "Cumulative Exposure and Events",
                 ...)

  # Cumulative events on secondary axis
  graphics::par(new = TRUE)
  graphics::plot(x$time, x$cum_events,
                 type = "s", lty = 2, lwd = 2, col = "steelblue",
                 axes = FALSE, xlab = "", ylab = "")
  graphics::axis(side = 4, col = "steelblue", col.axis = "steelblue")
  graphics::mtext("Cumulative Events", side = 4, line = 2.5, col = "steelblue")

  if (legend) {
    graphics::legend(
      legend_pos,
      legend = c("Cum. Exposure", "Cum. Events"),
      lty = c(1, 2),
      lwd = c(2, 2),
      col = c("black", "steelblue"),
      bty = "n"
    )
  }
}


# Internal: Plot number of systems at risk over time.
.plot_at_risk_panel <- function(x, ...) {
  graphics::plot(x$time, x$n_at_risk,
                 type = "s", lwd = 2,
                 xlab = "Time",
                 ylab = "Systems at Risk",
                 main = "Number of Systems at Risk",
                 ...)
}


# Internal: Plot event rate over time.
.plot_event_rate_panel <- function(x, ...) {
  graphics::plot(x$time, x$event_rate,
                 type = "s", lwd = 2,
                 xlab = "Time",
                 ylab = "Event Rate (events / exposure)",
                 main = "Cumulative Event Rate",
                 ...)
}

#' @srrstats {G5.0} Tests use standard recurrent event data.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results.
#' @srrstats {G5.2a} Every message produced by stop() is unique.
#' @srrstats {G5.4} Correctness tests with known exposure values.
#' @srrstats {G5.8a} Checks for zero-length data.
#' @srrstats {G5.8b} Checks for unsupported data types.
#' @srrstats {G5.8c} Checks for data with NA fields.

# --- Input validation tests ---

test_that("exposure errors on missing id", {
  expect_error(exposure(time = c(1, 2)),
               "'id' must be provided to exposure().", fixed = TRUE)
})

test_that("exposure errors on missing time", {
  expect_error(exposure(id = c(1, 1)),
               "'time' must be provided to exposure().", fixed = TRUE)
})

test_that("exposure errors on NULL id", {
  expect_error(exposure(id = NULL, time = c(1, 2)),
               "'id' must be provided to exposure().", fixed = TRUE)
})

test_that("exposure errors on NULL time", {
  expect_error(exposure(id = c(1), time = NULL),
               "'time' must be provided to exposure().", fixed = TRUE)
})

test_that("exposure errors on non-numeric time", {
  expect_error(
    exposure(id = c(1, 1), time = c("a", "b")),
    "'time' must be a numeric vector in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on NA in time", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, NA)),
    "'time' contains missing (NA) or NaN values in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on NaN in time", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, NaN)),
    "'time' contains missing (NA) or NaN values in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on non-positive time", {
  expect_error(
    exposure(id = c(1, 1), time = c(0, 100)),
    "All values in 'time' must be finite and > 0 in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on infinite time", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, Inf)),
    "All values in 'time' must be finite and > 0 in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on mismatched id and time lengths", {
  expect_error(
    exposure(id = c(1, 2, 3), time = c(100, 200)),
    "'id' and 'time' must have the same length in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on non-numeric event", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, 200), event = c("a", "b")),
    "'event' must be a numeric vector in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on mismatched event length", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, 200), event = c(1)),
    "'event' and 'time' must have the same length in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on NA in event", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, 200), event = c(1, NA)),
    "'event' contains missing (NA) or NaN values in exposure().", fixed = TRUE
  )
})

test_that("exposure errors on invalid event values", {
  expect_error(
    exposure(id = c(1, 1), time = c(100, 200), event = c(1, 2)),
    "'event' must contain only 0 (censored) or 1 (event) in exposure().", fixed = TRUE
  )
})

# --- Data frame input tests ---

test_that("exposure works with data frame input", {
  df <- data.frame(id = c(1, 1, 2, 2), time = c(100, 200, 150, 300))
  result <- exposure(data = df)
  expect_s3_class(result, "exposure")
  expect_equal(result$n_systems, 2)
  expect_equal(result$total_events, 4)
})

test_that("exposure data frame errors on missing columns", {
  expect_error(
    exposure(data = data.frame(x = 1:3)),
    "'data' must contain a column named 'id' for exposure().", fixed = TRUE
  )
  expect_error(
    exposure(data = data.frame(id = 1:3)),
    "'data' must contain a column named 'time' for exposure().", fixed = TRUE
  )
})

test_that("exposure errors on non-data-frame data", {
  expect_error(
    exposure(data = list(id = 1, time = 1)),
    "'data' must be a data frame when provided to exposure().", fixed = TRUE
  )
})

test_that("exposure data frame with event column works", {
  df <- data.frame(
    id = c(1, 1, 2, 2),
    time = c(100, 200, 150, 300),
    event = c(1, 0, 1, 1)
  )
  result <- exposure(data = df)
  expect_s3_class(result, "exposure")
  expect_equal(result$total_events, 3)
})

# --- Correctness tests ---

test_that("exposure computes correct values for single-system case", {
  # Single system, events at 100 and 200
  # max_time = 200
  # At t=100: cum_exposure = min(100, 200) = 100, cum_events = 1
  # At t=200: cum_exposure = min(200, 200) = 200, cum_events = 2
  id <- c(1, 1)
  time <- c(100, 200)
  result <- exposure(id, time)

  expect_equal(result$time, c(100, 200))
  expect_equal(result$n_at_risk, c(1, 1))
  expect_equal(result$cum_exposure, c(100, 200))
  expect_equal(result$cum_events, c(1, 2))
  expect_equal(result$total_exposure, 200)
  expect_equal(result$total_events, 2)
  expect_equal(result$n_systems, 1)
})

test_that("exposure computes correct values for multi-system case", {
  # System 1: events at 100, 300; max_time = 300
  # System 2: events at 200; max_time = 200
  #
  # At t=100: n_at_risk=2, cum_exposure = min(100,300) + min(100,200) = 200
  # At t=200: n_at_risk=2, cum_exposure = min(200,300) + min(200,200) = 400
  # At t=300: n_at_risk=1, cum_exposure = min(300,300) + min(300,200) = 500
  id <- c(1, 1, 2)
  time <- c(100, 300, 200)
  result <- exposure(id, time)

  expect_equal(result$time, c(100, 200, 300))
  expect_equal(result$n_at_risk, c(2, 2, 1))
  expect_equal(result$cum_exposure, c(200, 400, 500))
  expect_equal(result$cum_events, c(1, 2, 3))
  expect_equal(result$total_exposure, 500)
  expect_equal(result$total_events, 3)
})

test_that("exposure event rate is correct", {
  id <- c(1, 1, 2)
  time <- c(100, 300, 200)
  result <- exposure(id, time)

  # Event rate = cum_events / cum_exposure
  expect_equal(result$event_rate, result$cum_events / result$cum_exposure)
})

test_that("exposure with censoring correctly limits observation", {
  # System 1: event at 100, censored at 300
  # System 2: event at 200
  # max_times: sys1=300, sys2=200
  id    <- c(1, 1, 2)
  time  <- c(100, 300, 200)
  event <- c(1, 0, 1)
  result <- exposure(id, time, event)

  # Only event times: 100, 200
  expect_equal(result$time, c(100, 200))
  expect_equal(result$cum_events, c(1, 2))
  expect_equal(result$total_events, 2)
  # Total exposure is still sum of max_times = 300 + 200 = 500
  expect_equal(result$total_exposure, 500)
})

test_that("exposure cum_exposure is monotonically non-decreasing", {
  id <- c(1, 1, 1, 2, 2, 3, 3, 3)
  time <- c(100, 200, 400, 150, 350, 50, 250, 500)
  result <- exposure(id, time)
  expect_true(all(diff(result$cum_exposure) >= 0))
})

test_that("exposure cum_events is monotonically non-decreasing", {
  id <- c(1, 1, 1, 2, 2, 3, 3, 3)
  time <- c(100, 200, 400, 150, 350, 50, 250, 500)
  result <- exposure(id, time)
  expect_true(all(diff(result$cum_events) >= 0))
})

test_that("exposure n_at_risk is monotonically non-increasing", {
  id <- c(1, 1, 2, 2, 3)
  time <- c(100, 200, 100, 300, 100)
  result <- exposure(id, time)
  expect_true(all(diff(result$n_at_risk) <= 0))
})

test_that("exposure handles simultaneous events across systems", {
  # All 3 systems have events at t=100
  id <- c(1, 2, 3)
  time <- c(100, 100, 100)
  result <- exposure(id, time)

  expect_equal(result$time, 100)
  expect_equal(result$n_at_risk, 3)
  expect_equal(result$cum_events, 3)
  expect_equal(result$cum_exposure, 300)
  expect_equal(result$total_exposure, 300)
})

# --- Print and plot tests ---

test_that("print.exposure produces output", {
  result <- exposure(c(1, 1, 2, 2), c(100, 200, 150, 300))
  expect_output(print(result), "Exposure Analysis")
})

test_that("print.exposure returns invisible", {
  result <- exposure(c(1, 1, 2, 2), c(100, 200, 150, 300))
  expect_invisible(print(result))
})

test_that("print.exposure errors on wrong class", {
  expect_error(
    print.exposure(list(a = 1)),
    "'x' must be an object of class 'exposure'.", fixed = TRUE
  )
})

test_that("plot.exposure produces a plot without error", {
  result <- exposure(c(1, 1, 2, 2), c(100, 200, 150, 300))
  expect_silent(plot(result))
})

test_that("plot.exposure validates inputs", {
  result <- exposure(c(1, 1, 2, 2), c(100, 200, 150, 300))
  expect_error(plot.exposure(list(a = 1)),
               "'x' must be an object of class 'exposure'.", fixed = TRUE)
  expect_error(plot(result, legend = "yes"),
               "'legend' must be a single logical value in plot.exposure().", fixed = TRUE)
  expect_error(plot(result, legend_pos = 123),
               "'legend_pos' must be a single character string in plot.exposure().", fixed = TRUE)
})

test_that("plot.exposure works with single panels", {
  result <- exposure(c(1, 1, 2, 2), c(100, 200, 150, 300))
  expect_silent(plot(result, which = "exposure"))
  expect_silent(plot(result, which = "at_risk"))
  expect_silent(plot(result, which = "event_rate"))
})

test_that("plot.exposure works without legend", {
  result <- exposure(c(1, 1, 2, 2), c(100, 200, 150, 300))
  expect_silent(plot(result, legend = FALSE))
})

#' Duane Analysis
#'
#' This function performs a Duane analysis (1962) <doi:10.1109/TA.1964.4319640>
#' on failure data by fitting a log-log linear regression of cumulative Mean Time
#' Between Failures (MTBF) versus cumulative time. The function accepts either
#' two numeric vectors (`times`, `failures`) or a data frame containing both.
#'
#' @srrstats {G1.0} The primary reference for Duane Analysis is provided in the
#' description.
#' @srrstats {G1.1} The `duane` function is the first implementation of the Duane
#' model within an R package on CRAN.
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} [roxygen2](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.4b} Explicit conversion of log-likelihood to continuous is made via `as.numeric()`.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.7} Both one-dimensional vectors and data frames are accepted as input.
#' @srrstats {G2.8} Sub-functions `print.duane` and `plot.duane` are provided for the `duane` class.
#' @srrstats {G2.10} `data.frame` objects are appropriately processed to ensure consistent behavior.
#' @srrstats {G2.11} `data.frame` inputs  are appropriately processed and do not error without reason.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produce expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as data changes.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#' @srrstats {RE1.2} Documentation includes expected format for inputting predictor variables (`times`, `failures`).
#' @srrstats {RE1.3} Output retains all relevant aspects of input data.
#' @srrstats {RE1.3a} Output retains all relevant aspects of input data.
#' @srrstats {RE1.4} Documentation includes assumptions for the input data (i.e., positive, finite values).
#' @srrstats {RE2.1} NA, NaN, and Inf values in input data result in an error.
#' @srrstats {RE2.2} Missing values in input data results in an error.
#' @srrstats {RE4.0} Software returns a “model” object, which is a modified `lm` model object.
#' @srrstats {RE4.2} Model coefficients are included in the output object.
#' @srrstats {RE4.3} Standard errors on the coefficients are included in the output object.
#' @srrstats {RE4.4} The model specification is included in the output object.
#' @srrstats {RE4.5} The numbers of observations is included in the output object.
#' @srrstats {RE4.8} The Response variable (MTBF) is included in the output object.
#' @srrstats {RE4.9} Modeled values of the response variable are included in the output object.
#' @srrstats {RE4.10} Model Residuals, including documentation is included in the output object.
#' @srrstats {RE4.11} Goodness-of-fit statistics (log-likelihood, AIC, BIC) are included
#' in the output object.
#' @srrstats {RE4.13} All input variables are included in the output object.
#' @srrstats {RE4.17} Model objects are extended by a default `print` method which
#' provides an on-screen summary of model (input) parameters and (output) coefficients.
#' @srrstats {RE5.0} Scaling relationships between sizes of input data and
#' speed of algorithm are documented in the function documentation.
#' @srrstats {RE6.0} Model objects have default plot methods.
#' @srrstats {RE6.2} The default plot method produces a plot of the fitted values
#' of the model, with optional visualisation of confidence intervals.
#' @srrstats {RE7.1} Unit tests check for noiseless, exact relationships between
#' predictor (independent) and response (dependent) data.
#' @srrstats {RE7.1a} Unit tests confirm that model fitting is at least as fast
#' or faster than testing with equivalent noisy data.
#' @srrstats {RE7.2} Unit tests demonstrate that output objects retain aspects
#' of input data such as case names.
#' @srrstats {RE7.3} Unit tests demonstrate expected behavior when `duane` object
#' is submitted to the accessor methods `print` and `plot`.
#'
#' @param times Either:
#'   - A numeric vector of exact failure times, or
#'   - A data frame containing two columns: `times` and `failures`. The `times`
#'     column contains exact failure times, and the `failures` column
#'     contains the number of failures at each corresponding time.
#' @param failures A numeric vector of the number of failures at each corresponding
#'   time in `times`. Ignored if `times` is a data frame. Must be the same length as
#'   `times` if both are vectors.
#'   All values must be positive and finite.
#' @param conf.level Confidence level for the confidence bounds (default: `0.95`).
#'   Must be between 0 and 1 (exclusive).
#'
#' @family Duane functions
#'
#' @return A list of class \code{"duane"} containing:
#' \item{times}{The input exact failure times.}
#' \item{failures}{The input number of failures.}
#' \item{n_obs}{The number of observations (failures).}
#' \item{MTBF}{The cumulative mean time between failures.}
#' \item{model}{The fitted \code{lm} (linear model) object containing the regression results.}
#' \item{logLik}{The log-likelihood of the fitted model.}
#' \item{AIC}{Akaike Information Criterion (AIC).}
#' \item{BIC}{Bayesian Information Criterion (BIC).}
#' \item{conf.level}{The confidence level.}
#' \item{Cumulative_Time}{The cumulative operating times.}
#' \item{Cumulative_MTBF}{The cumulative mean time between failures.}
#' \item{Fitted_Values}{The fitted values on the MTBF scale.}
#' \item{Confidence_Bounds}{Matrix of fitted values and confidence bounds on the MTBF scale.}
#' \item{Residuals_Log}{Residuals on the log(MTBF) scale (from the regression).}
#' \item{Residuals_MTBF}{Residuals on the MTBF scale (observed - fitted).}
#'
#' @details
#' The scaling relationship between the size of input data (numbers of observations)
#' and speed of algorithm execution is approximately linear (O(n)). The function is
#' efficient and can handle large data sets (e.g., thousands of observations) quickly.
#' The function uses base R functions and does not require any additional packages.
#' The function includes comprehensive input validation and error handling to ensure
#' robustness. The function is tested with a standard data set from a published paper
#' and includes unit tests to verify correctness and performance.
#'
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit1 <- duane(times, failures, conf.level = 0.90)
#' print(fit1)
#'
#' df <- data.frame(times = times, failures = failures)
#' fit2 <- duane(df, conf.level = 0.95)
#' print(fit2)
#'
#' @importFrom stats lm AIC BIC logLik predict residuals
#' @export
duane <- function(times, failures = NULL, conf.level = 0.95) {
  # Case 1: data frame input
  if (is.data.frame(times)) {
    if (!all(c("times", "failures") %in% names(times))) {
      stop(
        "Data frame input must contain columns named 'times' and 'failures'."
      )
    }
    failures <- times$failures
    times <- times$times
  }

  # Input validation
  if (!is.numeric(times) || !is.vector(times)) {
    stop("'times' must be a numeric vector.")
  }
  if (!is.numeric(failures) || !is.vector(failures)) {
    stop("'failures' must be a numeric vector.")
  }
  if (any(is.na(times)) || any(is.nan(times))) {
    stop("'times' contains missing (NA) or NaN values.")
  }
  if (any(is.na(failures)) || any(is.nan(failures))) {
    stop("'failures' contains missing (NA) or NaN values.")
  }
  if (length(times) == 0) {
    stop("'times' cannot be empty.")
  }
  if (length(failures) == 0) {
    stop("'failures' cannot be empty.")
  }
  if (length(times) != length(failures)) {
    stop("The length of 'times' and 'failures' must be equal.")
  }
  if (any(!is.finite(times)) || any(times <= 0)) {
    stop("All values in 'times' must be finite and > 0.")
  }
  if (any(!is.finite(failures)) || any(failures <= 0)) {
    stop("All values in 'failures' must be finite and > 0.")
  }
  if (!is.numeric(conf.level) || length(conf.level) != 1) {
    stop("'conf.level' must be a single numeric value.")
  }
  if (conf.level <= 0 || conf.level >= 1) {
    stop("'conf.level' must be between 0 and 1 (exclusive).")
  }

  # Cumulative times & failures
  cum_failures <- cumsum(failures)
  cum_times <- cumsum(times)

  # cumulative MTBF
  cum_mtbf <- cum_times / cum_failures

  # log-log transform
  log_cum_times <- log(cum_times)
  log_cum_failures <- log(cum_failures)
  log_cum_mtbf <- log(cum_mtbf)

  # Fit model (log-log)
  fit <- stats::lm(log_cum_mtbf ~ log_cum_times)

  # fit stats
  aic <- stats::AIC(fit)
  bic <- stats::BIC(fit)
  loglik <- as.numeric(stats::logLik(fit))

  # Fitted values and residuals
  fitted_log <- predict(fit)
  fitted_values <- exp(fitted_log)
  residuals_log <- residuals(fit)
  residuals_mtbf <- cum_mtbf - fitted_values

  # confidence intervals on MTBF scale
  pred <- predict(fit, interval = "confidence", level = conf.level)
  conf_intervals <- stats::predict(fit, interval = "confidence", level = conf.level)
  ci_bounds <- exp(pred)
  lower_bounds <- exp(conf_intervals[, "lwr"])
  upper_bounds <- exp(conf_intervals[, "upr"])

  result <- list(
    times = times,
    failures = failures,
    n_obs = length(failures),
    MTBF = cum_mtbf,
    model = fit,
    logLik = loglik,
    AIC = aic,
    BIC = bic,
    conf.level = conf.level,
    Cumulative_Time = cum_times,
    Cumulative_MTBF = cum_mtbf,
    Fitted_Values = fitted_values,
    Confidence_Bounds = ci_bounds,
    lower_bounds = lower_bounds,
    upper_bounds = upper_bounds,
    Residuals_Log = residuals_log,
    Residuals_MTBF = residuals_mtbf
  )
  class(result) <- "duane"
  return(result)
}


#' Print method for duane objects.
#'
#' This function prints a summary of the Duane analysis result.
#'
#' @srrstats {G1.0} The primary reference for Duane Analysis is provided in the
#' description.
#' @srrstats {G1.1} The `duane` function is the first implementation of the Duane
#' model within an R package on CRAN.
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} [roxygen2](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.4b} Explicit conversion of log-likelihood to continuous is made via `as.numeric()`.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.7} Both one-dimensional vectors and data frames are accepted as input.
#' @srrstats {G2.8} Sub-functions `print.duane` and `plot.duane` are provided for the `duane` class.
#' @srrstats {G2.10} `data.frame` objects are appropriately processed to ensure consistent behavior.
#' @srrstats {G2.11} `data.frame` inputs  are appropriately processed and do not error without reason.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produce expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as data changes.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class "duane" returned by the duane_plot function.
#' @param ... Additional arguments (not used).
#' @family Duane functions
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- duane(times, failures)
#' print(fit)
#' @return Invisibly returns the input object.
#'
#' @export
print.duane <- function(x, ...) {
  # Input validation
  if (!inherits(x, "duane")) {
    stop("'x' must be an object of class 'duane'.")
  }

  cat("Duane Analysis Result\n")
  cat("----------------------\n")
  cat("Linear model (log-log scale): log(MTBF) ~ log(Time)\n")

  # Number of observations (failures)
  cat(sprintf("\nNumber of observations (failures): %d\n", x$n_obs))

  # Coefficient estimates and standard errors
  smry <- summary(x$model)
  coefs <- smry$coefficients

  cat("\nCoefficients:\n")
  print(coefs[, c("Estimate", "Std. Error")])

  # Model fit statistics
  cat(sprintf("\nLog-likelihood: %.2f", x$logLik))
  cat(sprintf("\nAIC: %.2f, BIC: %.2f", x$AIC, x$BIC))

  # Confidence level, if used
  if (!is.null(x$conf.level)) {
    cat(sprintf("\nConfidence level: %.1f%%", 100 * x$conf.level))
  }

  cat("\n")
  invisible(x)
}

#' Plot Method for Duane Analysis
#'
#' Generates a Duane plot (log-log or linear scale) with fitted regression line
#' and optional confidence bounds.
#'
#' @srrstats {G1.0} The primary reference for Duane Analysis is provided in the
#' description.
#' @srrstats {G1.1} The `duane` function is the first implementation of the Duane
#' model within an R package on CRAN.
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} [roxygen2](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.4b} Explicit conversion of log-likelihood to continuous is made via `as.numeric()`.
#' @srrstats {G2.6} One-dimensional inputs are appropriately pre-processed.
#' @srrstats {G2.7} Both one-dimensional vectors and data frames are accepted as input.
#' @srrstats {G2.8} Sub-functions `print.duane` and `plot.duane` are provided for the `duane` class.
#' @srrstats {G2.10} `data.frame` objects are appropriately processed to ensure consistent behavior.
#' @srrstats {G2.11} `data.frame` inputs  are appropriately processed and do not error without reason.
#' @srrstats {G2.13} The function checks for missing data and errors if any is found.
#' @srrstats {G2.14} See sub-tags for responses.
#' @srrstats {G2.14a} Missing data results in an error.
#' @srrstats {G2.14b} Missing data results in an error.
#' @srrstats {G2.14c} Missing data results in an error.
#' @srrstats {G2.15} The function checks for missing data and errors if any is found.
#' @srrstats {G2.16} The function checks for NA and NaN values and errors if any are found.
#' @srrstats {G5.0} The function is tested with a standard data set from a published paper.
#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produce expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as data changes.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class \code{"duane"}.
#' @param log Logical; whether to use logarithmic scales for axes (default: \code{TRUE}).
#' @param conf.int Logical; whether to plot confidence bounds (default: \code{TRUE}).
#' @param legend Logical; whether to include a legend (default: TRUE).
#' @param legend.pos Position of the legend (default: "topleft").
#' @param ... Further arguments passed to \code{plot()}.
#' @family Duane functions
#'
#' @return Invisibly returns \code{NULL}.
#' @examples
#' times <- c(100, 200, 300, 400, 500)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- duane(times, failures)
#' plot(fit, main = "Duane Plot", xlab = "Cumulative Time", ylab = "Cumulative MTBF")
#' @importFrom graphics legend lines plot
#' @export
plot.duane <- function(
    x,
    log = TRUE,
    conf.int = TRUE,
    legend = TRUE,
    legend.pos = "topleft",
    ...) {
  # Input validation
  if (!inherits(x, "duane")) {
    stop("'x' must be an object of class 'duane'.")
  }
  if (!is.logical(log) || length(log) != 1) {
    stop("'log' must be a single logical value.")
  }
  if (!is.logical(conf.int) || length(conf.int) != 1) {
    stop("'conf.int' must be a single logical value.")
  }
  if (!is.logical(legend) || length(legend) != 1) {
    stop("'legend' must be a single logical value.")
  }
  if (!is.character(legend.pos) || length(legend.pos) != 1) {
    stop("'legend.pos' must be a single character string.")
  }

  cum_time <- x$Cumulative_Time
  cum_mtbf <- x$Cumulative_MTBF

  plot_args <- list(
    x = cum_time,
    y = cum_mtbf,
    pch = 16,
    ...
  )
  if (log) {
    plot_args$log <- "xy"
  }
  do.call(graphics::plot, plot_args)

  # fitted line
  graphics::lines(cum_time, x$Fitted_Values, lty = 1)

  # confidence bounds (optional)
  if (conf.int && !is.null(x$Confidence_Bounds)) {
    graphics::lines(cum_time, x$Confidence_Bounds[, "lwr"], lty = 2)
    graphics::lines(cum_time, x$Confidence_Bounds[, "upr"], lty = 2)
  }

  # legend
  if (legend) {
    legend_items <- c("Observed", "Fitted Line")
    pch <- c(16, NA)
    lty <- c(NA, 1)

    if (conf.int && !is.null(x$Confidence_Bounds)) {
      legend_items <- c(legend_items, "Confidence Bounds")
      pch <- c(pch, NA)
      lty <- c(lty, 2)
    }

    graphics::legend(
      legend.pos,
      legend = legend_items,
      pch = pch,
      lty = lty,
      bty = "n"
    )
  }

  invisible(NULL)
}

#' @srrstats {G5.1} The function is tested with a standard data set. The data set is
#' created within and used to test the package. The data set is exported so that users
#' can confirm tests and run examples.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.4} Unit tests include correctness tests to test that statistical algorithms produce expected results to some fixed test data sets.
#' @srrstats {G5.4c} Unit tests include stored values that are drawn from a published paper output.
#' @srrstats {G5.5} Correctness tests are run with a fixed random seed.
#' @srrstats {G5.6} Unit tests include parameter recovery checks to test that the implementation produce expected results given data with known properties.
#' @srrstats {G5.6a} Parameter recovery tests are expected to be within a defined tolerance rather than exact values.
#' @srrstats {G5.7} Unit tests include algorithm performance checks to test that the function performs as expected as data changes.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.9} Unit tests include noise susceptibility tests for expected stochastic behavior.
#' @srrstats {G5.9a} Unit tests check that adding trivial noise to data does not meaningfully change results.
#' @srrstats {G5.9b} Unit tests check that different random seeds do not meaningfully change results.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#' @srrstats {RE7.1} Unit tests check for noiseless, exact relationships between
#' predictor (independent) and response (dependent) data.
#' @srrstats {RE7.1a} Unit tests confirm that model fitting is at least as fast
#' or faster than testing with equivalent noisy data.
#' @srrstats {RE7.2} Unit tests demonstrate that output objects retain aspects
#' of input data such as case names.
#' @srrstats {RE7.3} Unit tests demonstrate expected behavior when `duane` object
#' is submitted to the accessor methods `print` and `plot`.

set.seed(123)

test_that("data.frame input works the same as separate vectors", {
  times <- c(100, 200, 300)
  failures <- c(1, 2, 1)
  df <- data.frame(times = times, failures = failures)

  res_df <- duane(df)
  res_vec <- duane(times, failures)

  expect_s3_class(res_df, "duane")
  expect_s3_class(res_vec, "duane")
  expect_equal(res_df$Cumulative_Time, res_vec$Cumulative_Time)
  expect_equal(res_df$Cumulative_MTBF, res_vec$Cumulative_MTBF)
  expect_equal(res_df$Fitted_Values, res_vec$Fitted_Values)
})

test_that("data.frame without required columns errors", {
  df1 <- data.frame(x = 1:3, y = 1:3)
  expect_error(
    duane(df1),
    "must contain columns named 'times' and 'failures'"
  )

  df2 <- data.frame(times = 1:3)
  expect_error(
    duane(df2),
    "must contain columns named 'times' and 'failures'"
  )
})

test_that("NA or NaN in data.frame input throws error", {
  df_na <- data.frame(times = c(100, 200, NA), failures = c(1, 2, 1))
  df_nan <- data.frame(times = c(100, NaN, 300), failures = c(1, 2, 1))
  df_fail_na <- data.frame(times = c(100, 200, 300), failures = c(1, NA, 1))

  expect_error(duane(df_na), "'times' contains missing")
  expect_error(duane(df_nan), "'times' contains missing")
  expect_error(duane(df_fail_na), "'failures' contains missing")
})

test_that("data.frame with non-positive or infinite values errors", {
  df_zero_time <- data.frame(times = c(0, 100, 200), failures = c(1, 2, 1))
  df_neg_fail <- data.frame(times = c(100, 200, 300), failures = c(1, -1, 1))
  df_inf_time <- data.frame(times = c(100, Inf, 200), failures = c(1, 2, 1))

  expect_error(duane(df_zero_time), "must be finite and > 0")
  expect_error(duane(df_neg_fail), "must be finite and > 0")
  expect_error(duane(df_inf_time), "must be finite and > 0")
})

test_that("data.frame input respects conf.level parameter", {
  times <- c(100, 200, 300)
  failures <- c(1, 2, 1)
  df <- data.frame(times = times, failures = failures)

  res <- duane(df, conf.level = 0.90)
  expect_equal(res$conf.level, 0.90)
})

# Helper: minimal valid inputs
valid_times <- c(100, 200, 300)
valid_failures <- c(1, 2, 1)

test_that("data frame without required columns errors", {
  df_bad <- data.frame(a = 1:3, b = 1:3)
  expect_error(duane(df_bad),
    "Data frame input must contain columns named 'times' and 'failures'.",
    fixed = TRUE
  )
})

test_that("NA / NaN checks for times and failures", {
  expect_error(duane(c(1, NA), valid_failures),
    "'times' contains missing (NA) or NaN values.",
    fixed = TRUE
  )

  expect_error(duane(valid_times, c(1, NaN, 1)),
    "'failures' contains missing (NA) or NaN values.",
    fixed = TRUE
  )
})

test_that("type checks for times and failures", {
  expect_error(duane(list(1, 2, 3), valid_failures),
    "'times' must be a numeric vector.",
    fixed = TRUE
  )

  expect_error(duane(valid_times, list(1, 2, 3)),
    "'failures' must be a numeric vector.",
    fixed = TRUE
  )
})

test_that("empty vector checks", {
  expect_error(duane(numeric(0), numeric(0)),
    "'times' cannot be empty.",
    fixed = TRUE
  )

  expect_error(duane(valid_times, numeric(0)),
    "'failures' cannot be empty.",
    fixed = TRUE
  )
})

test_that("length mismatch check", {
  expect_error(duane(c(1, 2), c(1, 2, 3)),
    "The length of 'times' and 'failures' must be equal.",
    fixed = TRUE
  )
})

test_that("finite and >0 checks for times and failures", {
  expect_error(duane(c(1, Inf, 3), c(1, 1, 1)),
    "All values in 'times' must be finite and > 0.",
    fixed = TRUE
  )

  expect_error(duane(valid_times, c(1, 0, 2)),
    "All values in 'failures' must be finite and > 0.",
    fixed = TRUE
  )
})

test_that("conf.level validation", {
  expect_error(duane(valid_times, valid_failures, conf.level = c(0.9, 0.95)),
    "'conf.level' must be a single numeric value.",
    fixed = TRUE
  )

  expect_error(duane(valid_times, valid_failures, conf.level = 1),
    "'conf.level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )

  expect_error(duane(valid_times, valid_failures, conf.level = 0),
    "'conf.level' must be between 0 and 1 (exclusive).",
    fixed = TRUE
  )
})

test_that("print.duane errors when input is not duane", {
  expect_error(print.duane(list()),
    "'x' must be an object of class 'duane'.",
    fixed = TRUE
  )
})

test_that("plot.duane argument type checks", {
  # Minimal valid duane object
  dummy_fit <- duane(valid_times, valid_failures)

  # inherits check
  expect_error(plot.duane(list()),
    "'x' must be an object of class 'duane'.",
    fixed = TRUE
  )

  # log must be single logical
  expect_error(plot.duane(dummy_fit, log = "nope"),
    "'log' must be a single logical value.",
    fixed = TRUE
  )

  # conf.int must be single logical
  expect_error(plot.duane(dummy_fit, log = TRUE, conf.int = "nope"),
    "'conf.int' must be a single logical value.",
    fixed = TRUE
  )

  # legend must be single logical
  expect_error(plot.duane(dummy_fit, log = TRUE, conf.int = TRUE, legend = "nope"),
    "'legend' must be a single logical value.",
    fixed = TRUE
  )

  # legend.pos must be single character
  expect_error(plot.duane(dummy_fit, log = TRUE, conf.int = TRUE, legend = TRUE, legend.pos = c("a", "b")),
    "'legend.pos' must be a single character string.",
    fixed = TRUE
  )
})

test_that("duane recovers known parameters (linear log-log relationship)", {
  # Generate synthetic data for constant MTBF process
  times <- rep(100, 10) # equal spacing of 100
  failures <- rep(1, 10) # one failure per interval

  fit <- duane(times, failures, conf.level = 0.95)

  coef_est <- coef(fit$model)

  # Expected: slope ~ 1, intercept ~ log(constant)
  expect_equal(unname(exp(coef_est[2])), 1, tolerance = 0.2)
})

test_that("duane returns same result for vector and data.frame inputs", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)

  fit_vec <- duane(times, failures)
  fit_df <- duane(data.frame(times = times, failures = failures))

  expect_equal(fit_vec$Cumulative_MTBF,
    fit_df$Cumulative_MTBF,
    tolerance = 1e-12
  )

  expect_equal(coef(fit_vec$model),
    coef(fit_df$model),
    tolerance = 1e-12
  )
})

test_that("confidence level is respected", {
  times <- c(100, 200, 300, 400, 500)
  failures <- c(1, 2, 1, 3, 2)

  fit90 <- duane(times, failures, conf.level = 0.90)
  fit95 <- duane(times, failures, conf.level = 0.95)

  # Wider interval for higher confidence level
  width90 <- mean(fit90$Confidence_Bounds[, "upr"] - fit90$Confidence_Bounds[, "lwr"])
  width95 <- mean(fit95$Confidence_Bounds[, "upr"] - fit95$Confidence_Bounds[, "lwr"])

  expect_gt(width95, width90)
})

test_that("slope approaches 1 for constant MTBF process", {
  # Failures occur at constant intervals
  times <- rep(100, 20)
  failures <- rep(1, 20)

  fit <- duane(times, failures)
  slope <- unname(exp(coef(fit$model)[2]))

  expect_equal(slope, 1, tolerance = 0.2)
})

test_that("slope is less than 1 for reliability growth process", {
  # Failures get further apart over time (improving reliability)
  times <- seq(100, 1000, length.out = 20)
  failures <- rep(1, 20)

  fit <- duane(times, failures)
  slope <- coef(fit$model)[2]

  expect_lt(slope, 1)
})

test_that("cumulative MTBF increases with cumulative time if failures constant", {
  times <- rep(100, 10)
  failures <- rep(1, 10)

  fit <- duane(times, failures)

  expect_true(all(diff(fit$Cumulative_MTBF) >= 0))
})

# test_that("loglik increases with more data", {
#   times_small <- rep(100, 5)
#   failures_small <- rep(1, 5)
#   fit_small <- duane(times_small, failures_small)
#
#   times_large <- rep(100, 50)
#   failures_large <- rep(1, 50)
#   fit_large <- duane(times_large, failures_large)
#
#   # Larger datasets should yield smaller logLik but larger penalties
#   expect_lt(fit_small$logLik, fit_large$logLik) # logLik increases with more data
#
# })

test_that("adding small noise to times does not change slope significantly", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  times <- rep(100, 20)
  failures <- rep(1, 20)

  fit_clean <- duane(times, failures)

  # Add ±1% random noise
  noise <- rnorm(length(times), mean = 0, sd = 1)
  times_noisy <- times + times * noise * 0.01
  fit_noisy <- duane(times_noisy, failures)

  slope_clean <- coef(fit_clean$model)[2]
  slope_noisy <- coef(fit_noisy$model)[2]

  expect_equal(slope_clean, slope_noisy, tolerance = 0.05)
})

test_that("adding small noise preserves fitted MTBF values", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  times <- seq(100, 1000, length.out = 20)
  failures <- rep(1, 20)

  fit_clean <- duane(times, failures)

  noise <- rnorm(length(times), mean = 0, sd = 1)
  times_noisy <- times + times * noise * 0.01
  fit_noisy <- duane(times_noisy, failures)

  # Compare fitted values via correlation
  cor_val <- cor(fit_clean$Fitted_Values, fit_noisy$Fitted_Values)
  expect_gt(cor_val, 0.99)
})

test_that("duane works with input from testdata", {
  # testdata is lazy-loaded via LazyData: true; no explicit data() call needed
  # Subset one LRU to get a simple series
  g1 <- subset(testdata, LRU == "G1")

  # Use cumulative ETI as times and Failure_Count as failures
  times <- g1$Cum_ETI
  failures <- g1$Failure_Count

  fit <- duane(times, failures, conf.level = 0.95)

  # Check structure
  expect_true(all(fit$Cumulative_MTBF > 0))
  expect_true(all(fit$Fitted_Values > 0))
})

# Example data
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- duane(times, failures, conf.level = 0.95)

test_that("print.duane works on valid duane object", {
  expect_invisible(out <- print(fit))
  expect_s3_class(out, "duane")
})

test_that("print.duane shows expected content", {
  output <- capture.output(print(fit))
  expect_true(any(grepl("Duane Analysis Result", output)))
  expect_true(any(grepl("Coefficients:", output)))
  expect_true(any(grepl("Log-likelihood:", output)))
  expect_true(any(grepl("AIC:", output)))
  expect_true(any(grepl("Confidence level:", output)))
})

test_that("print.duane errors on wrong input type", {
  expect_error(print.duane(123), "'x' must be an object of class 'duane'")
  expect_error(print.duane(list()), "'x' must be an object of class 'duane'")
})

test_that("plot.duane works on valid duane object", {
  expect_invisible(plot(fit)) # default args
  expect_invisible(plot(fit, log = FALSE, conf.int = FALSE, legend = FALSE))
})

test_that("plot.duane errors on wrong input type", {
  expect_error(plot.duane(123), "'x' must be an object of class 'duane'")
  expect_error(plot.duane(list()), "'x' must be an object of class 'duane'")
})

test_that("plot.duane input validation works", {
  expect_error(plot(fit, log = "yes"), "'log' must be a single logical value")
  expect_error(plot(fit, conf.int = "yes"), "'conf.int' must be a single logical value")
  expect_error(plot(fit, legend = "yes"), "'legend' must be a single logical value")
  expect_error(plot(fit, legend.pos = TRUE), "'legend.pos' must be a single character string")
})

test_that("duane() handles noiseless, exact relationships efficiently", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()

  set.seed(123)

  # Generate synthetic data with a perfect log-log linear relationship
  n <- 1000
  times <- seq(1, n)
  alpha <- 2.0
  beta <- 0.5
  # Perfect noiseless relationship (log-log linear)
  failures_noiseless <- exp((log(times) - log(alpha)) / beta)

  # Slightly noisy version of the same
  noise <- rnorm(n, mean = 0, sd = 0.1)
  failures_noisy <- exp((log(times) - log(alpha)) / beta + noise)

  # Measure performance (timing)
  t_noiseless <- system.time({
    fit_noiseless <- duane(times, failures_noiseless)
  })[["elapsed"]]

  t_noisy <- system.time({
    fit_noisy <- duane(times, failures_noisy)
  })[["elapsed"]]

  # Check that noiseless fit is at least as fast or faster
  expect_true(
    t_noiseless <= t_noisy * 1.2 + 0.1, # allow small tolerance
    info = sprintf("Noiseless fit took %.3fs vs noisy %.3fs", t_noiseless, t_noisy)
  )

  # Check that coefficients are nearly identical in noiseless case
  coef_noiseless <- coef(fit_noiseless$model)
  coef_noisy <- coef(fit_noisy$model)
})

test_that("output retains row or case names from input data", {
  # Case 1: Named numeric vectors
  times <- c(A = 100, B = 200, C = 300, D = 400, E = 500)
  failures <- c(A = 1, B = 2, C = 1, D = 3, E = 2)

  fit_named <- duane(times, failures)

  # Expect that output retains names
  expect_equal(names(fit_named$times), names(times))
  expect_equal(names(fit_named$failures), names(failures))
})

#' Q-Q Plot for RGA Objects
#'
#' This function creates a Q-Q plot for a fitted Reliability Growth Analysis (RGA) model
#' Currently only supports the Crow-AMSAA model. A Q-Q plot compares the quantiles of
#' the empirical data to the quantiles of the theoretical distribution specified by the model.
#' If the model fits well, the points should fall approximately along a straight line.
#'
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.8} Sub-functions `qqplot.rga` and `ppplot.rga` are provided for the `rga` class.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class \code{rga}.
#' @param main Title of the plot.
#' @param ... Additional arguments passed to \code{stats::qqplot()}.
#' @examples
#' times <- c(5, 10, 15, 20, 25)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- rga(times, failures)
#' qqplot.rga(fit)
#' @return A Q-Q plot comparing empirical and theoretical quantiles.
#' @family goodness-of-fit
#' @importFrom stats lsfit
#' @export
qqplot.rga <- function(x,
                       main = "Q-Q Plot",
                       ...) {
  # Input validation
  if (!inherits(x, "rga")) {
    stop("'x' must be an object of class 'rga'.")
  }
  if (!is.character(main) || length(main) != 1) {
    stop("'main' must be a single character string.")
  }

  # Extract parameters (Crow-AMSAA case)
  beta <- if (is.list(x$betas)) x$betas$log_times[1, "Est."] else x$betas[1]
  lambda <- if (is.matrix(x$lambdas)) x$lambdas[1, "Est."] else x$lambdas[1]

  # Observed cumulative times
  times <- sort(
    exp(x$model$model$log_times)
  )
  n <- length(times)

  # Empirical probs
  emp_probs <- (1:n) / (n + 1)

  # Theoretical quantiles
  theo_quants <- (-log(1 - emp_probs) / lambda)^(1 / beta)

  stats::qqplot(theo_quants, times,
    main = main,
    xlab = "Theoretical Quantiles",
    ylab = "Observed Quantiles",
    ...
  )
  abline(lsfit(theo_quants, times))
}

#' P-P Plot for RGA Objects
#'
#' This function creates a P-P plot for a fitted Reliability Growth Analysis (RGA)
#' model. Currently only supports the Crow-AMSAA model. A P-P plot compares the
#' empirical cumulative distribution function (CDF) to the theoretical CDF specified
#' by the model. If the model fits well, the points should fall approximately along a straight line.
#'
#' @srrstats {G1.2} The Life Cycle Statement is in the CONTRIBUTING.md file.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G1.4} `roxygen2`](https://roxygen2.r-lib.org/) documentation is used
#' to document all functions.
#' @srrstats {G1.3} All statistical terminology is explicitly defined in the documentation.
#' @srrstats {G2.0} Inputs are validated for length.
#' @srrstats {G2.1} Inputs are validated for type.
#' @srrstats {G2.2} Multivariate inputs are prohibited where only univariate are allowed.
#' @srrstats {G2.8} Sub-functions `qqplot.rga` and `ppplot.rga` are provided for the `rga` class.
#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.
#'
#' @param x An object of class \code{rga}.
#' @param main Title of the plot.
#' @param ... Additional arguments passed to \code{plot()}.
#' @examples
#' times <- c(5, 10, 15, 20, 25)
#' failures <- c(1, 2, 1, 3, 2)
#' fit <- rga(times, failures)
#' ppplot.rga(fit)
#' @returns A P-P plot comparing empirical and theoretical CDFs.
#' @family goodness-of-fit
#' @importFrom stats lsfit
#' @export
ppplot.rga <- function(x,
                       main = "P-P Plot",
                       ...) {
  # Input validation
  if (!inherits(x, "rga")) {
    stop("'x' must be an object of class 'rga'.")
  }
  if (!is.character(main) || length(main) != 1) {
    stop("'main' must be a single character string.")
  }

  # Extract parameters (Crow-AMSAA case)
  beta <- if (is.list(x$betas)) x$betas$log_times[1, "Est."] else x$betas[1]
  lambda <- if (is.matrix(x$lambdas)) x$lambdas[1, "Est."] else x$lambdas[1]

  # Observed cumulative times
  times <- sort(
    exp(x$model$model$log_times)
  )
  n <- length(times)

  # Empirical CDF (median rank plotting position)
  emp_cdf <- (1:n) / (n + 1)

  # Theoretical CDF from Weibull model
  theo_cdf <- lambda * times^beta

  # Plot with probability scaling
  plot(theo_cdf, emp_cdf,
    main = main,
    xlab = "Theoretical CDF",
    ylab = "Empirical CDF",
    ...
  )

  # 45-degree reference line
  abline(lsfit(theo_cdf, emp_cdf))
}

#' @srrstats {G5.2} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.2a} Every message produced by `stop()` is unique.
#' @srrstats {G5.2b} Unit tests demonstrate error messages and compare results with expected values.
#' @srrstats {G5.8} See sub-tags for responses.
#' @srrstats {G5.8a} Unit tests include checks for zero-length data.
#' @srrstats {G5.8b} Unit tests include checks for unsupported data types.
#' @srrstats {G5.8c} Unit tests include checks for data with 'NA' fields.
#' @srrstats {G5.8d} Unit tests include checks for data outside the scope of the algorithm.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.

test_that("qqplot.rga() errors on invalid inputs", {
  # Not an rga object
  expect_error(
    qqplot.rga(list()),
    "'x' must be an object of class 'rga'."
  )

  # Invalid 'main': numeric instead of string
  dummy <- structure(list(), class = "rga")
  expect_error(
    qqplot.rga(dummy, main = 123),
    "'main' must be a single character string."
  )

  # Invalid 'main': character vector of length > 1
  expect_error(
    qqplot.rga(dummy, main = c("a", "b")),
    "'main' must be a single character string."
  )
})

test_that("ppplot.rga() errors on invalid inputs", {
  # Not an rga object
  expect_error(
    ppplot.rga(list()),
    "'x' must be an object of class 'rga'."
  )

  # Invalid 'main': numeric instead of string
  dummy <- structure(list(), class = "rga")
  expect_error(
    ppplot.rga(dummy, main = 999),
    "'main' must be a single character string."
  )

  # Invalid 'main': character vector of length > 1
  expect_error(
    ppplot.rga(dummy, main = c("p", "q")),
    "'main' must be a single character string."
  )
})

# Build a minimal reproducible rga object for testing
make_rga_numeric <- function() {
  times <- c(5, 10, 15, 20, 25)
  failures <- c(1, 2, 1, 3, 2)
  rga(times, failures) # assumes your rga() is available
}

make_rga_matrix <- function() {
  fit <- make_rga_numeric()
  # Overwrite betas and lambdas to matrix/list form to cover branches
  fit$betas <- list(log_times = matrix(
    c(0.9, 0.1),
    nrow = 1, dimnames = list(NULL, c("Est.", "SE"))
  ))
  fit$lambdas <- matrix(
    c(0.002, 0.0005),
    nrow = 1, dimnames = list(NULL, c("Est.", "SE"))
  )
  fit
}

test_that("qqplot.rga runs silently on valid input (numeric params)", {
  fit <- make_rga_numeric()
  expect_silent(qqplot.rga(fit))
  expect_silent(qqplot.rga(fit, main = "Custom QQ", pch = 19, col = "blue"))
})

test_that("qqplot.rga runs silently on valid input (matrix/list params)", {
  fit <- make_rga_matrix()
  expect_silent(qqplot.rga(fit))
})

test_that("ppplot.rga runs silently on valid input (numeric params)", {
  fit <- make_rga_numeric()
  expect_silent(ppplot.rga(fit))
  expect_silent(ppplot.rga(fit, main = "Custom PP", pch = 17, col = "red"))
})

test_that("ppplot.rga runs silently on valid input (matrix/list params)", {
  fit <- make_rga_matrix()
  expect_silent(ppplot.rga(fit))
})

#' ReliaGrowR: Reliability Growth Analysis
#'
#' @description
#' Modeling and plotting functions for Reliability Growth Analysis (RGA).
#' The package implements three families of models and provides a REST API
#' interface via [plumber][plumber::plumber].
#'
#' @section Reliability Growth Models:
#'
#' **Crow-AMSAA (NHPP power-law)**
#'
#' The core model fits a Non-Homogeneous Poisson Process (NHPP) with a
#' Weibull intensity function to cumulative failure data. Parameters are
#' estimated by least-squares log-log regression (default) or maximum
#' likelihood.  A growth rate > 0 indicates reliability improvement.
#'
#' **Piecewise NHPP**
#'
#' Extends Crow-AMSAA by fitting separate NHPP segments separated by
#' change points.  Change points can be detected automatically via the
#' [segmented][segmented::segmented] package or supplied by the user.
#'
#' **Duane**
#'
#' Log-log regression of cumulative MTBF versus cumulative time, providing
#' a graphical and analytical representation of reliability growth.
#'
#' @section Main Functions:
#'
#' | Function | Description |
#' |---|---|
#' | [rga()] | Fit Crow-AMSAA or Piecewise NHPP model |
#' | [predict_rga()] | Forecast cumulative failures from a fitted model |
#' | [duane()] | Fit Duane model |
#' | [rdt()] | Reliability Demonstration Test plan calculator |
#' | [weibull_to_rga()] | Convert Weibull data to RGA format |
#' | [sim_failures()] | Simulate failures from a conditional Weibull model |
#' | [qqplot.rga()] | Q-Q goodness-of-fit plot for an `rga` object |
#' | [ppplot.rga()] | P-P goodness-of-fit plot for an `rga` object |
#' | [grwr_api()] | Launch the plumber REST API |
#'
#' @section S3 Classes and Methods:
#'
#' | Class | Methods |
#' |---|---|
#' | `rga` | [print.rga()], [plot.rga()] |
#' | `rga_predict` | [print.rga_predict()], [plot.rga_predict()] |
#' | `duane` | [print.duane()], [plot.duane()] |
#' | `rdt` | [print.rdt()] |
#'
#' @section References:
#' Crow, L. H. (1975). *Reliability Analysis for Complex Repairable Systems.*
#' AMSAA Technical Report No. 138. US Army Materiel Systems Analysis Activity.
#'
#' Duane, J. T. (1964). Learning curve approach to reliability monitoring.
#' *IEEE Transactions on Aerospace*, 2(2), 563–566.
#' \doi{10.1109/TA.1964.4319640}
#'
#' Guo, H., Mettas, A., Sarakakis, G., & Niu, P. (2010). Piecewise NHPP
#' models with maximum likelihood estimation for repairable systems. In
#' *Proceedings of the 2010 Annual Reliability and Maintainability Symposium*
#' (pp. 1–6). IEEE. \doi{10.1109/RAMS.2010.5448029}
#'
#' Muggeo, V. M. R. (2024). *segmented: Regression Models with Break-Points /
#' Change-Points Estimation.* R package.
#' \url{https://cran.r-project.org/package=segmented}
#'
#' @aliases ReliaGrowR-package
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

#' NA_standards
#'
#' @srrstatsNA {G1.4a} There are no internal (non-exported) functions.
#' @srrstatsNA {G1.5} There are no performance claims made in an associated publication.
#' @srrstatsNA {G1.6} There are no performance claims compared to other R packages.
#' @srrstatsNA {G2.0a} The primary documentation covers all expectations on lengths of inputs.
#' @srrstatsNA {G2.1a} The primary documentation covers all expectations on data types of all vector inputs.
#' @srrstatsNA {G2.4} See sub-tags for responses.
#' @srrstatsNA {G2.4a} `integer` conversion is not required.
#' @srrstatsNA {G2.4c} `as.character()` conversion is not required.
#' @srrstatsNA {G2.4d} `as.factor()` conversion is not required.
#' @srrstatsNA {G2.4e} `as...()` conversion is not required.
#' @srrstatsNA {G2.5} There are no factor inputs.
#' @srrstatsNA {G2.9} No information is lost in type conversions.
#' @srrstatsNA {G2.12} list columns in `data.frame` objects are not used.
#' @srrstatsNA {G3.0} There are no comparisons made between floating point numbers.
#' @srrstatsNA {G3.1} Currently, there are no covariance calculations since all models
#' are univariate. However, this may change in future versions.
#' @srrstatsNA {G3.1a} Currently, there are no covariance calculations since all models
#' are univariate. However, this may change in future versions.
#' @srrstatsNA {G4.0} There are no local file outputs at this time.
#' @srrstatsNA {G5.3} Missing values return an error.
#' @srrstatsNA {G5.4a} There are new implementations of existing methods but no new methods.
#' @srrstatsNA {G5.4b} There are no known implementations of these methods in R.
#' @srrstatsNA {G5.6b} Parameter recovery tests do not include random components.
#' @srrstatsNA {G5.11} Unit tests do not require large amounts of data.
#' @srrstatsNA {G5.11a} Unit tests do not require downloads of additional data.
#' @srrstatsNA {G5.12} Unit tests do not require any special platform requirements,
#' memory, expected runtime, or artefacts.
#' @srrstatsNA {RE1.0} The dependent variable (failures) and independent
#' variable (times) are defined in the function arguments, not in a formula interface.
#' @srrstatsNA {RE1.1} The dependent variable (failures) and independent
#' variable (times) are defined in the function arguments, not in a formula interface.
#' @srrstatsNA {RE2.0} Input data is transformed using simple mathematical operations
#' (e.g., cumsum) that do not require special treatment.
#' @srrstatsNA {RE2.3} Data centering is not required.
#' @srrstatsNA {RE3.0} Algorithms are closed form solutions and do not require iterative
#' methods.
#' @srrstatsNA {RE3.1} Algorithms are closed form solutions and do not require iterative
#' methods.
#' @srrstatsNA {RE3.2} Algorithms are closed form solutions and do not require iterative
#' methods.
#' @srrstatsNA {RE3.3} Algorithms are closed form solutions and do not require iterative
#' methods.
#' @srrstatsNA {RE4.1} Software does not enable the ability to generate a model object
#' without actually fitting values.
#' @srrstatsNA {RE4.6} The current functionality is for univariate analysis only.
#' As such, the variance-covariance matrix of the model parameters is not applicable.
#' @srrstatsNA {RE4.7} Convergence is not applicable as the algorithms are closed form solutions.
#' @srrstatsNA {RE4.12} Input data is transformed using simple mathematical operations
#' (e.g., cumsum) that do not require special treatment.
#' @srrstatsNA {RE4.14} Model objects are not used to generate a forecast, though
#' prediction is possible in a future version.
#' @srrstatsNA {RE4.15} Model objects are not used to generate a forecast, though
#' prediction is possible in a future version.
#' @srrstatsNA {RE4.16} Model objects are not used to generate a forecast, though
#' prediction is possible in a future version.
#' @srrstatsNA {RE4.18} Summary statistics are trivial and do not require special
#' methods.
#' @srrstatsNA {RE6.1} Default plot method is a generic plot method dispatched on
#' the class of return objects.
#' @srrstatsNA {RE6.3} Model objects are not used to generate a forecast, though
#' prediction is possible in a future version.
#' @srrstatsNA {RE7.0} The current functionality is for univariate analysis only.
#' As such, tests involving relationships between predictor (independent) variables
#' are not applicable.
#' @srrstatsNA {RE7.0a} The current functionality is for univariate analysis only.
#' As such, tests involving relationships between predictor (independent) variables
#' are not applicable.
#' @srrstatsNA {RE7.4} Model objects are not used to generate a forecast, though
#' prediction is possible in a future version.
#' @noRd
NULL

#' @srrstats {G5.1} The data set is created within and used to test the package.
#' The data set is exported so that users can confirm tests and run examples.
#' @srrstats {G5.10} All unit tests run as part of continuous integration.

test_that("testdata dataset has expected structure", {
  data("testdata")

  # check class
  expect_s3_class(testdata, "data.frame")

  # check dimensions
  expect_equal(nrow(testdata), 25)
  expect_equal(ncol(testdata), 7)

  # check column names
  expected_cols <- c(
    "LRU", "Cum_ETI", "Failure_Count",
    "Cum_MTBF", "Report_No", "Flag", "Cause"
  )
  expect_equal(names(testdata), expected_cols)

  # check column types
  expect_type(testdata$LRU, "character") # could be factor, depends how saved
  expect_type(testdata$Cum_ETI, "double")
  expect_type(testdata$Failure_Count, "integer") # or "double"
  expect_type(testdata$Cum_MTBF, "double")
  expect_type(testdata$Report_No, "double") # or "double"
  expect_type(testdata$Flag, "double") # or factor
  expect_type(testdata$Cause, "character") # or factor

  # sanity checks on values
  expect_true(all(testdata$Cum_ETI >= 0))
  expect_true(all(testdata$Failure_Count >= 0))
  expect_true(all(testdata$Cum_MTBF > 0))
  expect_true(all(testdata$LRU %in% c("G1", "G2")))
  expect_true(all(testdata$Cause %in% c("D", "M", "R", "NR")))
})

---
title: "Reliability Data Management"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

This vignette demonstrates various data management techniques commonly used in reliability analysis. It covers importing data from different sources, arranging and cleaning the data, handling missing values, and exporting the cleaned data for further analysis.

## Importing Data

Most examples in this package use data that is included with the package. However, in practice, you will often need to import data from external sources. Here are some common methods for importing data into R:

### Importing from CSV Files

You can use the `read.csv()` function to import data from CSV files. For example:
```{r eval=FALSE}
data <- read.csv("path/to/your/data.csv")
head(data)
```

### Importing from Excel Files

You can use the `readxl` package to import data from Excel files. For example:
```{r eval=FALSE}
library(readxl)
data <- read_excel("path/to/your/data.xlsx")
head(data)
```

There are many other data import methods available in R, including importing from databases, JSON files, and more. The choice of method will depend on the format of your data and your specific needs. To learn more about data import in R, you can refer to the [R Data Import/Export](https://cran.r-project.org/doc/manuals/r-release/R-data.html) documentation.

## Arranging Data

Once you have imported your data, you may need to arrange it in a format suitable for reliability analysis.
`ReliaGrowR` functions typically expect data in a "long" format, where each row represents a single observation or event. Specifically, for reliability data, you will need data for:

- **Times**: The time points at which failures were observed.
- **Failures**: The number of failures observed at each time point.

This data can be passed to functions as separate vectors or as a data frame with appropriate column names. For example, data vectors can be created as follows:
```{r}
times <- c(100, 200, 300, 400, 500)
failures <- c(2, 3, 5, 7, 11)
```

Alternatively, you can create a data frame:
```{r}
data <- data.frame(col1 = times, col2 = failures)
head(data)
```

If importing data using the `read.csv()` or `read_excel()` functions, you may need to rename columns to match the expected format. You can use the `colnames()` function to rename columns:

```{r}
colnames(data) <- c("times", "failures")
head(data)
```

## Censored Data

In reliability analysis, censored data refers to observations where the exact failure time is not known. This can occur in various scenarios, such as when a test is terminated before all units have failed or when some units are still functioning at the end of the observation period. Censored data can be classified into different types:

- **Right Censoring**: This occurs when the failure time is known to be greater than a certain value. For example, if a unit is still functioning at the end of the test period, its failure time is right-censored.
- **Left Censoring**: This occurs when the failure time is known to be less than a certain value. For example, if a unit fails before the start of the observation period, its failure time is left-censored.
- **Interval Censoring**: This occurs when the failure time is known to lie within a certain interval. For example, if a unit is inspected at regular intervals, and it is known to have failed between two inspection times, its failure time is interval
censored.

The `weibull_to_rga` utility function can be used to convert censored data into a format suitable for reliability growth analysis. This function converts data in the following ways:

- **Failure times**: Failure times are converted to cumulative times and failure counts.
- **Right-censored times**: Right-censored times are included in the cumulative time calculations but do not contribute to failure counts.
- **Interval-censored times**: Interval-censored times are handled by taking the midpoint of the interval for the failure time. 

For example, consider the following data that includes failure times, right-censored times, and interval-censored times:

```{r}
failures <- c(100, 200, 200, 400)
right_censored <- c(250, 350, 450)
interval_starts <- c(150, 300)
interval_ends <- c(180, 320)
```

To convert this data for reliability growth analysis, you can use the `weibull_to_rga` function as follows:

```{r}
library(ReliaGrowR)
result <- weibull_to_rga(failures, right_censored, interval_starts, interval_ends)
head(result)
```

The resulting data frame contains cumulative times and failure counts, which can then be passed directly to the `rga` function for reliability growth analysis. 

## Missing, NAs, NANs and Infs

When working with real-world data, you may encounter missing values, represented as `NA`, `NaN`, or `Inf` in R. It is important to handle these values appropriately before performing any analysis. Here are some common techniques for dealing with missing values:

### Removing Missing Values

You can use the `na.omit()` function to remove rows with missing values from your data frame. For example:
```{r}
data <- data.frame(
  times = c(100, 200, NA, 400, 500),
  failures = c(2, NA, 5, 7, 11)
)
cleaned_data <- na.omit(data)
head(cleaned_data)
```

### Replacing Missing Values

You can replace missing values with a specific value, such as the mean or median failure time. For example, to replace missing values in the `times` and `failures` columns with the mean values, you can use the following:

```{r}
data <- data.frame(
  times = c(100, 200, NA, 400, 500),
  failures = c(2, NA, 5, 7, 11)
)
data$times[is.na(data$times)] <-
  mean(
    data$times,
    na.rm = TRUE
  )
data$failures[is.na(data$failures)] <-
  ceiling(
    mean(
      data$failures,
      na.rm = TRUE
    )
  )
head(data)
```

## Exporting Data

Once you have cleaned and arranged your data, you may want to export it for further analysis or sharing. Here are some common methods for exporting data from R:

### Exporting to CSV Files

You can use the `write.csv()` function to export data to CSV files. For example:
```{r eval=FALSE}
write.csv(data, "path/to/your/cleaned_data.csv", row.names = FALSE)
```

### Exporting to Excel Files

You can use the `writexl` package to export data to Excel files. For example:

```{r eval=FALSE}
library(writexl)
write_xlsx(data, "path/to/your/cleaned_data.xlsx")
```

To learn more about data export in R, you can refer to the [R Data Import/Export](https://cran.r-project.org/doc/manuals/r-release/R-data.html) documentation.

---
title: "Repairable Systems Analysis"
output:
  html_vignette:
    fig_width: 7
    fig_height: 5
vignette: >
  %\VignetteIndexEntry{Repairable Systems Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

**Repairable systems** are systems that can be restored to a functioning state after a failure. Unlike non-repairable items where we model time-to-first-failure, repairable systems generate **recurrent events** -- a sequence of failure times over the system's life. The **Non-Homogeneous Poisson Process (NHPP)** provides a natural framework for modeling these recurrence patterns.

ReliaGrowR provides four key tools for analyzing repairable systems:

1. **`mcf()`** -- Non-parametric Mean Cumulative Function estimation
2. **`exposure()`** -- Exposure analysis (total operating time at risk)
3. **`nhpp()`** -- Parametric NHPP model fitting (Power Law and Log-Linear)
4. **`predict_nhpp()`** -- Forecasting future events from a fitted model

## Mean Cumulative Function

The **Mean Cumulative Function (MCF)** is a non-parametric estimate of the expected cumulative number of events per system over time. It makes no assumptions about the underlying process and is useful as an exploratory tool before fitting parametric models.

For a fleet of systems, the MCF at time $t$ is estimated using the Nelson-Aalen estimator:

$$
\hat{M}(t) = \sum_{t_j \le t} \frac{d_j}{n_j}
$$

where $d_j$ is the number of events at time $t_j$ and $n_j$ is the number of systems still under observation.

### Example

Consider three systems with the following event histories:

```{r}
library(ReliaGrowR)

id   <- c(1, 1, 1, 2, 2, 2, 3, 3, 3, 3)
time <- c(100, 350, 500, 80, 300, 600, 150, 250, 400, 700)
```

Compute and plot the MCF:

```{r}
result <- mcf(id, time)
plot(result, main = "Mean Cumulative Function",
     xlab = "Time", ylab = "MCF")
```

The step-function plot shows the estimated average cumulative events per system over time, with dashed confidence bounds.

### Using Data Frames

The `mcf()` function also accepts a data frame:

```{r}
df <- data.frame(id = id, time = time)
result2 <- mcf(data = df)
```

### Censored Observations

If some systems are withdrawn from observation before the end of the study, use the `event` indicator (1 = event, 0 = censored):

```{r}
id    <- c(1, 1, 1, 2, 2, 2, 3, 3, 3)
time  <- c(100, 350, 500, 80, 300, 400, 150, 250, 700)
event <- c(  1,   1,   0,  1,   1,   0,   1,   1,   1)

result <- mcf(id, time, event)
```

System 1 was censored at time 500 and System 2 at time 400. The MCF accounts for the reduced number of systems at risk after these times.

### Exposure-Adjusted MCF

When systems are observed beyond their last event time (i.e., the system was running but no event occurred), it is important to specify the actual end-of-observation time. Otherwise, the MCF assumes a system left observation at its last event, which inflates the estimated recurrence rate.

The `end_time` parameter specifies the actual observation window per system:

```{r}
id   <- c(1, 1, 2, 2, 3, 3)
time <- c(100, 300, 150, 400, 200, 350)

# Without end_time: system observation ends at last event
mcf_basic <- mcf(id, time)

# With end_time: all systems observed until time 800
mcf_adj <- mcf(id, time, end_time = c("1" = 800, "2" = 800, "3" = 800))
```

Compare the two MCF estimates:

```{r}
par(mfrow = c(1, 2))
plot(mcf_basic, main = "MCF (inferred exposure)",
     xlab = "Time", ylab = "MCF")
plot(mcf_adj, main = "MCF (explicit exposure)",
     xlab = "Time", ylab = "MCF")
```

The exposure-adjusted MCF produces lower estimates because all three systems remain in the risk set for the full observation period -- even at times beyond their last event.

### Using Exposure Results with MCF

The `exposure()` function returns per-system end-of-observation times in its `end_times` field, which can be passed directly to `mcf()`:

```{r}
id    <- c(1, 1, 1, 2, 2, 2, 3, 3, 3)
time  <- c(100, 350, 500, 80, 300, 400, 150, 250, 700)
event <- c(  1,   1,   0,  1,   1,   0,   1,   1,   1)

exp_result <- exposure(id, time, event)
mcf_result <- mcf(id, time, event, end_time = exp_result$end_times)
```

This workflow ensures the MCF and exposure calculations use the same observation windows.

## Exposure Analysis

**Exposure** measures the total operating time during which events can occur across all systems in a fleet. It is fundamental to repairable systems analysis because event rates are only meaningful when normalized by the time during which events could have been observed.

For $k$ systems observed up to times $T_1, T_2, \ldots, T_k$, the total exposure is:

$$
E = \sum_{i=1}^{k} T_i
$$

The cumulative exposure at time $t$ is:

$$
E(t) = \sum_{i=1}^{k} \min(t, T_i)
$$

Each system contributes time up to the lesser of $t$ or its observation end. The **event rate** is then $r(t) = N(t) / E(t)$, the cumulative number of events per unit exposure.

### Example

Using the same three-system fleet from the MCF example:

```{r}
id   <- c(1, 1, 1, 2, 2, 2, 3, 3, 3, 3)
time <- c(100, 350, 500, 80, 300, 600, 150, 250, 400, 700)
result <- exposure(id, time)
```

The `exposure()` function returns a table showing, at each event time, the number of systems at risk, cumulative exposure, cumulative events, and the event rate.

### Multi-Panel Plot

The default plot method produces a three-panel dashboard:

```{r fig.height=8}
plot(result)
```

The top panel shows cumulative exposure (left axis) and cumulative events (right axis, blue). The middle panel tracks the number of systems under observation. The bottom panel shows the cumulative event rate (events per unit exposure).

### Single Panel Plots

Individual panels can be selected with the `which` argument:

```{r}
plot(result, which = "exposure")
```

```{r}
plot(result, which = "event_rate")
```

### Exposure with Censoring

When systems are withdrawn before the end of study, their observation time still contributes to exposure up to the censoring point:

```{r}
id    <- c(1, 1, 1, 2, 2, 2, 3, 3, 3)
time  <- c(100, 350, 500, 80, 300, 400, 150, 250, 700)
event <- c(  1,   1,   0,  1,   1,   0,   1,   1,   1)

result <- exposure(id, time, event)
```

System 1 (censored at 500) and System 2 (censored at 400) contribute exposure up to their respective censoring times, but only actual events are counted in the cumulative event tally.

## Power Law NHPP

The **Power Law NHPP** models the cumulative number of events as:

$$
N(t) = \lambda\, t^{\beta}
$$

The parameter $\beta$ indicates the trend:

* $\beta > 1$: deteriorating system (increasing event rate)
* $\beta = 1$: constant rate (Homogeneous Poisson Process)
* $\beta < 1$: improving system (decreasing event rate)

### Example

```{r}
time  <- c(200, 400, 600, 800, 1000, 1200, 1400, 1600, 1800, 2000)
event <- c(  3,   5,   4,   7,    6,    8,    5,    9,    7,   10)
```

Fit using Maximum Likelihood Estimation (default):

```{r}
fit_mle <- nhpp(time, event, method = "MLE")
plot(fit_mle, main = "Power Law NHPP (MLE)",
     xlab = "Cumulative Time", ylab = "Cumulative Events")
```

Fit using Least Squares:

```{r}
fit_ls <- nhpp(time, event, method = "LS")
```

Both methods estimate similar parameters. MLE is generally preferred for its statistical properties, while LS provides a simple regression-based approach and supports piecewise models.

## Log-Linear NHPP

The **Log-Linear NHPP** models the event intensity as:

$$
\lambda(t) = \exp(a + bt)
$$

with cumulative function:

$$
\Lambda(t) = \frac{e^a}{b}\left(e^{bt} - 1\right)
$$

The parameter $b$ controls the trend: $b > 0$ means an increasing rate, $b < 0$ a decreasing rate, and $b \approx 0$ a constant rate.

### Example

```{r}
result_ll <- nhpp(time, event, model_type = "Log-Linear")
plot(result_ll, main = "Log-Linear NHPP",
     xlab = "Cumulative Time", ylab = "Cumulative Events")
```

## Segmented NHPP

The **Piecewise Power Law NHPP** allows different parameters in different time segments. This is useful when a system's behavior changes -- for example, after a major overhaul or design change.

### With User-Specified Breakpoints

```{r}
time  <- c(100, 200, 300, 400, 500, 600, 700, 800, 900, 1000,
           1100, 1200, 1300, 1400, 1500)
event <- c(  1,   1,   2,   4,   4,   1,   1,   2,   1,    4,
             1,   1,   3,   3,   4)

result_pw <- nhpp(time, event, breaks = c(500), method = "LS")
plot(result_pw, main = "Piecewise Power Law NHPP",
     xlab = "Cumulative Time", ylab = "Cumulative Events")
```

The vertical dashed line indicates the change point at time 500. Each segment has its own $\beta$ and $\lambda$ parameters.

## Forecasting

The `predict_nhpp()` function forecasts cumulative events at future times using a fitted NHPP model.

### Example

```{r}
time  <- c(200, 400, 600, 800, 1000, 1200, 1400, 1600, 1800, 2000)
event <- c(  3,   5,   4,   7,    6,    8,    5,    9,    7,   10)

fit <- nhpp(time, event, method = "MLE")
fc <- predict_nhpp(fit, time = c(2001, 2500, 3000, 4000, 5000))
```

Plot the observed data with the forecast:

```{r}
plot(fc, main = "NHPP Forecast",
     xlab = "Cumulative Time", ylab = "Cumulative Events")
```

The plot shows the observed data (points), the fitted model (solid line), and the forecast (dashed line) with confidence bounds. The vertical gray line marks the boundary between observed and forecast periods.

Forecasting also works with Log-Linear models:

```{r}
fit_ll <- nhpp(time, event, model_type = "Log-Linear")
fc_ll <- predict_nhpp(fit_ll, time = c(2500, 3000))
```

## Summary

NHPP models provide a flexible framework for analyzing repairable systems:

* The **MCF** gives a non-parametric view of the recurrence pattern, useful for exploratory analysis and comparing systems.
* **Exposure analysis** quantifies the total time at risk and event rates, providing essential context for interpreting event counts.
* The **Power Law NHPP** is the most widely used parametric model, with $\beta$ directly indicating whether the system is improving or deteriorating.
* The **Log-Linear NHPP** offers an alternative parametric form for modeling time-dependent intensity.
* **Piecewise models** capture changes in system behavior across different operational phases.
* **Forecasting** with `predict_nhpp()` supports planning for spare parts, maintenance scheduling, and warranty analysis.

---
title: "AI Integration via MCP"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{AI Integration via MCP}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

## Overview

ReliaGrowR can expose its core analysis functions as **Model Context Protocol (MCP)
tools**, allowing AI assistants such as Claude to call them directly during a
conversation. This is powered by the
[mcptools](https://posit-dev.github.io/mcptools/) package from Posit.

Once configured, an AI assistant can:

- Fit reliability growth models on data you describe
- Forecast future failures or MTBF
- Run goodness-of-fit tests
- Plan reliability demonstration tests
- Analyze repairable systems with MCF and NHPP models

## Installation

Install the required packages:

```r
install.packages("mcptools")   # MCP server framework
install.packages("ellmer")     # Tool definition helpers (already in ReliaGrowR Suggests)
```

## Starting the MCP Server

The server is started with a single call:

```r
ReliaGrowR::rga_mcp_server()
```

By default this uses stdio transport (suitable for Claude Code and Claude
Desktop). To use HTTP transport instead:

```r
ReliaGrowR::rga_mcp_server(type = "http", port = 8080)
```

## Configuring Claude Code

Add the server to Claude Code from your terminal:

```bash
claude mcp add -s user reliagrowR -- Rscript -e "ReliaGrowR::rga_mcp_server()"
```

The `-s user` flag stores the configuration in your user-level settings so it is
available in every project.

## Configuring Claude Desktop

Add the following block to `claude_desktop_config.json`
(`~/Library/Application Support/Claude/claude_desktop_config.json` on macOS):

```json
{
  "mcpServers": {
    "reliagrowR": {
      "command": "Rscript",
      "args": ["-e", "ReliaGrowR::rga_mcp_server()"]
    }
  }
}
```

Restart Claude Desktop after saving.

## Available Tools

| Tool | Function | Description |
|---|---|---|
| `rga` | `rga()` | Crow-AMSAA reliability growth model |
| `nhpp` | `nhpp()` | NHPP Power Law / Log-Linear for repairable systems |
| `duane` | `duane()` | Duane log-log regression |
| `mcf` | `mcf()` | Mean Cumulative Function (Nelson-Aalen) |
| `predict_rga` | `predict_rga()` | Forecast cumulative failures from RGA model |
| `predict_duane` | `predict_duane()` | Forecast MTBF from Duane model |
| `rdt` | `rdt()` | Reliability Demonstration Test planning |
| `gof_rga` | `gof()` | Goodness-of-fit statistics (CvM, K-S) |

## Example Session

With the MCP server running, you can ask Claude questions like:

> "I have failure data with times [100, 200, 300, 400, 500] and failure counts
> [1, 2, 1, 3, 2]. Fit a Crow-AMSAA reliability growth model and forecast the
> cumulative failures at 1000 and 2000 hours."

Claude will call `rga` and `predict_rga` on your behalf and return the results
in plain language.

> "Plan a reliability demonstration test for 90% reliability at 500 hours with
> 90% confidence, using a Weibull model with beta = 1.5 and 10 test units."

Claude will call `rdt` and explain the required test duration.

## Security Considerations

The MCP server runs R code in your local R session. Only share the server
endpoint with trusted clients. For multi-user deployments, consider running the
server in a sandboxed environment.

---
title: "Reliability Demonstration Test Planning"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reliability Demonstration Test Planning}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

```{r setup}
library(ReliaGrowR)
```

## Overview

A **Reliability Demonstration Test (RDT)** provides statistical evidence that a
product meets its reliability requirement. The `rdt()` function computes either:

- the **required test time** given a fixed sample size, or
- the **required sample size** given a fixed test duration.

The calculation uses the Weibull distribution; setting `beta = 1` (the default)
gives the exponential special case.

## Test Time

For a zero-failure test plan (`f = 0`), the required cumulative test time $T$
for $n$ units satisfying reliability $R$ at mission time $t_m$ with confidence
$C$ is:

$$T = t_m \left(\frac{-\ln C}{-\ln R}\right)^{1/\beta}$$

When allowable failures $f > 0$, the chi-squared quantile replaces $-\ln C$:

$$T = \frac{t_m}{(-\ln R)^{1/\beta}} \cdot \left(\frac{\chi^2_{1-C,\,2(f+1)}}{2n}\right)^{1/\beta}$$

## Solving for Test Time

Suppose you need to demonstrate 90% reliability at 500 hours with 90%
confidence, and you have 20 units available for testing:

```{r solve-time}
plan <- rdt(
  target       = 0.90,   # 90% reliability
  mission_time = 500,    # hours
  conf_level   = 0.90,   # 90% confidence
  n            = 20      # 20 test units
)
print(plan)
```

Each unit must be tested for `Required_Test_Time` hours (or failure, whichever
comes first) with zero failures allowed.

## Solving for Sample Size

Alternatively, fix the test duration at 800 hours per unit:

```{r solve-n}
plan2 <- rdt(
  target       = 0.90,
  mission_time = 500,
  conf_level   = 0.90,
  test_time    = 800
)
print(plan2)
```

## Effect of Weibull Shape

For products with wear-out (`beta > 1`) or infant mortality (`beta < 1`), the
Weibull shape parameter changes the required test time. Here we compare three
scenarios:

```{r weibull-shape}
betas <- c(0.8, 1.0, 1.5)
for (b in betas) {
  p <- rdt(target = 0.90, mission_time = 500, conf_level = 0.90, n = 20, beta = b)
  cat(sprintf("beta = %.1f  ->  required test time = %.1f hours\n",
              b, p$Required_Test_Time))
}
```

Higher beta (wear-out) demands shorter tests because failures concentrate near
end-of-life; lower beta (infant mortality) demands longer tests.

## Allowing Failures

A zero-failure plan is conservative. Allowing a small number of failures can
substantially reduce the test burden:

```{r allow-failures}
for (f in 0:3) {
  p <- rdt(target = 0.90, mission_time = 500, conf_level = 0.90, n = 20, f = f)
  cat(sprintf("f = %d  ->  required test time = %.1f hours\n",
              f, p$Required_Test_Time))
}
```

## Summary

| Parameter | Role |
|---|---|
| `target` | Required reliability R(t_m) |
| `mission_time` | The time at which reliability is evaluated |
| `conf_level` | Statistical confidence in the demonstration |
| `beta` | Weibull shape (1 = exponential) |
| `f` | Allowable failures (0 = most conservative) |
| `n` | Sample size → solves for test time |
| `test_time` | Test duration per unit → solves for sample size |

---
title: "Reliability Growth Forecasting"
output:
  html_vignette:
    fig_width: 7
    fig_height: 6
vignette: >
  %\VignetteIndexEntry{Reliability Growth Forecasting}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, message = FALSE, warning = FALSE}
library(ReliaGrowR)
library(WeibullR)
```

## Introduction

Reliability Growth Analysis (RGA) tracks the improvement (or degradation) of system reliability over a test campaign as changes are made. However, translating an observed growth trend into a fleet-level life forecast requires several additional steps that are often performed separately, leading to potential inconsistencies, loss of fidelity in reliability projections, and an incomplete picture of future system performance. This study introduces a novel integrated workflow that systematically connects RGA to life estimation through stochastic simulation, providing a robust framework to propagate observed reliability growth into predictive fleet-level life distributions.  The RGA forecast determines the cumulative time over which a target number of additional failures is expected, `sim_failures()` generates conditional failure times for the surviving units over that horizon, and the simulated failures are combined with observed test data to fit a growth-adjusted Weibull distribution, offering a dynamic and data-driven representation of reliability that evolves with observed improvements. This integration provides a more comprehensive and realistic fleet-level life distribution forecast by explicitly accounting for reliability improvements, thereby avoiding underestimation of future reliability or misallocation of resources often associated with fragmented analyses. A parallel Weibull fit to the test data alone (without growth) provides a baseline for the comparison. Monte Carlo analysis quantifies the variability introduced by the simulation, and sensitivity analyses examine how the results depend on the number of test failures and the strength of the reliability growth trend. 

## Life Data Analysis

Suppose a reliability test is conducted on 100 units over a 1000 time-unit campaign, during which 20 failures occur. The failure mechanism is wear-out ($\beta$>1), so the hazard rate increases with age. During the test, design improvements were introduced to mitigate the wear-out mechanism, producing a reliability growth trend in the system-level failure intensity. Eighty surviving units remained at risk during the forecast period. The failure and suspension times were as follows:

```{r}
failures <- c(
  500, 600, 650, 700, 730, 760, 790, 810, 830, 845,
  860, 875, 890, 905, 920, 935, 945, 960, 975, 990
)
suspensions <- rep(1000, 80)

n_test_failures <- length(failures)
n_surviving <- length(suspensions)
```

The `WeibullR` package is used to fit a Weibull distribution to the failure and suspension data. The `wblr()` function creates a Weibull object, and `wblr.fit()` fits the model:

```{r}
obj <- wblr(failures, suspensions,
  col = "steelblue", label = "Test Data", is.plot.cb = FALSE
)
obj <- wblr.fit(obj)
sim_beta <- obj$fit[[1]]$beta
eta_orig <- obj$fit[[1]]$eta
plot(obj,
  main = "Weibull Fit to Test Data",
  xlab = "Operating Time", ylab = "Unreliability"
)
```

The estimated shape parameter $\beta$ is `r round(sim_beta, 3)` and the scale parameter $\eta$ is `r round(eta_orig, 1)`. A shape parameter greater than one confirms the wear-out failure mode.

## Reliability Growth Analysis

The Weibull life data are converted into the cumulative-time and failure-count format required for reliability growth analysis with `weibull_to_rga()`. The function handles the exact failure times and right-censored suspensions, producing a data frame with cumulative time and failure counts that can be used to fit a Crow-AMSAA model.

```{r}
rga_data <- weibull_to_rga(failures, suspensions)
rga_input <- data.frame(
  times = rga_data$CumulativeTime,
  failures = rga_data$Failures
)
```

The Crow-AMSAA model is fitted with `rga()`. The growth parameter $\beta_{\text{growth}}$ governs the system-level failure intensity trend and is distinct from the Weibull shape parameter $\beta$ estimated above: the Crow-AMSAA $\beta_{\text{growth}}$ describes whether the rate of failure accumulation is increasing or decreasing over cumulative operating time, whereas the Weibull $\beta$ describes the unit-level hazard function.

```{r}
test_end_cum_time <- max(rga_input$times)
fit <- rga(rga_input, times_type = "cumulative_failure_times")
growth_beta <- fit$betas
growth_lambda <- fit$lambdas
```

```{r results = "asis"}
plot(fit,
  main = "Reliability Growth Analysis",
  xlab = "Cumulative Time", ylab = "Cumulative Failures"
)
```

The cumulative operating time at test end is `r format(test_end_cum_time, big.mark = ",")` units. The fitted growth parameter $\beta_{\text{growth}}$ is `r round(growth_beta, 3)`. A value less than one indicates reliability improvement over the test campaign.

## Reliability Growth Forecast

The Crow-AMSAA model provides an intensity function that can be extrapolated to forecast future failures. The cumulative failure function is $N(t) = \lambda \, t^{\beta_{\text{growth}}}$, and solving for the cumulative time $t_f$ at which a target number of total failures is reached gives:

$$t_f = \left(\frac{N_{\text{target}}}{\lambda}\right)^{1/\beta_{\text{growth}}}$$

The forecast targets 60 additional failures beyond the `r n_test_failures` observed during the test, for a total of `r n_test_failures + 60` cumulative failures. This represents 75% of the surviving fleet, leaving 20 units as right-censored suspensions in the combined dataset.

```{r}
n_forecast <- 60
n_target <- n_test_failures + n_forecast

t_forecast <- (n_target / growth_lambda)^(1 / growth_beta)
delta_cum_time <- t_forecast - test_end_cum_time
effective_fleet <- n_surviving - n_forecast / 2
per_unit_time <- delta_cum_time / effective_fleet
```

The forecasted cumulative time to `r n_target` total failures is `r format(round(t_forecast), big.mark = ",")` units, requiring an additional `r format(round(delta_cum_time), big.mark = ",")` cumulative time units beyond the test end. Because units that fail during the forecast period stop accumulating operating time, a simple division by `r n_surviving` would underestimate the per-unit horizon. An attrition-adjusted effective fleet size of `r effective_fleet` accounts for this by assuming that, on average, each failing unit contributes roughly half a window of operating time. The resulting per-unit simulation horizon is `r round(per_unit_time, 1)` additional operating time units.

```{r warnings = FALSE}
forecast_times <- seq(test_end_cum_time, t_forecast, length.out = 50)
fc <- predict_rga(fit, forecast_times)
plot(fc,
  main = "Reliability Growth Forecast",
  xlab = "Cumulative Time", ylab = "Cumulative Failures"
)
```

## Simulating Failures

The `sim_failures()` function generates conditional Weibull failure times for the surviving units. Each unit has already accumulated 1000 operating time units, and the simulation draws failure times from the conditional Weibull distribution over the forecast horizon derived above. The function internally calibrates the Weibull scale parameter $\eta$ so that the expected number of conditional failures over the horizon matches the target count of `r n_forecast`.

```{r}
set.seed(123)
sim_result <- sim_failures(
  n = n_forecast,
  runtimes = rep(1000, n_surviving),
  window = per_unit_time,
  beta = sim_beta
)
sim_eta <- attr(sim_result, "weibull_eta")
```

The calibrated scale parameter is $\eta$ = `r round(sim_eta, 1)`, compared to the original test-data estimate of $\eta$ = `r round(eta_orig, 1)`. The increase in $\eta$ reflects the reliability growth: the growth-adjusted failure rate is lower, so a larger characteristic life is needed to match the forecasted failure count. Of the `r n_surviving` surviving units, `r n_forecast` receive simulated failure times and `r n_surviving - n_forecast` are right-censored at the end of the forecast horizon.

The simulated failures are combined with the `r n_test_failures` test failures:

```{r}
sim_fail_times <- sim_result$runtime[sim_result$type == "Failure"]
sim_susp_times <- sim_result$runtime[sim_result$type == "Suspension"]
combined_failures <- c(failures, sim_fail_times)
combined_suspensions <- sim_susp_times
```

### Weibull Comparison

Two Weibull models are fitted to contrast the effect of reliability growth:

- **Without growth**: fitted to the original test data (`r n_test_failures` failures, `r n_surviving` suspensions at $t = 1000$). This represents the life distribution implied by the test alone, without projecting the growth trend forward.
- **With growth**: fitted to the combined dataset (`r n_test_failures` test failures + `r n_forecast` simulated failures + `r n_surviving - n_forecast` suspensions). This represents the life distribution when growth-adjusted failure times are included.

```{r}
obj_growth <- wblr(combined_failures, combined_suspensions,
  col = "tomato", label = "With Growth", is.plot.cb = FALSE
)
obj_growth <- wblr.fit(obj_growth)

obj_nogrowth <- wblr(failures, suspensions,
  col = "grey40", label = "Without Growth", is.plot.cb = FALSE
)
obj_nogrowth <- wblr.fit(obj_nogrowth)

plot.wblr(list(obj_nogrowth, obj_growth),
  main = "Weibull Comparison: With vs. Without Reliability Growth",
  is.plot.legend = TRUE
)
```

```{r}
growth_wb_beta <- obj_growth$fit[[1]]$beta
growth_wb_eta <- obj_growth$fit[[1]]$eta
nogrowth_wb_beta <- obj_nogrowth$fit[[1]]$beta
nogrowth_wb_eta <- obj_nogrowth$fit[[1]]$eta

params <- data.frame(
  Scenario = c("Without Growth", "With Growth"),
  Beta = round(c(nogrowth_wb_beta, growth_wb_beta), 3),
  Eta = round(c(nogrowth_wb_eta, growth_wb_eta), 1)
)

knitr::kable(params,
  caption = "Weibull parameters: with vs. without reliability growth",
  col.names = c("Scenario", "\u03b2", "\u03b7"),
  row.names = FALSE
)
```

Under reliability growth, the combined dataset includes simulated failure times that extend beyond the original test horizon, producing a larger estimated $\eta$ (rightward shift in the life distribution). Both $\beta$ and $\eta$ are freely estimated in each fit.

Because the combined dataset merges test-era failures (generated under the original failure distribution) with simulated post-growth failures (generated under a larger $\eta$), the data do not follow a single Weibull distribution exactly. On a Weibull probability plot the two populations produce a concave pattern, which pulls the fitted $\beta$ downward. The $\eta$ comparison remains the primary indicator of the growth effect; the $\beta$ change should be interpreted as a mixture artifact rather than a shift in the underlying failure mechanism.

## Monte Carlo Analysis

A single call to `sim_failures()` produces one realization of the `r n_forecast` simulated failure times. Different random draws yield different combined datasets and therefore different Weibull parameter estimates. The Monte Carlo analysis repeats the simulation 500 times to characterize this variability.

### Run the Monte Carlo Loop

Each iteration draws a new set of `r n_forecast` simulated failure times, combines them with the `r n_test_failures` test failures, and fits a Weibull distribution. Both $\beta$ and $\eta$ are freely estimated in every iteration.

```{r mc-helpers, include = FALSE}
sim_and_fit <- function(test_failures, n_sim, runtimes, window, beta) {
  sim_i <- sim_failures(
    n = n_sim, runtimes = runtimes, window = window, beta = beta
  )
  sim_f <- sim_i$runtime[sim_i$type == "Failure"]
  sim_s <- sim_i$runtime[sim_i$type == "Suspension"]
  obj_i <- wblr(c(test_failures, sim_f), sim_s, is.plot.cb = FALSE)
  obj_i <- wblr.fit(obj_i)
  data.frame(beta = obj_i$fit[[1]]$beta, eta = obj_i$fit[[1]]$eta)
}
```

```{r mc-loop, message = FALSE, warning = FALSE, results = "hide"}
set.seed(99)
n_mc <- 500
mc_results <- vector("list", n_mc)

for (i in seq_len(n_mc)) {
  tryCatch(
    {
      mc_results[[i]] <- sim_and_fit(
        failures, n_forecast, rep(1000, n_surviving), per_unit_time, sim_beta
      )
    },
    error = function(e) NULL
  )
}

mc_df <- do.call(rbind, Filter(Negate(is.null), mc_results))
```

### Visualize the Parameter Distributions

Histograms of the fitted $\hat{\beta}$ and $\hat{\eta}$ across all valid iterations show the spread introduced by the stochastic simulation. The dashed line marks the Monte Carlo median, and the dotted grey line marks the no-growth baseline.

```{r mc-histograms, fig.height = 4}
par(mfrow = c(1, 2), mar = c(4, 4, 3, 1))

hist(mc_df$beta,
  breaks = "Sturges", col = "steelblue", border = "white",
  main = "MC Distribution of \u03b2",
  xlab = expression(hat(beta)), ylab = "Count", freq = TRUE
)
abline(v = median(mc_df$beta), lty = 2, lwd = 2)
abline(v = nogrowth_wb_beta, lty = 3, lwd = 2, col = "grey40")

hist(mc_df$eta,
  breaks = "Sturges", col = "tomato", border = "white",
  main = "MC Distribution of \u03b7",
  xlab = expression(hat(eta)), ylab = "Count", freq = TRUE
)
abline(v = median(mc_df$eta), lty = 2, lwd = 2)
abline(v = nogrowth_wb_eta, lty = 3, lwd = 2, col = "grey40")

par(mfrow = c(1, 1))
```

### Monte Carlo Summary

```{r mc-table}
mc_summary <- data.frame(
  Parameter = c("\u03b2", "\u03b7"),
  NoGrowth = round(c(nogrowth_wb_beta, nogrowth_wb_eta), 3),
  Median = round(c(median(mc_df$beta), median(mc_df$eta)), 3),
  Mean = round(c(mean(mc_df$beta), mean(mc_df$eta)), 3),
  CI_lo = round(c(
    quantile(mc_df$beta, 0.025),
    quantile(mc_df$eta, 0.025)
  ), 3),
  CI_hi = round(c(
    quantile(mc_df$beta, 0.975),
    quantile(mc_df$eta, 0.975)
  ), 3)
)

knitr::kable(mc_summary,
  caption = paste0(
    "Monte Carlo summary of Weibull parameters (",
    nrow(mc_df), " valid iterations)"
  ),
  col.names = c(
    "Parameter", "No Growth", "Median", "Mean", "2.5%", "97.5%"
  ),
  row.names = FALSE
)
```

The Monte Carlo distributions quantify how much the Weibull parameter estimates vary due to the randomness in the simulated failure times. The no-growth baseline falls outside the Monte Carlo distribution when the growth trend produces a meaningfully different life distribution.

## Sensitivity Analysis

The preceding Monte Carlo analysis held the RGA forecast and Weibull input shape parameter fixed while varying the simulated failure draws. This section examines how two key inputs affect the growth-adjusted Weibull: (a) the number of test failures observed during the campaign, and (b) the Crow-AMSAA growth parameter.

### Effect of Number of Test Failures

The number of failures observed during the test affects both the Weibull fit to the test data and the RGA model. More test failures provide more information about the failure distribution but also change the number of remaining units to forecast. Four scenarios are examined: 10, 20, 30, and 40 test failures out of 100 units.

```{r sens-fail-setup}
# Build failure scenarios by subsampling or extending the base failures
fail_10 <- failures[seq(1, 20, by = 2)]
fail_20 <- failures
fail_30 <- sort(c(failures, seq(505, 995, length.out = 10)))
fail_40 <- sort(c(failures, seq(510, 998, length.out = 20)))

test_fail_scenarios <- list(
  "10 failures" = list(f = fail_10, s = rep(1000, 90)),
  "20 failures" = list(f = fail_20, s = rep(1000, 80)),
  "30 failures" = list(f = fail_30, s = rep(1000, 70)),
  "40 failures" = list(f = fail_40, s = rep(1000, 60))
)
```

For each scenario, the full pipeline is executed: fit the Weibull, fit the RGA, forecast remaining failures, simulate with `sim_failures()`, and fit the combined Weibull. A Monte Carlo loop of 200 iterations per scenario captures variability. In each scenario, 75% of the surviving units are targeted for simulated failure (matching the base analysis), with the remainder treated as suspensions.

```{r sens-fail-mc, message = FALSE, warning = FALSE, results = "hide"}
set.seed(42)
n_mc_sens <- 200

sens_fail_list <- lapply(names(test_fail_scenarios), function(sc_name) {
  sc <- test_fail_scenarios[[sc_name]]
  n_f <- length(sc$f)
  n_s <- length(sc$s)
  n_fc <- round(n_s * 0.75) # forecast 75% of survivors

  tryCatch(
    {
      # Fit Weibull
      obj_sc <- wblr(sc$f, sc$s, is.plot.cb = FALSE)
      obj_sc <- wblr.fit(obj_sc)
      beta_sc <- obj_sc$fit[[1]]$beta

      # Fit RGA
      rga_sc <- weibull_to_rga(sc$f, sc$s)
      rga_in <- data.frame(
        times = rga_sc$CumulativeTime,
        failures = rga_sc$Failures
      )
      fit_sc <- rga(rga_in, times_type = "cumulative_failure_times")

      # Forecast
      t_end_sc <- max(rga_in$times)
      n_tgt <- n_f + n_fc
      t_fc <- (n_tgt / fit_sc$lambdas)^(1 / fit_sc$betas)
      delta_sc <- t_fc - t_end_sc
      effective_sc <- n_s - n_fc / 2
      put_sc <- delta_sc / effective_sc

      if (put_sc <= 0) {
        return(NULL)
      }

      # MC loop
      rows <- lapply(seq_len(n_mc_sens), function(i) {
        tryCatch(
          {
            r <- sim_and_fit(sc$f, n_fc, rep(1000, n_s), put_sc, beta_sc)
            cbind(scenario = sc_name, r)
          },
          error = function(e) NULL
        )
      })
      do.call(rbind, Filter(Negate(is.null), rows))
    },
    error = function(e) NULL
  )
})

sens_fail_df <- do.call(rbind, Filter(Negate(is.null), sens_fail_list))
sens_fail_df$scenario <- factor(
  sens_fail_df$scenario,
  levels = names(test_fail_scenarios)
)
```

```{r sens-fail-boxplot, fig.height = 4}
par(mfrow = c(1, 2), mar = c(5, 4, 3, 1))

fail_cols <- c("#2E86C1", "#27AE60", "#F39C12", "#C0392B")
boxplot(beta ~ scenario,
  data = sens_fail_df,
  col = fail_cols, outline = FALSE,
  main = "Fitted \u03b2 by Test Failures",
  xlab = "Scenario", ylab = expression(hat(beta)),
  las = 2, cex.axis = 0.8
)

boxplot(eta ~ scenario,
  data = sens_fail_df,
  col = fail_cols, outline = FALSE,
  main = "Fitted \u03b7 by Test Failures",
  xlab = "Scenario", ylab = expression(hat(eta)),
  las = 2, cex.axis = 0.8
)

par(mfrow = c(1, 1))
```

```{r sens-helpers, include = FALSE}
summarize_mc <- function(df) {
  do.call(rbind, lapply(levels(df$scenario), function(lbl) {
    sub <- df[df$scenario == lbl, ]
    if (nrow(sub) == 0) return(NULL)
    data.frame(
      Scenario = lbl, Valid = nrow(sub),
      Beta_med = round(median(sub$beta), 3),
      Eta_med = round(median(sub$eta), 1),
      Eta_lo = round(quantile(sub$eta, 0.025), 1),
      Eta_hi = round(quantile(sub$eta, 0.975), 1)
    )
  }))
}
```

```{r sens-fail-table}
sens_fail_tbl <- summarize_mc(sens_fail_df)

knitr::kable(sens_fail_tbl,
  caption = paste0(
    "Growth-adjusted Weibull by test-failure scenario (",
    n_mc_sens, " iterations each)"
  ),
  col.names = c(
    "Scenario", "Valid", "Med. \u03b2",
    "Med. \u03b7", "2.5% \u03b7", "97.5% \u03b7"
  ),
  row.names = FALSE
)
```

More test failures leave fewer units to forecast and provide a more constrained Weibull fit from the test data, while fewer test failures leave a larger fleet for simulation and greater forecast uncertainty.

### Effect of Growth Parameter

The Crow-AMSAA growth parameter $\beta_{\text{growth}}$ controls the forecast horizon: stronger growth (smaller $\beta_{\text{growth}}$) implies that more cumulative operating time is needed before the target number of additional failures accumulates, producing longer simulated lifetimes and a larger $\eta$. Four growth scenarios are examined while holding all other inputs at their base values. For each scenario, $\lambda$ is re-anchored so that the model matches the observed failure count at the test end ($\lambda_k = N_{\text{current}} / t_{\text{end}}^{\,\beta_{g,k}}$), isolating the effect of the extrapolation rate from the model fit.

```{r growth-setup}
growth_scenarios <- c(0.4, 0.6, 0.8, 1.0)
growth_labels <- c(
  "\u03b2g = 0.4 (strong)",
  "\u03b2g = 0.6 (moderate)",
  "\u03b2g = 0.8 (mild)",
  "\u03b2g = 1.0 (none)"
)
```

```{r growth-mc, message = FALSE, warning = FALSE, results = "hide"}
set.seed(77)

sens_growth_list <- lapply(seq_along(growth_scenarios), function(k) {
  gb <- growth_scenarios[k]

  # Re-anchor lambda so model matches observed data, then forecast
  lambda_k <- n_test_failures / test_end_cum_time^gb
  t_fc_k <- (n_target / lambda_k)^(1 / gb)
  delta_k <- t_fc_k - test_end_cum_time
  put_k <- delta_k / effective_fleet

  if (put_k <= 0) {
    return(NULL)
  }

  rows <- lapply(seq_len(n_mc_sens), function(i) {
    tryCatch(
      {
        r <- sim_and_fit(
          failures, n_forecast, rep(1000, n_surviving), put_k, sim_beta
        )
        cbind(scenario = growth_labels[k], r)
      },
      error = function(e) NULL
    )
  })
  do.call(rbind, Filter(Negate(is.null), rows))
})

sens_growth_df <- do.call(rbind, Filter(Negate(is.null), sens_growth_list))
sens_growth_df$scenario <- factor(sens_growth_df$scenario, levels = growth_labels)
```

```{r growth-boxplot, fig.height = 4}
par(mfrow = c(1, 2), mar = c(5, 4, 3, 1))

growth_cols <- c("#2E86C1", "#27AE60", "#F39C12", "#C0392B")
boxplot(beta ~ scenario,
  data = sens_growth_df,
  col = growth_cols, outline = FALSE,
  main = "Fitted \u03b2 by Growth Strength",
  xlab = "Growth scenario", ylab = expression(hat(beta)),
  las = 2, cex.axis = 0.75
)

boxplot(eta ~ scenario,
  data = sens_growth_df,
  col = growth_cols, outline = FALSE,
  main = "Fitted \u03b7 by Growth Strength",
  xlab = "Growth scenario", ylab = expression(hat(eta)),
  las = 2, cex.axis = 0.75
)

par(mfrow = c(1, 1))
```

```{r growth-table}
sens_growth_tbl <- summarize_mc(sens_growth_df)

knitr::kable(sens_growth_tbl,
  caption = paste0(
    "Growth-adjusted Weibull by growth parameter scenario (",
    n_mc_sens, " iterations each)"
  ),
  col.names = c(
    "Scenario", "Valid", "Med. \u03b2",
    "Med. \u03b7", "2.5% \u03b7", "97.5% \u03b7"
  ),
  row.names = FALSE
)
```

Stronger reliability growth (smaller $\beta_{\text{growth}}$) extends the forecast horizon, producing longer simulated lifetimes and a larger fitted $\eta$ in the combined Weibull. At $\beta_{\text{growth}} = 1$ (no growth), the forecast horizon is shortest and the growth-adjusted distribution is closest to the no-growth baseline.

## Limitations

1. **Extrapolation uncertainty.** The reliability growth forecast extrapolates the observed improvement trend beyond the test horizon. Confidence in the forecast degrades with distance from the observed data, suggesting that forecasts further beyond the test horizon should be interpreted with increasing caution, potentially visualized with widening uncertainty bands. The simulation results should be interpreted as conditional on the growth trend continuing unchanged.

2. **Model risk.** The Crow-AMSAA power-law model is one of several possible reliability growth models. Alternative models (e.g., piecewise, AMSAA-Bingham) may yield different forecast horizons and therefore different simulated life distributions. Future work could involve comparing the robustness of forecasts across different growth models or incorporating model uncertainty into the overall forecast.

3. **Homogeneous fleet.** The simulation assumes all surviving units follow a single conditional Weibull distribution. If the fleet contains distinct subpopulations (e.g., different duty cycles or hardware revisions), separate analyses should be performed for each group.

4. **Simulation horizon sensitivity.** The per-unit remaining time derived from the RGA forecast directly controls the simulated failure distribution. Errors in the growth parameters propagate through to the Weibull comparison, as demonstrated in the sensitivity analysis.

5. **Repair policy.** The simulation assumes failed units are not replaced. The combined dataset treats all simulated failures as first-failure events for each unit.

6. **Input shape parameter.** The `sim_failures()` call uses the Weibull shape parameter from the initial life data fit as input to the conditional failure model. The Monte Carlo analysis captures variability in the fitted parameters on the combined data, but the input shape to the simulation is held constant across iterations. This simplification may lead to an underestimation of total forecast uncertainty; future enhancements could involve sampling this input shape from a distribution to more fully capture its variability.

7. **Mixture distribution.** The combined dataset is a mixture of pre-growth and post-growth failure populations. A single Weibull fitted to this mixture is an approximation; the reduction in $\beta$ relative to the no-growth fit reflects the mixture shape rather than a change in the wear-out mechanism.

## Conclusion

This analysis demonstrated an end-to-end reliability growth forecasting pipeline. The Crow-AMSAA growth model determined the cumulative time over which a target number of additional failures is expected, and `sim_failures()` generated conditional Weibull failure times for the surviving units over that horizon. Combining the simulated failures with the observed test data produced a growth-adjusted Weibull distribution that was compared against a no-growth baseline fitted to the test data alone. Monte Carlo simulation quantified the variability in both $\beta$ and $\eta$ arising from the stochastic simulation, and sensitivity analyses showed how the results depend on the number of test failures and the strength of the growth trend. Together, these components provide a repeatable methodology for translating observed reliability growth into fleet-level life distribution estimates.

---
title: "Reliability Growth Analysis"
output:
  html_vignette:
    fig_width: 7
    fig_height: 5
vignette: >
  %\VignetteIndexEntry{Reliability Growth Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Introduction

**Reliability Growth Analysis (RGA)** is an important part of reliability engineering, focused on improving system reliability during development and testing. By analyzing failure data, RGA helps engineers identify trends, estimate reliability parameters, and track improvements over time. These insights are essential for assessing the impact of design changes, guiding engineering decisions, and managing risks effectively.

## The Crow-AMSAA Model

The Army Materiel Systems Analysis Activity model by Crow **(Crow-AMSAA)** takes failure behavior as a **Non-Homogeneous Poisson Process (NHPP)** governed by a power law, making the model particularly effective for systems undergoing reliability growth due to continuous improvements.

The Crow-AMSAA model is a statistical model that characterizes the relationship between the cumulative number of failures and cumulative time as the following:

\[
N(t) = \lambda\, t^{\beta}
\]

where \(N(t)\) denotes the cumulative number of failures by time \(t\), \(\lambda\) is a scaling parameter, and \(\beta\) is the shape parameter.  

The shape parameter \(\beta\) indicates whether the system’s reliability is improving or deteriorating.

* If \(\beta\) < 1, then the system is improving (reliability is growing, the failure rate is decreasing).
* If \(\beta\) = 1, then there is no change in reliability (the system is stable).
* If \(\beta\) > 1, then reliability is worsening (the failure rate is increasing).

### Example

To use the Crow-AMSAA model in ReliaGrowR, use the `rga` function. This function takes a vector of failure times and a vector of failure counts, and generates a plot of cumulative MTBF vs cumulative time with the fitted model.

First, load the package:
```{r}
library(ReliaGrowR)
```

Next, set up some time and failure data:
```{r}
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

Then run the rga and plot the results:
```{r}
result <- rga(times, failures)
plot(result, main = "Crow-AMSAA Model", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

The plot shows the cumulative failures on the y-axis and cumulative time on the x-axis, with a fitted curve indicating the reliability growth trend.

## The Piecewise NHPP Model

**The Piecewise NHPP model** is an extension of the standard NHPP model that includes different segments or phases of time that follow separate failure distributions. This model is particularly useful when a system experiences changes in failure behavior over different development phases, such as the initial, interim and final phases of a development process.

For a Piecewise NHPP model, the cumulative number of failures is modeled as a piecewise function, where each segment has its own parameters. Formally, for time t within phase i, the relationship between the cumulative number of failures and cumulative time is given by:

\[
N_i(t) = N(t_{i-1}) + \lambda_i (t - t_{i-1})^{\beta_i}, \, \text{for } t_{i-1} < t \leq t_i
\]

where \(N_i(t)\) is the cumulative number of failures by time \(t\) in phase \(i\), \(\lambda_i\) is the scale parameter for phase \(i\), and \(\beta_i\) is the shape parameter for phase \(i\).

### Example

To use the Piecewise NHPP model in ReliaGrowR, first, set up some time and failure data and specify a breakpoint:
```{r}
times <- c(25, 55, 97, 146, 201, 268, 341, 423, 513, 609, 710, 820, 940, 1072, 1217)
failures <- c(1, 1, 2, 4, 4, 1, 1, 2, 1, 4, 1, 1, 3, 3, 4)
breaks <- 500
```

Then run the rga and plot the results:
```{r}
result <- rga(times, failures, model_type = "Piecewise NHPP", breaks = c(breaks))
plot(result, main = "Piecewise NHPP Model", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

## The Piecewise NHPP with Change Point Detection

The Piecewise NHPP with **Change Point Detection** is an advanced model to identify changes in failure behavior and model system reliability. This method builds on the Piecewise NHPP model by introducing the concept of change points, which represent the time when the underlying failure behavior changes. Detection of change points involves statistical techniques that analyze failure data to automatically identify when the behavior changes, allowing for a more precise segmentation of the model into different distributions.

### Example

To use the Piecewise NHPP with Change Point Detection in ReliaGrowR, use the `rga` function with the model type set to "Piecewise NHPP" and breaks set to NULL. The function will automatically detect change points based on the provided failure data. First, set up some time and failure data:
```{r}
times <- c(25, 55, 97, 146, 201, 268, 341, 423, 513, 609, 710, 820, 940, 1072, 1217)
failures <- c(1, 1, 2, 4, 4, 1, 1, 2, 1, 4, 1, 1, 3, 3, 4)
```

Then run the rga and plot the results:
```{r}
result <- rga(times, failures, model_type = "Piecewise NHPP")
plot(result, main = "Piecewise NHPP with Change Point Detection", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

## The Duane Model

**The Duane model** provides a simple and graphical way to observe and analyze whether failure rates are improving as changes are made to a product or system. The Duane Model is a log-log plot of the cumulative Mean Time Between Failures (MTBF) vs cumulative time.

The slope of the line on the plot indicates the rate of reliability growth:

* A positive slope means that the system is improving (reliability is growing, the failure rate is decreasing).
* A zero slope means there is no change in reliability (the system is stable).
* A negative slope indicates that reliability is worsening (the failure rate is increasing).

To use the Duane Model in ReliaGrowR, use the `duane_plot` function. This function takes a a vector of failure times and a vector of failure counts, and generates a log-log plot of cumulative MTBF vs cumulative time.

### Example

First, set up some time and failure data:
```{r}
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
```

Then fit the model and plot the results:
```{r}
fit <- duane(times, failures)
plot(fit, main = "Duane Plot", xlab = "Cumulative Time", ylab = "Cumulative MTBF")
```

The plot shows the cumulative MTBF on the y-axis and cumulative time on the x-axis, with a fitted line indicating the reliability growth trend. 

## Goodness of Fit

Goodness of fit plots help assess how well a model matches the observed data. Two key functions are `qqplot.rga` for Q-Q plots and `ppplot.rga` for P-P plots. Both functions take an rga object as input.

```{r echo=TRUE}
times <- c(50, 100, 150, 200, 300, 400, 600, 800, 1000)
failures <- c(1, 1, 2, 2, 3, 4, 5, 6, 8)
result <- rga(times, failures)
qqplot.rga(result)
```

The `qqplot.rga` function produces a Q-Q plot that compares the quantiles of the observed data with those of the fitted model. A good fit is indicated when the points closely follow the 45-degree reference line. 

Similarly, a P-P plot can be created with:

```{r echo=TRUE}
ppplot.rga(result)
```

The `ppplot.rga` function compares the cumulative probabilities of the observed data with those of the fitted model. As with the Q-Q plot, a good fit is indicated when the points closely follow the 45-degree line.

## Data Conversion

Reliability data can be recorded in different formats, such as exact failure times, right-censored times, or failure times grouped into intervals. The `weibull_to_rga` utility function converts data from the format used in the WeibullR package into the format required for RGA analysis in ReliaGrowR. It accepts a vector of failure times and optional vectors of right-censored (suspension) times and interval-censored times, then returns a data frame with cumulative times and failure counts suitable for use with the rga function.

For example, consider the following dataset with failure times, right-censored times, and interval-censored times:

```{r echo=TRUE}
failures <- c(100, 200, 200, 400)
suspensions <- c(250, 350, 450)
interval_starts <- c(150, 300)
interval_ends <- c(180, 320)
```

The data can be converted using `weibull_to_rga`:

```{r echo=TRUE}
result <- weibull_to_rga(failures, suspensions, interval_starts, interval_ends)
```

The resulting data frame contains cumulative times and failure counts, which can then be passed directly to the `rga` function for reliability growth analysis.

```{r echo=TRUE}
fit <- rga(result$CumulativeTime, result$Failures)
plot(fit, main = "RGA from Converted Data", xlab = "Cumulative Time", ylab = "Cumulative Failures")
```

## Summary 

RGA provides a powerful statistical framework for tracking and predicting how a system's reliability evolves throughout development. Models like Crow-AMSAA, Piecewise NHPP, and the Duane Model allow engineers and analysts to quantify improvements (or regressions) in reliability, understand system behavior across development phases, detect significant shifts in failure patterns, and guide design changes and process improvements.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exposure.R
\name{print.exposure}
\alias{print.exposure}
\title{Print Method for exposure Objects.}
\usage{
\method{print}{exposure}(x, ...)
}
\arguments{
\item{x}{An object of class \code{exposure}.}

\item{...}{Additional arguments (not used).}
}
\value{
Invisibly returns the input object.
}
\description{
Prints a summary of the exposure analysis results.
}
\examples{
id   <- c(1, 1, 2, 2)
time <- c(100, 200, 150, 300)
result <- exposure(id, time)
print(result)
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdt.R
\name{print.rdt}
\alias{print.rdt}
\title{Print method for rdt objects}
\usage{
\method{print}{rdt}(x, ...)
}
\arguments{
\item{x}{An object of class \code{rdt}.}

\item{...}{Additional arguments (not used).}
}
\value{
Invisibly returns the input object.
}
\description{
This function provides a formatted print method for objects of class \code{rdt}.
}
\examples{
plan <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, n = 10)
print(plan)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exposure.R
\name{plot.exposure}
\alias{plot.exposure}
\title{Plot Method for exposure Objects.}
\usage{
\method{plot}{exposure}(
  x,
  which = c("all", "exposure", "at_risk", "event_rate"),
  legend = TRUE,
  legend_pos = "topleft",
  ...
)
}
\arguments{
\item{x}{An object of class \code{exposure}.}

\item{which}{Character string selecting which panel(s) to plot. One of
\code{"all"} (default), \code{"exposure"}, \code{"at_risk"}, or
\code{"event_rate"}.}

\item{legend}{Logical; show the legend (default: TRUE).}

\item{legend_pos}{Position of the legend (default: "topleft").}

\item{...}{Additional arguments passed to the underlying \code{plot()}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Produces a multi-panel plot of exposure analysis results. The default
layout shows cumulative exposure and cumulative events versus time
(top panel), the number of systems at risk over time (middle panel),
and the event rate over time (bottom panel). Alternatively, a single
\code{which} panel can be selected.
}
\examples{
id   <- c(1, 1, 1, 2, 2, 2, 3, 3, 3, 3)
time <- c(100, 350, 500, 80, 300, 600, 150, 250, 400, 700)
result <- exposure(id, time)
plot(result)
plot(result, which = "exposure")
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{print.rga}
\alias{print.rga}
\title{Print method for rga objects.}
\usage{
\method{print}{rga}(x, ...)
}
\arguments{
\item{x}{An object of class \code{rga}, which contains the results from the RGA model.}

\item{...}{Additional arguments (not used).}
}
\value{
Invisibly returns the input object.
}
\description{
This function prints a summary of the results from an object of class \code{rga}.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
result <- rga(times, failures)
print(result)
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{overlay_rga}()},
\code{\link{plot.rga}()},
\code{\link{plot.rga_predict}()},
\code{\link{predict_rga}()},
\code{\link{print.rga_predict}()},
\code{\link{rga}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{plot.rga_predict}
\alias{plot.rga_predict}
\title{Plot Method for rga_predict Objects}
\usage{
\method{plot}{rga_predict}(x, conf_bounds = TRUE, legend = TRUE, legend_pos = "bottomright", ...)
}
\arguments{
\item{x}{An object of class \code{rga_predict}.}

\item{conf_bounds}{Logical; include confidence bounds (default: \code{TRUE}).}

\item{legend}{Logical; show the legend (default: \code{TRUE}).}

\item{legend_pos}{Position of the legend (default: \code{"bottomright"}).}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Plots observed data, the fitted reliability growth curve, and the forecast
with optional confidence bounds for an \code{rga_predict} object.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- rga(times, failures)
fc <- predict_rga(fit, times = c(1500, 2000))
plot(fc, main = "RGA Forecast", xlab = "Cumulative Time", ylab = "Cumulative Failures")
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{overlay_rga}()},
\code{\link{plot.rga}()},
\code{\link{predict_rga}()},
\code{\link{print.rga}()},
\code{\link{print.rga_predict}()},
\code{\link{rga}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{predict_rga}
\alias{predict_rga}
\title{Forecast Cumulative Failures from a Reliability Growth Model}
\usage{
predict_rga(object, times, conf_level = 0.95)
}
\arguments{
\item{object}{An object of class \code{rga} returned by \code{rga()}.}

\item{times}{A numeric vector of cumulative times at which to forecast.
All values must be finite and > 0. A warning is issued if any value is
at or below the maximum observed cumulative time (hindcasting).}

\item{conf_level}{The desired confidence level (default \code{0.95}). Must
be a single finite numeric in (0, 1).}
}
\value{
An object of class \code{rga_predict} containing:
\item{times}{The forecast cumulative times.}
\item{cum_failures}{Predicted cumulative failures.}
\item{lower_bounds}{Lower confidence bounds.}
\item{upper_bounds}{Upper confidence bounds.}
\item{conf_level}{The confidence level used.}
\item{model_type}{Either \code{"Crow-AMSAA"} or \code{"Piecewise NHPP"}.}
\item{rga_object}{The original \code{rga} object (used by the plot method).}
}
\description{
Takes a fitted \code{rga} object and a vector of cumulative times, returning
predicted cumulative failures with confidence bounds as an \code{rga_predict}
S3 object.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- rga(times, failures)
fc <- predict_rga(fit, times = c(1500, 2000))
print(fc)
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{overlay_rga}()},
\code{\link{plot.rga}()},
\code{\link{plot.rga_predict}()},
\code{\link{print.rga}()},
\code{\link{print.rga_predict}()},
\code{\link{rga}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testdata.r
\docType{data}
\name{testdata}
\alias{testdata}
\title{Reliability Test Data}
\format{
@format ## \code{testdata}
A data frame with 25 rows and 6 variables:
\describe{
\item{LRU}{The Line Replaceable Unit identifier (G1 or G2).}
\item{Cum_ETI}{Cumulative Equivalent Test Hours (ETI).}
\item{Failure_Count}{Cumulative number of failures observed.}
\item{Cum_MTBF}{Cumulative Mean Time Between Failures (MTBF).}
\item{Report_No}{Report number associated with the failure.}
\item{Flag}{A flag indicating special conditions or notes.}
\item{Cause}{Cause of the failure (e.g., D for Design, M for Manufacturing, R for Random, NR for No Report).}
}
@usage data(testdata)
}
\usage{
testdata
}
\description{
A dataset containing example reliability test data from the military report
"Reliability Growth Prediction" (1986) by The Analytical Sciences Corporation.
This dataset includes cumulative ETI, failure counts, cumulative MTBF, report
numbers, flags, and causes for two different LRUs (G1 and G2).
}
\examples{
data(testdata)
head(testdata)
summary(testdata)
str(testdata)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nhpp.R
\name{predict_nhpp}
\alias{predict_nhpp}
\title{Forecast Cumulative Events from an NHPP Model.}
\usage{
predict_nhpp(object, time, conf_level = 0.95)
}
\arguments{
\item{object}{An object of class \code{nhpp} returned by \code{nhpp()}.}

\item{time}{A numeric vector of cumulative times at which to forecast.
All values must be finite and > 0.}

\item{conf_level}{Confidence level (default 0.95).}
}
\value{
An object of class \code{nhpp_predict} containing:
\item{time}{Forecast times.}
\item{cum_events}{Predicted cumulative events.}
\item{lower_bounds}{Lower confidence bounds.}
\item{upper_bounds}{Upper confidence bounds.}
\item{conf_level}{Confidence level used.}
\item{model_type}{Model type.}
\item{nhpp_object}{The original nhpp object.}
}
\description{
Takes a fitted \code{nhpp} object and a vector of future cumulative times,
returning predicted cumulative events with confidence bounds.
}
\examples{
time <- c(200, 400, 600, 800, 1000)
event <- c(3, 5, 4, 7, 6)
fit <- nhpp(time, event)
fc <- predict_nhpp(fit, time = c(1500, 2000))
print(fc)
plot(fc, main = "NHPP Forecast", xlab = "Time", ylab = "Cumulative Events")
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nhpp.R
\name{print.nhpp}
\alias{print.nhpp}
\title{Print Method for nhpp Objects.}
\usage{
\method{print}{nhpp}(x, ...)
}
\arguments{
\item{x}{An object of class \code{nhpp}.}

\item{...}{Additional arguments (not used).}
}
\value{
Invisibly returns the input object.
}
\description{
Prints a summary of the NHPP model results.
}
\examples{
time <- c(200, 400, 600, 800, 1000)
event <- c(3, 5, 4, 7, 6)
result <- nhpp(time, event)
print(result)
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exposure.R
\name{exposure}
\alias{exposure}
\title{Exposure Analysis for Repairable Systems.}
\usage{
exposure(id = NULL, time = NULL, event = NULL, data = NULL)
}
\arguments{
\item{id}{A vector of system/unit identifiers. Each unique value
represents a distinct system.}

\item{time}{A numeric vector of event or censoring times. Must be
positive and finite.}

\item{event}{An optional numeric vector of event indicators: 1 for an
event, 0 for censoring (end of observation). If \code{NULL} (default),
all observations are treated as events, and the maximum time per system
is treated as the end of observation.}

\item{data}{An optional data frame containing columns named \code{id},
\code{time}, and optionally \code{event}.}
}
\value{
An object of class \code{exposure} containing:
\item{time}{Sorted unique event times (excluding censoring-only times).}
\item{n_at_risk}{Number of systems under observation at each event time.}
\item{cum_exposure}{Cumulative total exposure (system-time) up to each
event time.}
\item{cum_events}{Cumulative number of events up to each event time.}
\item{event_rate}{Cumulative event rate (cum_events / cum_exposure) at
each event time.}
\item{total_exposure}{Total exposure across all systems and the full
observation period.}
\item{total_events}{Total number of events.}
\item{n_systems}{Number of distinct systems.}
\item{end_times}{Named numeric vector of end-of-observation times per
system. Can be passed directly to \code{mcf(end_time = ...)} to ensure
the MCF properly accounts for system exposure.}
}
\description{
Computes exposure (total operating time at risk) across one or more
repairable systems as a function of time. Exposure is defined as the
total accumulated observation time summed across all systems still under
observation. The function also computes the number of systems at risk
and the event rate (events per unit exposure) at each event time.
}
\details{
\strong{Exposure} is the total amount of operating time during which events
can occur. For a fleet of \eqn{k} systems observed up to times
\eqn{T_1, T_2, \ldots, T_k}, the total exposure is
\eqn{E = \sum_{i=1}^{k} T_i}{E = T_1 + T_2 + ... + T_k}.

The \strong{cumulative exposure} at time \eqn{t} is
\eqn{E(t) = \sum_{i=1}^{k} \min(t, T_i)}{E(t) = sum of min(t, T_i)},
i.e., each system contributes time up to the lesser of \eqn{t} or its
observation end.

The \strong{event rate} at time \eqn{t} is the cumulative number of events
divided by the cumulative exposure: \eqn{r(t) = N(t) / E(t)}.
}
\examples{
id   <- c(1, 1, 1, 2, 2, 2, 3, 3, 3, 3)
time <- c(100, 350, 500, 80, 300, 600, 150, 250, 400, 700)
result <- exposure(id, time)
print(result)
plot(result)

# With censoring
id    <- c(1, 1, 1, 2, 2, 2, 3, 3, 3)
time  <- c(100, 350, 500, 80, 300, 400, 150, 250, 700)
event <- c(  1,   1,   0,  1,   1,   0,   1,   1,   1)
result2 <- exposure(id, time, event)
print(result2)
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof.R
\name{qqplot.rga}
\alias{qqplot.rga}
\title{Q-Q Plot for RGA Objects}
\usage{
qqplot.rga(x, main = "Q-Q Plot", ...)
}
\arguments{
\item{x}{An object of class \code{rga}.}

\item{main}{Title of the plot.}

\item{...}{Additional arguments passed to \code{stats::qqplot()}.}
}
\value{
A Q-Q plot comparing empirical and theoretical quantiles.
}
\description{
This function creates a Q-Q plot for a fitted Reliability Growth Analysis (RGA) model
Currently only supports the Crow-AMSAA model. A Q-Q plot compares the quantiles of
the empirical data to the quantiles of the theoretical distribution specified by the model.
If the model fits well, the points should fall approximately along a straight line.
}
\examples{
times <- c(5, 10, 15, 20, 25)
failures <- c(1, 2, 1, 3, 2)
fit <- rga(times, failures)
qqplot.rga(fit)
}
\seealso{
Other goodness-of-fit: 
\code{\link{ppplot.rga}()}
}
\concept{goodness-of-fit}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{rga}
\alias{rga}
\title{Reliability Growth Analysis.}
\usage{
rga(
  times,
  failures,
  times_type = c("failure_times", "cumulative_failure_times"),
  model_type = "Crow-AMSAA",
  breaks = NULL,
  conf_level = 0.95,
  method = c("LS", "MLE")
)
}
\arguments{
\item{times}{Either a numeric vector of failure-time inputs or a data frame
containing both time inputs and failure counts. If \code{times_type = "failure_times"}
(default), \code{times} is treated exactly as in previous versions of the function
and is cumulatively summed inside \code{rga()}. If
\code{times_type = "cumulative_failure_times"}, \code{times} is treated as already
cumulative and is used directly without applying \code{cumsum()}. If a data frame
is provided, it must contain two columns: \code{times} and \code{failures}.}

\item{failures}{A numeric vector of the number of failures at each corresponding time
in times. Must be the same length as \code{times} if both are vectors. All values must be
positive and finite. Ignored if \code{times} is a data frame.}

\item{times_type}{Character scalar indicating how to interpret \code{times}.
\code{"failure_times"} (default) preserves the current behavior and cumulatively
sums \code{times} inside \code{rga()}. \code{"cumulative_failure_times"} treats \code{times}
as already cumulative and skips that internal \code{cumsum()}.}

\item{model_type}{The model type. Either \code{Crow-AMSAA} (default) or \verb{Piecewise NHPP} with change point detection.}

\item{breaks}{An optional vector of breakpoints for the \verb{Piecewise NHPP} model.}

\item{conf_level}{The desired confidence level, which defaults to 95\%. The confidence
level is the probability that the confidence interval contains the true mean response.}

\item{method}{Estimation method: \code{"LS"} (default) for least-squares
log-log regression, or \code{"MLE"} for maximum likelihood estimation of
the Crow-AMSAA model. \code{"MLE"} is not supported for
\code{model_type = "Piecewise NHPP"}.}
}
\value{
The function returns an object of class \code{rga} that contains:
\item{times}{The input time vector, stored exactly as supplied.}
\item{cum_times}{The cumulative time vector used for fitting.}
\item{times_type}{How \code{times} was interpreted: \code{"failure_times"} or \code{"cumulative_failure_times"}.}
\item{failures}{The input number of failures.}
\item{n_obs}{The number of observations (failures).}
\item{cum_failures}{Cumulative failures.}
\item{model}{The fitted model object (lm (linear model) or segmented).}
\item{residuals}{Model residuals on the log-log scale. These represent deviations of the observed
log cumulative failures from the fitted values and are useful for diagnostic checking.}
\item{logLik}{The log-likelihood of the fitted model. The log-likelihood is a
measure of model fit, with higher values indicating a better fit.}
\item{AIC}{Akaike Information Criterion (AIC). AIC is a measure used for model selection,
with lower values indicating a better fit.}
\item{BIC}{Bayesian Information Criterion(BIC). BIC is another criterion for model selection}
\item{breakpoints}{Breakpoints (log scale) if applicable.}
\item{fitted_values}{Fitted cumulative failures on the original scale.}
\item{lower_bounds}{Lower confidence bounds (original scale).}
\item{upper_bounds}{Upper confidence bounds (original scale).}
\item{betas}{Estimated beta(s). Betas are the slopes of the log-log plot.}
\item{betas_se}{Standard error(s) of the estimated beta(s).}
\item{growth_rate}{Estimated growth rate(s). Growth rates are calculated as 1 - beta.}
\item{lambdas}{Estimated lambda(s). Lambdas are the intercepts of the log-log plot.}
}
\description{
This function performs reliability growth analysis using the Crow-AMSAA model by
Crow (1975) (AMSAATR138) or piecewise
NHPP model by Guo et al. (2010) \url{doi:10.1109/RAMS.2010.5448029}. It fits
a log-log linear regression of cumulative failures versus cumulative time. The
function accepts either two numeric vectors (\code{times}, \code{failures}) or a data frame
containing both. The \verb{Piecewise NHPP} model can automatically detect change points
or use user-specified breakpoints.
}
\details{
The scaling relationship between the size of input data (numbers of observations)
and speed of algorithm execution is approximately linear (O(n)). The function is
efficient and can handle large data sets (e.g., thousands of observations) quickly.
The function uses the \code{segmented} package for piecewise regression, which employs
an iterative algorithm to estimate breakpoints. The number of iterations required
for convergence may vary depending on the data and initial values.
In practice, the function typically converges within a few iterations for most data sets.
However, in some cases, especially with complex data or poor initial values,
it may take more iterations.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
result1 <- rga(times, failures)
print(result1)

df <- data.frame(times = times, failures = failures)
result2 <- rga(df)
print(result2)

cum_times <- cumsum(times)
result2b <- rga(cum_times, failures, times_type = "cumulative_failure_times")
print(result2b)

result3 <- rga(times, failures, model_type = "Piecewise NHPP")
print(result3)

result4 <- rga(times, failures, model_type = "Piecewise NHPP", breaks = c(450))
print(result4)
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{overlay_rga}()},
\code{\link{plot.rga}()},
\code{\link{plot.rga_predict}()},
\code{\link{predict_rga}()},
\code{\link{print.rga}()},
\code{\link{print.rga_predict}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim_failures.R
\name{sim_failures}
\alias{sim_failures}
\title{Simulate Failures from a Conditional Weibull Model}
\usage{
sim_failures(n, runtimes, replace = FALSE, window = NULL, beta = 1, eta = NULL)
}
\arguments{
\item{n}{Positive integer. Number of failures to simulate.}

\item{runtimes}{Numeric vector of positive values. The current operating
runtime of each unit in the non-failed population.}

\item{replace}{Logical scalar. If \code{TRUE}, sampling is done with replacement
(a unit may be selected more than once). Default is \code{FALSE}.}

\item{window}{\code{NULL} or a single positive numeric. The width of the
observation window. When \code{NULL} (default), event times equal current
runtimes. When provided, failure times are sampled from the conditional
Weibull distribution over \verb{(runtime, runtime + window]}, and suspension
times are \code{runtime + window}.}

\item{beta}{Positive numeric scalar. Weibull shape parameter used to model
the age-dependent hazard. Defaults to \code{1}, corresponding to an
exponential-process assumption.}

\item{eta}{\code{NULL} or a single positive numeric. Weibull scale parameter.
When \code{NULL} and \code{window} is supplied, the scale is calibrated so that the
expected number of failures across the fleet during the window matches
\code{n}.}
}
\value{
A data frame with \code{length(runtimes)} rows sorted ascending by
\code{runtime}, containing:
\item{index}{Integer index of the unit in \code{runtimes}.}
\item{runtime}{Simulated event time.}
\item{type}{Character; \code{"Failure"} for selected units, \code{"Suspension"} for
the rest.}
The returned object also carries attributes \code{weibull_beta} and
\code{weibull_eta} describing the Weibull parameters used for the simulation.
}
\description{
Simulates which units in a non-failed population fail next by using a
Weibull life model conditional on each unit's current runtime. When a
positive \code{window} is supplied, the function calibrates a Weibull scale
parameter (unless provided directly) so that the expected number of failures
within the window matches \code{n}. Units are then sampled with probability
proportional to their conditional Weibull failure probability over the
window, and failure times are drawn from the truncated conditional Weibull
distribution. The full fleet is returned: selected units are labelled
\code{"Failure"} and the remaining units are labelled \code{"Suspension"}.
}
\details{
When \code{window = NULL}, the function returns the current fleet state at the
supplied runtimes. In this case, failing units are selected using relative
Weibull hazard weights implied by \code{beta}.
}
\examples{
set.seed(42)
runtimes <- c(100, 500, 200, 800, 300)
result <- sim_failures(2, runtimes, beta = 1.5)
print(result)

# With an observation window
set.seed(42)
result_w <- sim_failures(2, runtimes, window = 50, beta = 1.5)
print(result_w)
}
\seealso{
Other data preparation: 
\code{\link{weibull_to_rga}()}
}
\concept{data preparation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReliaGrowR-package.R
\docType{package}
\name{ReliaGrowR-package}
\alias{ReliaGrowR}
\alias{ReliaGrowR-package}
\title{ReliaGrowR: Reliability Growth Analysis}
\description{
Modeling and plotting functions for Reliability Growth Analysis (RGA).
The package implements three families of models and provides a REST API
interface via \link[plumber:deprecated_r6]{plumber}.
}
\section{Reliability Growth Models}{


\strong{Crow-AMSAA (NHPP power-law)}

The core model fits a Non-Homogeneous Poisson Process (NHPP) with a
Weibull intensity function to cumulative failure data. Parameters are
estimated by least-squares log-log regression (default) or maximum
likelihood.  A growth rate > 0 indicates reliability improvement.

\strong{Piecewise NHPP}

Extends Crow-AMSAA by fitting separate NHPP segments separated by
change points.  Change points can be detected automatically via the
\link[segmented:segmented]{segmented} package or supplied by the user.

\strong{Duane}

Log-log regression of cumulative MTBF versus cumulative time, providing
a graphical and analytical representation of reliability growth.
}

\section{Main Functions}{
\tabular{ll}{
   Function \tab Description \cr
   \code{\link[=rga]{rga()}} \tab Fit Crow-AMSAA or Piecewise NHPP model \cr
   \code{\link[=predict_rga]{predict_rga()}} \tab Forecast cumulative failures from a fitted model \cr
   \code{\link[=duane]{duane()}} \tab Fit Duane model \cr
   \code{\link[=rdt]{rdt()}} \tab Reliability Demonstration Test plan calculator \cr
   \code{\link[=weibull_to_rga]{weibull_to_rga()}} \tab Convert Weibull data to RGA format \cr
   \code{\link[=sim_failures]{sim_failures()}} \tab Simulate failures from a conditional Weibull model \cr
   \code{\link[=qqplot.rga]{qqplot.rga()}} \tab Q-Q goodness-of-fit plot for an \code{rga} object \cr
   \code{\link[=ppplot.rga]{ppplot.rga()}} \tab P-P goodness-of-fit plot for an \code{rga} object \cr
   \code{\link[=grwr_api]{grwr_api()}} \tab Launch the plumber REST API \cr
}
}

\section{S3 Classes and Methods}{
\tabular{ll}{
   Class \tab Methods \cr
   \code{rga} \tab \code{\link[=print.rga]{print.rga()}}, \code{\link[=plot.rga]{plot.rga()}} \cr
   \code{rga_predict} \tab \code{\link[=print.rga_predict]{print.rga_predict()}}, \code{\link[=plot.rga_predict]{plot.rga_predict()}} \cr
   \code{duane} \tab \code{\link[=print.duane]{print.duane()}}, \code{\link[=plot.duane]{plot.duane()}} \cr
   \code{rdt} \tab \code{\link[=print.rdt]{print.rdt()}} \cr
}
}

\section{References}{

Crow, L. H. (1975). \emph{Reliability Analysis for Complex Repairable Systems.}
AMSAA Technical Report No. 138. US Army Materiel Systems Analysis Activity.

Duane, J. T. (1964). Learning curve approach to reliability monitoring.
\emph{IEEE Transactions on Aerospace}, 2(2), 563–566.
\doi{10.1109/TA.1964.4319640}

Guo, H., Mettas, A., Sarakakis, G., & Niu, P. (2010). Piecewise NHPP
models with maximum likelihood estimation for repairable systems. In
\emph{Proceedings of the 2010 Annual Reliability and Maintainability Symposium}
(pp. 1–6). IEEE. \doi{10.1109/RAMS.2010.5448029}

Muggeo, V. M. R. (2024). \emph{segmented: Regression Models with Break-Points /
Change-Points Estimation.} R package.
\url{https://cran.r-project.org/package=segmented}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://paulgovan.github.io/ReliaGrowR/}
  \item \url{https://github.com/paulgovan/ReliaGrowR}
  \item Report bugs at \url{https://github.com/paulgovan/ReliaGrowR/issues}
}

}
\author{
\strong{Maintainer}: Paul Govan \email{paul.govan2@gmail.com} (\href{https://orcid.org/0000-0002-1821-8492}{ORCID}) [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duane.R
\name{print.duane}
\alias{print.duane}
\title{Print method for duane objects.}
\usage{
\method{print}{duane}(x, ...)
}
\arguments{
\item{x}{An object of class "duane" returned by the duane_plot function.}

\item{...}{Additional arguments (not used).}
}
\value{
Invisibly returns the input object.
}
\description{
This function prints a summary of the Duane analysis result.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- duane(times, failures)
print(fit)
}
\seealso{
Other Duane functions: 
\code{\link{duane}()},
\code{\link{plot.duane}()}
}
\concept{Duane functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcf.R
\name{plot.mcf}
\alias{plot.mcf}
\title{Plot Method for mcf Objects.}
\usage{
\method{plot}{mcf}(x, conf_bounds = TRUE, legend = TRUE, legend_pos = "topleft", ...)
}
\arguments{
\item{x}{An object of class \code{mcf}.}

\item{conf_bounds}{Logical; include confidence bounds (default: TRUE).}

\item{legend}{Logical; show the legend (default: TRUE).}

\item{legend_pos}{Position of the legend (default: "topleft").}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Plots the Mean Cumulative Function with optional confidence bounds.
}
\examples{
id <- c(1, 1, 1, 2, 2, 3, 3, 3, 3)
time <- c(100, 300, 500, 150, 400, 50, 200, 350, 600)
result <- mcf(id, time)
plot(result, main = "Mean Cumulative Function")
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{print.rga_predict}
\alias{print.rga_predict}
\title{Print Method for rga_predict Objects}
\usage{
\method{print}{rga_predict}(x, ...)
}
\arguments{
\item{x}{An object of class \code{rga_predict}.}

\item{...}{Additional arguments (not used).}
}
\value{
Invisibly returns the input object.
}
\description{
Prints a formatted table of forecast cumulative failures with confidence
bounds for an \code{rga_predict} object.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- rga(times, failures)
fc <- predict_rga(fit, times = c(1500, 2000))
print(fc)
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{overlay_rga}()},
\code{\link{plot.rga}()},
\code{\link{plot.rga_predict}()},
\code{\link{predict_rga}()},
\code{\link{print.rga}()},
\code{\link{rga}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nhpp.R
\name{plot.nhpp}
\alias{plot.nhpp}
\title{Plot Method for nhpp Objects.}
\usage{
\method{plot}{nhpp}(x, conf_bounds = TRUE, legend = TRUE, legend_pos = "topleft", ...)
}
\arguments{
\item{x}{An object of class \code{nhpp}.}

\item{conf_bounds}{Logical; include confidence bounds (default: TRUE).}

\item{legend}{Logical; show the legend (default: TRUE).}

\item{legend_pos}{Position of the legend (default: "topleft").}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Plots observed cumulative events with the fitted NHPP model and optional
confidence bounds.
}
\examples{
time <- c(200, 400, 600, 800, 1000)
event <- c(3, 5, 4, 7, 6)
result <- nhpp(time, event)
plot(result, main = "Power Law NHPP", xlab = "Time", ylab = "Cumulative Events")
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcf.R
\name{mcf}
\alias{mcf}
\title{Mean Cumulative Function for Repairable Systems.}
\usage{
mcf(
  id = NULL,
  time = NULL,
  event = NULL,
  end_time = NULL,
  data = NULL,
  conf_level = 0.95
)
}
\arguments{
\item{id}{A vector of system/unit identifiers. Each unique value represents
a distinct system. Ignored if \code{data} is provided.}

\item{time}{A numeric vector of event or censoring times. Must be positive
and finite. Ignored if \code{data} is provided.}

\item{event}{An optional numeric vector of event indicators: 1 for an event,
0 for censoring (end of observation). If \code{NULL} (default), all
observations are treated as events.}

\item{end_time}{An optional named numeric vector of end-of-observation times
per system, where names correspond to system identifiers. This defines
the actual exposure window for each system. When provided, a system
remains in the risk set until its \code{end_time}, even if its last event
occurred earlier. If \code{NULL} (default), the end of observation is
inferred as the maximum time recorded for each system (from both events
and censoring records).}

\item{data}{An optional data frame containing columns named \code{id},
\code{time}, and optionally \code{event} and \code{end_time}.}

\item{conf_level}{Confidence level for bounds (default 0.95).}
}
\value{
An object of class \code{mcf} containing:
\item{time}{Unique event times.}
\item{mcf}{MCF values at each event time.}
\item{variance}{Variance estimates at each event time.}
\item{lower_bounds}{Lower confidence bounds.}
\item{upper_bounds}{Upper confidence bounds.}
\item{conf_level}{Confidence level used.}
\item{n_systems}{Number of distinct systems.}
\item{n_events}{Total number of events.}
\item{end_times}{Named vector of end-of-observation times per system.}
}
\description{
Computes the non-parametric Mean Cumulative Function (MCF) for recurrent
event data from one or more repairable systems, using the Nelson-Aalen
estimator. The MCF estimates the expected cumulative number of events per
system as a function of time, properly accounting for system exposure
(observation windows).
}
\details{
The MCF at time \eqn{t} is estimated as:
\deqn{\hat{M}(t) = \sum_{t_j \le t} \frac{d_j}{n_j}}
where \eqn{d_j} is the number of events at time \eqn{t_j} and \eqn{n_j} is
the number of systems still under observation at \eqn{t_j}. Variance is
estimated as \eqn{\hat{V}(t) = \sum_{t_j \le t} d_j / n_j^2}.

The risk set \eqn{n_j} is determined by each system's \strong{exposure window}.
A system is considered at risk at time \eqn{t_j} if its end-of-observation
time (from \code{end_time}, censoring records, or last event) is
\eqn{\ge t_j}. Specifying \code{end_time} is important when systems were
observed beyond their last event -- without it, the MCF may overestimate
the true recurrence rate because systems with no late events are assumed to
have left observation at their last event time.
}
\examples{
# Basic usage (end of observation inferred from last event)
id <- c(1, 1, 1, 2, 2, 3, 3, 3, 3)
time <- c(100, 300, 500, 150, 400, 50, 200, 350, 600)
result <- mcf(id, time)
print(result)
plot(result, main = "Mean Cumulative Function")

# With explicit end-of-observation times (exposure-adjusted)
end_time <- c("1" = 800, "2" = 800, "3" = 800)
result2 <- mcf(id, time, end_time = end_time)
print(result2)

df <- data.frame(id = id, time = time)
result3 <- mcf(data = df)
print(result3)
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{nhpp}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weibull_to_rga.R
\name{weibull_to_rga}
\alias{weibull_to_rga}
\title{Weibull to RGA}
\usage{
weibull_to_rga(
  failures,
  suspensions = NULL,
  interval_starts = NULL,
  interval_ends = NULL
)
}
\arguments{
\item{failures}{A numeric vector of exact failure times. Each failure time
indicates when an item failed during the observation period.}

\item{suspensions}{A numeric vector of suspension (right-censored) times. A suspension
indicates that the item was removed from observation at that time without failure.
This parameter is optional and can be NULL if there are no suspensions.}

\item{interval_starts}{A numeric vector of interval start times (lower bound of censoring).
This parameter is optional and can be NULL if there are no interval-censored data.
If provided, it must be the same length as \code{interval_ends}.}

\item{interval_ends}{A numeric vector of interval end times (upper bound of censoring).
This parameter is optional and can be NULL if there are no interval-censored data.
If provided, it must be the same length as \code{interval_starts}.}
}
\value{
The data frame contains two columns:
\item{CumulativeTime}{Cumulative time at each failure event.}
\item{Failures}{Number of failures at each cumulative time point.}
The function approximates interval-censored failures by placing them at the midpoint
of the interval.
}
\description{
Converts Weibull data (failure, suspension, and interval-censored times)
into a format suitable for reliability growth analysis (RGA). The function
handles exact failure times, right-censored suspensions, and interval-censored data.
It approximates interval-censored failures by placing them at the midpoint of the interval.
The output is a data frame with cumulative time and failure counts. This format
can be used with RGA models such as Crow-AMSAA.
}
\examples{
failures <- c(100, 200, 200, 400)
suspensions <- c(250, 350, 450)
interval_starts <- c(150, 300)
interval_ends <- c(180, 320)
result <- weibull_to_rga(failures, suspensions, interval_starts, interval_ends)
print(result)
}
\seealso{
Other data preparation: 
\code{\link{sim_failures}()}
}
\concept{data preparation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duane.R
\name{duane}
\alias{duane}
\title{Duane Analysis}
\usage{
duane(times, failures = NULL, conf.level = 0.95)
}
\arguments{
\item{times}{Either:
\itemize{
\item A numeric vector of exact failure times, or
\item A data frame containing two columns: \code{times} and \code{failures}. The \code{times}
column contains exact failure times, and the \code{failures} column
contains the number of failures at each corresponding time.
}}

\item{failures}{A numeric vector of the number of failures at each corresponding
time in \code{times}. Ignored if \code{times} is a data frame. Must be the same length as
\code{times} if both are vectors.
All values must be positive and finite.}

\item{conf.level}{Confidence level for the confidence bounds (default: \code{0.95}).
Must be between 0 and 1 (exclusive).}
}
\value{
A list of class \code{"duane"} containing:
\item{times}{The input exact failure times.}
\item{failures}{The input number of failures.}
\item{n_obs}{The number of observations (failures).}
\item{MTBF}{The cumulative mean time between failures.}
\item{model}{The fitted \code{lm} (linear model) object containing the regression results.}
\item{logLik}{The log-likelihood of the fitted model.}
\item{AIC}{Akaike Information Criterion (AIC).}
\item{BIC}{Bayesian Information Criterion (BIC).}
\item{conf.level}{The confidence level.}
\item{Cumulative_Time}{The cumulative operating times.}
\item{Cumulative_MTBF}{The cumulative mean time between failures.}
\item{Fitted_Values}{The fitted values on the MTBF scale.}
\item{Confidence_Bounds}{Matrix of fitted values and confidence bounds on the MTBF scale.}
\item{Residuals_Log}{Residuals on the log(MTBF) scale (from the regression).}
\item{Residuals_MTBF}{Residuals on the MTBF scale (observed - fitted).}
}
\description{
This function performs a Duane analysis (1962) \url{doi:10.1109/TA.1964.4319640}
on failure data by fitting a log-log linear regression of cumulative Mean Time
Between Failures (MTBF) versus cumulative time. The function accepts either
two numeric vectors (\code{times}, \code{failures}) or a data frame containing both.
}
\details{
The scaling relationship between the size of input data (numbers of observations)
and speed of algorithm execution is approximately linear (O(n)). The function is
efficient and can handle large data sets (e.g., thousands of observations) quickly.
The function uses base R functions and does not require any additional packages.
The function includes comprehensive input validation and error handling to ensure
robustness. The function is tested with a standard data set from a published paper
and includes unit tests to verify correctness and performance.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit1 <- duane(times, failures, conf.level = 0.90)
print(fit1)

df <- data.frame(times = times, failures = failures)
fit2 <- duane(df, conf.level = 0.95)
print(fit2)

}
\seealso{
Other Duane functions: 
\code{\link{plot.duane}()},
\code{\link{print.duane}()}
}
\concept{Duane functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{plot.rga}
\alias{plot.rga}
\title{Plot Method for RGA Objects}
\usage{
\method{plot}{rga}(
  x,
  conf_bounds = TRUE,
  legend = TRUE,
  log = FALSE,
  legend_pos = "bottomright",
  ...
)
}
\arguments{
\item{x}{An object of class \code{rga}, which contains the results from the RGA model.}

\item{conf_bounds}{Logical; include confidence bounds (default: TRUE).}

\item{legend}{Logical; show the legend (default: TRUE).}

\item{log}{Logical; use a log-log scale (default: FALSE).}

\item{legend_pos}{Position of the legend (default: "bottomright").}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
This function generates plots for objects of class \code{rga}.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
result <- rga(times, failures)
plot(result,
  main = "Reliability Growth Analysis",
  xlab = "Cumulative Time", ylab = "Cumulative Failures"
)
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{overlay_rga}()},
\code{\link{plot.rga_predict}()},
\code{\link{predict_rga}()},
\code{\link{print.rga}()},
\code{\link{print.rga_predict}()},
\code{\link{rga}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rga.R
\name{overlay_rga}
\alias{overlay_rga}
\title{Overlay Plot for Multiple RGA Models}
\usage{
overlay_rga(
  models,
  conf_bounds = TRUE,
  legend = TRUE,
  legend_pos = "bottomright",
  colors = NULL,
  log = FALSE,
  ...
)
}
\arguments{
\item{models}{A named or unnamed list of objects of class \code{rga}.
At least one model must be provided. If the list is named, those names
are used as legend labels; otherwise labels default to
\code{"Model 1"}, \code{"Model 2"}, etc.}

\item{conf_bounds}{Logical; draw confidence bounds for each model
(default: \code{TRUE}).}

\item{legend}{Logical; draw a legend (default: \code{TRUE}).}

\item{legend_pos}{Legend position keyword (default: \code{"bottomright"}).}

\item{colors}{Optional character vector of colors, one per model. If
\code{NULL} (default), \code{palette()} colors are cycled.}

\item{log}{Logical; use log-log axes (default: \code{FALSE}).}

\item{...}{Additional arguments passed to the initial \code{plot()} call
(e.g., \code{main}, \code{xlab}, \code{ylab}). Not forwarded to subsequent
\code{lines()} or \code{points()} calls.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Plots multiple fitted \code{rga} objects on a single set of axes, using
distinct colors per model. Observed data points, fitted lines, and optional
confidence bounds are drawn for every model. Models may have been fit to
different datasets.
}
\examples{
t1 <- c(100, 200, 300, 400, 500)
f1 <- c(1, 2, 1, 3, 2)
t2 <- c(150, 300, 450, 600, 750)
f2 <- c(2, 1, 3, 2, 4)
m1 <- rga(t1, f1)
m2 <- rga(t2, f2)
overlay_rga(list(System_A = m1, System_B = m2),
  main = "RGA Overlay", xlab = "Cumulative Time",
  ylab = "Cumulative Failures"
)
}
\seealso{
Other Reliability Growth Analysis: 
\code{\link{plot.rga}()},
\code{\link{plot.rga_predict}()},
\code{\link{predict_rga}()},
\code{\link{print.rga}()},
\code{\link{print.rga_predict}()},
\code{\link{rga}()}
}
\concept{Reliability Growth Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/duane.R
\name{plot.duane}
\alias{plot.duane}
\title{Plot Method for Duane Analysis}
\usage{
\method{plot}{duane}(
  x,
  log = TRUE,
  conf.int = TRUE,
  legend = TRUE,
  legend.pos = "topleft",
  ...
)
}
\arguments{
\item{x}{An object of class \code{"duane"}.}

\item{log}{Logical; whether to use logarithmic scales for axes (default: \code{TRUE}).}

\item{conf.int}{Logical; whether to plot confidence bounds (default: \code{TRUE}).}

\item{legend}{Logical; whether to include a legend (default: TRUE).}

\item{legend.pos}{Position of the legend (default: "topleft").}

\item{...}{Further arguments passed to \code{plot()}.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Generates a Duane plot (log-log or linear scale) with fitted regression line
and optional confidence bounds.
}
\examples{
times <- c(100, 200, 300, 400, 500)
failures <- c(1, 2, 1, 3, 2)
fit <- duane(times, failures)
plot(fit, main = "Duane Plot", xlab = "Cumulative Time", ylab = "Cumulative MTBF")
}
\seealso{
Other Duane functions: 
\code{\link{duane}()},
\code{\link{print.duane}()}
}
\concept{Duane functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nhpp.R
\name{nhpp}
\alias{nhpp}
\title{Non-Homogeneous Poisson Process Model for Repairable Systems.}
\usage{
nhpp(
  time,
  event = NULL,
  data = NULL,
  model_type = "Power Law",
  breaks = NULL,
  method = c("MLE", "LS"),
  conf_level = 0.95
)
}
\arguments{
\item{time}{A numeric vector of cumulative event times, or a data frame
containing columns \code{time} and optionally \code{event}. All values
must be positive, finite, and strictly increasing.}

\item{event}{An optional numeric vector of event counts at each time. If
\code{NULL} (default), each time is treated as a single event.}

\item{data}{An optional data frame containing columns \code{time} and
optionally \code{event}.}

\item{model_type}{Model type: \code{"Power Law"} (default) or
\code{"Log-Linear"}.}

\item{breaks}{Optional vector of breakpoints for piecewise Power Law model.}

\item{method}{Estimation method: \code{"MLE"} (default) or \code{"LS"}.
\code{"LS"} is not supported for \code{"Log-Linear"} models.}

\item{conf_level}{Confidence level for bounds (default 0.95).}
}
\value{
An object of class \code{nhpp} containing:
\item{time}{The input cumulative event times.}
\item{event}{The event counts.}
\item{cum_events}{Cumulative event counts.}
\item{n_obs}{Number of observations.}
\item{model}{Fitted model object (lm or segmented), or NULL for MLE.}
\item{model_type}{\code{"Power Law"} or \code{"Log-Linear"}.}
\item{method}{\code{"MLE"} or \code{"LS"}.}
\item{params}{Named list of estimated parameters.}
\item{params_se}{Named list of standard errors.}
\item{vcov}{Variance-covariance matrix (MLE only).}
\item{fitted_values}{Fitted cumulative events.}
\item{lower_bounds}{Lower confidence bounds.}
\item{upper_bounds}{Upper confidence bounds.}
\item{residuals}{Model residuals.}
\item{logLik}{Log-likelihood.}
\item{AIC}{Akaike Information Criterion.}
\item{BIC}{Bayesian Information Criterion.}
\item{breakpoints}{Breakpoints (log scale) if piecewise model.}
\item{conf_level}{Confidence level used.}
}
\description{
Fits a parametric NHPP model to recurrent event data from repairable systems.
Supported models include the Power Law process and the Log-Linear process.
The Power Law model can also be fit as a piecewise (segmented) model with
automatic change point detection or user-specified breakpoints.
}
\details{
The \strong{Power Law NHPP} models the cumulative number of events as
\eqn{N(t) = \lambda t^\beta}{N(t) = lambda * t^beta}. The parameter
\eqn{\beta > 1} indicates a deteriorating system (increasing event rate),
\eqn{\beta < 1} an improving system, and \eqn{\beta = 1} a constant rate
(HPP).

The \strong{Log-Linear NHPP} models the intensity as
\eqn{\lambda(t) = \exp(a + bt)}{lambda(t) = exp(a + b*t)} with cumulative
function \eqn{\Lambda(t) = \frac{e^a}{b}(e^{bt} - 1)}{Lambda(t) = (exp(a)/b)(exp(bt) - 1)}.
}
\examples{
time <- c(200, 400, 600, 800, 1000)
event <- c(3, 5, 4, 7, 6)
result <- nhpp(time, event)
print(result)
plot(result, main = "Power Law NHPP")

result_ll <- nhpp(time, event, model_type = "Log-Linear")
print(result_ll)
}
\seealso{
Other Repairable Systems Analysis: 
\code{\link{exposure}()},
\code{\link{mcf}()},
\code{\link{overlay_nhpp}()},
\code{\link{plot.exposure}()},
\code{\link{plot.mcf}()},
\code{\link{plot.nhpp}()},
\code{\link{plot.nhpp_predict}()},
\code{\link{predict_nhpp}()},
\code{\link{print.exposure}()},
\code{\link{print.mcf}()},
\code{\link{print.nhpp}()},
\code{\link{print.nhpp_predict}()}
}
\concept{Repairable Systems Analysis}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grwr_api.R
\name{grwr_api}
\alias{grwr_api}
\title{ReliaGrowR API}
\usage{
grwr_api()
}
\value{
Launches the ReliaGrowR API on a local server.
}
\description{
This function provides an interface to the ReliaGrowR API.#' This function provides an interface to the ReliaGrowR API.
}
\examples{
\dontrun{
grwr_api()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdt.R
\name{rdt}
\alias{rdt}
\title{Reliability Demonstration Test (RDT) Plan Calculator}
\usage{
rdt(
  target,
  mission_time,
  conf_level,
  beta = 1,
  f = 0,
  n = NULL,
  test_time = NULL
)
}
\arguments{
\item{target}{Required reliability at mission time (0 < target < 1).}

\item{mission_time}{Mission duration (time units). Must be greater than 0.}

\item{conf_level}{Desired confidence level (e.g., 0.9 for 90\% confidence). The
confidence level must be between 0 and 1 (exclusive).}

\item{beta}{Weibull shape parameter (beta=1 corresponds to exponential distribution).
Must be greater than 0. Default is 1.}

\item{f}{Number of allowable failures during the test (non-negative integer). Default is 0
(zero-failure test plan). Increasing \code{f} reduces the required test time or sample size at
the cost of accepting more observed failures.}

\item{n}{Sample size (optional, supply if solving for test_time). Must be a positive integer.}

\item{test_time}{Test time per unit (optional, supply if solving for n). Must be greater than 0.}
}
\value{
The function returns an object of class \code{rdt} that contains:
\item{Distribution}{Type of distribution used (Exponential or Weibull).}
\item{Beta}{Weibull shape parameter.}
\item{Allowed_Failures}{Number of allowable failures during the test.}
\item{Target_Reliability}{Specified target reliability.}
\item{Mission_Time}{Specified mission time.}
\item{Required_Test_Time}{Calculated required test time (if n is provided).}
\item{Input_Sample_Size}{Provided sample size (if test_time is calculated).}
\item{Required_Sample_Size}{Calculated required sample size (if test_time is provided).}
\item{Input_Test_Time}{Provided test time (if n is calculated).}
}
\description{
This function calculates the required test time or sample size for a Reliability
Demonstration Test (RDT) based on specified reliability, mission time, confidence
level, and Weibull shape parameter.
}
\examples{
#' # Example 1: Calculate required test time
plan1 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, n = 10)
print(plan1)
# Example 2: Calculate required sample size
plan2 <- rdt(target = 0.9, mission_time = 1000, conf_level = 0.9, beta = 1, test_time = 2000)
print(plan2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gof.R
\name{ppplot.rga}
\alias{ppplot.rga}
\title{P-P Plot for RGA Objects}
\usage{
ppplot.rga(x, main = "P-P Plot", ...)
}
\arguments{
\item{x}{An object of class \code{rga}.}

\item{main}{Title of the plot.}

\item{...}{Additional arguments passed to \code{plot()}.}
}
\value{
A P-P plot comparing empirical and theoretical CDFs.
}
\description{
This function creates a P-P plot for a fitted Reliability Growth Analysis (RGA)
model. Currently only supports the Crow-AMSAA model. A P-P plot compares the
empirical cumulative distribution function (CDF) to the theoretical CDF specified
by the model. If the model fits well, the points should fall approximately along a straight line.
}
\examples{
times <- c(5, 10, 15, 20, 25)
failures <- c(1, 2, 1, 3, 2)
fit <- rga(times, failures)
ppplot.rga(fit)
}
\seealso{
Other goodness-of-fit: 
\code{\link{qqplot.rga}()}
}
\concept{goodness-of-fit}
